<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Mail;

class CancelOrderMailAdminCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'CancelOrder:EmailAdmin';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Send email for cancelled orders';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $orders = $this->cleanFields($this->getCancelledOrders());
        if($orders->count()>0){
            $file = $this->excel($orders);
            $this->email(['sanjayjdn@gmail.com'],$file);
        }
    }

    protected function getCancelledOrders( )
        {
            try{
                $start = \Carbon\Carbon::now()->subDays(1)->startOfDay();
                //$start = \Carbon\Carbon::now()->startOfDay();
                //$start = \Carbon\Carbon::createFromFormat('m/d/Y', '04/03/2019', 'Asia/Kolkata')->startOfDay();//->subHour(8);
                $end = $start->copy()->endOfDay();
                $orders = \App\Models\Order::
                            where(function ($q) use($start,$end) {
                                $q->where('deliveryDateTime', '>=', $start)->where('deliveryDateTime', '<=', $end);
                              })->where(function ($q) {
                                $q->where('status', '=', 'ORDER_CANCELLED');
                              })->get()->map(function($f) {
                                    $f->outletDetails = \App\Outlet::find($f->outlet_id);
                                    $f->deliveryStation = json_decode($f->deliveryDetails)->stationCode;
                                    return $f;
                              });
                $vendorOrders = \App\Models\VendorOrder::
                            where(function ($q) use($start,$end) {
                                $q->where('deliveryDateTime', '>=', $start)->where('deliveryDateTime', '<=', $end);
                              })->where(function ($q) {
                                $q->where('status', '=', 'ORDER_CANCELLED');
                              })->get()->map(function($f) {
                    $x =$f;
                    $x->deliveryStation = $x->stationCode;
                    $x->outletDetails = \App\Outlet::find($x->outlet);

                    return $x;
                    });
                $orders = $orders->toBase()->merge($vendorOrders);
                return $orders;
            }catch(\Exception $e){
                dd($e);
                return response([ 'message'=>'invalid request'],200);
            }
        }

        public function cleanFields( $invoice)
        {
            $invoice = $invoice->map(function($o, $i){
                $invoiceItem = new \stdClass;
                $invoiceItem->serialNo = $i+1;
                $invoiceItem->orderId = $o->orderId;
                //$invoiceItem->vendorName = 'OLF STORE';//todo
                //$invoiceItem->vendorType = 'AGGREGATOR';//todo
                $invoiceItem->outletName = $o->outletDetails->outletName;
                $invoiceItem->deliveryStation = $o->deliveryStation;
                $invoiceItem->orderStatus = $o->status;
                $invoiceItem->reason = $o->status_reason;

                return $invoiceItem;
            });

            return $invoice;
        }
    public function excel($invoice) {

        $paymentsArray = [];

        $invoice = json_decode(json_encode($invoice), true);
        foreach ($invoice as $payment) {
            $paymentsArray[] = $payment;
        }
        $excelFile = 'cancelled_orders_OLFSTORES_admin'.'-'. \Carbon\Carbon::now()->format('Y-M-d').'.xlsx';
        \Excel::store(new \App\Exports\CancelOrderExcel($paymentsArray), $excelFile);
        return storage_path().'/app/'.$excelFile;
    }

	private function email($email, $file){
		Mail::send('mail.emailVendorCancelled',
			  compact('outlet'),
			   function ($message) use($email,$file){
					$message->from('noreply@olfstores.com', 'OLF STORE');
					$message->sender('noreply@olfstores.com','OLF STORE');
					$message->to($email);
					$message->subject('Mark order cancelled for date '. \Carbon\Carbon::now()->subDays(1)->format('m/d/Y').' for olfstores');
					$message->priority(3);
                                        $message->attach($file);
				});

    }

}
