<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use App\Models\Order;
use stdClass;
use App\Models\MenuItem;

define('None', null);

class DataCleaner
{
    public static function extract($row, $index, $format)
    {
        $row['formated'] = sscanf($row[$index], $format);
        return $row;
    }
}

class DataPatcher
{
    static $missing = 0;

    public static function patchItems($processed, $outletID)
    {

        $item = MenuItem::where(function ($query) use ($processed, $outletID) {
            return $query->where('itemName', '=', $processed[0])
                ->where('outlet_id', $outletID);
        })->first();

        if (! $item){
            var_dump("PANIC {$processed[0]} : ({$outletID})");
            DataPatcher::$missing += 1;
        }

        return [
            "itemId" => $item ? $item->id : 0,
            "itemName" =>  $item ? $item->itemName : "",
            "basePrice" =>  $item ? $item->basePrice : 100,
            "description" =>  $item ? $item->description : 0,
            "quantity" => $processed[1],
            "tax" =>  $item ? $item->tax : 0,
            "sellingPrice" =>  $item ? $item->sellingPrice : 0,
            "option" => "",
            "isVegetarian" => $item ? $item->isVegetarian : 'NA',
            "isNonVegetarian" => $item ? (! $item->isVegetarian) : 'NA',
            "cuisine" => $item ? $item->cuisine: "[MULICUISINE]",
            'is_null' => $item ? false : true
        ];
    }

    public static function aggrigatorDetails($outletDetals)
    {
        return [
            'id' => 82261,
            'name' => 'Olf Store',
            'customerCareNumbers' => ['9424331299'],
            'outletId' => $outletDetals[0],
            'outletName' => $outletDetals[1]
        ];
    }

    public static function priceDetails($value)
    {
        return [
            'deliveryCharge' => 0,
            'discountAmount' => $value[20] == '' ? 0 : $value[20],
            'totalAmount' => $value[22],
            'totalMargin' => 0.12, # required
            'otherCharges' => 0,
            'gst' => $value[19],
            'amountPayable' => $value[22]
        ];
    }

    public static function orderStatus($value){
        $status = [
            'Undelivered'=> 'ORDER_UNDELIVERED',
            'delivered'=> 'ORDER_DELIVERED',
            'cancelled'=> 'ORDER_CANCELLED',
            'partial delivered' => 'ORDER_PARTIALLY_DELIVERED'
        ];

        return $status[$value[12]];
    }

    public static function deliveryDetails($payload)
    {

        return [
            'pnr' => '2249012587',
            'trainNo' => $payload[15],
            'trainName' => $payload[15],
            'station' => $payload[10],
            'stationCode' => $payload[10],
            'berth' => $payload[14],
            'coach' => $payload[13],
            'ETA' => $payload[8]
        ];
    }

    public static function vendorPayment($value){
        $orderValue = round(floatval($value[18]), 2);
        $less = $orderValue * 0.31;
        return  $orderValue - $less;
    }

    public static function formatedOrder($order, $menuItems, $deliveryDetails, $priceDetails, $outletDetails, $status, $vendorPayment)
    {

        return [
            'orderId' => $order[0],
            'aggregatorOrderId' => '',
            'customerDetails' => '{"customerId":19567,"customerName":"Nikhil Mishra","email":"nikhil6.mishra@gmail.com","mobile":"9716221914"}',
            'aggregatorDetails' => json_encode($outletDetails), 
            'isAggregatorOrder' => '0',
            'outlet_id' => $outletDetails['outletId'],
            'bookingDate' => $order[7],
            'deliveryDate' => $order[8],
            'priceDetails' => json_encode($priceDetails), 
            'remarks' => 'NA',
            'status' => $status,# 'ORDER_CONFIRMED', # required
            'deleted' => '0',
            'deliveryDetails' => json_encode($deliveryDetails), 
            'orderItems' => json_encode($menuItems), 
            'paymentType' => $order[11],
            'created_at' => $order[7],
            'updated_at' =>  $order[9],
            'comment' => None,
            'status_reason' => None,
            'status_status' => None,
            'pending_status' => None,
            'requestor_user_id' => None,
            'status_history' => None,
            'deliveryDateTime' => $order[8],
            'customerEmail' => 'olforder@gmail.com',
            'discount_code' => None,
            'customerMobile' => '9716221914',
            'deliveryBoy' => None,
            'amountPayable' => $order[22],
            'vendorPayment' => $vendorPayment,
            'remember_token' => None,
            'password' => None,
            'previous_id' => None,
            'previous_type' => None,
            'next_id' => '93348',
            'next_type' => 'App\\Order',
            'vendor' => '0',
            'updated' => '0',
            'api_token' => None,
            'paymentReason' => ''
        ];
    }
}
$KEYS = [
    'orderId',
    'aggregatorOrderId',
    'customerDetails',
    'aggregatorDetails',
    'isAggregatorOrder',
    'outlet_id',
    'bookingDate',
    'deliveryDate',
    'priceDetails',
    'remarks',
    'status',
    'deleted',
    'deliveryDetails',
    'orderItems',
    'paymentType',
    'created_at',
    'updated_at',
    'comment',
    'status_reason',
    'status_status',
    'pending_status',
    'requestor_user_id',
    'status_history',
    'deliveryDateTime',
    'customerEmail',
    'discount_code',
    'customerMobile',
    'deliveryBoy',
    'amountPayable',
    'vendorPayment',
    'remember_token',
    'password',
    'previous_id',
    'previous_type',
    'next_id',
    'next_type',
    'vendor',
    'updated',
    'api_token',
    'paymentReason'
];

class SeedMissing extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    
    protected $signature = 'seed:missing {file}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command to seed Missing Data in Orders from 01/01/2023 to 10/01-2023';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $data = [];
        echo "Reading " . public_path() . '/' . $this->argument('file') . "\n";
        ini_set('auto_detect_line_endings', TRUE);
        $count = 1;
        $order_ids = [];
        $handle = fopen(public_path() . '/' . $this->argument('file'), 'r');
        $seed_data = [];
        
        while (($data = fgetcsv($handle)) !== FALSE) {
            $count += 1;
            $order_ids[] += $data[0];
            $seed_data[] = $data;
        }
        ini_set('auto_detect_line_endings', TRUE);
        $orders = Order::whereIn('orderId', $order_ids)->count();
        echo "Duplicated Entries : " . $orders . "\n";
        echo "Processed " . count($seed_data) . "\n";
        $test_data = array_slice($seed_data, 0, 10, true);
        echo "Testing Data: " . count($test_data) . "\n";

        foreach ($seed_data as $key => $value) {
            $splited = preg_split("/\,/", $value[16]);
            $components = [];

            $item = new stdClass();
            foreach ($splited as $spl) {
                $trimmed = preg_split("/\s/", trim($spl));
                $item->name = join(" ", array_slice($trimmed, 0, -1));
                $item->quantity =  intval(str_split(array_pop($trimmed))[1]);
                $components[] = $item;
            }

            $outlet = \App\Models\Outlet::where('outletName', '=', $value[3])->first();
            if (! $outlet){
                var_dump("PANIC ! {$value[3]} #{$value[0]} Not Found");
            }

            $menuItems = [];
            foreach ($components as $_value) {
                $_item = DataPatcher::patchItems([trim($_value->name), $_value->quantity], $outlet->id ?? 0);
                $menuItems[] = $_item ? $_item: NULL;
            }


            
            $data[] = DataPatcher::formatedOrder($value, $menuItems, DataPatcher::deliveryDetails($value), DataPatcher::priceDetails($value),DataPatcher::aggrigatorDetails([$outlet->id ?? 0, $outlet->outletName ?? 'Not Available']), DataPatcher::orderStatus($value), DataPatcher::vendorPayment($value));
        }

        var_dump("Missing Items :".DataPatcher::$missing." \n Conflicted Orders:  ".$orders."\n");
    }
}
