<?php

namespace App\Domain;

class InvoiceCompiler
{

    private $order;
    private $outlet;
    private $amountInWords;

    function __construct($order, $outlet, $amountInWords)
    {
        $this->order = $order;
        $this->outlet = $outlet;
        $this->amountInWords = $amountInWords;
    }

    public function journeyInfo()
    {
        $customer = json_decode($this->order->customer);
        return [
            ['Billed To', $customer->fullName],
            ['Order_Number', $this->order->orderId],
            ['Payment Mode', $this->order->paymentType],
            ['Booking Date', $this->order->bookingDate],
            ['Delivery Date', $this->order->deliveryDate],
            ['Train No', $this->order->trainNo],
            ['Coach', $this->order->coach],
            ['Seat No', $this->order->berth],
            ['Station', $this->order->stationName],
        ];
    }

    public function deliveryDetails()
    {
        return [
            ['Sold By', $this->outlet->outletName],
            ['Address', $this->outlet->address],
            ['GST No', '23AACCO8285A1ZN'],
            ['FSSAI No', $this->outlet->fssaiNo],
            ['Invoice No', "Invoice/{$this->order->orderId}"],
            ['SAC Code', '996331'],
            ['HSN Code', '999799'],
            ['Place Of Supply', $this->outlet->city.', '.$this->outlet->state],
        ];
    }

    public function getTable()
    {

        $items = (json_decode($this->order->orderItems));
        $subTotal = round(collect($items)->sum(function ($item) {
            return $item->basePrice * $item->quantity;
        }), 3);

	
        $target = $this->outlet->state;
	
        $against = ["M.P", "Madhya Pradesh"];

        
        $cgVaild = in_array($target, $against);
        $sgValid = in_array($target, $against);
        $igValid = !in_array($target, $against);

        $cgst = $cgVaild ? round($this->order->gst / 2, 2) : "";
        $sgst = $sgValid ? round($this->order->gst / 2, 2) :  "";
        $igst = $igValid ? round($this->order->gst, 2) : "";

        $cgL = $cgVaild ? "CGST 2.5%" : "CGST";
        $sgL = $sgValid ? "SGST 2.5%" : "SGST";
        $igL = $igValid ? "IGST 5%" : "IGST";

        $template = '
        <style>
        td {
          padding: 5px;
        }
      </style>
      <!-- 
          [{"itemId":1557,"itemName":"Veg Maharaja Thali","description":"Kadahi Paneer(100g), Seasonal Veg (100g), Dal Makhni(100g), Rice (150g) , Raita(50gm), Butter Chapati (3 pieces), Sweet(1 piece),Papad,Salad and Pickle","basePrice":178.09523809523807358345948159694671630859375,"sellingPrice":187,"taxRate":8.90476190476192641654051840305328369140625,"isVegetarian":true,"quantity":7,"option":null,"foodType":"THALI"}]
       -->
      
      <table>
        <tr>
          <td colspan="2">
            <center>
              <h3 style="text-align: center; padding: 5px">Tax Invoice</h3>
            </center>
          </td>
        </tr>
      
        <tr>
          <td>
            <table cellspacing="5">
              <tr>
                <td colspan="2"><strong>Journey Info</strong></td>
              </tr>
              
              @foreach($jcells as $cell)
              <tr>
                <td><strong>{{$cell[0]}}</strong></td>
                <td>{{$cell[1]}}</td>
              </tr>
            
              @endforeach
            </table>
          </td>
          <td>
            <table cellspacing="5">
              <tr>
                <td colspan="2"><strong>Order/Delivery Info</strong></td>
              </tr>
      
              @foreach($dcells as $cell)
              <tr>
                <td><strong>{{$cell[0]}}</strong></td>
                <td>{{$cell[1]}}</td>
              </tr>
            
              @endforeach
            </table>
          </td>
        </tr>
      </table>
      
      <table border="1" cellpadding="7">
        <thead>
          <tr>
            <td style="width: 60%">Item</td>
            <td style="width: 10%">Qty</td>
            <td style="width: 20%">Rate</td>
            <td style="width: 10%">Amount</td>
          </tr>
        </thead>
        <tbody>
         @foreach($items as $item)
         <tr>
         <td style="width: 60%">
           <strong>{{$item[0]}}</strong>
         </td>
         <td style="width: 10%">{{$item[1]}}</td>
         <td style="width: 20%">{{round($item[2], 2)}}</td>
         <td style="width: 10%">{{$item[3]}}</td>
       </tr>
         @endforeach
          
         @foreach($cells as $cell)
          <tr>
            <td></td>
            <td></td>
            <td>{{$cell[0]}}</td>
            <td>{{$cell[1]}}</td>
          </tr>
          @endforeach
          
      
          <tr>
            <td colspan="4">Grand Total: {{$grandTotal}}</td>
          </tr>
        </tbody>
      </table>
      
      <ul>
        <li>
          This is an electronically generated invoice, hence, does not require a
          signature.
        </li>
        <li>
          The seller shall be responsible for issues relating to statutory purposes
          like price, quality, taxation, etc.
        </li>
        <li>
          A receipt of delivery charges(if any) shall be issued and handed over to the
          customer by the delivery agent at the time of delivery.
        </li>
      
        <li>
          Get the Food on OLF App on Google Play or App Store or call 9522996999 for a
          hassle-free experience.
        </li>
      </ul>';


        $contents = StringParser::render($template, 
        ['grandTotal' => $this->amountInWords,
         'items' => $this->getItemCells($items),
         'cells' => $this->getCellInfo($subTotal, $cgL, $cgst, $igL, $igst, $sgL, $sgst),
         'jcells' => $this->journeyInfo(),
         'dcells' => $this->deliveryDetails()
        ]);

        return $contents;
    }

    public function getCellInfo($subTotal, $cgL, $cgst, $igL, $igst, $sgL, $sgst)
    {
        return [
            ['Sub Total', $subTotal],
            [$cgL, $cgst],
            [$sgL, $sgst],
            [$igL, $igst],
            ['Delivery Charges', 0],
            ['Discounts(-)', $this->order->discountAmount ],
            ['Grand Total', $this->order->amountPayable]
        ];
    }

    public function getItemCells($items)
    {
        $_ = array_map(function ($item) {
            return [$item->itemName, $item->quantity, $item->basePrice, round($item->basePrice, 2)];
        }, $items);

        return $_;
    }
}
