<?php
namespace App\Domain;

use Config;

class OrderService {
    private $data;
    
    function __construct(){
        $this->data[] = [];
        
    }
    
    function getOrder($coupon){
        /*
         @returns [configutaionChanges , actualOrder ]
         false if no history found
        */
        
        $o = \App\Models\Order::where('discount_code', $coupon)->first();
        $vo = \App\Models\VendorOrder::where('discount_code', $coupon)->first();
        
        if($o == null && $vo == null){
            return [false, null];
        }
        
        if($o!=null){
            return [true ,$o];
        }
            
        return [ false,$vo ];
    }
    
    
    function addDeliveryCharges($fields){
        $data = $fields;
        $data['amountPayable'] = $fields['amountPayable']+ $fields['deliveryCharge'];
        unset($data['coupon']);
        return $data;
                        
    }
    
    
    function createParcel($fields,$details){
        $removables = ['fullName','email','mobile','foodType'];
        $orderData = [];
        
        $orderData['customer'] = [
                'fullName' => $fields['fullName'],'email'=> $fields['email'],'mobile' => $fields['mobile']
            ];
        foreach($fields as $key=>$value){
            if(!in_array($key,$removables))
                $orderData[$key] = $value; 
        }
        
        $orderData['vendorPaymentType'] = $fields['paymentType'];
        return $orderData;
    }
    
    
    function resetConfigs(){
        Config::set('vouchers.relation_table','vendor_order_voucher');
        Config::set('vouchers.user_model',\App\Models\VendorOrder::class);
    }
    
    function calculatePayable($orderData, $couponApplicable, $walletApplicable){
        $oData = $orderData;
        $items = array_map(function($item){
            $basePrice = $item['price']/1.05;
            return [                
                    "itemId"      => $item['id'],
                    "itemName"    => $item['itemName'],
                    "description" => $item['description'],
                    "basePrice"   => $basePrice,
                    "sellingPrice"=> $item['price'],
                    "taxRate"     => $item['price']-$basePrice,
                    "isVegetarian"=> $item['isVegeterian'] == 1? true:false,
                    "quantity"    => $item['quantity'],
                    "option"      => null,
                ];
                
        },$orderData['orderItems']);
        
        $subtotal = collect($items)->sum(function($item){
                            return $item['sellingPrice'] * $item['quantity'];
                    });
                    
        $txn = collect($items)->sum(function($item){
                            return ($item['taxRate']* $item['quantity']);
                    });
                    
        $oData['orderItems'] = $items;
        $oData['gst'] = $txn;
        
        $oData['totalAmount'] = $subtotal;
        $oData['amountPayable'] = round($subtotal);
        
        
        if($couponApplicable){
            list($configChanged, $order) = $this->getOrder($oData['coupon']);
            
            if($order){
                list($couponApplied,$discount,$amount) = $this->redimCode($order, $oData);
                // Call this after applying coupon 
                if($configChanged){
                    $this->resetConfigs();
                }
            }
            
        }
        
        if($walletApplicable){
            echo("APPLYING FROM WALLET");
        }
        
        return $oData;
    }
    
    
    function redimCode($order, $currentOrder){
        /*
         @returns [couponApopliedOrNot true or false , discount numeric or 0, amount new Amount]
         need to update existing amount to thois amount 
        */
        
        dd("REEDIMING CODE");
    }
    
    function validateDate($date1,$date2, $currentDate, $debug){
        $d1 = \Carbon\Carbon::parse($date1);
        $d2 = \Carbon\Carbon::parse($date2);
        if($debug){
            return $d1->eq($d2);    
        }else{
            return $d1->eq($d2) && $d1->greaterThanOrEqualTo($currentDate);
        }
        
    }
    
    function validateRestraunt($station, $rest, $debug){
        $currentTime = \Carbon\Carbon::now();
        $arrivalTime = \Carbon\Carbon::createFromFormat("Y-m-d H:i",$station->arrDate." ".$station->arrival);
        $orderTime =   \Carbon\Carbon::createFromFormat("Y-m-d H:i",$station->arrDate." ".$station->arrival)->subMinutes($rest->order_timing);
        $openingTime = \Carbon\Carbon::createFromFormat("Y-m-d H:i",$station->arrDate." ".$rest->openingTime);
        $closingTime = \Carbon\Carbon::createFromFormat("Y-m-d H:i",$station->arrDate." ".$rest->closingTime);
        $statusAtCurrentTime = $currentTime->between($openingTime,$closingTime);
        $statusAtTrainArrival = $arrivalTime->between($openingTime,$closingTime);
        
        if($debug){
            if(!$statusAtCurrentTime)
                return [
                    true,
                    "Cannot post order in past date"
                ];
        }
            
        if(!$statusAtTrainArrival)
            return [
                true,
                "Restraunt is closed"
            ];
            
        if($debug){
            if($currentTime->greaterThanOrEqualTo($orderTime)){
                return [
                    true,
                    "Order service not available at this Time"
                    ];
        }
        }
        
        
        return [false, [
                'deliveryDate' => $station->arrDate ." ".$station->arrival." IST" ?? $station->schArrivalTime." IST",
                'bookingDate' => \Carbon\Carbon::now()->format('Y-m-d H:i')." IST"
            ]];
    }
    
    function validateParcel ($fields,$details){
        $station = collect($details->stations)
                        ->filter(function($st) use($fields){ return $st->code == $fields['stationCode'];})
                        ->flatten()
                        ->first() ?? false;
        
        if($current = $station){
            if($this->validateDate(
                    $current->arrDate,
                    $fields['deliveryDate'],
                    \Carbon\Carbon::now(),
                    true
                )
            ){
                return [true,  $station];
            }else{
                return [false, "Date Not matching"];
            }
        }else{
            return [false, "Station Not Found"];
        }
    }
    
    
    
}