<?php

namespace App\Domain;

use App\Models\VendorOrder;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\Log;

trait PersistableTrait
{

    function persist($payload)
    {

        // Log::info("Persist");

        $formatted = $this->formatOrder($payload);
        // Log::info($formatted);
        $order=null;
        
        try {
            $order = VendorOrder::create($formatted);
        } catch (Exception $th) {
            // dd($th->getMessage());
        }
        return $order;

    }
    private function formatOrder($payload)
    {
        $order = [];
    
        $order['id'] = $payload->vendorOrderId; //todo
        $order['orderId'] = $payload->orderId;
        $order['customer'] = json_encode($payload->customer);
        $order['customerEmail'] = $payload->customer->email;
        $order['customerMobile'] = $payload->customer->mobile;
        $order['outlet'] = $payload->outlet->id; //get the local outlet id
        $order['bookingDate'] = $payload->bookingDate;
        $order['deliveryDate'] = $payload->deliveryDate;
        $deliveryDate1 = $this->getFormatedDate($payload->deliveryDate);
        $dateTime = ($deliveryDate1->format('Y-m-d H:i'));
        $order['deliveryDateTime'] = $dateTime . 'IST';

        $order['pnr'] = $payload->pnr;
        $order['trainNo'] = $payload->trainNo;
        $order['trainName'] = $payload->trainName;
        $order['stationCode'] = $payload->stationCode;
        $order['stationName'] = $payload->stationName;
        $order['coach'] = $payload->coach;
        $order['berth'] = $payload->berth;
        $order['totalAmount'] = $payload->amountPayable; //todo
        $order['deliveryCharge'] = $payload->deliveryCharge;
        $order['gst'] = $payload->gst;
        $order['orderItems'] = json_encode($payload->orderItems);
        $order['discountAmount'] = $payload->discount;
        $order['amountPayable'] = $payload->amountPayable; //todo
        $order['paymentType'] = $payload->paymentType;
        $order['status'] = "ORDER_CONFIRMED";


        $outlet = \App\Models\Outlet::find($payload->outlet->id);

        $items = array_column($outlet->menu->items->toArray(), 'vendorPrice', 'id');
        $orderItems = collect($payload->orderItems)->map(function ($oi) use ($items) {
            $oi = (array)$oi;
            if (isset($items[$oi['itemId']]))
                $oi['costPrice'] = $items[$oi['itemId']];
            return $oi;
        });


        $unavailable = false;
        $subtotal1 = collect($orderItems)->sum(function ($item) {
            if (isset($item['costPrice']) && $item['costPrice'] != 'N.A.' && $item['costPrice'] != '') {
                return $item['costPrice'] * $item['quantity'];
            }
            $unavailable = true;
        });
        if ($unavailable)
            $order['vendorPayment'] = 0;
        else
            $order['vendorPayment'] = $order['amountPayable'] - $subtotal1;

        if(!$payload->pay == 0){
            $order['razorpay_payment_id'] = $payload->razorpay_payment_id;
        }

        return $order;
    }


    private function getFormatedDate($str)
    {
        $_parts = preg_split("/\s/", $str);
        $_tz = trim($_parts[2]);

        $_time = join(" ", array_merge([$_parts[0], $_parts[1]]));
        $_ctime = null;
        if ($_tz == 'IST')
            $_ctime  = Carbon::createFromFormat('Y-m-d H:i', $_time, 'Asia/Kolkata');
        else
            $_ctime =  Carbon::createFromFormat('Y-m-d H:i', $_time, $_tz)->setTimeZone('Asia/Kolkata');

        return $_ctime;
    }
}
