<?php

namespace App\Exports;

use App\Invoice;
use Illuminate\Contracts\View\View;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Concerns\Exportable;
use Illuminate\Support\Carbon;
use Maatwebsite\Excel\Concerns\FromQuery;
use Maatwebsite\Excel\Concerns\FromView;

class ConsolidatedReport implements FromView
{
    use Exportable;
    private $start;
    private $end;

    function __construct($start, $end)
    {
        $this->start = $start;
        $this->end = $end;
    }

    public function view(): View
    {
        $blackList = ['Madhya Pradesh', 'M.P', 'MP'];
        $headings = ['SNo', 'Invoice No', 'Generation Date', 'Outlet', 'HSN', 'Outlet GST', 'Commision', 'IGST', 'SGST', 'CGST'];

        $mapper = function ($obj, $index) {
            return [
                $index,
                'Invoice/' . $obj->id,
                Carbon::parse($obj->created_at)->format('Y/M/d h:m'),
                $obj->outlet_name,
                $obj->hsn,
                $obj->gstin_outlet,
                $obj->commision,
                $obj->igst,
                $obj->sgst,
                $obj->cgst,
            ];
        };

        $rows = DB::table('invoices')
            ->join('outlets', 'invoices.outlet_id', '=', 'outlets.id')
            ->select('invoices.*', 'outlets.city', 'outlets.state')
            ->whereDate('date', '>=', $this->start,)
            ->whereDate('date', '<=', $this->end)
            ->get()
            ->map(function ($obj, $index) use ($blackList, $mapper) {
                $tax = $obj->igst;
                if (in_array($obj->state, $blackList)) {
                    $obj->cgst = $tax / 2;
                    $obj->sgst = $tax / 2;
                    $obj->igst = 0;
                } else {
                    $obj->cgst = 0;
                    $obj->sgst = 0;
                }

                $obj->hsn = 999799;

                return $mapper($obj, $index);
            })->toArray();

        return view('invoices.consolidatedReport', compact('headings', 'rows'));
    }
}
