<?php

namespace App\Http\Controllers\Auth;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Validator;
use App\Models\User;
use App\Models\Token;
use Auth;
use Log;

class ApiLoginController extends Controller
{
    private $apiToken = null;
    private $hash = false;

	function __constuct(){
		$this->apiToken = uniqid(base64_encode(str_random(60)));
	}

    public function getToken($apiToken)
    {
        return $apiToken;
    }

    public function saveToken($apiToken, $user)
    {
        $postArray = ['api_token' => $this->getToken($apiToken)];
        return User::where('email',$user->email)->update($postArray);
    }
    public function removeToken($apiToken, $user)
    {
        return $this->saveToken(null, $user);
    }

    public function getTokenUser($token)
    {
        return User::where('api_token',$this->getToken($token))->first();
    }

    public function hasRole($user)
    {
        if(sizeof($this->getRole())>0&&!$user->hasRole(...$this->getRole())) {
        	return false;
		}

		return true;
	}
    public function getRole()
    {
    	return [];
	}
	public function postLogin(Request $request)
    {
            $this->apiToken = str_random(60);
    // Validations
   $rules = [
      'email'=>'required|email',
      'password'=>'required|min:6'
    ];
    $validator = Validator::make($request->all(), $rules);
    if ($validator->fails()) {
      // Validation failed
      return response()->json([
        'status' => 'error',
        'message' => $validator->messages(),
      ]);
    } else {
      // Fetch User
      $user = User::where('email',$request->email)->first();
      if($user&&$this->hasRole($user)==true) {
        // Verify the password
        if( password_verify($request->password, $user->password) ) {
          // Update Token
          $postArray = ['api_token' => $this->apiToken];
          $login = $this->saveToken($this->apiToken, $user);
          $signed = 0;
          
          $outlets = $user->outlets->map(function($outlet){
              $stn = \App\Models\Station::find($outlet->station_id);
              //return $outlet;
              return (object)[
                  "id"=>$outlet->id,
                  "name"=>$outlet->outletName,
                  "station" => $stn->name." (".$stn->code.")",
                  "active" => $outlet->active ==1? true : false
                 ];
          });
          if($user->documentSign)
              $signed = $user->documentSign->signed;

          if($login) {
            return response()->json([
              'status' => 'success',
              'name'         => $user->name,
              'email'        => $user->email,
              'mobile'       => explode(',', $user->mobile)[0],
              'document_signed' => 1,//$signed,
              'api_token' => $this->apiToken,
              "outlet" => $outlets
            ]);
          }
        } else {
          return response()->json([
            'status' => 'error',
            'message' => 'Invalid Password',
          ]);
        }
      } else {
        return response()->json([
          'status' => 'error',
          'message' => 'User not found',
        ]);
      }
    }
  }


//logout

public function postLogout(Request $request)
  {
    //$token = $request->header('Authorization');
    $token = $request->query('api_token');
    $user = $this->getTokenUser($token);
    if($user) {
      $logout = $this->removeToken($token, $user);
      if($logout) {
        return response()->json([
          'status' => 'success',
          'message' => 'User Logged Out',
        ]);
      }
    } else {
      return response()->json([
        'status' => 'error',
        'message' => 'User not found',
      ]);
    }
  }
    public function getAuthUser()
    {
        return Auth::guard('api')->user();
    }
    public function isLoggedIn(Request $request)
    {
        $user = $this->getAuthUser();
        if($user){
            return response()->json([
                    'status' => 'success',
                    'logged' => 1
                ], 200);
        }
        return response()->json([
            'status' => 'success',
            'logged' => 0
        ], 200);
    }

}
