<?php

namespace App\Http\Controllers\Auth;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Validator;
use App\Models\UserMobile;
use App\Models\Token;
use App\Models\User;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;

class ApiMobileLoginController extends Controller
{
    private $apiToken = null;
    private $hash = false;

	function __constuct(){
		$this->apiToken = uniqid(base64_encode(str_random(60)));
	}

    public function getToken($apiToken)
    {
        if($apiToken==null)
            return $apiToken;
        return hash('sha256', $apiToken);
    }

    public function saveToken($apiToken, $user, $firebaseToken=null)
    {
        if(isset($firebaseToken))
            $postArray = ['access_token' => $this->getToken($apiToken), 'user_id' => $user->id, 'firebase_token'=>$firebaseToken];
        else
            $postArray = ['access_token' => $this->getToken($apiToken), 'user_id' => $user->id];
        return Token::create($postArray);
    }
    public function removeToken($apiToken, $user)
    {
        return $this->deleteToken($apiToken);
    }
    public function deleteToken($apiToken)
    {
        $token = Token::where('access_token', $this->getToken($apiToken));
        return $token->delete();
    }

    public function getTokenUser($token)
    {
        $token = Token::where('access_token',$this->getToken($token))->first();
        return User::find($token->user_id);
    }

    public function hasRole($user)
    {
        if(sizeof($this->getRole())>0&&!$user->hasRole(...$this->getRole())) {
        	return false;
		}

		return true;
	}
    public function getRole()
    {
    	return ['vendor'];
	}
	public function postLogin(Request $request)
    {
            $this->apiToken = str_random(60);
    // Validations
   $rules = [
      'email'=>'required|email',
      'mobile'=>'required',
      'otp'=>'required|min:6'
    ];
    $validator = Validator::make($request->all(), $rules);
    if ($validator->fails()) {
      // Validation failed
      return response()->json([
        'status' => 'error',
        'message' => $validator->messages(),
      ]);
    } else {
      // Fetch User
      $admin = \App\Models\User::where('email', $request->email)->first();
      if(!$admin)
          return response()->json([
            'status' => 'error',
            'message' => 'Email is not registered',
          ]);
          
      $user = UserMobile::where('mobile',$request->mobile)->where('user_id',$admin->id)->first();
      if($user&&$this->hasRole($user->user)==true) {
        // Verify the password
        if( password_verify($request->otp, $user->password)) {
          // Update Token
            $signed = 0;
          $postArray = ['api_token' => $this->apiToken];
          if($request->has('firebase_token'))
              $login = $this->saveToken($this->apiToken, $user->user, $request->firebase_token);
          else
              $login = $this->saveToken($this->apiToken, $user->user);
          if($user->user->documentSign)
              $signed = $user->user->documentSign->signed;
          if($login) {
              $user->password = null;
              $user->save();
              
              $userActual = $user->user;
              $outlets = $user->user->outlets->map(function($outlet){
                    $stn = \App\Models\Station::find($outlet->station_id);
                    $x = $outlet;
                    $x['agreementStart'] = \Carbon\Carbon::now()->format('Y-m-d');        
                    $x["id"]      = $outlet->id;
                    $x["name"]    = $outlet->outletName;
                    $x["station"] = $stn->name." (".$stn->code.")";
                    $x["active"]  = $outlet->active ==1? true : false ;
                    return $x;   
                });
            return response()->json([
              'status' => 'success',
              'name'         => $userActual->name,
              'email'        => $userActual->email,
              'mobile'       => explode(',', $user->user->mobile)[0],
              'document_signed' => $signed,
              'api_token' => $this->apiToken,
              "outlet" => $outlets
            ]);
          }
        } else {
          return response()->json([
            'status' => 'error',
            'message' => 'Invalid otp',
          ]);
        }
      } else {
        return response()->json([
          'status' => 'error',
          'message' => 'User not found',
        ]);
      }
    }
  }


//logout

public function postLogout(Request $request)
  {
    //$token = $request->header('Authorization');
    $token = $request->query('api_token');
    $user = $this->getTokenUser($token);
    if($user) {
      $logout = $this->removeToken($token, $user);
      if($logout) {
        return response()->json([
          'status' => 'success',
          'message' => 'User Logged Out',
        ]);
      }
    } else {
      return response()->json([
        'status' => 'error',
        'message' => 'User not found',
      ]);
    }
  }
    public function getAuthUser()
    {
        return Auth::guard('api_mobile')->user();
    }
    public function isLoggedIn(Request $request)
    {
        $user = $this->getAuthUser();
        if($user){
            return response()->json([
                    'status' => 'success',
                    'logged' => 1
                ], 200);
        }
        return response()->json([
            'status' => 'success',
            'logged' => 0
        ], 200);
    }

    public function sendOtp(Request $request) {
        if (!$request->has('mobile')||!$request->has('email')) {
            return json_encode(array('status' => 'failure', 'msg' => "input parameters missing"));
        } else {
            //Auth::logout();
            //put in session
            $mobile = $request->input('mobile');
            $email = $request->input('email');
            $admin = \App\Models\User::where('email', $request->email)->first();
            if(!$admin)
                return response()->json([
                  'status' => 'error',
                  'message' => 'Email is not registered',
                ]);
            $mobileUser = UserMobile::where('mobile',$mobile)->where('user_id',$admin->id)->first();
            
            if(!$mobileUser)
                return json_encode(array('status' => 'failure', 'msg' => "mobile is not registered"));
            if(!$mobileUser->user->active)
                return json_encode(array('status' => 'failure', 'msg' => "user needs approval from admin"));
            if($mobileUser->user->email!=$email)
                return json_encode(array('status' => 'failure', 'msg' => "email is not registered"));
            
            $otp = $this->generateOTP();
            $mob = "XXX";
            $message = 'your otp for signing is '.$otp.'on https://olfstores.com';
            Log::info('___ otp request  __');
            Log::info(json_encode(compact('otp','mob')));
            Log::info('__ otp request  __');
            ob_start();
            $this->sendMessage($message,$admin->mobileAdmin->mobile, env('LOGIN_OTP_TEMLPLATE_ID', NULL));
            $res = ob_get_clean();
            $mobileUser->password = \Illuminate\Support\Facades\Hash::make($otp);
            $mobileUser->save();
            return json_encode(array('status' => 'success', "otp" => $otp, 'message' => 'otp sent successfully'));
        }
    }

    private function sendMessage($string, $numbers, $DLT_TE_ID=null){
            $key = "191164Aup6j2p05a4cb419";
            $url = sprintf('https://api.msg91.com/api/sendhttp.php?authkey=%s&mobiles=%s&route=%s&sender=%s&message=%s&country=%s',$key,$numbers,'4',urlencode('OLFSTR'),urlencode($string),'91');

            if($DLT_TE_ID!=null){
                $url = $url.sprintf('&DLT_TE_ID=%s', $DLT_TE_ID);
            }
            
            $curl = curl_init();

            curl_setopt_array($curl, array(
                                    CURLOPT_URL => "http://api.msg91.com/api/v2/sendsms",
                                    CURLOPT_URL => $url,
                                    CURLOPT_RETURNTRANSFER => true,
                                    CURLOPT_ENCODING => "",
                                    CURLOPT_MAXREDIRS => 10,
                                    CURLOPT_TIMEOUT => 30,
                                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                                    //CURLOPT_CUSTOMREQUEST => "POST",
                                    CURLOPT_CUSTOMREQUEST => "GET",
                                    //CURLOPT_POSTFIELDS => json_encode($data),
                                    CURLOPT_SSL_VERIFYHOST => 0,
                                    CURLOPT_SSL_VERIFYPEER => 0,
                /*
                                    CURLOPT_HTTPHEADER => array(
                                            "content-type: application/json"
                                            ),*/
                                    ));

		$response = curl_exec($curl);
		$err = curl_error($curl);

		curl_close($curl);

		
		if ($err) {
		    Log::channel('otpLog')->info("______________ERROR_____________");
			Log::channel('otpLog')->info($err);
			Log::channel('otpLog')->info("______________ERROR_____________");

		} else {
			Log::channel('otpLog')->info("______________OTP_____________");
			Log::channel('otpLog')->info($response);
			Log::channel('otpLog')->info("______________OTP_____________");
		}
		print_r($response);
	}

    public function generateOTP(){
        $otp = mt_rand(100000,999999);
        return $otp;
    }
    public function updateFirebaseToken(Request $request) {
        if (!$request->has('firebase_token')) {
            return json_encode(array('status' => 'failure', 'msg' => "input parameters missing"));
        } else {
            $token = $this->getTokenForRequest($request);
            $token = \App\Models\Token::where('access_token', hash('sha256', $token))->first();
            //Log::channel('dailyInfo')->info('here:'.print_r($token, true));
            $token->firebase_token = $request->firebase_token;
            if($token->save()){
                return json_encode(array('status' => 'success', 'msg' => "firebase token saved successfully"));
            }else{
                return json_encode(array('status' => 'failure', 'msg' => "firebase token couldnt be saved"));
            }
        }        
    }
    public function getTokenForRequest ($request) {
            $inputKey = 'api_token';
            $token = $request->query($inputKey);
            if (empty($token)) {
                    $token = $request->input($inputKey);
            }
            if (empty($token)) {
                    $token = $request->bearerToken();
            }
            return $token;
    }
}
