<?php

namespace App\Http\Controllers\Auth;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Support\Facades\Auth;

class MemoryUserLoginController extends Controller
{
    use AuthenticatesUsers;

    protected $redirectTo = '/customer/home';
    public $loginPath = '/customer/login';

    public function __construct()
    {   
        $this->middleware('guest:memoryuser')->except('logout');
    }

    public function performLogout(Request $request)
    {
        $this->guard('memoryuser')->logout();
        return response()->json(['status' => 'success', 'msg' => 'Logged out successfully']);
    }

    protected function loggedOut(Request $request)
    {
        return response()->json(['status' => 'success', 'msg' => 'Logged out successfully']);
    }

    public function showLoginForm()
    {
        return view('auth.login.order');
    }

    public function username()
    {
        return 'mobile';
    }   

    protected function guard()
    {
        return Auth::guard('memoryuser');
    }

    public function loginApi(Request $request)
    {
        if (Auth::check()) {
            return response()->json(['status' => 'success'], 200);
        }

        $this->validateLogin($request);

        if ($this->hasTooManyLoginAttempts($request)) {
            $this->fireLockoutEvent($request);

            return response()->json(['status' => 'error', 'message' => 'Too many login attempts.'], 401);
        }

        if ($this->attemptLogin($request)) {

            return response()->json(['status' => 'success'], 200);
        }

        $this->incrementLoginAttempts($request);

        return response()->json(['status' => 'error', 'message' => 'Credentials do not match our database.'], 200);
    }
    
    public function home()
    {   
        $redirect = session()->has('items') ? route('cart.confirm') : url()->current();
        return response()->json(['status' => 'success', 'msg' => 'Login successful', 'redirect' => $redirect]);
    }    

    protected function credentials(Request $request)
    {
        return $request->only($this->username(), 'password');
    }

    protected function redirectTo()
    {
        $redirect = 'https://olf.invoidea.app/yesy';
        return $redirect;
    }
}
