<?php

namespace App\Http\Controllers\Auth;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use App\Models\PhoneLog;

class OrderLoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers {
        logout as performLogout;
    }

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/customer/home';
    public $loginPath = '/customer/login';
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function logout(Request $request)
    {
        $count = 0;
        if (Auth::user() != null)
            $count = $count + 1;
        if (Auth::guard('order')->user() != null)
            $count = $count + 1;
        if (Auth::guard('vendororder')->user() != null)
            $count = $count + 1;
        if (Auth::guard('memoryuser')->user() != null)
            $count = $count + 1;
        $this->guard()->logout();
        if ($count == 1) {
            //Log::channel('dailyInfo')->info('--invalidating session:');
            // $request->session()->invalidate();
        }

        return json_encode(array('status' => 'success', 'msg' => "logged out successfully"));

        //return $this->loggedOut($request) ?: redirect('/');
    }
    /*
    public function performLogout(Request $request)
    {
        //$this->guard('order')->logout();
        Auth::guard('order')->logout();
        Auth::logout();
        return json_encode(array('status'=>'success','msg'=>"logged out successfully"));
    }
     * 
     */

    public function __construct()
    {
        $this->middleware('guest:order')->except('logout');
    }
    public function showLoginForm()
    {
        return view('auth.login.order');
    }
    public function username()
    {
        return 'orderId';
    }
    protected function guard()
    {
        return Auth::guard('order');
    }
    protected function loggedOut(Request $request)
    {
        return json_encode(array('status' => 'success', 'msg' => "logged out successfully"));
    }
    public function loginApi(Request $request)
    {
        $credentials = $request->only('orderId', 'password');
        if (Auth::guard('order')->attempt($credentials)) {
            $user = Auth::guard('order')->user();
            $wallet = $user->wallet;
            if ($wallet == null) {
                $user->createWallet([
                    'name' => 'New Wallet',
                    'slug' => 'my-wallet',
                ]);
            }
            return response()->json([
                'msg' => "login successful",
                'status' => 'success',
            ], 200);
        }
        return response()->json([
            'status' => 'error',
            'message' => 'Credentials do not match our database.'
        ], 200);
        /*
        //Auth::login($request);
		if(Auth::check()){
            return response()->json([
                    'status' => 'success',
                ], 200);
		}

        $this->validateLogin($request);

        // If the class is using the ThrottlesLogins trait, we can automatically throttle
        // the login attempts for this application. We'll key this by the username and
        // the IP address of the client making these requests into this application.
        if ($this->hasTooManyLoginAttempts($request)) {
            $this->fireLockoutEvent($request);

            return response()->json([
                    'status' => 'error',
                    'message' => 'Too many login attempts.'
                ], 401);
        }

        if (Auth::attempt($credentials)) {
            return response()->json([
                    'msg'=>"login successful",
                    'status' => 'success',
                ], 200);
        }

        // If the login attempt was unsuccessful we will increment the number of attempts
        // to login and redirect the user back to the login form. Of course, when this
        // user surpasses their maximum number of attempts they will get locked out.
        $this->incrementLoginAttempts($request);

        return response()->json([
                'status' => 'error',
                'message' => 'Credentials do not match our database.'
            ], 200);
         * 
         */
    }
    public function logoutCustomer(Request $request)
    {
        Log::channel('dailyInfo')->info('--logout customer:');
        $user = Auth::guard('vendororder')->user();
        $user1 = Auth::guard('order')->user();
        $user2 = Auth::guard('memoryuser')->user();
        if ($user != null) {
            if ($user1 != null && $user1->created_at->gt($user->created_at))
                $user = $user1;
        } else {
            $user = $user1;
        }

        if ($user == null) {
            if ($user2 != null)
                return redirect()->route('customer__.logout');
            else
                return json_encode(array('status' => 'error', 'msg' => 'Login to proceed'));
        }
        Log::channel('dailyInfo')->info('--logout customer1:');
        if ($user->vendor)
            return redirect()->route('customer_.logout');
        return redirect()->route('customer.logout');
    }
    public function checkLogged(Request $request)
    {
        $user = Auth::guard('vendororder')->user();
        $user1 = Auth::guard('order')->user();
        $user2 = Auth::guard('memoryuser')->user();
        $loggedIn = false;
        if ($user != null) {
            $loggedIn = true;
        } else {
            if ($user1 != null) {
                $loggedIn = true;
            } else if ($user2 != null) {
                $loggedIn = true;
            }
        }
        if (!$loggedIn) {
            //Auth::logout();
            return json_encode(array('status' => 'success', 'logged' => 0, "session_id" => session()->getId()));
        }
        return json_encode(array('status' => 'success', 'logged' => 1, "session_id" => session()->getId()));
    }

    public function home()
    {
        $redirect = session()->has('items') ? route('cart.confirm') : url()->current();
        return json_encode(array('status' => 'success', 'msg' => "login successful", 'redirect' => $redirect));
      //,'balance'=>$user->balance));
    }
    public function loginCustomer(Request $request)
    {      
        Log::channel('dailyInfo')->info('--login:');
    
        if ($request->has('mobile') && $request->has('otp')) {
     
            $otp = $request->input('otp');
            $mobile = $request->input('mobile');
            $_mobile = $mobile;
            if ($otp == session('otp') && $mobile == session('mobile')) {
                // dd("hii if");
                $mobile = session('mobile');
            } 
            
            else if(PhoneLog::isOTPValid($mobile, $otp)){
                // dd("hii else if");
                PhoneLog::resetOTP($mobile, $otp);
                $mobile = $_mobile;
                
            } else {
                // dd("hii else");
                return json_encode(array('status' => 'error', 'msg' => "Please enter valid otp."));
            }
            
            $res = \App\Models\Order::latest($request->input('mobile'));
            // dd($res);
            //$res = $this->getUser($request->input('mobile'));
            if ($res == null) {
                return redirect()->route('customer__.login', ['mobile' => $mobile, 'password' => $request->input('otp')]);
            } else {
                $vo = $res;
                $vendor = $vo->vendor;
            }
        } else {
            return response()->json(['status' => 'error', 'message' => 'otp is invalid'], 200);
        }
       
        if ($vendor)
            return redirect()->route('customer_.login', ['orderId' => $vo->orderId, 'password' => $request->input('otp')]);
        else
            return redirect()->route('customer.login', ['orderId' => $vo->orderId, 'password' => $request->input('otp')]);
    }

    public function submitOtp($request)
    {
        $otp = trim(request('otp'));
        if ($otp == '') {
            return json_encode(array('status' => 'error', 'message' => "Please enter valid otp."));
        } else {
            if ($otp == session('otp')) {
                $mobile = session('mobile');
                json_encode(array('status' => 200, 'msg' => 'sucess'));
            } else {
                return json_encode(array('status' => 'error', 'message' => "Please enter valid otp."));
            }
        }
    }
    protected function credentials(Request $request)
    {
        return $request->only($this->username(), 'password');
    }
    
    
    
    
    public function getUser($mobile, $wallet = false)
    {
        $oWhere = \App\Models\Order::where('customerMobile', $mobile);
        $voWhere = \App\Models\VendorOrder::where('customerMobile', $mobile);
        if ($wallet) {
            $oWhere->where('wallet_id', '!=', 0);
            $voWhere->where('wallet_id', '!=', 0);
        }
        $o = $oWhere->orderBy('created_at', 'DESC')
            ->first();
        $vo = $voWhere->orderBy('created_at', 'DESC')
            ->first();
        $vendor = false;
        if ($o != null) {
            if ($vo != null) {
                if ($o->created_at->gt($vo->created_at)) {
                    $vo = $o;
                } else {
                    $vendor = true;
                }
            } else {
                $vo = $o;
            }
        } else {
            $vendor = true;
            if ($vo == null) {
                return array('user' => null);
            }
        }

        return array('user' => $vo, 'wallet' => $vo, 'vendor' => $vendor);
    }
    public function test(Request $request)
    {
        Log::channel('dailyInfo')->info('--getting:' . \Illuminate\Support\Facades\Hash::make(599440));
    }
}
