<?php

namespace App\Http\Controllers;

use App\Models\Post;
use App\Repositories\BlogRepository;
use Illuminate\Http\Request;
use DB;

class BlogController extends Controller
{
    private $repository;

    function __construct(BlogRepository $repository)
    {  
        $this->repository = $repository;
    }

    public function index()
    {   
        $categories = $this->repository->categorySet();
        $recentPosts = $this->repository->recentPosts();
        $lastPost = $this->repository->lastPost();

        $title = "OLF | Blog";

        $description = "OLF | Blog | get updated information and about food & journey planing and even more..";
        $image = '';

        $payload = (compact('categories', 'recentPosts', 'lastPost','title', 'description','image'));

        return view('v2.blog.index', $payload);
    }

    public function postByCategory($slug)
    {
        list($category, $posts) = $this->repository->postByCategory($slug, true);

        $categories = $this->repository->categorySet();
        $recentPosts = $this->repository->recentPosts();
        $title = "OLF | Blog / {$category->name}";

        $description = "OLF | Blog | {$category->name}";
        $image = '';

        $payload = (compact('categories', 'recentPosts', 'posts','title', 'description','image'));

        return view('v2.blog.postByCategory', $payload);


    }

    public function createPost(Request $request)
    {
        
        $isGet = $request->method() == 'GET';

        if ($isGet) {
            $categories = $this->repository->categorySet();
            return view('v2.blog.create', compact('categories'));
        } else if ($request->method() == 'POST') {

            $catName = $request->input('categoryName');
            $payload = $request->except('categoryId');

            $payload['slug'] = $this->repository->generateSlug($payload['title']);
            $payload['content'] = base64_encode($request->input('content'));


            try {
                $cat = $this->repository->createCategory(['name' => $catName, 'slug' => $this->repository->generateSlug($catName)]);
                $result = $this->repository->createPost($cat->id, $payload);
                return back();
            } catch (\Exception $th) {
                dd($th);
            }
        }
    }

    public function setBanner($image)
    {
        dd($image);
    }

    public function backendPosts()
    {
        $categories = \App\Models\BlogCategory::with('posts')->get();
        return view('v2.blog.private', compact('categories'));
    }

    public function editPost(Request $request, \App\Models\Post $id)
    {
        if($request->method() == 'POST'){
            $this->repository->editPost($id->id, $request->all());
            return back();

        }
        else{
            $post = $id;
            $categories = $this->repository->categorySet();
           return view('v2.blog.edit', compact('post', 'categories'));
        }

    }

    public function postByTitle($slug)
    {
        $post =  $this->repository->getPost($slug);
        $categories = $this->repository->categorySet();
        $recentPosts = $this->repository->recentPosts();
        $lastPost = $post;


        $_title = $post != null ? $post->page_title : "";

        $title = "{$_title}";

        $description = $post != null ? $post->description : "";
        $image = '';

        $payload = (compact('categories', 'recentPosts', 'lastPost','title', 'description','image'));

        return view('v2.blog.index', $payload);
    }


    public function ListingBlog(){

       $this->authorize('blog-list');

       $AllData =  $this->repository->recentPosts();

       return view('backend.blog.view',['AllData'=>$AllData]);
    }

    public function BlogEdit(Request $request, $category_id){

        $this->authorize('edit-blog');
      // $getMyData = BlogRepository::first();
         // Access the 'name' input from the request
         $totalData =[];
         if($request->method() == 'POST'){
            $file = $request->file('image');
            if(isset($file)){
            $fileName = date('Ymdhis').$file->getClientOriginalName();
             $destinationPath = public_path('assets/img');
             $file->move($destinationPath,$fileName);
               $link = 'public/assets/img/'.$fileName;
               $totalData['image'] = $link;
            }


            $totalData['title'] = $request->input('title');
            $totalData['content'] = base64_encode($request->input('content'));

            DB::table('posts')->where('id',$category_id)->update($totalData);
          //  $this->repository->editPost($category_id, $totalData);
            return back();

         }
         else{
            $categories =   \App\Models\Post::find($category_id);
            $blog_categores = [];
            return view('backend.blog.edit',['categories'=>$categories]);
         }
    }

    public function BlogInsert(Request $request){

        $this->authorize('create-blog');
         if($request->method()=="POST"){
            $title = $request->title;
            $page_title = $request->page_title;
            $description = $request->description;
            $slug = str_replace(" ","-",$title);
            $content = base64_encode($request->content);
          //  $image = base64_encode($request->image);
          $file = $request->file('image');
         $fileName = date('Ymdhis').$file->getClientOriginalName();
          $destinationPath = public_path('assets/img');
          $file->move($destinationPath,$fileName);
            $link = 'public/assets/img/'.$fileName;

         DB::table('posts')->insert(["title" => $title,"page_title"=>$page_title,"description"=>$description,"slug"=>$slug,"content"=>$content, "image" => $link]);
         return redirect(url('backend/Blog/view'));
         }
         else{
            $category = DB::table('blog_categores')->get();
            return view('backend.blog.add',["category" => $category]);
         }

    }
}
