<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use App\Http\Requests;
use File;
use Log;
use DB;

class ExportDataController extends Controller{

    public function getOrders($status, $start, $end)
    {
        DB::connection()->enableQueryLog();
        $orders = \App\Models\Order::
                    where(function ($q) use($start,$end,$status){
                                $q = $q->where('created_at', '>=', $start)
                                        ->where('created_at', '<=', $end);
                                if(isset($status) && $status!='All')
                                    $q->where('status', $status);

                      })->orderBy('created_at', 'desc')->get()->map(function($f) {
                        $x = array();
                        $x['orderId'] = $f['orderId'];
                        $customer = json_decode($f['customerDetails']);
                        $x['customer'] = $customer->customerName;
                        $orderItems = json_decode($f['orderItems']);
                        $itemStr = '';
                        foreach($orderItems as $item){
                            $itemStr .= $item->quantity.' '.$item->itemName.', ';
                        }
                        $x['items'] = $itemStr; 
                        $x['station'] = json_decode($f['deliveryDetails'])->station;
                        $x['paymentType'] = $f['paymentType'];
                        $x['status'] = $f['status'];
                        $x['amountPayable'] = $f['amountPayable'];
                        $x['customerMobile'] = $f['customerMobile'];
                        $x['outletName'] = $f['outlet']['outletName'];
                        $x['deliveryDate'] = $f['deliveryDate'];
                        $x['vendorOrder'] = 'false';

                        return $x;
                    });
        $vendorOrders = \App\Models\VendorOrder::
                    where(function ($q) use($start,$end,$status){
                                $q = $q->where('created_at', '>=', $start)
                                        ->where('created_at', '<=', $end);
                                if(isset($status) && $status!='All')
                                    $q->where('status', $status);
                      })->orderBy('created_at', 'desc')->get()->map(function($f) {
            Log::channel('dailyInfo')->info('order:'.print_r($f, true));
            $x = array();
            $x['orderId'] = $f['orderId'];
            $customer = json_decode($f['customer']);
            $x['customer'] = $customer->fullName;
            $orderItems = json_decode($f['orderItems']);
            $itemStr = '';
            foreach($orderItems as $item){
                $itemStr .= $item->quantity.' '.$item->itemName.', ';
            }
            $x['item'] = $itemStr; 
            $x['station'] = $f['stationName'];
            $x['paymentType'] = $f['paymentType'];
            $x['status'] = $f['status'];
            $x['amountPayable'] = $f['amountPayable'];
            $x['customerMobile'] = $f['customerMobile'];
            $x['outletName'] = $f->outlet1->outletName;
            $x['deliveryDate'] = $f['deliveryDate'];
            $x['vendorOrder'] = 'true';
            //$x['outletName'] = $f['outlet']['outletName'];

            return $x;
        });
        $orders = $orders->toBase()->merge($vendorOrders);
        //Log::channel('dailyInfo')->info('query log:'.print_r($orders, true));

        return $orders;
    }
    
    public function viewOrders($status, $start, $end)
    {
        $range = $this->getRangeDates($start, $end);
        $start1 = $range[0];
        $end1 = $range[1];

        $orders = $this->getOrders($status, $start1, $end1);

        return view('backend.orders.view', ['orders' => $orders, 'status' => $status, 'start' => $start, 'end' => $end]);
    }

    public function getRangeDates($start, $end)
    {
        if(isset($start)){
            $start = \Carbon\Carbon::createFromFormat('Y-m-d', $start, 'Asia/Kolkata')->startOfDay();
        }else{
            $start = \Carbon\Carbon::today()->startOfDay();
        }
        if(isset($end)){
            $end = \Carbon\Carbon::createFromFormat('Y-m-d', $end, 'Asia/Kolkata')->endOfDay();
        }else{
            $end = $start->copy()->addHours(23)->addMinutes(59)->addSeconds(59);
        }

        return [$start, $end];        
    }
    
    public function downloadOrders($status, $start, $end)
    {
        $range = $this->getRangeDates($start, $end);
        $start = $range[0];
        $end = $range[1];
        $orders = $this->getOrders($status, $start, $end)->toArray();
        $export = new \App\Exports\OrderExcel($orders);
        $excelFile = 'orders_'.'cancelled_'. $start->format('Y-M-d').' - to - '.$end->format('Y-M-d').'.xlsx';
        \Excel::store($export, $excelFile);
        $headers = [
              'Content-Type' => 'application/vnd.ms-excel; charset=utf-8',
           ];
        return response()->download(storage_path().'/app/'.$excelFile, $excelFile, $headers);
    }

}
