<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class ItemsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request, $outletId, $filter='ALL')
    {
        $_data = $filter == 'ALL' ? \App\Models\Outlet::find($outletId)->items->groupBy('foodType') : (object)[$filter => \App\Models\Outlet::find($outletId)->items->filter(function($item) use($filter){
            return $item->foodType == $filter;
        })->flatten()];
        
        $_categories = \App\Models\Outlet::find($outletId)->items->groupBy('foodType')->map(function($c){
            return collect($c)->map(function($a) {
                return $a->visibility == 1 ? true :false;
            })->reduce(function($carry = false, $item){
                return $carry || $item;
            });
        });
        
        
        return response()->json(['items'=> $_data, 'status'=>200, 'filter' => $filter, 'categories'=>json_decode(json_encode($_categories), false)]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    
    public function toggle(Request $request, $itemId)
    {
        $item = \App\Models\MenuItem::find($itemId);
        $item->visibility = $item->visibility == 1 ? 0 :1;
        $item->save();
        
        return response()->json(['message' => 'Item updated', 'status' => '200']);
        
    }
    
    
    

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function toggleBlock(Request $request, $outletId)
    {
        $category = $request->input('categoryName');
        $action = $request->input('action');
        $ids = \App\Models\Outlet::find($outletId)->items->filter(function($item) use($category){
            return $category == $item->foodType;
        })->pluck('id');
        
        
        DB::table('menu_items')->whereIn('id',$ids)->update(['visibility'=> $action == 'ON'? 1:0]);
        
        return response()->json(['message' => 'Item updated', 'status' => '200']);
       
        
    }

    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
       $item = \App\Models\MenuItem::find($id);
       $item->fill($request->only(['itemName','description','openingTime','closingTime','basePrice', 'sellingPrice','vendorPrice','foodType','isVegeterian','isNonVegeterian']));
       $item->save();
       
       return response()->json(['message' => 'Item updated', 'status' => '200']);
       
    }
    
    public function remove(Request $request, $id)
    {
       $item = \App\Models\MenuItem::find($id);
       $item->delete();
       return response()->json(['message' => 'Item Removed', 'status' => '200']);
       
    }
    
    public function removeAllInOutlet(Request $request, $id)
    {
       $items = DB::table('menu_items')->where('outlet_id',$id)->delete();
        
       return response()->json(['message' => 'Item Removed', 'status' => '200','txn' => $items]);
       
    }
    
    public function removeAllInOutletByCategory(Request $request, $id, $category)
    {
       $items = DB::table('menu_items')->where('outlet_id',$id)->where('foodType', $category)->delete();
        
       return response()->json(['message' => 'Item Removed', 'status' => '200', 'txn' => $items]);
       
    }

    
}
