<?php

namespace App\Http\Controllers;

use App\GupshupAPI\RequestI;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Requests;
use App\Jobs\CustomerInvoiceJob;
use App\Models\Outlet;
use App\Services\LoginOtpService;
use App\Services\WaService;
use BeyondCode\Vouchers\Models\Voucher;
use BeyondCode\Vouchers\Vouchers;
use Carbon\Carbon;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Razorpay\Api\Api;
use DB;
use Illuminate\Support\Facades\Log;
use Exception;
use Event;
use App\Models\PhoneLog;
use App\Models\User;

function processOutlet($id)
{
    $outlet = \App\Models\Outlet::find($id);
    return (object)[
        'id' => $outlet->id,
        'name' => $outlet->outletName,
        'city' => $outlet->city,
        'address' => $outlet->address,
        'fssaiNumber' => $outlet->fssaiNo,
        'gstNumber' => $outlet->gstNo,
        'fssaiCutOffDate' => $outlet->fssaiValidUpto . " IST",
        'state' => $outlet->state,
        'contactNumbers' => $outlet->mobile,
        'relationshipManagerName' => "Sanjay Jadoun",
        'relationshipManagerPhone' => $outlet->mobile,
        'relationshipManagerEmail' => 'olfstores@gmail.com',
        'pinCode' => '474011',
        'deliveryCost' => $outlet->deliveryCost,
        'payment_mode' => $outlet->payment_mode
    ];
}

class OrderController extends Controller
{

    private $service = null;
    private $loginOtpService;
    public function __construct(WaService $service, LoginOtpService $loginOtpService)
    {
        $this->service = $service;
        $this->loginOtpService = $loginOtpService;
    }
    public function getAllOutletsByStation(Request $request, $start = 1, $end = 20, $total = null,$sid)
    {   
        switch ($request->method()) {
            case 'POST':
                if ($request->submit == "next") {
                    $start += 20;
                    $end = $start + 20 - 1;
                } else if ($request->submit == "previous") {
                    $start -= 20;
                    $end = $start + 20 - 1;
                }
                break;

            default:
                // invalid request
                break;
        }
        Log::channel('dailyInfo')->info('input123' . $start . ' ' . $end);

        if ($start == 1)
            $total = \App\Models\Outlet::where('station_id',$sid)->count();
        if ($end > $total)
            $end = $total;
        if ($start > $total)
            $outlets = null;
        else
            $outlets = \App\Models\Outlet::where('station_id',$sid)->skip($start - 1)->take($end - $start + 1)->get();
        $pageCount = (int)($total / 20) + 1;
        Log::channel('dailyInfo')->info('input' . print_r($outlets, true));

        if (Auth::user()->hasRole('vendor')) {
            if (Auth::user()->outlets != null)
                $outlets = Auth::user()->outlets;
            else
                $outlets = null;
        } else

        {   

              $total = \App\Models\Outlet::where('station_id',$sid)->count();
              if($total >=1)
              {
               $outlets = $outlets->map(function ($out, $i) {
                if ($out->closingPeriod == '')
                    $period = json_decode(json_encode([]));
                else
                    $period = json_decode($out->closingPeriod);

                $now = \Carbon\Carbon::now();
                $out->open = true;
                $weekMap = [
                    0 => 'SUN',
                    1 => 'MON',
                    2 => 'TUE',
                    3 => 'WED',
                    4 => 'THU',
                    5 => 'FRI',
                    6 => 'SAT',
                ];
                array_map(function ($p) use ($out, $now) {
                    $f = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedFrom)->startOfDay();
                    $t = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedTo)->endOfDay();
                    if ($now->gte($f) && $now->lte($t))
                        $out->open = false;
                }, $period);
                if ($out->open) {
                    if ($out->weeklyClosed == '')
                        $weekly = json_decode(json_encode([]));
                    else
                        $weekly = json_decode($out->weeklyClosed);
                    array_map(function ($w) use ($out, $now, $weekMap) {
                        if ($weekMap[$now->dayOfWeek] == $w)
                            $out->open = false;
                    }, $weekly);
                }
                if ($out->open)
                    $out->open = 'checked';
                else
                    unset($out->open);
                $out->weeklyClosed = json_decode($out->weeklyClosed);
                $out->closingPeriod = json_decode($out->closingPeriod);

                if ($this->isVegNonvegClosed($out)) {
                    if ($out->veg_non_veg == 0)
                        $out->veg_non_veg = 0;
                    else
                        $out->veg_non_veg = 1;
                } else
                    $out->veg_non_veg = 2;
                $out->weeklyClosed_veg_non_veg = json_decode($out->weeklyClosed_veg_non_veg);
                $out->closingPeriod_veg_non_veg = json_decode($out->closingPeriod_veg_non_veg);

                return $out;
            });
           }
           else
           {
             return view('backend.outlets.outlatesByStation', ['outlets' => [], 'pageCount' => $pageCount, 'start' => $start, 'end' => $end, 'total' => $total,'sid'=>$sid]);
           }
        }
        if ($request->has('api') && (int)$request->input('api'))
            return response()->json(['status' => 'success', 'outlets' => $outlets->toArray()], 200);

        return view('backend.outlets.outlatesByStation', ['outlets' => $outlets, 'pageCount' => $pageCount, 'start' => $start, 'end' => $end, 'total' => $total,'sid'=>$sid]);
    }

    public function getAllOutlets(Request $request, $start = 1, $end = 20, $total = null)
    {   
        $this->authorize('outlet-list');
        switch ($request->method()) {
            case 'POST':
                if ($request->submit == "next") {
                    $start += 20;
                    $end = $start + 20 - 1;
                } else if ($request->submit == "previous") {
                    $start -= 20;
                    $end = $start + 20 - 1;
                }
                break;

            default:
                // invalid request
                break;
        }

        // Log::channel('dailyInfo')->info('input123' . $start . ' ' . $end);

        $search='';
        if(isset($request->searchOutlets) && $request->searchOutlets !='')
        {
            $search=$request->searchOutlets;
        }
       
        if ($start == 1)
            $total = \App\Models\Outlet::Where(function ($query) use ($search){ 
                $query ->orWhere('outletName', 'like', '%' . $search . '%');
                })->where('is_delete',0)->count();

        if ($end > $total)
            $end = $total;
        if ($start > $total)
            $outlets = null;
        else
        if(isset($request->searchOutlets) && $request->searchOutlets !='')
        {
            $search=$request->searchOutlets;
            $outlets = \App\Models\Outlet::Where(function ($query) use ($search){ 
                $query ->orWhere('outletName', 'like', '%' . $search . '%');
                
                })->where('is_delete',0)->get();
        }
        else{
            $outlets = \App\Models\Outlet::skip($start - 1)->take($end - $start + 1)->where('is_delete',0)->get();
        }
           
              
               
        $pageCount = (int)($total / 20) + 1;
        
        // Log::channel('dailyInfo')->info('input' . print_r($outlets, true));

        if (Auth::user()->hasRole('vendor')) {
            if (Auth::user()->outlets != null)
                $outlets = Auth::user()->outlets;
            else
                $outlets = null;
        } else
            if(!empty($outlets) && sizeof($outlets)>0){
            $outlets = $outlets->map(function ($out, $i) {

                if (!empty($out->pushed_by))
                {   
                    $userData=User::where('id',$out->pushed_by)->first();
                    $out->pushed_by_name=$userData->first_name.' '.$userData->last_name;
                }

                if ($out->closingPeriod == '')
                    $period = json_decode(json_encode([]));
                else
                    $period = json_decode($out->closingPeriod);

                $now = \Carbon\Carbon::now();
                $out->open = true;
                $weekMap = [
                    0 => 'SUN',
                    1 => 'MON',
                    2 => 'TUE',
                    3 => 'WED',
                    4 => 'THU',
                    5 => 'FRI',
                    6 => 'SAT',
                ];
                array_map(function ($p) use ($out, $now) {
                    $f = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedFrom)->startOfDay();
                    $t = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedTo)->endOfDay();
                    if ($now->gte($f) && $now->lte($t))
                        $out->open = false;
                }, $period);

                if ($out->open) {

                    if ($out->weeklyClosed == '')
                        $weekly = json_decode(json_encode([]));
                    else
                        $weekly = json_decode($out->weeklyClosed);
                        array_map(function ($w) use ($out, $now, $weekMap) {
                            if ($weekMap[$now->dayOfWeek] == $w)
                                $out->open = false;
                        }, $weekly);
                }
                if ($out->open)
                    $out->open = true;
                else
                    unset($out->open);
                
                $out->weeklyClosed = json_decode($out->weeklyClosed);
                $out->closingPeriod = json_decode($out->closingPeriod);

                if ($this->isVegNonvegClosed($out)) {
                    if ($out->veg_non_veg == 0)
                        $out->veg_non_veg = 0;
                    else
                        $out->veg_non_veg = 1;
                } else
                    $out->veg_non_veg = 2;
                $out->weeklyClosed_veg_non_veg = json_decode($out->weeklyClosed_veg_non_veg);
                $out->closingPeriod_veg_non_veg = json_decode($out->closingPeriod_veg_non_veg);

                return $out;
            });
            }
        if ($request->has('api') && (int)$request->input('api'))
            return response()->json(['status' => 'success', 'outlets' => $outlets->toArray()], 200);
        
        return view('backend.outlets.index', ['outlets' => $outlets, 'pageCount' => $pageCount, 'start' => $start, 'end' => $end, 'total' => $total,'?searchOutlets='.$search,"searchVal" => $search]);
    }

    public function searchOutlets(Request $request, $search)
    {  
        $this->authorize('outlet-list');
        /*
        switch ($request->method()) {
            case 'GET':
                break;

            default:
                // invalid request
                return view('backend.outlets.search.result', ['outlets' => null]);
        }
        */
        $outlets = \App\Models\Outlet::where("outletName", "like", "%".$search."%")->get()->map(function ($out, $i) {
                if ($out->closingPeriod == '')
                    $period = json_decode(json_encode([]));
                else
                    $period = json_decode($out->closingPeriod);

                $now = \Carbon\Carbon::now();
                $out->open = true;
                $weekMap = [
                    0 => 'SUN',
                    1 => 'MON',
                    2 => 'TUE',
                    3 => 'WED',
                    4 => 'THU',
                    5 => 'FRI',
                    6 => 'SAT',
                ];
                array_map(function ($p) use ($out, $now) {
                    $f = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedFrom)->startOfDay();
                    $t = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedTo)->endOfDay();
                    if ($now->gte($f) && $now->lte($t))
                        $out->open = false;
                }, $period);
                if ($out->open) {
                    if ($out->weeklyClosed == '')
                        $weekly = json_decode(json_encode([]));
                    else
                        $weekly = json_decode($out->weeklyClosed);
                    array_map(function ($w) use ($out, $now, $weekMap) {
                        if ($weekMap[$now->dayOfWeek] == $w)
                            $out->open = false;
                    }, $weekly);
                }
                if ($out->open)
                    $out->open = 'checked';
                else
                    unset($out->open);
                $out->weeklyClosed = json_decode($out->weeklyClosed);
                $out->closingPeriod = json_decode($out->closingPeriod);

                if ($this->isVegNonvegClosed($out)) {
                    if ($out->veg_non_veg == 0)
                        $out->veg_non_veg = 0;
                    else
                        $out->veg_non_veg = 1;
                } else
                    $out->veg_non_veg = 2;
                $out->weeklyClosed_veg_non_veg = json_decode($out->weeklyClosed_veg_non_veg);
                $out->closingPeriod_veg_non_veg = json_decode($out->closingPeriod_veg_non_veg);

                return $out;
            });

        return view('backend.outlets.search.result', ['outlets' => $outlets]);
    }

    public function getAllOutletsInTabular(Request $request, $start = 1, $end = 20, $total = null)
    {
        switch ($request->method()) {
            case 'POST':
                if ($request->submit == "next") {
                    $start += 20;
                    $end = $start + 20 - 1;
                } else if ($request->submit == "previous") {
                    $start -= 20;
                    $end = $start + 20 - 1;
                }
                break;

            default:
                // invalid request
                break;
        }
        Log::channel('dailyInfo')->info('input123' . $start . ' ' . $end);

        if ($start == 1)
            $total = \App\Models\Outlet::count();
        if ($end > $total)
            $end = $total;
        if ($start > $total)
            $outlets = null;
        else
            $outlets = \App\Models\Outlet::skip($start - 1)->take($end - $start + 1)->get();
        $pageCount = (int)($total / 20) + 1;
        Log::channel('dailyInfo')->info('input' . print_r($outlets, true));

        if (auth()->user()->hasRole('vendor')) {
            if (Auth::user()->outlets != null)
                $outlets = Auth::user()->outlets;
            else
                $outlets = null;
        } else
            $outlets = $outlets->map(function ($out, $i) {
                if ($out->closingPeriod == '')
                    $period = json_decode(json_encode([]));
                else
                    $period = json_decode($out->closingPeriod);

                $now = \Carbon\Carbon::now();
                $out->open = true;
                $weekMap = [
                    0 => 'SUN',
                    1 => 'MON',
                    2 => 'TUE',
                    3 => 'WED',
                    4 => 'THU',
                    5 => 'FRI',
                    6 => 'SAT',
                ];
                array_map(function ($p) use ($out, $now) {
                    $f = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedFrom)->startOfDay();
                    $t = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedTo)->endOfDay();
                    if ($now->gte($f) && $now->lte($t))
                        $out->open = false;
                }, $period);
                if ($out->open) {
                    if ($out->weeklyClosed == '')
                        $weekly = json_decode(json_encode([]));
                    else
                        $weekly = json_decode($out->weeklyClosed);
                    array_map(function ($w) use ($out, $now, $weekMap) {
                        if ($weekMap[$now->dayOfWeek] == $w)
                            $out->open = false;
                    }, $weekly);
                }
                if ($out->open)
                    $out->open = 'checked';
                else
                    unset($out->open);
                $out->weeklyClosed = json_decode($out->weeklyClosed);
                $out->closingPeriod = json_decode($out->closingPeriod);

                if ($this->isVegNonvegClosed($out)) {
                    if ($out->veg_non_veg == 0)
                        $out->veg_non_veg = 0;
                    else
                        $out->veg_non_veg = 1;
                } else
                    $out->veg_non_veg = 2;
                $out->weeklyClosed_veg_non_veg = json_decode($out->weeklyClosed_veg_non_veg);
                $out->closingPeriod_veg_non_veg = json_decode($out->closingPeriod_veg_non_veg);

                return $out;
            });
        if ($request->has('api') && (int)$request->input('api'))
            return response()->json(['status' => 'success', 'outlets' => $outlets->toArray()], 200);

        return view('backend.outlets.tabular', ['outlets' => $outlets, 'pageCount' => $pageCount, 'start' => $start, 'end' => $end, 'total' => $total]);
    }
    public function isVegNonvegClosed($outlet)
    {
        if ($outlet->closingPeriod_veg_non_veg == null)
            $closingPeriods = json_decode('[]');
        else
            $closingPeriods = json_decode($outlet->closingPeriod_veg_non_veg);
        $outlet->veg_non_veg_closed = false;
        $now = \Carbon\Carbon::now();
        if(isset($closingPeriods)){
        foreach ($closingPeriods as $closingPeriod) {
            if (
                $now->greaterThanOrEqualTo(\Carbon\Carbon::createFromFormat('Y-m-d', $closingPeriod->closedFrom_veg_non_veg)->startOfDay())
                && $now->lessThanOrEqualTo((\Carbon\Carbon::createFromFormat('Y-m-d', $closingPeriod->closedTo_veg_non_veg)->endOfDay()))
            ) {
                $outlet->veg_non_veg_closed = true;
                break;
            }
        }
    }
        if (!$outlet->veg_non_veg_closed) {
            $weekMap = [
                0 => 'SUN',
                1 => 'MON',
                2 => 'TUE',
                3 => 'WED',
                4 => 'THU',
                5 => 'FRI',
                6 => 'SAT',
            ];
            $now = \Carbon\Carbon::now();
            if ($outlet->weeklyClosed_veg_non_veg == '' || $outlet->weeklyClosed_veg_non_veg == null)
                $weekly = json_decode(json_encode([]));
            else
                $weekly = json_decode($outlet->weeklyClosed_veg_non_veg);
            array_map(function ($w) use ($outlet, $now, $weekMap) {
                if ($weekMap[$now->dayOfWeek] == $w)
                    $outlet->veg_non_veg_closed = true;
            }, $weekly);
        }
        $closed = $outlet->veg_non_veg_closed;
        unset($outlet->veg_non_veg_closed);
        return $closed;
    }

    public function createOrder(Request $request)
    {
        return response()->json($request->all(), 200);
    }

    public function createOrderTemplate(Request $request)
    {
        return view('backend.orders.create');
    }


    
    public function orders(Request $request, $approve, $json)
    {  
        // if(sizeof($request->all())>0){

            $this->authorize('view-order-history');
            $this->authorize('approve-order');

        // }

        try {

            if ($request->filled('date')) {
                $start = \Carbon\Carbon::createFromFormat('m/d/Y', $request->input('date'), 'Asia/Kolkata')->startOfDay();
            } else {
                $start = \Carbon\Carbon::today()->startOfDay();
            }
            
            
            $cancelled = null;
            
            if($request->has('cancelled')){
                $cancelled = $request->input('cancelled');
            }
            
            #IRCTC ORDER

            $end = $start->copy()->addHours(24);

            $orders = \App\Models\Order::where(function ($q) use ($start, $end, $cancelled) {
                $q->where('created_at', '>=', $start)
                    ->where('created_at', '<', $end);
                    
                    if($cancelled){
                        $q->where('status', '=', 'ORDER_CANCELLED');
                    }
                    
                if ($this->__api && $this->__outlet_id) {
                    $q->where('outlet_id', (int)$this->__outlet_id);
                }
                /*})
                                    ->where(function ($q){
                                        $q->where('status_status', 'PENDING');*/
            })->orderBy('created_at', 'desc')->get()->map(function ($f) {
                $x = $f;
                $x->outletDetails = \App\Models\Outlet::find($x->outlet_id);
                if(isset($x->outletDetails->deliveryBoys)){
                $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                if ($x->deliveryBoy != null) {
                    $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                    $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                } else
                    $x->deliveryBoyName = '';
                $x->vendorOrder = false;
                $x->vendorOrder1 = 'false';

                return $x;
            }else
            {
              $x->deliveryBoyName = '';
                $x->vendorOrder = false;
                $x->vendorOrder1 = 'false';   
            }
            }); 

            #Website

            $vendorOrders = \App\Models\VendorOrder::where(function ($q) use ($start, $end, $cancelled) {
                $q->where('created_at', '>=', $start)
                    ->where('created_at', '<', $end);
            
                if($cancelled){
                        $q->where('status', '=', 'ORDER_CANCELLED');
                }
                    
                if ($this->__api && $this->__outlet_id) {
                    $q->where('outlet', (int)$this->__outlet_id);
                }
                /*})
                                    ->where(function ($q){
                                        $q->where('status_status', 'PENDING');*/
            })->orderBy('created_at', 'desc')->get()->map(function ($f) {

                $x = $f;
                $deliveryDetails = new \stdClass;
                $deliveryDetails->pnr = $x->pnr;
                $deliveryDetails->trainNo = $x->trainNo;
                $deliveryDetails->trainName = $x->trainName;
                $deliveryDetails->station = $x->stationName;
                $deliveryDetails->stationCode = $x->stationCode;
                $deliveryDetails->berth = $x->berth;
                $deliveryDetails->coach = $x->coach;
                $deliveryDetails->ETA = $x->deliveryDate;
                $x->deliveryDetails = json_encode($deliveryDetails);

                $c = json_decode($x->customer);

                $x->customerDetails = json_encode($c);

                $priceDetails = new \stdClass;
                $priceDetails->totalAmount = $x->totalAmount;
                $x->priceDetails = json_encode($priceDetails);
                $x->outletDetails = \App\Models\Outlet::find($x->outlet);
                if(isset($x->outletDetails->deliveryBoys)){
                $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                if ($x->deliveryBoy != null) {
                    $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                    $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                } else
                    $x->deliveryBoyName = '';

                $x->vendorOrder = true;
                $x->vendorOrder1 = 'true';

                return $x;
            }
                else
                {
                 $x->deliveryBoyName = '';

                $x->vendorOrder = true;
                $x->vendorOrder1 = 'true';

                return $x;   
                }
            });

            

            $orders = $orders->toBase()->merge($vendorOrders);
            
            $orders = $orders->sortBy(function ($order) {
                if(isset($order->created_at))
                {
                 return $order->created_at;
                }
                else
                {
                  return '';
                }
                
            })->reverse()->values();

            if ((isset($json) && $json == 'json') || $this->__api) {
                $orders = $orders->map(function ($o) {
                    $x = $o;
                    
                   if(!empty($o->aggregatorDetails))
                   {
                     $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                   }
                   if(!empty($o->customerDetails))
                   {
                    $x->customerDetails = json_decode($o->customerDetails);
                   }
                   if(!empty($o->priceDetails))
                   {
                      $x->priceDetails = json_decode($o->priceDetails);
                   }
                   if(!empty($o->orderItems))
                   {
                      $x->orderItems = json_decode($o->orderItems);
                   }

                    return $x;
                });
                
                return response()->json($orders, 200);

            } else if ($request->filled('object') && $request->input('object') == true) {
                $orders = $orders->map(function ($o) {
                    $x = $o;
                    if (!$o->updated) {
                        $x->orderBackColor = '#d65151';
                        $x->color = '#fff';
                    } else {
                        $x->orderBackColor = '#fff';
                        $x->color = '#000';
                    }
                    $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                    $x->customerDetails = json_decode($o->customerDetails);
                    $x->priceDetails = json_decode($o->priceDetails);
                    $x->orderItems = json_decode($o->orderItems);

                    return $x;
                });
                return $orders;
            }
        

            return view('backend.admins.order', compact('orders'));
        } catch (\Exception $e) {
            // dd($e);
            return response(['message' => 'invalid request'], 200);
        }
    }
    
    
    public function ordersV2(Request $request){
        

        $input = $request->input('date');
        
        if(! $input){
            return response()->json(['message' => 'Input Error']);
        }
        
        try{
            $_orders = DB::select("CALL orders_fetch_combined(?)", [$input]);
            $orders = collect($_orders)->map(function($order){
                $order->orderItems = json_decode($order->orderItems);
                $order->deliveryDetails = json_decode($order->deliveryDetails);
                $order->priceDetails = json_decode($order->priceDetails);
                $order->outletDetails = json_decode($order->outletDetails);
                $order->customerDetails = json_decode($order->customerDetails);
                return $order;
            }); 
        }catch(\Exception $e){
            $orders = [];
        }
        
        
        return response()->json(compact('orders'));
    }
    

    public function update(Request $request, $orderId)
    {
        //Log::channel('dailyInfo')->info('input'.$request->input('vendor'));
        if (!isset($orderId) || !$request->filled('vendor')) {
            session()->flash('error', 'Request input parameters are missing.');
            return redirect()->back();
        }

        if ($request->input('vendor') == 'true') {
            //Log::channel('dailyInfo')->info('vendor'.$request->input('vendor'));
            $order = \App\Models\VendorOrder::where('orderId', $orderId)->first();
            $order->updated = 1;
        } else {
            //Log::channel('dailyInfo')->info($request->input('vendor'));
            $order = \App\Models\Order::where('orderId', $orderId)->first();
            $order->updated = 1;
        }
        $order->save();
        session()->flash('success', 'Order Updated successfully.');
        return redirect()->back();
    }
    public function massUpdateStatusApprove(Request $request)
    {
        try {
            $fields = json_decode($request->getContent());
            $orders = $fields->orders;
            $status = $fields->status;
            $dataVendor = array();
            $data = array();
            foreach ($orders as $item) {
                if ($item->vendorOrder)
                    $dataVendor[] = $item->orderId;
                else
                    $data[] = $item->orderId;
            }
            if (sizeof($dataVendor) > 0) {
                $vendorOrderUpdate = \App\Models\VendorOrder::where(function ($q) use ($dataVendor) {
                    $q->where('orderId', $dataVendor[0]);
                    foreach ($dataVendor as $vendorOrder) {
                        $q->orWhere('orderId', $vendorOrder);
                    }
                });
                $vendorOrderUpdate->update(array('status' => $status));
            }

            if (sizeof($data) > 0) {
                $orderUpdate = \App\Models\Order::where(function ($q) use ($data) {
                    $q->where('orderId', $data[0]);
                    foreach ($data as $order) {
                        $q->orWhere('orderId', $order);
                    }
                });
                $orderUpdate->update(array('status' => $status));
            }
            if (isset($fields->date)) {
                $start = \Carbon\Carbon::createFromFormat('m/d/Y', $fields->date, 'Asia/Kolkata')->startOfDay();
            } else {
                $start = \Carbon\Carbon::today()->startOfDay();
            }
            $end = $start->copy()->addHours(24);
            $orders = \App\Models\Order::where(function ($q) use ($start, $end) {
                $q->where('created_at', '>=', $start)
                    ->where('created_at', '<', $end);
                /*})
                                    ->where(function ($q){
                                        $q->where('status_status', 'PENDING');*/
            })->orderBy('created_at', 'desc')->get()->map(function ($f) {
                $x = $f;
                $x->outletDetails = \App\Models\Outlet::find($x->outlet_id);
                $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                if ($x->deliveryBoy != null) {
                    $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                    $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                } else
                    $x->deliveryBoyName = '';
                $x->vendorOrder = false;
                return $x;
            });
            $vendorOrders = \App\Models\VendorOrder::where(function ($q) use ($start, $end) {
                $q->where('created_at', '>=', $start)
                    ->where('created_at', '<', $end);
                /*})
                                    ->where(function ($q){
                                        $q->where('status_status', 'PENDING');*/
            })->orderBy('created_at', 'desc')->get()->map(function ($f) {
                $x = $f;
                $deliveryDetails = new \stdClass;
                $deliveryDetails->pnr = $x->pnr;
                $deliveryDetails->trainNo = $x->trainNo;
                $deliveryDetails->trainName = $x->trainName;
                $deliveryDetails->station = $x->stationName;
                $deliveryDetails->stationCode = $x->stationCode;
                $deliveryDetails->berth = $x->berth;
                $deliveryDetails->coach = $x->coach;
                $deliveryDetails->ETA = $x->deliveryDate;
                $x->deliveryDetails = json_encode($deliveryDetails);

                $x->customerDetails = json_decode($x->customer);
                $x->customerDetails->customerName = $x->customerDetails->fullName;
                $x->customerDetails = json_encode($x->customerDetails);

                $priceDetails = new \stdClass;
                $priceDetails->totalAmount = $x->totalAmount;
                $x->priceDetails = json_encode($priceDetails);
                $x->outletDetails = \App\Models\Outlet::find($x->outlet);
                $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                if ($x->deliveryBoy != null) {
                    $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                    $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                } else
                    $x->deliveryBoyName = '';

                $x->vendorOrder = true;
                return $x;
            });
            $orders = $orders->toBase()->merge($vendorOrders);
            $orders = $orders->sortBy(function ($order) {
                return $order->created_at;
            })->reverse()->values();

            $orders = $orders->map(function ($o) {
                $x = $o;
                $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                $x->customerDetails = json_decode($o->customerDetails);
                $x->priceDetails = json_decode($o->priceDetails);
                $x->orderItems = json_decode($o->orderItems);

                return $x;
            });
            return response()->json($orders, 200);
        } catch (\Exception $e) {
            dd($e);
            return response(['message' => 'invalid request'], 200);
        }
    }
    public function ordersByOutlet(Request $request, $outlet)
    {
        try {
            $out = \App\Models\Outlet::find($outlet);
            $this->authorize('pol_view_orders', $out);
            $start = \Carbon\Carbon::today()->startOfDay();
            $end = \Carbon\Carbon::today()->startOfDay()->addHours(24);
            $orders = \App\Models\Outlet::find($outlet)->orders()
                ->where(function ($q) use ($start, $end) {
                    $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                })->orderBy('created_at', 'desc')->get()->map(function ($f) use ($out) {
                    $x = $f;
                    $x->outletDetails = $out;
                    $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                    if ($x->deliveryBoy != null) {
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                    } else
                        $x->deliveryBoyName = '';
                    $x->vendorOrder = false;
                    return $x;
                });
            $vendorOrders = \App\Models\Outlet::find($outlet)->vendor_orders()
                ->where(function ($q) use ($start, $end) {
                    $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                })->orderBy('created_at', 'desc')->get()->map(function ($f) use ($out) {
                    $x = $f;
                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $x->pnr;
                    $deliveryDetails->trainNo = $x->trainNo;
                    $deliveryDetails->trainName = $x->trainName;
                    $deliveryDetails->station = $x->stationName;
                    $deliveryDetails->stationCode = $x->stationCode;
                    $deliveryDetails->berth = $x->berth;
                    $deliveryDetails->coach = $x->coach;
                    $deliveryDetails->ETA = $x->deliveryDate;
                    $x->deliveryDetails = json_encode($deliveryDetails);

                    $x->customerDetails = json_decode($x->customer);
                    $x->customerDetails->customerName = $x->customerDetails->fullName;
                    $x->customerDetails = json_encode($x->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $x->totalAmount;
                    $x->priceDetails = json_encode($priceDetails);

                    $x->outletDetails = $out;
                    if ($x->deliveryBoy != null) {
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                    } else
                        $x->deliveryBoyName = '';
                    $x->vendorOrder = true;
                    return $x;
                });
            $orders = $orders->toBase()->merge($vendorOrders);
            $orders = $orders->sortBy(function ($order) {
                return $order->created_at;
            })->reverse()->values();

            return view('backend.orders.index', compact('orders', 'outlet'));
        } catch (\Exception $e) {
            dd($e);
            return response(['message' => 'invalid request'], 200);
        }
    }

    public function ordersByOutletJson(Request $request, $outlet)
    {
        try {
            if ($request->filled('date')) {
                $start = \Carbon\Carbon::createFromFormat('m/d/Y', $request->input('date'), 'Asia/Kolkata')->startOfDay();
                $end = \Carbon\Carbon::createFromFormat('m/d/Y', $request->input('date'), 'Asia/Kolkata')->startOfDay()->addHours(24);
            } else {
                $start = \Carbon\Carbon::today()->startOfDay();
                $end = \Carbon\Carbon::today()->startOfDay()->addHours(24);
            }
            $out = \App\Models\Outlet::find($outlet);
            $this->authorize('pol_view_orders', $out);
            $orders = $out->orders()
                ->where(function ($q) use ($start, $end) {
                    $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                })->orderBy('created_at', 'desc')->get()->map(function ($f) use ($out) {
                    $x = $f;
                    $x->outletDetails = $out;
                    $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                    if ($x->deliveryBoy != null) {
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                    } else
                        $x->deliveryBoyName = '';
                    $x->vendorOrder = false;

                    return $x;
                });
            $vendorOrders = $out->vendor_orders()
                ->where(function ($q) use ($start, $end) {
                    $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                })->orderBy('created_at', 'desc')->get()->map(function ($f) use ($out) {
                    $x = $f;
                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $x->pnr;
                    $deliveryDetails->trainNo = $x->trainNo;
                    $deliveryDetails->trainName = $x->trainName;
                    $deliveryDetails->station = $x->stationName;
                    $deliveryDetails->stationCode = $x->stationCode;
                    $deliveryDetails->berth = $x->berth;
                    $deliveryDetails->coach = $x->coach;
                    $deliveryDetails->ETA = $x->deliveryDate;
                    $x->deliveryDetails = json_encode($deliveryDetails);

                    $x->customerDetails = json_decode($x->customer);
                    $x->customerDetails->customerName = $x->customerDetails->fullName;
                    $x->customerDetails = json_encode($x->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $x->totalAmount;
                    $x->priceDetails = json_encode($priceDetails);

                    $x->outletDetails = $out;
                    $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                    if ($x->deliveryBoy != null) {
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                    } else
                        $x->deliveryBoyName = '';
                    $x->vendorOrder = true;
                    return $x;
                });
            $orders = $orders->toBase()->merge($vendorOrders);
            $orders = $orders->sortBy(function ($order) {
                return $order->created_at;
            })->reverse()->values();

            $orders = $orders->map(function ($o) use ($out) {
                $x = $o;
                $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                $x->outletDetails = $out;
                $x->customerDetails = json_decode($o->customerDetails);
                $x->priceDetails = json_decode($o->priceDetails);
                $x->orderItems = json_decode($o->orderItems);

                return $x;
            });
            //print_r(collect($orders));
            //echo json_encode($orders);
            return response()->json($orders, 200);
        } catch (\Exception $e) {
            dd($e);
            return response()->json(['message' => 'invalid request'], 200);
        }
    }
    public function massUpdateStatus(Request $request, $outlet)
    {
        try {
            $fields = json_decode($request->getContent());
            $orders = $fields->orders;
            $status = $fields->status;
            $dataVendor = array();
            $data = array();
            foreach ($orders as $item) {
                if ($item->vendorOrder)
                    $dataVendor[] = $item->orderId;
                else
                    $data[] = $item->orderId;
            }
            if (sizeof($dataVendor) > 0) {
                $vendorOrderUpdate = \App\Models\VendorOrder::where(function ($q) use ($dataVendor) {
                    $q->where('orderId', $dataVendor[0]);
                    foreach ($dataVendor as $vendorOrder) {
                        $q->orWhere('orderId', $vendorOrder);
                    }
                });
                $vendorOrderUpdate->update(array('status' => $status));
            }

            if (sizeof($data) > 0) {
                $orderUpdate = \App\Models\Order::where(function ($q) use ($data) {
                    $q->where('orderId', $data[0]);
                    foreach ($data as $order) {
                        $q->orWhere('orderId', $order);
                    }
                });
                $orderUpdate->update(array('status' => $status));
            }


            if ($fields->date) {
                $start = \Carbon\Carbon::createFromFormat('m/d/Y', $fields->date, 'Asia/Kolkata')->startOfDay();
                $end = \Carbon\Carbon::createFromFormat('m/d/Y', $fields->date, 'Asia/Kolkata')->startOfDay()->addHours(24);
            } else {
                $start = \Carbon\Carbon::today()->startOfDay();
                $end = \Carbon\Carbon::today()->startOfDay()->addHours(24);
            }
            $out = \App\Models\Outlet::find($outlet);
            $this->authorize('pol_view_orders', $out);
            $orders = $out->orders()
                ->where(function ($q) use ($start, $end) {
                    $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                })->orderBy('created_at', 'desc')->get()->map(function ($f) use ($out) {
                    $x = $f;
                    $x->outletDetails = $out;
                    $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                    if ($x->deliveryBoy != null) {
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                    } else
                        $x->deliveryBoyName = '';
                    $x->vendorOrder = false;

                    return $x;
                });
            $vendorOrders = $out->vendor_orders()
                ->where(function ($q) use ($start, $end) {
                    $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                })->orderBy('created_at', 'desc')->get()->map(function ($f) use ($out) {
                    $x = $f;
                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $x->pnr;
                    $deliveryDetails->trainNo = $x->trainNo;
                    $deliveryDetails->trainName = $x->trainName;
                    $deliveryDetails->station = $x->stationName;
                    $deliveryDetails->stationCode = $x->stationCode;
                    $deliveryDetails->berth = $x->berth;
                    $deliveryDetails->coach = $x->coach;
                    $deliveryDetails->ETA = $x->deliveryDate;
                    $x->deliveryDetails = json_encode($deliveryDetails);

                    $x->customerDetails = json_decode($x->customer);
                    $x->customerDetails->customerName = $x->customerDetails->fullName;
                    $x->customerDetails = json_encode($x->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $x->totalAmount;
                    $x->priceDetails = json_encode($priceDetails);

                    $x->outletDetails = $out;
                    $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                    if ($x->deliveryBoy != null) {
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                    } else
                        $x->deliveryBoyName = '';
                    $x->vendorOrder = true;
                    return $x;
                });
            $orders = $orders->toBase()->merge($vendorOrders);
            $orders = $orders->sortBy(function ($order) {
                return $order->created_at;
            })->reverse()->values();

            $orders = $orders->map(function ($o) use ($out) {
                $x = $o;
                $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                $x->outletDetails = $out;
                $x->customerDetails = json_decode($o->customerDetails);
                $x->priceDetails = json_decode($o->priceDetails);
                $x->orderItems = json_decode($o->orderItems);

                return $x;
            });
            //print_r(collect($orders));
            //echo json_encode($orders);
            return response()->json($orders, 200);
        } catch (\Exception $e) {
            dd($e);
            return response()->json(['message' => 'invalid request'], 200);
        }
    }

    public function Search($vender, $critaria, $term)
    {
        $orders = \App\Models\Order::where('outlet_id', $vender)->get();
        switch ($critaria) {
            case 'orderId':
                # code...
                return $results = $orders->filter(function ($item) use ($term) {
                    return $item->orderId == $term;
                });

            case 'customer':
                if (strlen($term) > 3) {
                    return $results = $orders->filter(function ($item) use ($term) {
                        $temp = json_decode($item->customerDetails);
                        return $retVal = (strstr($temp->customerName, $term)) ? $item : false;
                    });
                } else {
                    return (object)['message' => 'invalid Query'];
                }


            case 'mobile':
                return $results = $orders->filter(function ($item) use ($term) {
                    $temp = json_decode($item->customerDetails);
                    return $temp->mobile == $term;
                });

            default:
                return $orders;
        }
    }
    public function orderSearch(Request $request, $vendorId, $critaria, $term)
    {
        $check = $vendorId && $critaria && $term;
        try {
            if ($check) {
                $results = $this->Search($vendorId, $critaria, $term);
                header('Content-Type', 'application/json');
                return response()->json($results, 200);
            } else {
                dd('invalid Query');
            }
        } catch (\Exception $e) {
            dd($e);
        }
    }


    public function canRedeemGeneralCode($voucher)
    {
        try {
            return $voucher->check();
        } catch (\Throwable $th) {
            return false;
        }
    }

    public function orderUpdate(Request $request, $orderId)
    {
        try {
            //$this->authorize('pol_update_order', \App\Order::where('orderId',$orderId));
            $fields = $request->all();
            $vendorOrder = $fields['vendorOrder'];
            $fields['pending_status'] = $fields['status'];
            $fields['requestor_user_id'] = Auth::user()->id;
            $fields['status_status'] = 'PENDING';
            unset($fields['vendorOrder']);
            unset($fields['status']);
            if (isset($fields['reason1']))
                unset($fields['reason1']);
            if ($vendorOrder) {
                $update = \App\Models\VendorOrder::where('orderId', $orderId)->update($fields);
                $order = \App\Models\VendorOrder::where('orderId', $orderId)->first();
                if (\App\Models\VendorOrder::where('orderId', $orderId)->approvalReq()->first() == null) {
                    if ($order->status_history == null)
                        $status_history = '';
                    else
                        $status_history = $order->status_history;

                    $status = array(
                        'status' => $order->pending_status,
                        'pending_status' => null,
                        'status_history' => $status_history . '->' . $order->pending_status,
                        'status_status' => null
                    );
                    $update1 = \App\Models\VendorOrder::where('orderId', $orderId)->update($status);
                    if($status == 'ORDER_DELIVERED'){
                        event(new CustomerInvoiceJob($update1));
                    }
                    
                    $order = \App\Models\VendorOrder::where('orderId', $orderId)->first();
                }


                $deliveryDetails = new \stdClass;
                $deliveryDetails->pnr = $order->pnr;
                $deliveryDetails->trainNo = $order->trainNo;
                $deliveryDetails->trainName = $order->trainName;
                $deliveryDetails->station = $order->stationName;
                $deliveryDetails->stationCode = $order->stationCode;
                $deliveryDetails->berth = $order->berth;
                $deliveryDetails->coach = $order->coach;
                $deliveryDetails->ETA = $order->deliveryDate;
                $order->deliveryDetails = json_encode($deliveryDetails);

                $order->customerDetails = json_decode($order->customer);
                $order->customerDetails->customerName = $order->customerDetails->fullName;
                $order->customerDetails = json_encode($order->customerDetails);

                $priceDetails = new \stdClass;
                $priceDetails->totalAmount = $order->totalAmount;
                $order->priceDetails = json_encode($priceDetails);

                $order->outletDetails = \App\Models\Outlet::find($order->outlet);
                $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
                if ($order->deliveryBoy != null) {
                    $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                    $order->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                } else
                    $order->deliveryBoyName = '';
                $order->vendorOrder = true;
            } else {
                $update = \App\Models\Order::where('orderId', $orderId)->update($fields);
                $order = \App\Models\Order::where('orderId', $orderId)->first();
                if (\App\Models\Order::where('orderId', $orderId)->approvalReq()->first() == null) {
                    if ($order->status_history == null)
                        $status_history = '';
                    else
                        $status_history = $order->status_history;

                    $status = array(
                        'status' => $order->pending_status,
                        'pending_status' => null,
                        'status_history' => $status_history . '->' . $order->pending_status,
                        'status_status' => null
                    );
                    $update1 = \App\Models\Order::where('orderId', $orderId)->update($status);
                    $order = \App\Models\Order::where('orderId', $orderId)->first();
                }
                $deliveryDetails = json_decode($order->deliveryDetails);
                
                $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
                if ($order->deliveryBoy != null) {
                    $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                    $order->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                } else
                    $order->deliveryBoyName = '';
                $order->outletDetails = \App\Models\Outlet::find($order->outlet_id);
                $order->vendorOrder = false;
            }
            $order->aggregatorDetails = json_decode($order->aggregatorDetails);
            $order->customerDetails = json_decode($order->customerDetails);
            $order->priceDetails = json_decode($order->priceDetails);
            $order->orderItems = json_decode($order->orderItems);

            if ($update1) {
                if ($order->status == 'ORDER_UNDELIVERED') {
                    $message = "You have been charged Rs 100 against Non Delivery of Order id: %s";
                    $this->sendStatusMessage($order, $message);
                } else if ($order->status == 'ORDER_PARTIALLY_DELIVERED') {
                    $outletName = $order->outletDetails->outletName;
                    $message = "Customer has raised a complaint against $outletName for Order id: %s, at station \"$deliveryDetails->station\" As resolution of complaint an appropriate amount has been debited from Vendor Price";
                    $this->sendStatusMessage($order, $message);
                }
            }
            if ($update) {
                //CustomerInvoiceJob::dispatch($order);
                return response()->json($order, 200);
            }
            return response()->json(['message' => 'Updation failed'], 200);
        } catch (\Exception $e) {
            // dd($e);
            return response()->json(['message' => $e->getMessage()], 501);
        }
    }

    public function orderUpdateAdmin(Request $request, $orderId, $approve)
    {
        //try{
        //$this->authorize('pol_update_order', \App\Order::where('orderId',$orderId));
        $fields = $request->all();
        $vendorOrder = $fields['vendorOrder'];
        $fields['requestor_user_id'] = Auth::user()->id;
        unset($fields['vendorOrder']);
        if (isset($fields['reason1']))
            unset($fields['reason1']);
        $fields['status_status'] = null;
        $fields['pending_status'] = null;
        //$fields['status_reason'] = null;

        if ($vendorOrder) {
            $order = \App\Models\VendorOrder::where('orderId', $orderId)->first();
            if (isset($approve) && $approve == 'approve') {
                $fields['status'] = $order->pending_status;
            }
            if ($order->status_history == null)
                $status_history = '';
            else
                $status_history = $order->status_history;
            $fields['status_history'] = $status_history . '->' . $fields['status'];
            $update = \App\Models\VendorOrder::where('orderId', $orderId)->update($fields);
            $order = \App\Models\VendorOrder::where('orderId', $orderId)->first();

            $deliveryDetails = new \stdClass;
            $deliveryDetails->pnr = $order->pnr;
            $deliveryDetails->trainNo = $order->trainNo;
            $deliveryDetails->trainName = $order->trainName;
            $deliveryDetails->station = $order->stationName;
            $deliveryDetails->stationCode = $order->stationCode;
            $deliveryDetails->berth = $order->berth;
            $deliveryDetails->coach = $order->coach;
            $deliveryDetails->ETA = $order->deliveryDate;
            $order->deliveryDetails = json_encode($deliveryDetails);

            $order->customerDetails = json_decode($order->customer);
            $order->customerDetails->customerName = $order->customerDetails->fullName;
            $order->customerDetails = json_encode($order->customerDetails);

            $priceDetails = new \stdClass;
            $priceDetails->totalAmount = $order->totalAmount;
            $order->priceDetails = json_encode($priceDetails);

            $order->outletDetails = \App\Models\Outlet::find($order->outlet);
            $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
            if ($order->deliveryBoy != null) {
                $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                $order->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
            } else
                $order->deliveryBoyName = '';
            $order->vendorOrder = true;
        } else {
            $order = \App\Models\Order::where('orderId', $orderId)->first();
            if (isset($approve) && $approve == 'approve') {
                $fields['status'] = $order->pending_status;
            }
            if ($order->status_history == null)
                $status_history = '';
            else
                $status_history = $order->status_history;
            $fields['status_history'] = $status_history . '->' . $fields['status'];
            $update = \App\Models\Order::where('orderId', $orderId)->update($fields);
            $order = \App\Models\Order::where('orderId', $orderId)->first();

            $deliveryDetails = json_decode($order->deliveryDetails);
            $order->outletDetails = \App\Models\Outlet::find($order->outlet_id);
            $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
            if ($order->deliveryBoy != null) {
                $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                $order->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
            } else
                $order->deliveryBoyName = '';
            $order->vendorOrder = false;
        }
        $order->aggregatorDetails = json_decode($order->aggregatorDetails);
        $order->customerDetails = json_decode($order->customerDetails);
        $order->priceDetails = json_decode($order->priceDetails);
        $order->orderItems = json_decode($order->orderItems);

        if ($update) {
            if ($order->status == 'ORDER_UNDELIVERED') {
                $message = "You have been charged Rs 100 against Non Delivery of Order id: %s";
                $this->sendStatusMessage($order, $message);
            } else if ($order->status == 'ORDER_PARTIALLY_DELIVERED') {
                $outletName = $order->outletDetails->outletName;
                $message = "Customer has raised a complaint against $outletName for Order id: %s, at station \"$deliveryDetails->station\" As resolution of complaint an appropriate amount has been debited from Vendor Price";
                $this->sendStatusMessage($order, $message);
            }
            return response()->json($order, 200);
        }
        return response()->json(['message' => 'Updation failed'], 200);
        //}catch(\Exception $e){
        //return response()->json(['message'=>'Updation failed'],501);
        //}
    }

    public function orderRejectAdmin(Request $request, $orderId)
    {
        try {
            //$this->authorize('pol_update_order', \App\Order::where('orderId',$orderId));
            $fields = array();
            $vendorOrder = $request->input('vendorOrder');
            $fields['requestor_user_id'] = Auth::user()->id;
            $fields['requestor_user_id'] = null;
            $fields['status_status'] = null;
            $fields['pending_status'] = null;
            $fields['status_reason'] = null;
            if ($vendorOrder) {
                $update = \App\Models\VendorOrder::where('orderId', $orderId)->update($fields);
                $order = \App\Models\VendorOrder::where('orderId', $orderId)->first();

                $deliveryDetails = new \stdClass;
                $deliveryDetails->pnr = $order->pnr;
                $deliveryDetails->trainNo = $order->trainNo;
                $deliveryDetails->trainName = $order->trainName;
                $deliveryDetails->station = $order->stationName;
                $deliveryDetails->stationCode = $order->stationCode;
                $deliveryDetails->berth = $order->berth;
                $deliveryDetails->coach = $order->coach;
                $deliveryDetails->ETA = $order->deliveryDate;
                $order->deliveryDetails = json_encode($deliveryDetails);

                $order->customerDetails = json_decode($order->customer);
                $order->customerDetails->customerName = $order->customerDetails->fullName;
                $order->customerDetails = json_encode($order->customerDetails);

                $priceDetails = new \stdClass;
                $priceDetails->totalAmount = $order->totalAmount;
                $order->priceDetails = json_encode($priceDetails);

                $order->outletDetails = \App\Models\Outlet::find($order->outlet);
                $order->vendorOrder = true;
            } else {
                $update = \App\Models\Order::where('orderId', $orderId)->update($fields);
                $order = \App\Models\Order::where('orderId', $orderId)->first();

                $order->outletDetails = \App\Models\Outlet::find($order->outlet_id);
                $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
                if ($order->deliveryBoy != null) {
                    $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                    $order->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
                } else
                    $order->deliveryBoyName = '';
                $order->vendorOrder = false;
            }
            $order->aggregatorDetails = json_decode($order->aggregatorDetails);
            $order->customerDetails = json_decode($order->customerDetails);
            $order->priceDetails = json_decode($order->priceDetails);
            $order->orderItems = json_decode($order->orderItems);

            if ($update) {
                return response()->json($order, 200);
            }
            return response()->json(['message' => 'Updation failed'], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Updation failed'], 501);
        }
    }

    protected function sendOrder($url, $data)
    {
        $data_string = json_encode($data);
        //dd($data_string);
        //Log::channel('dailyInfo')->info(print_r($data_string,true));
        //echo $data_string;
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt(
            $ch,
            CURLOPT_HTTPHEADER,
            array(
                'Content-Type: application/json',
                'Content-Length: ' . strlen($data_string),
                //'Authorization:80d171c2-8bef-4b02-884f-fabffc769776')//dev-----SHOULD BE CORRECTED
                'Authorization:'.config('constkeys.irctc_token')
            ) //prod----
        );

        $result = curl_exec($ch);
        print_r($result);
        //Log::channel('dailyInfo')->info(print_r($result,true));

    }

    public function sendOrdertoIRCTC($url, $data)
    {
        $data_string = json_encode($data);
        // dd($data_string);
        //dd(compact('url','data'));
        //Log::channel('dailyInfo')->info(print_r($data_string,true));
        //echo $data_string;
        $token = config('constkeys.irctc_token');
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt(
            $ch,
            CURLOPT_HTTPHEADER,
            array(
                'Content-Type: application/json',
                'Content-Length: ' . strlen($data_string),
                //		'Authorization:80d171c2-8bef-4b02-884f-fabffc769776')//dev-----SHOULD BE CORRECTED
                'Authorization:'.$token
            ) //prod----
        );

        $result = curl_exec($ch);
        return $result;
        //Log::channel('dailyInfo')->info(print_r($result,true));

    }


    public function sendDebugOrdertoIRCTC($url, $data)
    {
        $data_string = json_encode($data);
        //dd($data_string);
        //dd(compact('url','data'));
        //Log::channel('dailyInfo')->info(print_r($data_string,true));
        //echo $data_string;
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt(
            $ch,
            CURLOPT_HTTPHEADER,
            array(
                'Content-Type: application/json',
                'Content-Length: ' . strlen($data_string),
                'Authorization:'.config('constkeys.irctc_token')
            ) //dev-----SHOULD BE CORRECTED
            //	'Authorization:80d171c2-8bef-4b02-884f-fabffc769776')//prod----
        );

        $result = curl_exec($ch);
        return $result;
        //Log::channel('dailyInfo')->info(print_r($result,true));

    }

    public function processHydrogen1(Request $request)
    {


        $fields = json_decode($request->getContent());

        $outlet = processOutlet($fields->outlet->id);
        $fields->outlet = $outlet;
        $fields->vendorOrderId = substr(str_shuffle(str_repeat("1234567890", 5)), 0, 10);
        $pnr = $fields->pnr->value;
        $fields->pnr = $pnr;
        $fields->berth = $fields->berth->value;
        $fields->coach = $fields->coach->value;
        $fields->customer = (object)[
            'fullName' => $fields->fullName->value,
            'email' => $fields->email->value,
            'mobile' => $fields->mobile->value
        ];
        $date = \Carbon\Carbon::now()->setTimezone('Asia/Kolkata')->format('Y-m-d h:i');
        $fields->bookingDate = "$date IST";
        $orderItems = array_map(function ($item) {
            return [
                "itemId" => $item->id,
                "itemName" => $item->itemName,
                "description" => $item->description,
                "basePrice" => $item->price,
                "sellingPrice" => $item->price + $item->price * 0.05,
                "taxRate" => $item->price * 0.05,
                "isVegetarian" => $item->isVegeterian == 1 ? true : false,
                "quantity" => $item->quantity,
                "option" => null,
            ];
        }, $fields->orderItems);

        unset($orderItems['st']);

        $subtotal = collect($orderItems)->sum(function ($item) {
            return $item['sellingPrice'] * $item['quantity'];
        });
        $txn = collect($orderItems)->sum(function ($item) {
            return ($item['taxRate'] * $item['quantity']);
        });
        $fields->orderItems = $orderItems;
        $fields->deliveryDate = "$date IST";
        $fields->amountPayable = $subtotal;
        $fields->totalAmount = $subtotal;
        $fields->vendorPaymentType = "CASH_ON_DELIVERY";
        $fields->gst = $txn;
        unset($fields->email);
        unset($fields->fullName);
        unset($fields->mobile);
        $this->sendOrder(config('constkeys.irctc_base_url').'/api/v1/order/vendor', $fields);
        return response()->json($fields, 200);
    }

    public function getAmountAfterDiscount(Request $request)
    {
        try {
            $fields = json_decode($request->getContent());

            $orderItems = array_map(function ($item) {
                //$basePrice = $item->price/1.05;
                return [
                    //"itemId"=> $item->id,
                    //"itemName"=> $item->itemName,
                    //"description"=> $item->description,
                    //"basePrice"=>$basePrice,
                    "sellingPrice" => $item->price,
                    //"taxRate"=> $item->price-$basePrice,
                    //"isVegetarian"=> $item->isVegeterian == 1? true:false,
                    "quantity" => $item->quantity,
                    //"option"=> null,
                ];
            }, $fields->orderItems->value);

            unset($orderItems['st']);

            $subtotal = collect($orderItems)->sum(function ($item) {
                return $item['sellingPrice'] * $item['quantity'];
            });
            $deliveryCost = \App\Models\Outlet::find($fields->outletId)->deliveryCost;
            //$fields->amountPayable = round($subtotal+$subtotal*0.05);
            $fields->amountPayable = round($subtotal);
            $o = null;
            $vo = null;
            $customerMobile = null;
            $d = 0;
            $wDiscount = 0;
            $balance = 0;
            $discountAmount = 0;
            $couponInvalid = false;
            $redeemCode = true;
            if (isset($fields->coupon->value) && $fields->coupon->value != null && $fields->coupon->value != '') {
                $o = \App\Models\Order::where('discount_code', $fields->coupon->value)
                    ->first();
                $vo = \App\Models\VendorOrder::where('discount_code', $fields->coupon->value)
                    ->first();
            }
            if ($o != null) {
                $vo = $o;
                Config::set('vouchers.relation_table', 'order_voucher');
                Config::set('vouchers.user_model', \App\Models\Order::class);
            }
            if ($vo != null) {
                $customerMobile = $vo->customerMobile;
                if ($vo->canRedeemCode($fields->coupon->value)) {
                    try {
                        $v1 = $vo->check($fields->coupon->value);
                        $amountP = $fields->amountPayable;
                        $discount = \App\Models\Discount::where(function ($q) use ($amountP) {
                            $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                                ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString())
                                ->where('amountFrom', '<=', $amountP)
                                ->where('amountTo', '>=', $amountP);
                        })->first();
                        if ($v1->data->get('amountFrom') <= $fields->amountPayable && $v1->data->get('amountTo') >= $fields->amountPayable) {
                            $d = $v1->data->get('discount');
                            $fields->couponCode = $v1->code;
                            $fields->discountAmount = round(($fields->amountPayable) * $d / 100);
                            if ($discount != null) {
                                if ($fields->discountAmount > $discount->upto)
                                    $fields->discountAmount = (int)$discount->upto;
                            }
                            $discountAmount = $fields->discountAmount;
                            $fields->amountPayable = $fields->amountPayable - $fields->discountAmount;
                        } else if ($discount != null) {
                            $d = $v1->data->get('discount');
                            $fields->couponCode = $v1->code;
                            $fields->discountAmount = (int)$discount->upto;
                            $discountAmount = $fields->discountAmount;
                            $fields->amountPayable = $fields->amountPayable - $fields->discountAmount;
                        }
                    } catch (\Exception $e) {
                        return  response()
                            ->json(array('status' => 'failure', 'balance' => $balance, 'wDiscount' => $wDiscount, 'discount' => $discountAmount, 'amountPayable' => $fields->amountPayable, 'message' => $e->getMessage()), 200);
                    }
                }
            } else {
                if (!empty($fields->coupon->value)) {
                    //added code for vouchers                           

                    $voucher1 = \BeyondCode\Vouchers\Models\Voucher::where('code', $fields->coupon->value)->first();
                    if ($voucher1 != null) {
                        $discnt1 = (int)$voucher1->data[0];
                        $discountAmount = (int)$discnt1 * $fields->amountPayable / 100;
                        $fields->amountPayable = $fields->amountPayable - $discountAmount;
                    } else {
                        $couponInvalid = true;
                    }

                    //added code for vouchers                           
                }
                /*
                            return  response()
                                        ->json(array('status'=>'failure', 'balance'=>$balance, 'wDiscount'=>$wDiscount,'discount'=>$discountAmount,'amountPayable'=>$fields->amountPayable,'message'=>'coupon code is invalid'),200);
                             *
                             */
            }
            $user = Auth::guard('order')->user();
            $user1 = Auth::guard('vendororder')->user();
            $user2 = Auth::guard('apiorder')->user();
            $wallet = 0;
            if ($user == null) {
                if ($user1 != null) {
                    $user = $user1;
                } else {
                    if ($user2 != null)
                        $user = $user2;
                    else
                        $user = Auth::guard('memoryuser')->user();
                }
            }
            if ($user != null && get_class($user->last()) != 'App\Models\MemoryUser') {
                $wDiscount = $user->last()->getAmountProduct1($fields->amountPayable);
                $fields->amountPayable -= $user->last()->getAmountProduct1($fields->amountPayable);
                $balance = $user->last()->balance;
            }
            if ($couponInvalid || !$redeemCode)
                return  response()
                    ->json(array(
                        'status' => 'success', 'couponStatus' => 'failure', 'message' => 'coupon code is invalid',
                        'amountPayable' => $fields->amountPayable, 'balance' => $balance, 'wDiscount' => $wDiscount, 'mobile' => $customerMobile, 'discount' => $discountAmount
                    ), 200);

            return  response()
                ->json(array(
                    'status' => 'success',
                    'amountPayable' => $fields->amountPayable, 'balance' => $balance, 'wDiscount' => $wDiscount, 'mobile' => $customerMobile, 'discount' => $discountAmount
                ), 200);
        } finally {
            Config::set('vouchers.relation_table', 'vendor_order_voucher');
            Config::set('vouchers.user_model', \App\Models\VendorOrder::class);
        }
    }

    # New Order API STARTS

    public function processHydrogenV2(Request $request)
    {
        $fields = $request['fields'];
        //dd($fields);
        $api = config('constkeys.irctc_base_url').'/api/v1/order/vendor';
        //$response =  $this->sendOrdertoIRCTC('https://stage-ecatering.ipsator.com/api/v1/order/vendor', $fields);//dev
        $response =  $this->sendOrdertoIRCTC($api, $fields); //dev

        return response()->json(
            json_decode($response),
            200
        );
    }


    public function createMessage($order)
    {
        return view('messages.index', compact('order'));
    }

    # New Order Api ENDS

    #________________________________________________________________________ changes will be started from here_________________________________________________________________
    public function processHydrogen(Request $request)
    {
        try {
            try {
                Log::channel('dailyInfo')->info('--------------Order Log Begin----------------------');
                $fields = json_decode($request->getContent());

                Log::channel('dailyInfo')->info($request->getContent());
                if (!empty($fields->pay) && $fields->pay == true && empty($fields->razorpay_payment_id)) {
                    return  response()
                        ->json(array(
                            'status' => 'failure',
                            'message' => 'Payment failed please retry'
                        ), 200);
                }

                $outlet = processOutlet($fields->outlet->value->id);
                $order_timing = $fields->outlet->value->order_timing;

                $fields->outlet = $outlet;
                $fields->vendorOrderId = substr(str_shuffle(str_repeat("1234567890", 5)), 0, 10);     //todo
                $pnr = $fields->pnr->value;
                $fields->pnr = $pnr;
                $fields->berth = $fields->berth->value;
                $fields->coach = $fields->coach->value;

                $fields->trainNo = $fields->trainNo->value;
                $fields->trainName = $fields->trainName->value;
                $fields->stationName = $fields->stationName->value;
                $fields->stationCode = $fields->stationCode->value;
                $fields->discountAmount = 0;
                $fields->foodType = $fields->foodType->value;
            } catch (\Exception $e) {
                throw new Exception("Could not read fields.", 0, $e);
            }
            Log::channel('dailyInfo')->info('Read the fields from request!');
            try {
                if ((isset($fields->query->value) && $fields->query->value == 'pnr')
                    || isset($fields->pnr)
                ) {
                    $deliveryDate = app(OrderCreationController::class)->getSchArrivalPnr(
                        $fields->pnr,
                        $fields->deliveryDate->value,
                        $fields->stationCode
                    );

                    Log::channel('dailyInfo')->info('deliveryDate:' . "##" . $deliveryDate);
                }
                if (!isset($deliveryDate)) {
                    $deliveryDate = app(OrderCreationController::class)->getSchArrival(
                        $fields->trainNo,
                        $fields->deliveryDate->value,
                        $fields->stationCode
                    );
                    if (!isset($deliveryDate)) {
                        return response()->json([
                            'status' => 'failure',
                            'message' => 'Train doesn\'t run on Delivery date: ' . $fields->deliveryDate->value
                        ]);
                    }
                    //}
                }
                Log::channel('dailyInfo')->info('deliveryDate:' . '**' . $deliveryDate);
                $fields->deliveryDate = $deliveryDate . ' IST';
                $now = \Carbon\Carbon::now()->setTimezone('Asia/Kolkata');
                $bookDate = $now->copy();
                if (\Carbon\Carbon::createFromFormat('Y-m-d H:i', $deliveryDate)->lessThan(($bookDate->addMinutes((int)$order_timing)))) {
                    return response()->json([
                        'status' => 'failure',
                        'message' => 'Train scheduled departure: ' . $deliveryDate
                    ]);
                }
            } catch (\Exception $e) {
                throw new Exception("Could not fetch delivery date.", 0, $e);
            }

            /*
            $checkTime = app(OrderCreationController::class)->checkTrain($fields->trainNo,
                    $fields->stationCode,$now,\Carbon\Carbon::createFromFormat('Y-m-d H:i', $deliveryDate),$order_timing);
             *
             */
            $fields->customer = (object)[
                'fullName' => $fields->fullName->value,
                'email' => $fields->email->value,
                'mobile' => $fields->mobile->value
            ];
            $date = $now->format('Y-m-d h:i');
            $fields->bookingDate = "$date IST";

            try {
                $orderItems = array_map(function ($item) {
                    $basePrice = $item->price / 1.05;
                    return [
                        "itemId" => $item->id,
                        "itemName" => $item->itemName,
                        "description" => $item->description,
                        "basePrice" => $basePrice,
                        "sellingPrice" => $item->price,
                        "taxRate" => $item->price - $basePrice,
                        "isVegetarian" => $item->isVegeterian == 1 ? true : false,
                        "quantity" => $item->quantity,
                        "option" => null,
                        "foodType" => $item->foodType,
                    ];
                }, $fields->orderItems->value);

                unset($orderItems['st']);

                $subtotal = collect($orderItems)->sum(function ($item) {
                    return $item['sellingPrice'] * $item['quantity'];
                });

                $txn = collect($orderItems)->sum(function ($item) {
                    return ($item['taxRate'] * $item['quantity']);
                });
                $fields->orderItems = $orderItems;
                $deliveryCost = (int)$outlet->deliveryCost;
                //$fields->amountPayable = round($subtotal+$subtotal*0.05);
                $fields->amountPayable = round($subtotal);
                $fields->deliveryCharge = $deliveryCost;
            } catch (\Exception $e) {
                throw new Exception("Could not calculate amount payable.", 0, $e);
            }
            Log::channel('dailyInfo')->info('amountPayable:' . $fields->amountPayable);

            try {
                $o = null;
                $vo = null;
                if ($fields->coupon->value != null && $fields->coupon->value != '') {
                    $o = \App\Models\Order::where('customerMobile', $fields->mobile->value)
                        ->where('discount_code', $fields->coupon->value)
                        ->first();
                    $vo = \App\Models\VendorOrder::where('customerMobile', $fields->mobile->value)
                        ->where('discount_code', $fields->coupon->value)
                        ->first();
                }
                if ($o != null) {
                    $vo = $o;
                    Config::set('vouchers.relation_table', 'order_voucher');
                    Config::set('vouchers.user_model', \App\Models\Order::class);
                }
                if ($vo != null) {
                    if ($vo->canRedeemCode($fields->coupon->value)) {
                        try {
                            $v2 = $vo->check($fields->coupon->value);
                            $amountP = $fields->amountPayable;
                            $discount = \App\Models\Discount::where(function ($q) use ($amountP) {
                                $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                                    ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString())
                                    ->where('amountFrom', '<=', $amountP)
                                    ->where('amountTo', '>=', $amountP);
                            })->first();
                            if ($v2->data->get('amountFrom') <= $fields->amountPayable && $v2->data->get('amountTo') >= $fields->amountPayable) {
                                $v1 = $vo->redeemCode($fields->coupon->value);
                                $d = $v1->data->get('discount');
                                $fields->couponCode = $v1->code;
                                $fields->discountAmount = round($fields->amountPayable * $d / 100);
                                if ($discount != null) {
                                    if ($fields->discountAmount > $discount->upto)
                                        $fields->discountAmount = (int)$discount->upto;
                                }
                                $fields->amountPayable = $fields->amountPayable - $fields->discountAmount;

                                Log::channel('dailyInfo')->info('Applied coupon discount:' . $d . ', discountAmount:' . $fields->discountAmount);
                                Log::channel('dailyInfo')->info('After discount amountPayable:' . $fields->amountPayable);
                            } else if ($discount != null) {
                                $v1 = $vo->redeemCode($fields->coupon->value);
                                $d = $v1->data->get('discount');
                                $fields->couponCode = $v1->code;
                                $fields->discountAmount = round($fields->amountPayable * $d / 100);
                                if ($fields->discountAmount > $discount->upto)
                                    $fields->discountAmount = (int)$discount->upto;
                                $fields->amountPayable = $fields->amountPayable - $fields->discountAmount;

                                Log::channel('dailyInfo')->info('Applied coupon discountAmount:' . $fields->discountAmount);
                                Log::channel('dailyInfo')->info('After discount amountPayable:' . $fields->amountPayable);
                            } else {
                            }
                        } catch (\Exception $e) {
                            //voucher doesn't exists or couldn't be redeemed
                        }
                    }
                } else {
                    try {
                        Log::channel('dailyInfo')->info('coupon:' . $fields->coupon->value);

                        if (!empty($fields->coupon->value)) {
                            $voucher1 = \BeyondCode\Vouchers\Models\Voucher::where('code', $fields->coupon->value)->where('type', 1)->first();
                            if ($voucher1 != null) {
                                /* CONCERNED LOC */
                                if ($this->canRedeemGeneralCode($voucher1)) {
                                    $fields->couponCode = $voucher1->code;
                                    $discnt1 = (int)$voucher1->data[0];
                                    $fields->discountAmount = (int)$discnt1 * $fields->amountPayable / 100;
                                    $fields->amountPayable = $fields->amountPayable - $fields->discountAmount;
                                    $voucher1->total_discount_amount += $fields->discountAmount;
                                    $voucher1->save();
                                } else
                                    $couponInvalid = true;
                            } else {
                                $couponInvalid = true;
                            }
                        }

                        //added code for vouchers

                    } catch (\Exception $e) {
                        report($e);
                        //Log::channel('dailyInfo')->info('coupon:'.print_r($e, true));
                        //voucher doesn't exists or couldn't be redeemed
                    }
                    /*
                     return  response()
                                 ->json(array('status'=>'failure', 'balance'=>$balance, 'wDiscount'=>$wDiscount,'discount'=>$discountAmount,'amountPayable'=>$fields->amountPayable,'message'=>'coupon code is invalid'),200);
                      *
                      */
                }
            } catch (\Exception $e) {
                Log::channel('dailyInfo')->error('Order sent to IRCTC' . $e);
                throw new Exception("Could not apply coupon.", 0, $e);
            }
            $user = Auth::guard('order')->user();
            $user1 = Auth::guard('vendororder')->user();
            $user2 = Auth::guard('apiorder')->user();

            if ($user == null) {
                if ($user1 != null) {
                    $user = $user1;
                } else {
                    if ($user2 != null)
                        $user = $user2;
                    else
                        $user = Auth::guard('memoryuser')->user();
                }
            }
            if ($user != null && get_class($user->last()) != 'App\Models\MemoryUser') {
                $wDiscount = $user->last()->getAmountProduct1($fields->amountPayable);
                $user->last()->withdraw($wDiscount);
                $fields->amountPayable -= $wDiscount;
                $fields->discountAmount += $wDiscount;
            }
            $fields->amountPayable += $deliveryCost;
            try {
                $fields->totalAmount = $subtotal;
                $fields->vendorPaymentType = "CASH_ON_DELIVERY";
                $fields->paymentType = $fields->paymentType->value;
                if ($fields->paymentType == 'PAY_ONLINE')
                    $fields->paymentType = 'PRE_PAID';
                if (property_exists($fields, 'razorpay_payment_id')) {
                    $razorpayId = $fields->razorpay_payment_id;
                    Log::channel('dailyInfo')->info('Payment Type is PRE_PAID');
                    unset($fields->razorpay_payment_id);
                } else if ($fields->outlet->payment_mode == 1)
                    return  response()->json(array(
                        'status' => 'failure',
                        'message' => 'Only prepaid is allowed, cash on delivery is not available on this order'
                    ), 200);
                $fields->gst = $txn;
                unset($fields->outlet->payment_mode);
                unset($fields->email);
                unset($fields->fullName);
                unset($fields->mobile);
                unset($fields->cart);
                unset($fields->modal);
                unset($fields->amountPayable1);
                unset($fields->discount);
                unset($fields->csrf);
                unset($fields->csrf_token);
                unset($fields->coupon);
                unset($fields->query);
                unset($fields->otp);
                unset($fields->foodType);

                Log::channel('dailyInfo')->info('Order sent to IRCTC');
                ob_start();
                /*SHOULD BE CORRECTED*/
                // $this->sendOrder('https://stage-ecatering.ipsator.com/api/v1/order/vendor', $fields);//dev
                $this->sendOrder(config('constkeys.irctc_base_url').'/api/v1/order/vendor', $fields); //prod
                $res = ob_get_clean();
                //echo $res;
                $resObj = json_decode($res);
                // dd($resObj);
                Log::channel('dailyInfo')->info('IRCTC response');
                Log::channel('dailyInfo')->info('IRCTC response' . print_r($resObj, true));

                //Log::channel('dailyInfo')->info(print_r($fields, true));
                //Log::channel('dailyInfo')->info($res);

                if (isset($razorpayId)) {
                    $fields->razorpay_payment_id = $razorpayId;
                }
                //echo $res;
                //echo $resObj->status;
                if ($resObj->status == 'success') {
                    $fields->orderId = $resObj->result->id;
                    Log::channel('dailyInfo')->info('order id:' . $resObj->result->id);
                    $result = $this->persist_order($fields);
                    // user event
                    $outlet1 = \App\Models\Outlet::find($outlet->id);
                    $outlet1->user->token->map(function ($token) use ($resObj) {
                        //Log::channel('dailyInfo')->info('token:'.$token->firebase_token);
                        if (isset($token->firebase_token))
                            event(new \App\Events\OrderCreatedFirebaseEvent([
                                'type' => "USER",
                                'token' => $token->firebase_token,
                                'message' => [
                                    'title' => 'Order Recieved',
                                    'content' => "Order placed with #{$resObj->result->id}"
                                ]
                            ]));
                    });
                    /*
                    // agent event
            // to be removed hared coded
                    event(new \App\Events\OrderCreatedFirebaseEvent([
                        'type'=>'AGENT',
                        'token'=>\App\User::find($r->id)->agentToken(),
                        'message'=>[
                            'title'=>"You've Recieved an Order",
                            'content'=>parseOrder((object)$data)
                        ]
                    ]));
                             *
                             */
                    if ($result->status) {
                        Log::channel('dailyInfo')->info('order placement successful');
                        Log::channel('dailyInfo')->info('--------------Order Log End----------------------');
                        return  response()->json(array(
                            'status' => 'failure',
                            'message' => $result->message
                        ), 200);
                    } else {
                        Log::channel('dailyInfo')->info('order placement failed:');
                        Log::channel('dailyInfo')->info('--------------Order Log End----------------------');
                    }
                }
            } catch (\Exception $e) {
                throw new Exception("exception sending and persisting.", 0, $e);
            }

            return response($res);
            //return response()->json($fields,200);
        } catch (\Exception $e) {
            Mail::raw($e->getMessage() . $e->getTraceAsString(), function ($message) {
                $message->from('no-reply@olfstores.com', 'olfstores.com');
                $message->to(['sanjayjdn@gmail.com', 'dharamjdn@gmail.com']);
                $message->cc('ar31213@gmail.com');
            });
            Log::channel('dailyInfo')->error($e);
            throw $e;
            /*
                    return  response()->json(array('status'=>'failure',
                                                    'message'=>$e->getMessage()),200);
                 *
                 */
        } finally {
            Config::set('vouchers.relation_table', 'vendor_order_voucher');
            Config::set('vouchers.user_model', \App\Models\VendorOrder::class);
        }
    }

    #________________________________________________________________________ changes will be End here_________________________________________________________________

    public function persist_order($fields, $dbg = false)
    {
        try {
            $order['id'] = $fields->vendorOrderId; //todo
            $order['orderId'] = $fields->orderId;
            $order['customer'] = json_encode($fields->customer);
            $order['customerEmail'] = $fields->customer->email;
            $order['customerMobile'] = $fields->customer->mobile;
            $order['outlet'] = $fields->outlet->id; //get the local outlet id
            $order['bookingDate'] = $fields->bookingDate;
            $order['deliveryDate'] = $fields->deliveryDate;
            $deliveryDate1 = str_replace(' IST', '', $order['deliveryDate']);
            $order['deliveryDateTime'] = \Carbon\Carbon::createFromFormat('Y-m-d H:i', $deliveryDate1);
            $order['pnr'] = $fields->pnr;
            $order['trainNo'] = $fields->trainNo;
            $order['trainName'] = $fields->trainName;
            $order['stationCode'] = $fields->stationCode;
            $order['stationName'] = $fields->stationName;
            $order['coach'] = $fields->coach;
            $order['berth'] = $fields->berth;
            $order['totalAmount'] = $fields->amountPayable; //todo
            $order['deliveryCharge'] = $fields->deliveryCharge;
            $order['gst'] = $fields->gst;
            //$order['deliveryCharge'] = $fields->amountPayable; //todo be found out
            //$order['discountAmount'] = $fields->discountAmount; //todo be found out
            $order['orderItems'] = json_encode($fields->orderItems);
            $order['discountAmount'] = $fields->discountAmount;
            $order['amountPayable'] = $fields->amountPayable; //todo
            $order['paymentType'] = $fields->paymentType;
            $order['status'] = "ORDER_CONFIRMED";
            //$order['createdAt'] = $fields->createdAt; //todo
            //$order['updatedAt'] = $fields->updatedAt; //todo


            $outlet = \App\Models\Outlet::find($fields->outlet->id);
            $items = array_column($outlet->menu->items->toArray(), 'vendorPrice', 'id');
            $orderItems = collect($fields->orderItems)->map(function ($oi) use ($items) {
                if (isset($items[$oi['itemId']]))
                    $oi['costPrice'] = $items[$oi['itemId']];
                return $oi;
            });
            $unavailable = false;
            $subtotal1 = collect($orderItems)->sum(function ($item) {
                if (isset($item['costPrice']) && $item['costPrice'] != 'N.A.' && $item['costPrice'] != '') {
                    return $item['costPrice'] * $item['quantity'];
                }
                $unavailable = true;
            });
            if ($unavailable)
                $order['vendorPayment'] = 0;
            else
                $order['vendorPayment'] = $order['amountPayable'] - $subtotal1;
            $orderCreate = $outlet->vendor_orders()->firstOrCreate($order);
            $order1 = \App\Models\VendorOrder::where('orderId', $orderCreate->orderId);
            $order2 = \App\Models\VendorOrder::where('orderId', $orderCreate->orderId)->first();
            //event(new \App\Events\VendorOrderCreatedWallet($order2));
            /*
            $walletDatabase = \Bavix\Wallet\Models\Wallet::where('holder_id', $orderCreate->id)->first();
            if($walletDatabase){
                $walletDatabase->transactions->map(function($transaction) {
                    $transaction->delete();
                });
                $walletDatabase->delete();
                $order2 = \App\VendorOrder::where('id',$order2->id)->first();
                event(new \App\Events\VendorOrderCreatedWallet($order2));        
            }
            */
            $wallet = $order2->wallet;
            $oWhere = \App\Models\VendorOrder::where('customerMobile', $order['customerMobile']);
            $count = $order2->count($order['customerMobile']);
            if ($count == 1)
                $totalAmt = ($orderCreate->amountPayable - $orderCreate->deliveryCharge) * (0.8);
            else
                $totalAmt = ($orderCreate->amountPayable - $orderCreate->deliveryCharge) * (0.5);

            Log::channel('dailyInfo')->info('--except:' . $orderCreate->amountPayable . '-' . $orderCreate->deliveryCharge . '-' . $totalAmt);
            $wallet->deposit($totalAmt);
            $walletCash = $totalAmt;
            //$order1->update(['wallet_id'=>$wallet->id]);
        } catch (\Exception $e) {
            Log::channel('dailyInfo')->info('exception:' . $e->getTraceAsString());
            throw new Exception("Could not create wallet.", 0, $e);
        }
        Log::channel('dailyInfo')->info('order request filled');

        try {
            $first = $order1->first();
            $amountP = $orderCreate->amountPayable;
            $discount = \App\Models\Discount::where(function ($q) use ($amountP) {
                $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                    ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString())
                    ->where('amountFrom', '<=', $amountP)
                    ->where('amountTo', '>=', $amountP);
            })->first();
            if ($discount != null) {
                $voucher = $first->createVouchers(1, [
                    'discount' => $discount->discount,
                    'amountFrom' => $discount->amountFrom,
                    'amountTo' => $discount->amountTo
                ], \Carbon\Carbon::now()->addDays($discount->expires))[0];
                $order1->update(['discount_code' => $voucher->code]);
            }
        } catch (\Exception $e) {
            throw new Exception("Could not create voucher.", 0, $e);
        }
        try {
            if (!empty($fields->pay) && $fields->pay == true) {
                $api = new Api(config('razorpay.razor_key'), config('razorpay.razor_secret'));
                $payment = $api->payment->fetch($fields->razorpay_payment_id);

                try {
                    $response = $api->payment->fetch($fields->razorpay_payment_id)->capture(array('amount' => $payment['amount']));
                    $order1->update(['razorpay_payment_id' => $fields->razorpay_payment_id]);
                    Log::channel('dailyInfo')->info('payment done');
                } catch (\Exception $e) {
                    $order1->update(['status' => 'PAYMENT_FAILED']);
                    throw new Exception("Could not update payment.", 0, $e);
                    /*
                        $orderCreate->status = 'PAYMENT_FAILED';
                        $orderCreate->save();
                        return (object)[
                            'orderId'=>$orderCreate->orderId,
                            'status'=>1, //todo
                            'message'=>$e->getMessage()
                        ];
                         *
                         */
                }
            }
        } catch (\Exception $e) {
            throw new Exception("Could not fetch payment.", 0, $e);
        }

        # for sending user message
        try {
            $message =  $this->formatMessage($orderCreate);
            $res1 = '';
            ob_start();
            $this->sendMessage($message['message'], $message['outlet']->mobile, env('VENDOR_MSG_TEMLPLATE_ID', NULL));
            $res1 = ob_get_clean();
            Log::channel('dailyInfo')->info('--sms sent to outlet:' . $outlet->outletName . ', message response:' . $res1);
            Log::channel('dailyInfo')->info('customer mobiles:' . $fields->customer->mobile . ', message response:' . $res1);
            Log::channel('dailyInfo')->info('sms response:' . $res1 . '--');
            $linkMsg = ", Download the app https://play.google.com/store/apps/details?id=com.olfstores.olf_user or visit https://www.olfstores.com. Terms and conditions apply";
            if ($discount != null) {
                $message =  $this->formatMessageCustomer($orderCreate, $discount, $voucher, $walletCash, $linkMsg);
            } else {
                $message =  $this->formatMessage($orderCreate, $linkMsg);
            }
            Log::channel('dailyInfo')->info('--sms sent to customer mobiles:' . $fields->customer->mobile);
            $res2 = '';
            ob_start();
            $this->sendMessage($message['message'], $fields->customer->mobile, env('CUSTOMER_MSG_TEMPLATE_ID', NULL));
            $res2 = ob_get_clean();
            $res1 .= $res2;
            Log::channel('dailyInfo')->info('message response:' . $res2 . '--');



            try {
                foreach (preg_split("/\,/", $message['outlet']->mobile) as  $value) {
                    $this->service->sendMessageVendorOrders($value, $orderCreate, false);
                }

                $this->service->sendMessageVendorOrders($fields->customer->mobile, $orderCreate, true);
            } catch (\Exception $th) {
                Log::info($th);
            }
        } catch (\Exception $e) {
            throw new Exception("Could not send text message." . (isset($res1) ? $res1 : ''), 0, $e);
        }



        try {
            # sending email to customer
            $this->passEmail($order, $fields);
            Log::channel('dailyInfo')->info('email sent to customer::' . $order['customerEmail'] . ', outlet email:' . \App\Models\User::find($outlet->user_id)->email);
        } catch (\Exception $e) {
            throw new Exception("Could not send email.", 0, $e);
        }
        try {
            # sending voice message to vendor
            $message1 = 'Hi ! You received a new order from o el f store for train no. ' . implode(' ', str_split($fields->trainNo)) . ' customer name ' . $fields->customer->fullName . ' Thank You';
            $mobiles = explode(",", $outlet->user->mobile);
            foreach ($mobiles as $value) {
                $res2 = '';
                ob_start();
                $this->sendVoiceMessage($message1, $value);
                $res2 = ob_get_clean();

                Log::channel('dailyInfo')->info('--voice message sent to outlet mobile:' . $value);
                Log::channel('dailyInfo')->info('voice message response:' . $res2 . '--');
            }
        } catch (\Exception $e) {
            throw new Exception("Could not send voice message." . (isset($res1) && isset($res2) ? $res1 . '--end' . $res2 : ''), 0, $e);
        }

        $result = (object)[
            'orderId' => $orderCreate->id,
            'status' => 0, //todo
            'message' => $message
        ];

        return $result;
    }
    public function sendWhatsAppMsgToMobile($mobile, $message)
    {
        /*                
        $optInRes1 = $this->sendWhatsAppOptIn($mobile);
        $optInRes = json_decode($optInRes1);
        if(isset($optInRes) && isset($optInRes->user) && $optInRes->user == $mobile){
            $sendMessageRes = $this->sendWhatsAppMessage($mobile, $message);
            $resJson = json_decode($sendMessageRes);
            if($resJson->status == 'submitted'){
                Log::channel('dailyInfo')->info('whats app message response:'.$sendMessageRes.'--');
                
            }else{
                Log::channel('dailyInfo')->info('whats app message could not be submitted successfully');
                Log::channel('dailyInfo')->info('whats app message response:'.$sendMessageRes.'--');
                
            }
        }else{
                Log::channel('dailyInfo')->info('whats app Optin is unsuccessful');
                Log::channel('dailyInfo')->info('whats app optin response:'.$optInRes1.'--');
            }
        */
    }
    public function saveDeliveryBoy($orderId, $deliveryBoyId, $vendorOrder)
    {
        if ($vendorOrder == 'true') {
            $orderWhere = \App\Models\VendorOrder::where('orderId', $orderId);
            $update = $orderWhere->update(['deliveryBoy' => $deliveryBoyId]);
            $order = \App\Models\VendorOrder::where('orderId', $orderId)->first();

            $deliveryDetails = new \stdClass;
            $deliveryDetails->pnr = $order->pnr;
            $deliveryDetails->trainNo = $order->trainNo;
            $deliveryDetails->trainName = $order->trainName;
            $deliveryDetails->station = $order->stationName;
            $deliveryDetails->stationCode = $order->stationCode;
            $deliveryDetails->berth = $order->berth;
            $deliveryDetails->coach = $order->coach;
            $deliveryDetails->ETA = $order->deliveryDate;
            $order->deliveryDetails = json_encode($deliveryDetails);

            $order->customerDetails = json_decode($order->customer);
            $order->customerDetails->customerName = $order->customerDetails->fullName;
            $order->customerDetails = json_encode($order->customerDetails);

            $priceDetails = new \stdClass;
            $priceDetails->totalAmount = $order->totalAmount;
            $order->priceDetails = json_encode($priceDetails);

            $order->outletDetails = \App\Models\Outlet::find($order->outlet);
            $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
            if ($order->deliveryBoy != null) {
                $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                $order->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
            } else
                $order->deliveryBoyName = '';

            $order->vendorOrder = true;
        } else if ($vendorOrder == 'false') {
            $orderWhere = \App\Models\Order::where('orderId', $orderId);
            $update = $orderWhere->update(['deliveryBoy' => $deliveryBoyId]);
            $order = \App\Models\Order::where('orderId', $orderId)->first();
            $deliveryDetails = json_decode($order->deliveryDetails);
            $order->outletDetails = \App\Models\Outlet::find($order->outlet_id);
            $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
            if ($order->deliveryBoy != null) {
                $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                $order->deliveryBoyName = $deliveryBoy->first_name . ' ' . $deliveryBoy->last_name;
            } else
                $order->deliveryBoyName = '';
            $order->vendorOrder = false;
        }
        $order->aggregatorDetails = json_decode($order->aggregatorDetails);
        $order->customerDetails = json_decode($order->customerDetails);
        $order->priceDetails = json_decode($order->priceDetails);
        $order->orderItems = json_decode($order->orderItems);
        $this->sendMessage('Hi Delivery person ' . $order->deliveryBoyName . ' is on the way to deliver your order. Kindly be on your seat, in case of any query please feel free to contact him on ' . $deliveryBoy->mobile . ' Regards https://www.olfstores.com', $order->customerDetails->mobile);
        if ($update) {
            return response()->json($order, 200);
        }
        return response()->json(['status' => 'failure', 'message' => 'Updation failed'], 200);
    }
    public function getDeliveryBoys($outletId)
    {
        $deliveryBoys = \App\Models\Outlet::find($outletId)->deliveryBoys;
        return response()->json($deliveryBoys, 200);
    }
    private function formatMessageCustomer($data, $discount, $voucher, $walletCash, $linkMsg = null)
    {
        $aggrigator = 'Olf Store'; //todo
        //$delivery = $data->deliveryDate; //todo
        $p = $data->totalAmount;
        $customer = json_decode($data->customer);
        $orderId =  $data->orderId;
        $items = json_decode($data->orderItems);
        $booking = $data->bookingDate;
        //$deliveryDate = substr($data->deliveryDate,0,10);
        $deliveryDate = $data->deliveryDate;

        $userId = \App\Models\Outlet::where('id', $data->outlet)->first()->user_id;
        //dd($userId);
        $outlet = \App\Models\User::find($userId);
        $customerName = $customer->fullName;

        $message = "Hi $customerName, Thanks for order from OLFSTORE, IRCTC Order id: $orderId";
        $message .= " will be delivered at $data->stationName, by $outlet->outletName. Congratulations! You have received discount coupon from OLF, use coupon code $voucher->code to get $discount->discount% discount on your next order. Additionally an amount of Rs." . $walletCash . " has been deposited to your wallet. To avail (coupon wallet) discount " . $linkMsg;
        $message .= " ,OLFSTORES";

        /*
		$message = "Order id:{$orderId} For: {$data->trainNo}/{$data->trainName} coach {$data->coach}-{$data->berth} at {$data->stationName} to {$customer->fullName} {$customer->mobile} on {$deliveryDate} \n";
		$oitems = array_map(function($item){
                                return "{$item->quantity} -{$item->itemName}";
			},$items);

		$message.= "ITEMS:".join($oitems);

		$message .= " ,Amount: {$data['amountPayable']}-{$data['paymentType']}";
		$message .= " ,Use coupon code $voucher->code on your next order and avail discount of $discount->discount% . Additionally an amount of Rs.".$walletCash." has been deposited to your wallet. Which you can use by ".$linkMsg;
		$message .= ",OLFSTORES";
		*/
        return ['message' => $message, 'outlet' => $outlet];
    }
    //todo
    private function formatMessage($data, $linkMsg = null)
    {
        $aggrigator = 'Olf Store'; //todo
        //$delivery = $data->deliveryDate; //todo
        $p = $data->totalAmount;
        $customer = json_decode($data->customer);
        $orderId =  $data->orderId;
        $items = json_decode($data->orderItems);
        $booking = $data->bookingDate;
        $deliveryDate = substr($data->deliveryDate, 0, 10);
        $eta = substr($data->deliveryDateTime, 0, 16);
        $userId = \App\Models\Outlet::where('id', $data->outlet)->first()->user_id;
        //dd($userId);
        $outlet = \App\Models\User::find($userId);
        $message = "IRCTC Order id:{$orderId} For: {$data->trainNo}/{$data->trainName} ETA {$eta} coach {$data->coach}-{$data->berth} at {$data->stationName} to {$customer->fullName} {$customer->mobile} on {$deliveryDate} ";
        $oitems = array_map(function ($item) {
            return "{$item->quantity} -{$item->itemName}";
        }, $items);

        $message .= "ITEMS:" . join($oitems);

        $message .= " Amount: {$data['amountPayable']}-{$data['paymentType']}";
        if (isset($linkMsg)) {
            $message .= $linkMsg;
        }
        $message .= ",olfstores";
        return ['message' => $message, 'outlet' => $outlet];
    }

    public function testMessage(Request $request)
    {
        $order = \App\Models\Order::where('orderId', '45134629')->first();

        $deliveryData = (object)json_decode($order->deliveryDetails);
        $deliveryData->deliveryDate = preg_split("/\s/", $order->deliveryDate)[0];

        $orderId = $order->orderId;
        $customerDetails = (object)json_decode($order->customerDetails);
        $orderItems = json_decode($order->orderItems);
        $priceDetails = json_decode($order->priceDetails);
        $paymentType = $order->paymentType;
        $eta = substr($deliveryData->ETA, 0, -4);
        $data = (object)compact('deliveryData', 'orderId', 'customerDetails', 'orderItems', 'priceDetails', 'paymentType', 'eta');

        $messageContent = $this->createMessage($data);
        //return ($messageContent);
        return $this->sendMessage($messageContent->render(), "9336166567", "1307164294783823783");
    }

    public function testMessage1(Request $request)
    {
        $this->sendMessage("test", "9945040843");
    }

    private function sendMessage($string, $numbers, $DLT_TE_ID = null)
    {
        $key = "191164Aup6j2p05a4cb419";
        $url = sprintf('https://api.msg91.com/api/sendhttp.php?authkey=%s&mobiles=%s&route=%s&sender=%s&message=%s&country=%s', $key, $numbers, '4', urlencode('OLFSTR'), urlencode(urlencode($string)), '91');

        if (isset($DLT_TE_ID)) {
            $url = $url . sprintf('&DLT_TE_ID=%s', $DLT_TE_ID);
        }
        /*
        	    $nums = explode(",",$numbers);
        		$data = (object)[
        			'sender'=>'OLFSTR',
        			'route'=>4,
        			'country'=>91,
        			'sms'=>[
        			   (object)['message'=>$string,'to'=>$nums]
        			]
        
        		];
            */

        /* dd(json_encode($data));*/
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => "http://api.msg91.com/api/v2/sendsms",
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,

            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,

            //CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_CUSTOMREQUEST => "GET",
            //CURLOPT_POSTFIELDS => json_encode($data),

            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0,

            /*
					   
					   CURLOPT_HTTPHEADER => array(
						"content-type: application/json"
						),
						
					*/
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        /*if ($err) {
			echo "cURL Error #:" . $err;
		} else {
			echo $response;
		}*/
        print_r($response);
    }


    private function sendStatusMessage($data, $message)
    {
        $orderId =  $data->orderId;
        if ($data->vendorOrder)
            $outletId = $data->outlet;
        else
            $outletId = $data->outlet_id;
        $userId = \App\Models\Outlet::where('id', $outletId)->first()->user_id;
        //dd($userId);
        $user = \App\Models\User::find($userId);
        $message = sprintf($message, $orderId);
        $message .= ",OLFSTORES";
        //echo $message;
        $this->sendMessage($message, $user->mobile);
    }
    private function sendVoiceMessage($string, $number)
    {

        $data = http_build_query(
            array(
                'from' => '919522996999',
                'message' => $string,
                'authkey' => '191164Aup6j2p05a4cb419',
                'to' => '91' . $number,
            ),
            '&'
        );

        //dd($data);
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => "http://control.msg91.com/api/sendVoiceCall.php",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => $data,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        /*if ($err) {
			echo "cURL Error #:" . $err;
		} else {
			echo $response;
		}*/

        print_r($response);
    }
    public function sendWhatsAppMessage($mobile, $message)
    {
        $message = 'Hi Testing from OLFSTORES';
        /*$message = ['isHSM' => 'false',
                    'type' => 'text',
                    'text' => $message];
        */
        $client = new \App\GupshupAPI\Client();
        $res = $client->handle('sendMessage', ['destination' => $mobile, 'message' => $message]);
        return $res;
    }

    public function sendWhatsAppOptIn($mobile)
    {
        $client = new \App\GupshupAPI\Client();
        $res = $client->handle('optIn', ['user' => $mobile]);
        return $res;
    }

    private function passEmail($data, $fields, $dbg = false)
    {
        $delivery = (object)array(
            'pnr' => $data['pnr'],
            'trainNo' => $data['trainNo'],
            'trainName' => $data['trainName'],
            'station' => $data['stationName'],
            'stationCode' => $data['stationCode'],
            'berth' => $data['berth'],
            'coach' => $data['coach'],
            'ETA' => $data['deliveryDate'], //todo
        );
        $items = json_decode($data['orderItems']);
        $customerIRCTC = json_decode($data['customer']);
        //$customerEmail = $customer['email'];
        //$customerName = $customer['fullName'];
        $discountAmount = 0;
        if (array_key_exists('discountAmount', $data))
            $discountAmount = $data['discountAmount'];
        $pricing = (object)array(
            'totalAmount' => $data['totalAmount'],
            'gst' => $data['gst'],
            'amountPayable' => $data['amountPayable'],
            'discountAmount' => $discountAmount
        ); //todo
        $customerIRCTC->customerName = $customerIRCTC->fullName;
        //$items = json_decode($data['orderItems']);
        $orderId = $data['orderId'];
        $bookingDate = $data['bookingDate'];

        $deliveryDate = $data['deliveryDate'];
        $paymentType = $data['paymentType'];
        $outlet = \App\Models\Outlet::find($data['outlet']);

        Mail::send(
            'mail.emailVendor',
            compact('delivery', 'items', 'customerIRCTC', 'outlet', 'delivery', 'orderId', 'bookingDate', 'deliveryDate', 'paymentType', 'pricing'),
            function ($message) use ($outlet, $customerIRCTC, $orderId) {
                $y = \App\Models\User::find($outlet->user_id);
                $message->from('no-reply@olfstores.com', 'OLF STORE');
                $message->sender('no-reply@olfstores.com', 'OLF STORE');
                $message->to($y->email, $outlet->outletName);
                
                if($customerIRCTC->email != 'xyz@xyz.com'){
                    $message->cc($customerIRCTC->email, $customerIRCTC->customerName);
                }
                
                $message->bcc('sanjayjdn@gmail.com', 'Sanjay Jadon');
                $message->subject('ORDER ID:' . $orderId);
                $message->priority(3);
            }
        );
    }
    public function test12()
    {
        Log::channel('dailyInfo')->info('--------------Testing----------------------');
        /*
            $order1 = \App\Order::where('orderId', 9175150);

            $o = $order1->first();

            Config::set('vouchers.relation_table','order_voucher');
            Config::set('vouchers.user_model',\App\Order::class);

            $discount = \App\Discount::
                        where(function ($q){
                                    $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                                            ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString());
                        })->first();
            if($discount!=null){
                $voucher = $o->createVouchers(1, [
                    'discount_id' => $discount->id,
                ])[0];
                echo $voucher->code;
                $o->update(['discount_code' => $voucher->code]);
            }
             *
             $code = 'EU3L-2FKV';
             $mobile = '9945040843';
             $fields = new \stdClass;
             $fields->amountPayable = 145;
             $fields->coupon = new \stdClass;
             $fields->coupon->value = $code;
             //print_r(Voucher::whereCode($code)->first());
            //$this->sendMessage("Order For: 11123/Barauni coach s4-28 at LUCKNOW to ajay 9945040843 on 2019-06-04 ITEMS:1 -Veg Mini Thali ,Amount: 145-CASH_ON_DELIVERY ,Use coupon code DNNL-T2GY on your next order and avail discount of 25% for order amount (0-1000),OLFSTORES", "9945040843");
            try{
                $o = \App\Order::where('customerMobile', $mobile)
                        ->where('discount_code', $code)
                        ->first();
                $vo = \App\VendorOrder::where('customerMobile', $mobile)
                        ->where('discount_code', $code)
                        ->first();
            if($o!=null){
                $vo = $o;
                Config::set('vouchers.relation_table','order_voucher');
                Config::set('vouchers.user_model',\App\Order::class);
            }
            	echo $vo->canRedeemCode($code);
            	if($vo->canRedeemCode($code)){
            		//try{
						$v2 = $vo->check($code);
						if($v2->data->get('amountFrom')<=$fields->amountPayable && $v2->data->get('amountTo')>=$fields->amountPayable){
							$v1 = $vo->redeemCode($fields->coupon->value);
							echo 'redeemed';
							$d = $v1->data->get('discount');
							$fields->couponCode = $v1->code;
							$fields->discountAmount = round($fields->amountPayable*$d/100);
							$fields->amountPayable = $fields->amountPayable-$fields->discountAmount;
						}
					//}catch (\Exception $e) {
						//voucher doesn't exists or couldn't be redeemed
						//echo $e->getMessage();
					//}

				}
            }finally {
                    Config::set('vouchers.relation_table','vendor_order_voucher');
                    Config::set('vouchers.user_model',\App\VendorOrder::class);
          }
*/
        //print(\App\Outlet::find(3)->deliveryBoys);

    }
    public function generateOTP()
    {
        $otp = mt_rand(100000, 999999);
        return $otp;
    }

    public function sendOtp(Request $request)
    {
        // try{
        if (!$request->has('mobile')) {
            return json_encode(array('status' => 'failure', 'msg' => "input parameters missing"));
        } else {
            //Auth::logout();
            //put in session
            $mobile = $request->input('mobile');
            $otp = $this->generateOTP();
            $message = 'your otp for signing is ' . $otp . "on https://olfstores.com";
            ob_start();
            //$this->sendMessage($message, $mobile, env('LOGIN_OTP_TEMLPLATE_ID', NULL));
            $this->loginOtpService->sendOtp($mobile, $otp);
            $res = ob_get_clean();
            session(['mobile' => $mobile]);
            session(['otp' => $otp]);
            
            PhoneLog::createOTP(session('mobile'), session('otp'));
            
            if ($request->has('mobile')) {
                $vo = \App\Models\VendorOrder::latest($mobile);
            }

            if ($vo != null) {
                $vo->password = \Illuminate\Support\Facades\Hash::make($otp);
                $vo->save();
            } else {
                $memoryUser = \App\Models\MemoryUser::where('mobile', $mobile)->first();
                if ($memoryUser == null)
                    \App\Models\MemoryUser::create(['password' => \Illuminate\Support\Facades\Hash::make($otp), 'mobile' => $mobile, 'memory' => 1]);
                else {
                    $memoryUser->password = \Illuminate\Support\Facades\Hash::make($otp);
                    $memoryUser->save();
                }
            }
            Log::info("[OTP] $mobile  -> $otp");

            return json_encode(array('status' => 'success', 'message' => 'otp sent successfully'));
        }
        // }
        // catch(Exception $e){
        //     dd($e->getMessage());
        // }
    }

    public function sendOtp1(Request $request)
    {
        if (!$request->has('mobile')) {
            return json_encode(array('status' => 'failure', 'msg' => "Mobile number not valid"));
        } else {
            //put in session
            $mobile = $request->input('mobile');
            $otp = $this->generateOTP();
            $message = 'your otp is ' . $otp;
            ob_start();
            //$this->sendMessage($message,$mobile);
            $res = ob_get_clean();
            session(['mobile' => $mobile]);
            session(['otp' => $otp]);
            if ($request->has('mobile')) {
                $controller = app()->make('\App\Http\Controllers\Auth\OrderLoginController');
                $orderUser = $controller->getUser($mobile);
                $vo = $orderUser['user'];
            }
            if ($vo != null) {
                $vo->password = \Illuminate\Support\Facades\Hash::make($otp);
                $vo->save();
            } else {
                $memoryUser = \App\Models\MemoryUser::where('mobile', $mobile)->first();
                if ($memoryUser == null)
                    \App\Models\MemoryUser::create(['password' => \Illuminate\Support\Facades\Hash::make($otp), 'mobile' => $mobile, 'memory' => 1]);
                else {
                    $memoryUser->password = \Illuminate\Support\Facades\Hash::make($otp);
                    $memoryUser->save();
                }
            }
            return json_encode(array('status' => 'success', 'message' => 'otp sent successfully'));
        }
    }
    public function testing(Request $request)
    {

        Log::channel('dailyInfo')->info('here' . print_r($request->user(), true));
    }
    public function wallet(Request $request)
    {
        $user = Auth::guard('vendororder')->user();
        $user1 = Auth::guard('order')->user();
        $user2 = Auth::guard('memoryuser')->user();
        $user3 = Auth::guard('apiorder')->user();

        if ($user != null) {
            if ($user1 != null && $user1->created_at->gt($user->created_at))
                $user = $user1;
        } else {
            $user = $user1;
        }

        if ($user == null) {
            if ($user2 != null) {
                /*
                $order = \App\Order::latest($user2->mobile);
                if($order==null)
                    $balance = 0;
                else
                    $balance = $order->balance;
                 *
                 */
                return json_encode(array('status' => 'error', 'balance' => $user2->balance));
            } else {
                if (!$user3)
                    return json_encode(array('status' => 'error', 'balance' => 'Login to proceed'));
                else
                    $user = $user3;
            }
        }
        if ($request->has("payable"))
            return json_encode(array('status' => 'success', 'orderId' => $user->orderId, 'balance' => $user->last()->balance, 'discount' => $user->last()->getAmountProduct1((int)$request->payable)));

        //Log::channel('dailyInfo')->info('--getting:previous'.print_r($user->first(),true));
        //Log::channel('dailyInfo')->info('--getting:last'.print_r($user->last(),true));
        //$user->setRelation('wallet',$user->wallet);
        return json_encode(array('status' => 'success', 'orderId' => $user->orderId, 'balance' => $user->last()->balance));
    }
    public function mobile()
    {

        $user = Auth::guard('vendororder')->user();
        $user1 = Auth::guard('order')->user();
        $user2 = Auth::guard('memoryuser')->user();
        $user3 = Auth::guard('apiorder')->user();
        if ($user != null) {
            if ($user1 != null && $user1->created_at->gt($user->created_at))
                $user = $user1;
        } else {
            $user = $user1;
        }

        if ($user == null) {
            if ($user2 != null) {
                /*
                $order = \App\Order::latest($user2->mobile);
                if($order==null)
                    $balance = 0;
                else
                    $balance = $order->balance;
                 *
                 */
                $user = $user2;
                $mobile = $user->mobile;
            } else {
                if (!$user3)
                    return json_encode(array('status' => 'error', 'mobile' => 'Login to proceed'));
                else
                    $user = $user3;
            }
        } else
            $mobile = $user->customerMobile;


        return json_encode(array('status' => 'success', 'mobile' => $mobile));
    }
    public function getPreviousOrders(Request $request)
    {
        $user = Auth::guard('vendororder')->user();
        $user1 = Auth::guard('order')->user();
        $user2 = Auth::guard('memoryuser')->user();
        $user3 = Auth::guard('apiorder')->user();

        if ($user != null) {
            if ($user1 != null && $user1->created_at->gt($user->created_at))
                $user = $user1;
        } else {
            $user = $user1;
        }

        if ($user == null) {
            if ($user2 != null) {
                return json_encode(array('status' => 'error', 'orders' => []));
            } else {
                if (!$user3)
                    return json_encode(array('status' => 'error', 'balance' => 'Login to proceed'));
                else
                    $user = $user3;
            }
        }
        $orders = [];
        return json_encode(array('status' => 'success', 'orders' => $user->last()->getPreviousOrders($orders)));
    }
    public function createOrderEvent(Request $request)
    {
        $token = \App\Models\Token::where('access_token', hash('sha256', $request->api_token))->first();
        event(new \App\Events\OrderCreatedFirebaseEvent([
            'type' => "USER",
            'token' => $token->firebase_token,
            'message' => [
                'title' => 'Order Recieved',
                'content' => "Order placed with #{123}"
            ]
        ]));
    }

    private function getFormatedDate($str)
    {
        $_parts = preg_split("/\s/", $str);
        $_tz = trim($_parts[2]);
        $_time = join(" ", array_merge([$_parts[0], $_parts[1]]));

        if ($_tz == 'IST')
            return Carbon::createFromFormat('Y-m-d H:i', $_time, 'Asia/Kolkata');
        else
            return Carbon::createFromFormat('Y-m-d H:i', $_time, $_tz)->setTimezone('Asia/Kolkata');
    }


    public function saveOrder($fields, $dbg = false)
    {
        try {
            $order['id'] = $fields->vendorOrderId; //todo
            $order['orderId'] = $fields->orderId;
            $order['customer'] = json_encode($fields->customer);
            $order['customerEmail'] = $fields->customer->email;
            $order['customerMobile'] = $fields->customer->mobile;
            $order['outlet'] = $fields->outlet->id; //get the local outlet id
            $order['bookingDate'] = $fields->bookingDate;
            $order['deliveryDate'] = $fields->deliveryDate;

            $order['deliveryDateTime'] = $this->getFormatedDate($fields->deliveryDate);

            $order['pnr'] = $fields->pnr;
            $order['trainNo'] = $fields->trainNo;
            $order['trainName'] = $fields->trainName;
            $order['stationCode'] = $fields->stationCode;
            $order['stationName'] = $fields->stationName;
            $order['coach'] = $fields->coach;
            $order['berth'] = $fields->berth;
            $order['totalAmount'] = $fields->amountPayable; //todo
            $order['deliveryCharge'] = $fields->deliveryCharge;
            $order['gst'] = $fields->gst;
            //$order['deliveryCharge'] = $fields->amountPayable; //todo be found out
            //$order['discountAmount'] = $fields->discountAmount; //todo be found out
            $order['orderItems'] = json_encode($fields->orderItems);
            $order['discountAmount'] = $fields->discountAmount ?? $fields->discount ?? 0;
            $order['amountPayable'] = $fields->amountPayable; //todo
            $order['paymentType'] = $fields->paymentType;
            $order['status'] = "ORDER_CONFIRMED";
            //$order['createdAt'] = $fields->createdAt; //todo
            //$order['updatedAt'] = $fields->updatedAt; //todo


            $outlet = \App\Models\Outlet::find($fields->outlet->id);
            $items = array_column($outlet->menu->items->toArray(), 'vendorPrice', 'id');
            $orderItems = collect($fields->orderItems)->map(function ($o) {
                return (array)$o;
            })->map(function ($oi) use ($items) {
                if (isset($items[$oi['itemId']]))
                    $oi['costPrice'] = $items[$oi['itemId']];
                return $oi;
            });
            $unavailable = false;
            $subtotal1 = collect($orderItems)->sum(function ($item) {
                if (isset($item['costPrice']) && $item['costPrice'] != 'N.A.' && $item['costPrice'] != '') {
                    return $item['costPrice'] * $item['quantity'];
                }
                $unavailable = true;
            });
            if ($unavailable)
                $order['vendorPayment'] = 0;
            else
                $order['vendorPayment'] = $order['amountPayable'] - $subtotal1;
            $orderCreate = $outlet->vendor_orders()->firstOrCreate($order);
            $order1 = \App\Models\VendorOrder::where('orderId', $orderCreate->orderId);
            $order2 = \App\Models\VendorOrder::where('orderId', $orderCreate->orderId)->first();
            //event(new \App\Events\VendorOrderCreatedWallet($order2));
            /*
            $walletDatabase = \Bavix\Wallet\Models\Wallet::where('holder_id', $orderCreate->id)->first();
            if($walletDatabase){
                $walletDatabase->transactions->map(function($transaction) {
                    $transaction->delete();
                });
                $walletDatabase->delete();
                $order2 = \App\VendorOrder::where('id',$order2->id)->first();
                event(new \App\Events\VendorOrderCreatedWallet($order2));        
            }
            */
            $wallet = $order2->wallet;
            $oWhere = \App\Models\VendorOrder::where('customerMobile', $order['customerMobile']);
            $count = $order2->count($order['customerMobile']);
            if ($count == 1)
                $totalAmt = ($orderCreate->amountPayable - $orderCreate->deliveryCharge) * (0.8);
            else
                $totalAmt = ($orderCreate->amountPayable - $orderCreate->deliveryCharge) * (0.5);

            Log::channel('dailyInfo')->info('--except:' . $orderCreate->amountPayable . '-' . $orderCreate->deliveryCharge . '-' . $totalAmt);
            $wallet->deposit($totalAmt);
            $walletCash = $totalAmt;
            //$order1->update(['wallet_id'=>$wallet->id]);
        } catch (\Exception $e) {
            Log::channel('dailyInfo')->info('exception:' . $e->getTraceAsString());
            throw new Exception("Could not create wallet.", 0, $e);
        }
        Log::channel('dailyInfo')->info('order request filled');

        try {
            $first = $order1->first();
            $amountP = $orderCreate->amountPayable;
            $discount = \App\Models\Discount::where(function ($q) use ($amountP) {
                $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                    ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString())
                    ->where('amountFrom', '<=', $amountP)
                    ->where('amountTo', '>=', $amountP);
            })->first();
            if ($discount != null) {
                $voucher = $first->createVouchers(1, [
                    'discount' => $discount->discount,
                    'amountFrom' => $discount->amountFrom,
                    'amountTo' => $discount->amountTo
                ], \Carbon\Carbon::now()->addDays($discount->expires))[0];
                $order1->update(['discount_code' => $voucher->code]);
            }
        } catch (\Exception $e) {
            throw new Exception("Could not create voucher.", 0, $e);
        }
        try {
            if (!empty($fields->pay) && $fields->pay == true) {
                $api = new Api(env('KEY_ID'), config('KEY_SECRET'));
                $payment = $api->payment->fetch($fields->razorpay_payment_id);

                try {
                    $response = $api->payment->fetch($fields->razorpay_payment_id)->capture(array('amount' => $payment['amount']));
                    $order1->update(['razorpay_payment_id' => $fields->razorpay_payment_id]);
                    Log::channel('dailyInfo')->info('payment done');
                } catch (\Exception $e) {
                    $order1->update(['status' => 'PAYMENT_FAILED']);
                    Log::channel('dailyInfo')->info('payment could net be fetched for Id: ' . $fields->razorpay_payment_id);
                }
            }
        } catch (\Exception $e) {
            Log::channel('dailyInfo')->info("Could not fetch payment." . " [$fields->razorpay_payment_id] " . $e);
        }

        # for sending user message
        try {
            $message =  $this->formatMessage($orderCreate);
            $res1 = '';
            ob_start();
            $this->sendMessage($message['message'], $dbg == false ? $message['outlet']->mobile : "9336166567", env('VENDOR_MSG_TEMLPLATE_ID', NULL));
            $res1 = ob_get_clean();
            Log::channel('dailyInfo')->info('--sms sent to outlet:' . $outlet->outletName . ', message response:' . $res1);
            Log::channel('dailyInfo')->info('customer mobiles:' . $fields->customer->mobile . ', message response:' . $res1);
            Log::channel('dailyInfo')->info('sms response:' . $res1 . '--');
            $linkMsg = ", Download the app https://play.google.com/store/apps/details?id=com.olfstores.olf_user or visit https://www.olfstores.com. Terms and conditions apply";
            if ($discount != null) {
                $message =  $this->formatMessageCustomer($orderCreate, $discount, $voucher, $walletCash, $linkMsg);
            } else {
                $message =  $this->formatMessage($orderCreate, $linkMsg);
            }
            Log::channel('dailyInfo')->info('--sms sent to customer mobiles:' . $fields->customer->mobile);
            $res2 = '';
            ob_start();
            $this->sendMessage($message['message'], $fields->customer->mobile, env('CUSTOMER_MSG_TEMPLATE_ID', NULL));
            $res2 = ob_get_clean();
            $res1 .= $res2;
            Log::channel('dailyInfo')->info('message response:' . $res2 . '--');
        } catch (\Exception $e) {
            throw new Exception("Could not send text message." . (isset($res1) ? $res1 : ''), 0, $e);
        }

        try {
            # sending email to customer
            $this->passEmail($order, $fields);
            Log::channel('dailyInfo')->info('email sent to customer::' . $order['customerEmail'] . ', outlet email:' . \App\User::find($outlet->user_id)->email);
        } catch (\Exception $e) {
            throw new Exception("Could not send email.", 0, $e);
        }
        try {
            # sending voice message to vendor
            $message1 = 'Hi ! You received a new order from o el f store for train no. ' . implode(' ', str_split($fields->trainNo)) . ' customer name ' . $fields->customer->fullName . ' Thank You';
            $mobiles = explode(",", $outlet->user->mobile);
            foreach ($mobiles as $value) {
                $res2 = '';
                ob_start();
                $this->sendVoiceMessage($message1, $dbg == false ? $value : '9336166567');
                $res2 = ob_get_clean();

                Log::channel('dailyInfo')->info('--voice message sent to outlet mobile:' . $value);
                Log::channel('dailyInfo')->info('voice message response:' . $res2 . '--');
            }
        } catch (\Exception $e) {
            throw new Exception("Could not send voice message." . (isset($res1) && isset($res2) ? $res1 . '--end' . $res2 : ''), 0, $e);
        }

        $result = (object)[
            'orderId' => $orderCreate->id,
            'status' => 0, //todo
            'message' => $message
        ];

        return $result;
    }


    public function getVendorDetails(Request $request)
    {
        $outlet = Outlet::findOrFail($request->outlet_id);

        $vendorData=null;

        if(!empty($outlet)){
            $vendorData=User::findOrFail($outlet->user_id);
        }

        return response()->json(['vendorData' => $vendorData,'outlet'=>$outlet]);
    }
}
