<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Requests;
use BeyondCode\Vouchers\Models\Voucher;
use BeyondCode\Vouchers\Vouchers;
use Mail;
use Auth;
use Config;
use Razorpay\Api\Api;
use DB;
use Log;
use Exception;

function processOutletTest($id){
    $outlet = \App\Models\Outlet::find($id);
    return [
        "usable" => [
        'id'=>$outlet->id,
    	'name'=> $outlet->outletName,
    	'city'=>$outlet->city,
    	'address'=>$outlet->address,
    	'fssaiNumber'=> $outlet->fssaiNo,
    	'gstNumber'=>$outlet->gstNo,
    	'fssaiCutOffDate'=>$outlet->fssaiValidUpto." IST",
    	'state'=>$outlet->state,
    	'contactNumbers'=>$outlet->mobile,
    	'relationshipManagerName'=>"Sanjay Jadoun",
    	'relationshipManagerPhone'=>$outlet->mobile,
    	'relationshipManagerEmail'=>'olfstores@gmail.com',
    	'pinCode'=>'474011',
        ],
        
        'crossCheck' => [
                'cutoff' => $outlet->cancel_cut_off_time,
                'weeklyClosed' => json_decode($outlet->weeklyClosed),
                'closingPeriod' => json_decode($outlet->closingPeriod),
                'timing'=> [
                        'opening'=> $outlet->openingTime,
                        'closing' => $outlet->closingTime
                    ]
            ]
        ];
}

class OrderTestController extends Controller{

    public function getAllOutlets(Request $request)
    {
        if(auth()->user()->hasRole('vendor')){
            if(Auth::user()->outlets!=null)
                $outlets = Auth::user()->outlets;
            else
                $outlets = null;
        }else
            $outlets = \App\Models\Outlet::all()->map (function($out,$i){
                if($out->closingPeriod=='')
                    $period = json_decode(json_encode([]));
                else
                    $period = json_decode($out->closingPeriod);

                $now = \Carbon\Carbon::now();
                $out->open = true;
                $weekMap = [
                    0 => 'SUN',
                    1 => 'MON',
                    2 => 'TUE',
                    3 => 'WED',
                    4 => 'THU',
                    5 => 'FRI',
                    6 => 'SAT',
                ];
                array_map(function($p) use($out,$now){
                    $f = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedFrom)->startOfDay();
                    $t = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedTo)->endOfDay();
                    if($now->gte($f) && $now->lte($t))
                        $out->open = false;
                }, $period);
                if($out->open){
                    if($out->weeklyClosed=='')
                        $weekly = json_decode(json_encode([]));
                    else
                        $weekly = json_decode($out->weeklyClosed);
                    array_map(function($w) use($out,$now,$weekMap){
                        if($weekMap[$now->dayOfWeek] == $w)
                            $out->open = false;
                    }, $weekly);
                }
                if($out->open)
                    $out->open = 'checked';
                else
                    unset($out->open);
                $out->weeklyClosed = json_decode($out->weeklyClosed);
                $out->closingPeriod = json_decode($out->closingPeriod);
                return $out;
            });
        if($request->has('api')&&(int)$request->input('api'))
            return response()->json(['status'=>'success','outlets'=>$outlets->toArray()],200);

        return view('backend.outlets.index',['outlets'=>$outlets]);
    }

    public function createOrder(Request $request){
        return response()->json($request->all(), 200);

    }

    public function createOrderTemplate(Request $request){
        return view('backend.orders.create');
    }

    public function orders( Request $request ,$approve,$json)
        {
            try{
                if($request->filled('date')){
                    $start = \Carbon\Carbon::createFromFormat('m/d/Y', $request->input('date'), 'Asia/Kolkata')->startOfDay();
                }else{
                    $start = \Carbon\Carbon::today()->startOfDay();
                }
                $end = $start->copy()->addHours(24);
                $orders = \App\Models\Order::
                            where(function ($q) use($start,$end){
                                        $q->where('created_at', '>=', $start)
                                                ->where('created_at', '<', $end);
                                        if($this->__api&&$this->__outlet_id){
                                            $q->where('outlet_id', (int)$this->__outlet_id);
                                        }
                                /*})
                                    ->where(function ($q){
                                        $q->where('status_status', 'PENDING');*/

                              })->orderBy('created_at', 'desc')->get()->map(function($f) {
                                $x =$f;
                                $x->outletDetails = \App\Models\Outlet::find($x->outlet_id);
                                $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                                if($x->deliveryBoy!=null){
                                    $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                                    $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                                }else
                                    $x->deliveryBoyName = '';
                                $x->vendorOrder = false;
                                $x->vendorOrder1 = 'false';

                                return $x;
                            });
                $vendorOrders = \App\Models\VendorOrder::
                            where(function ($q) use($start,$end){
                                        $q->where('created_at', '>=', $start)
                                                ->where('created_at', '<', $end);
                                        if($this->__api&&$this->__outlet_id){
                                            $q->where('outlet', (int)$this->__outlet_id);
                                        }
                                /*})
                                    ->where(function ($q){
                                        $q->where('status_status', 'PENDING');*/

                              })->orderBy('created_at', 'desc')->get()->map(function($f) {
                    $x =$f;
                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $x->pnr;
                    $deliveryDetails->trainNo = $x->trainNo;
                    $deliveryDetails->trainName = $x->trainName;
                    $deliveryDetails->station = $x->stationName;
                    $deliveryDetails->stationCode = $x->stationCode;
                    $deliveryDetails->berth = $x->berth;
                    $deliveryDetails->coach = $x->coach;
                    $deliveryDetails->ETA = $x->deliveryDate;
                    $x->deliveryDetails = json_encode($deliveryDetails);

                    $x->customerDetails = json_decode($x->customer);
                    $x->customerDetails->customerName = $x->customerDetails->fullName;
                    $x->customerDetails = json_encode($x->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $x->totalAmount;
                    $x->priceDetails = json_encode($priceDetails);
                    $x->outletDetails = \App\Models\Outlet::find($x->outlet);
                    $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                    if($x->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $x->deliveryBoyName = '';

                    $x->vendorOrder = true;
                    $x->vendorOrder1 = 'true';

                    return $x;
                });
                $orders = $orders->toBase()->merge($vendorOrders);
                $orders = $orders->sortBy(function($order){
                                return $order->created_at;
                            })->reverse()->values();

                if((isset($json) && $json=='json')||$this->__api){
                    $orders = $orders->map(function($o){
                        $x = $o;
                        $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                        $x->customerDetails = json_decode($o->customerDetails);
                        $x->priceDetails = json_decode($o->priceDetails);
                        $x->orderItems = json_decode($o->orderItems);

                        return $x;
                    });
                    return response()->json($orders,200);
                }else if($request->filled('object')&&$request->input('object')==true){
                    $orders = $orders->map(function($o){
                        $x = $o;
                        if(!$o->updated){
                            $x->orderBackColor = '#d65151';
                            $x->color = '#fff';
                        }else{
                            $x->orderBackColor = '#fff';
                            $x->color = '#000';
                        }
                        $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                        $x->customerDetails = json_decode($o->customerDetails);
                        $x->priceDetails = json_decode($o->priceDetails);
                        $x->orderItems = json_decode($o->orderItems);

                        return $x;
                    });
                    return $orders;
                }
                return view('backend.admins.order',compact('orders'));

            }catch(\Exception $e){
                dd($e);
                return response([ 'message'=>'invalid request'],200);
            }
        }

    public function update( Request $request, $orderId)
    {
        //Log::channel('dailyInfo')->info('input'.$request->input('vendor'));
        if(!isset($orderId)||!$request->filled('vendor')){
            session()->flash('error', 'Request input parameters are missing.');
            return redirect()->back();
        }

        if($request->input('vendor')=='true'){
            //Log::channel('dailyInfo')->info('vendor'.$request->input('vendor'));
            $order = \App\Models\VendorOrder::where('orderId',$orderId)->first();
            $order->updated = 1;
        }else{
            //Log::channel('dailyInfo')->info($request->input('vendor'));
            $order = \App\Models\Order::where('orderId',$orderId)->first();
            $order->updated = 1;
        }
        $order->save();
        session()->flash('success', 'Order Updated successfully.');
        return redirect()->back();
    }
    public function massUpdateStatusApprove( Request $request)
        {
            try{
                $fields = json_decode($request->getContent());
                $orders = $fields->orders;
                $status = $fields->status;
                $dataVendor = array();
                $data = array();
                foreach($orders as $item) {
                    if($item->vendorOrder)
                        $dataVendor[] = $item->orderId;
                    else
                        $data[] = $item->orderId;
                }
                if(sizeof($dataVendor)>0){
                    $vendorOrderUpdate = \App\Models\VendorOrder::
                            where(function ($q) use($dataVendor){
                                $q->where('orderId', $dataVendor[0]);
                                foreach($dataVendor as $vendorOrder){
                                    $q->orWhere('orderId', $vendorOrder);
                                }
                              });
                    $vendorOrderUpdate->update(array('status'=>$status));
                }

                if(sizeof($data)>0){
                    $orderUpdate = \App\Models\Order::
                            where(function ($q) use($data){
                                $q->where('orderId', $data[0]);
                                foreach($data as $order){
                                    $q->orWhere('orderId', $order);
                                }
                              });
                    $orderUpdate->update(array('status'=>$status));
                }
                if(isset($fields->date)){
                    $start = \Carbon\Carbon::createFromFormat('m/d/Y', $fields->date, 'Asia/Kolkata')->startOfDay();
                }else{
                    $start = \Carbon\Carbon::today()->startOfDay();
                }
                $end = $start->copy()->addHours(24);
                $orders = \App\Order::
                            where(function ($q) use($start,$end){
                                        $q->where('created_at', '>=', $start)
                                                ->where('created_at', '<', $end);
                                /*})
                                    ->where(function ($q){
                                        $q->where('status_status', 'PENDING');*/

                              })->orderBy('created_at', 'desc')->get()->map(function($f) {
                                $x =$f;
                                $x->outletDetails = \App\Models\Outlet::find($x->outlet_id);
                                $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                                if($x->deliveryBoy!=null){
                                    $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                                    $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                                }else
                                    $x->deliveryBoyName = '';
                                $x->vendorOrder = false;
                                return $x;
                            });
                $vendorOrders = \App\Models\VendorOrder::
                            where(function ($q) use($start,$end){
                                        $q->where('created_at', '>=', $start)
                                                ->where('created_at', '<', $end);
                                /*})
                                    ->where(function ($q){
                                        $q->where('status_status', 'PENDING');*/

                              })->orderBy('created_at', 'desc')->get()->map(function($f) {
                    $x =$f;
                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $x->pnr;
                    $deliveryDetails->trainNo = $x->trainNo;
                    $deliveryDetails->trainName = $x->trainName;
                    $deliveryDetails->station = $x->stationName;
                    $deliveryDetails->stationCode = $x->stationCode;
                    $deliveryDetails->berth = $x->berth;
                    $deliveryDetails->coach = $x->coach;
                    $deliveryDetails->ETA = $x->deliveryDate;
                    $x->deliveryDetails = json_encode($deliveryDetails);

                    $x->customerDetails = json_decode($x->customer);
                    $x->customerDetails->customerName = $x->customerDetails->fullName;
                    $x->customerDetails = json_encode($x->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $x->totalAmount;
                    $x->priceDetails = json_encode($priceDetails);
                    $x->outletDetails = \App\Models\Outlet::find($x->outlet);
                    $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                    if($x->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $x->deliveryBoyName = '';

                    $x->vendorOrder = true;
                    return $x;
                });
                $orders = $orders->toBase()->merge($vendorOrders);
                $orders = $orders->sortBy(function($order){
                                return $order->created_at;
                            })->reverse()->values();

                $orders = $orders->map(function($o){
                    $x = $o;
                    $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                    $x->customerDetails = json_decode($o->customerDetails);
                    $x->priceDetails = json_decode($o->priceDetails);
                    $x->orderItems = json_decode($o->orderItems);

                    return $x;
                });
                return response()->json($orders,200);

            }catch(\Exception $e){
                dd($e);
                return response([ 'message'=>'invalid request'],200);
            }
        }
    public function ordersByOutlet( Request $request ,$outlet)
        {
            try{
                $out = \App\Models\Outlet::find($outlet);
                $this->authorize('pol_view_orders', $out);
                $start = \Carbon\Carbon::today()->startOfDay();
                $end = \Carbon\Carbon::today()->startOfDay()->addHours(24);
                $orders = \App\Models\Outlet::find($outlet)->orders()
                            ->where(function ($q) use($start,$end){
                                $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                              })->orderBy('created_at', 'desc')->get()->map(function($f) use($out){
                                $x =$f;
                                $x->outletDetails = $out;
                                $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                                if($x->deliveryBoy!=null){
                                    $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                                    $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                                }else
                                    $x->deliveryBoyName = '';
                                $x->vendorOrder = false;
                                return $x;
                            });
                $vendorOrders = \App\Models\Outlet::find($outlet)->vendor_orders()
                                    ->where(function ($q) use($start,$end) {
                                        $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                                    })->orderBy('created_at', 'desc')->get()->map(function($f) use($out){
                    $x =$f;
                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $x->pnr;
                    $deliveryDetails->trainNo = $x->trainNo;
                    $deliveryDetails->trainName = $x->trainName;
                    $deliveryDetails->station = $x->stationName;
                    $deliveryDetails->stationCode = $x->stationCode;
                    $deliveryDetails->berth = $x->berth;
                    $deliveryDetails->coach = $x->coach;
                    $deliveryDetails->ETA = $x->deliveryDate;
                    $x->deliveryDetails = json_encode($deliveryDetails);

                    $x->customerDetails = json_decode($x->customer);
                    $x->customerDetails->customerName = $x->customerDetails->fullName;
                    $x->customerDetails = json_encode($x->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $x->totalAmount;
                    $x->priceDetails = json_encode($priceDetails);

                    $x->outletDetails = $out;
                    if($x->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $x->deliveryBoyName = '';
                    $x->vendorOrder = true;
                    return $x;
                });
                $orders = $orders->toBase()->merge($vendorOrders);
                $orders = $orders->sortBy(function($order){
                                return $order->created_at;
                            })->reverse()->values();

                return view('backend.orders.index',compact('orders','outlet'));

            }catch(\Exception $e){
                dd($e);
                return response([ 'message'=>'invalid request'],200);
            }
        }

    public function ordersByOutletJson( Request $request ,$outlet)
        {
            try{
                if($request->filled('date')){
                    $start = \Carbon\Carbon::createFromFormat('m/d/Y', $request->input('date'), 'Asia/Kolkata')->startOfDay();
                    $end = \Carbon\Carbon::createFromFormat('m/d/Y', $request->input('date'), 'Asia/Kolkata')->startOfDay()->addHours(24);

                }else{
                    $start = \Carbon\Carbon::today()->startOfDay();
                    $end = \Carbon\Carbon::today()->startOfDay()->addHours(24);
                }
                $out = \App\Models\Outlet::find($outlet);
                $this->authorize('pol_view_orders', $out);
                $orders = $out->orders()
                            ->where(function ($q) use($start,$end){
                                $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                              })->orderBy('created_at', 'desc')->get()->map(function($f) use($out){
                            $x =$f;
                            $x->outletDetails = $out;
                            $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                            if($x->deliveryBoy!=null){
                                $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                                $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                            }else
                                $x->deliveryBoyName = '';
                            $x->vendorOrder = false;

                            return $x;
                        });
                $vendorOrders = $out->vendor_orders()
                                    ->where(function ($q) use($start,$end) {
                                        $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                                    })->orderBy('created_at', 'desc')->get()->map(function($f) use($out){
                    $x =$f;
                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $x->pnr;
                    $deliveryDetails->trainNo = $x->trainNo;
                    $deliveryDetails->trainName = $x->trainName;
                    $deliveryDetails->station = $x->stationName;
                    $deliveryDetails->stationCode = $x->stationCode;
                    $deliveryDetails->berth = $x->berth;
                    $deliveryDetails->coach = $x->coach;
                    $deliveryDetails->ETA = $x->deliveryDate;
                    $x->deliveryDetails = json_encode($deliveryDetails);

                    $x->customerDetails = json_decode($x->customer);
                    $x->customerDetails->customerName = $x->customerDetails->fullName;
                    $x->customerDetails = json_encode($x->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $x->totalAmount;
                    $x->priceDetails = json_encode($priceDetails);

                    $x->outletDetails = $out;
                    $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                    if($x->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $x->deliveryBoyName = '';
                    $x->vendorOrder = true;
                    return $x;
                });
                $orders = $orders->toBase()->merge($vendorOrders);
                $orders = $orders->sortBy(function($order){
                                return $order->created_at;
                            })->reverse()->values();

                $orders = $orders->map(function($o) use($out){
                        $x = $o;
                        $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                        $x->outletDetails = $out;
                        $x->customerDetails = json_decode($o->customerDetails);
                        $x->priceDetails = json_decode($o->priceDetails);
                        $x->orderItems = json_decode($o->orderItems);

                        return $x;
                    });
                    //print_r(collect($orders));
                    //echo json_encode($orders);
                return response()->json($orders,200);
            }catch(\Exception $e){
                dd($e);
                return response()->json([ 'message'=>'invalid request'],200);
            }
        }
    public function massUpdateStatus( Request $request ,$outlet)
        {
            try{
                $fields = json_decode($request->getContent());
                $orders = $fields->orders;
                $status = $fields->status;
                $dataVendor = array();
                $data = array();
                foreach($orders as $item) {
                    if($item->vendorOrder)
                        $dataVendor[] = $item->orderId;
                    else
                        $data[] = $item->orderId;
                }
                if(sizeof($dataVendor)>0){
                    $vendorOrderUpdate = \App\Models\VendorOrder::
                            where(function ($q) use($dataVendor){
                                $q->where('orderId', $dataVendor[0]);
                                foreach($dataVendor as $vendorOrder){
                                    $q->orWhere('orderId', $vendorOrder);
                                }
                              });
                    $vendorOrderUpdate->update(array('status'=>$status));
                }

                if(sizeof($data)>0){
                    $orderUpdate = \App\Models\Order::
                            where(function ($q) use($data){
                                $q->where('orderId', $data[0]);
                                foreach($data as $order){
                                    $q->orWhere('orderId', $order);
                                }
                              });
                    $orderUpdate->update(array('status'=>$status));
                }


                if($fields->date){
                    $start = \Carbon\Carbon::createFromFormat('m/d/Y', $fields->date, 'Asia/Kolkata')->startOfDay();
                    $end = \Carbon\Carbon::createFromFormat('m/d/Y', $fields->date, 'Asia/Kolkata')->startOfDay()->addHours(24);

                }else{
                    $start = \Carbon\Carbon::today()->startOfDay();
                    $end = \Carbon\Carbon::today()->startOfDay()->addHours(24);
                }
                $out = \App\Models\Outlet::find($outlet);
                $this->authorize('pol_view_orders', $out);
                $orders = $out->orders()
                            ->where(function ($q) use($start,$end){
                                $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                              })->orderBy('created_at', 'desc')->get()->map(function($f) use($out){
                            $x =$f;
                            $x->outletDetails = $out;
                            $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                            if($x->deliveryBoy!=null){
                                $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                                $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                            }else
                                $x->deliveryBoyName = '';
                            $x->vendorOrder = false;

                            return $x;
                        });
                $vendorOrders = $out->vendor_orders()
                                    ->where(function ($q) use($start,$end) {
                                        $q->where('created_at', '>=', $start)->where('created_at', '<', $end);
                                    })->orderBy('created_at', 'desc')->get()->map(function($f) use($out){
                    $x =$f;
                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $x->pnr;
                    $deliveryDetails->trainNo = $x->trainNo;
                    $deliveryDetails->trainName = $x->trainName;
                    $deliveryDetails->station = $x->stationName;
                    $deliveryDetails->stationCode = $x->stationCode;
                    $deliveryDetails->berth = $x->berth;
                    $deliveryDetails->coach = $x->coach;
                    $deliveryDetails->ETA = $x->deliveryDate;
                    $x->deliveryDetails = json_encode($deliveryDetails);

                    $x->customerDetails = json_decode($x->customer);
                    $x->customerDetails->customerName = $x->customerDetails->fullName;
                    $x->customerDetails = json_encode($x->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $x->totalAmount;
                    $x->priceDetails = json_encode($priceDetails);

                    $x->outletDetails = $out;
                    $x->outletDetails->deliveryBoys1 = $x->outletDetails->deliveryBoys;
                    if($x->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($x->deliveryBoy);
                        $x->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $x->deliveryBoyName = '';
                    $x->vendorOrder = true;
                    return $x;
                });
                $orders = $orders->toBase()->merge($vendorOrders);
                $orders = $orders->sortBy(function($order){
                                return $order->created_at;
                            })->reverse()->values();

                $orders = $orders->map(function($o) use($out){
                        $x = $o;
                        $x->aggregatorDetails = json_decode($o->aggregatorDetails);
                        $x->outletDetails = $out;
                        $x->customerDetails = json_decode($o->customerDetails);
                        $x->priceDetails = json_decode($o->priceDetails);
                        $x->orderItems = json_decode($o->orderItems);

                        return $x;
                    });
                    //print_r(collect($orders));
                    //echo json_encode($orders);
                return response()->json($orders,200);
            }catch(\Exception $e){
                dd($e);
                return response()->json([ 'message'=>'invalid request'],200);
            }
        }

        public function Search($vender,$critaria,$term)
        {
            $orders = \App\Models\Order::where('outlet_id',$vender)->get();
            switch ($critaria) {
                case 'orderId':
                    # code...
                    return $results = $orders->filter(function($item) use($term){
                        return $item->orderId == $term;
                    });

                case 'customer':
                    if (strlen($term) > 3) {
                        return $results = $orders->filter(function($item) use($term){
                                $temp = json_decode($item->customerDetails);
                                return $retVal = (strstr($temp->customerName, $term)) ? $item : false ;
                            });

                    } else {
                        return (object)['message'=>'invalid Query'];
                    }


                case 'mobile':
                    return $results = $orders->filter(function($item) use($term){
                        $temp = json_decode($item->customerDetails);
                        return $temp->mobile == $term;
                    });

                default:
                    return $orders;
            }
        }
        public function orderSearch(Request $request,$vendorId,$critaria,$term)
        {
            $check = $vendorId && $critaria && $term;
            try{
                if ($check) {
                    $results = $this->Search($vendorId,$critaria,$term);
                    header('Content-Type','application/json');
                    return response()->json($results,200);
                } else {
                    dd('invalid Query');
                }

            }catch(\Exception $e){
                dd($e);
            }

        }

        public function orderUpdate(Request $request, $orderId)
        {
            try{
                //$this->authorize('pol_update_order', \App\Order::where('orderId',$orderId));
                $fields = $request->all();
                $vendorOrder = $fields['vendorOrder'];
                $fields['pending_status'] = $fields['status'];
                $fields['requestor_user_id'] = Auth::user()->id;
                $fields['status_status'] = 'PENDING';
                unset($fields['vendorOrder']);
                unset($fields['status']);
                if(isset($fields['reason1']))
                    unset($fields['reason1']);
                if($vendorOrder){
                    $update = \App\Models\VendorOrder::where('orderId',$orderId)->update($fields);
                    $order = \App\Models\VendorOrder::where('orderId',$orderId)->first();
                    if(\App\Models\VendorOrder::where('orderId',$orderId)->approvalReq()->first()==null){
                        if($order->status_history==null)
                            $status_history = '';
                        else
                            $status_history = $order->status_history;

                        $status = array('status'=>$order->pending_status,
                                        'pending_status'=>null,
                                        'status_history'=>$status_history.'->'.$order->pending_status,
                                        'status_status'=>null);
                        $update1 = \App\Models\VendorOrder::where('orderId',$orderId)->update($status);
                        $order = \App\Models\VendorOrder::where('orderId',$orderId)->first();
                    }


                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $order->pnr;
                    $deliveryDetails->trainNo = $order->trainNo;
                    $deliveryDetails->trainName = $order->trainName;
                    $deliveryDetails->station = $order->stationName;
                    $deliveryDetails->stationCode = $order->stationCode;
                    $deliveryDetails->berth = $order->berth;
                    $deliveryDetails->coach = $order->coach;
                    $deliveryDetails->ETA = $order->deliveryDate;
                    $order->deliveryDetails = json_encode($deliveryDetails);

                    $order->customerDetails = json_decode($order->customer);
                    $order->customerDetails->customerName = $order->customerDetails->fullName;
                    $order->customerDetails = json_encode($order->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $order->totalAmount;
                    $order->priceDetails = json_encode($priceDetails);

                    $order->outletDetails = \App\Models\Outlet::find($order->outlet);
                    $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
                    if($order->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                        $order->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $order->deliveryBoyName = '';
                    $order->vendorOrder = true;
                }else{
                    $update = \App\Models\Order::where('orderId',$orderId)->update($fields);
                    $order = \App\Models\Order::where('orderId',$orderId)->first();
                    if(\App\Models\Order::where('orderId',$orderId)->approvalReq()->first()==null){
                        if($order->status_history==null)
                            $status_history = '';
                        else
                            $status_history = $order->status_history;

                        $status = array('status'=>$order->pending_status,
                                        'pending_status'=>null,
                                        'status_history'=>$status_history.'->'.$order->pending_status,
                                        'status_status'=>null);
                        $update1 = \App\Models\Order::where('orderId',$orderId)->update($status);
                        $order = \App\Models\Order::where('orderId',$orderId)->first();
                    }
                    $deliveryDetails = json_decode($order->deliveryDetails);
                    $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
                    if($order->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                        $order->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $order->deliveryBoyName = '';
                    $order->outletDetails = \App\Models\Outlet::find($order->outlet_id);
                    $order->vendorOrder = false;
                }
                $order->aggregatorDetails = json_decode($order->aggregatorDetails);
                $order->customerDetails = json_decode($order->customerDetails);
                $order->priceDetails = json_decode($order->priceDetails);
                $order->orderItems = json_decode($order->orderItems);

                if($update1){
                    if($order->status=='ORDER_UNDELIVERED'){
                        $message = "You have been charged Rs 100 against Non Delivery of Order id: %s";
                        $this->sendStatusMessage($order, $message);
                    }else if($order->status=='ORDER_PARTIALLY_DELIVERED'){
                        $outletName = $order->outletDetails->outletName;
                        $message = "Customer has raised a complaint against $outletName for Order id: %s, at station \"$deliveryDetails->station\" As resolution of complaint an appropriate amount has been debited from Vendor Price";
                        $this->sendStatusMessage($order, $message);
                    }
                }
                if($update){
                    return response()->json($order,200);
                }
                return response()->json(['message'=>'Updation failed'],200);
            }catch(\Exception $e){
                return response()->json(['message'=>'Updation failed'],501);
            }

        }

        public function orderUpdateAdmin(Request $request, $orderId,$approve)
        {
            //try{
                //$this->authorize('pol_update_order', \App\Order::where('orderId',$orderId));
                $fields = $request->all();
                $vendorOrder = $fields['vendorOrder'];
                $fields['requestor_user_id'] = Auth::user()->id;
                unset($fields['vendorOrder']);
                if(isset($fields['reason1']))
                    unset($fields['reason1']);
                $fields['status_status'] = null;
                $fields['pending_status'] = null;
                //$fields['status_reason'] = null;

                if($vendorOrder){
                    $order = \App\Models\VendorOrder::where('orderId',$orderId)->first();
                    if(isset($approve) && $approve=='approve'){
                        $fields['status'] = $order->pending_status;
                    }
                    if($order->status_history==null)
                        $status_history = '';
                    else
                        $status_history = $order->status_history;
                    $fields['status_history'] = $status_history.'->'.$fields['status'];
                    $update = \App\Models\VendorOrder::where('orderId',$orderId)->update($fields);
                    $order = \App\Models\VendorOrder::where('orderId',$orderId)->first();

                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $order->pnr;
                    $deliveryDetails->trainNo = $order->trainNo;
                    $deliveryDetails->trainName = $order->trainName;
                    $deliveryDetails->station = $order->stationName;
                    $deliveryDetails->stationCode = $order->stationCode;
                    $deliveryDetails->berth = $order->berth;
                    $deliveryDetails->coach = $order->coach;
                    $deliveryDetails->ETA = $order->deliveryDate;
                    $order->deliveryDetails = json_encode($deliveryDetails);

                    $order->customerDetails = json_decode($order->customer);
                    $order->customerDetails->customerName = $order->customerDetails->fullName;
                    $order->customerDetails = json_encode($order->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $order->totalAmount;
                    $order->priceDetails = json_encode($priceDetails);

                    $order->outletDetails = \App\Models\Outlet::find($order->outlet);
                    $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
                    if($order->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                        $order->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $order->deliveryBoyName = '';
                    $order->vendorOrder = true;
                }else{
                    $order = \App\Models\Order::where('orderId',$orderId)->first();
                    if(isset($approve) && $approve=='approve'){
                        $fields['status'] = $order->pending_status;
                    }
                    if($order->status_history==null)
                        $status_history = '';
                    else
                        $status_history = $order->status_history;
                    $fields['status_history'] = $status_history.'->'.$fields['status'];
                    $update = \App\Models\Order::where('orderId',$orderId)->update($fields);
                    $order = \App\Models\Order::where('orderId',$orderId)->first();

                    $deliveryDetails = json_decode($order->deliveryDetails);
                    $order->outletDetails = \App\Models\Outlet::find($order->outlet_id);
                    $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
                    if($order->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                        $order->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $order->deliveryBoyName = '';
                    $order->vendorOrder = false;
                }
                $order->aggregatorDetails = json_decode($order->aggregatorDetails);
                $order->customerDetails = json_decode($order->customerDetails);
                $order->priceDetails = json_decode($order->priceDetails);
                $order->orderItems = json_decode($order->orderItems);

                if($update){
                    if($order->status=='ORDER_UNDELIVERED'){
                        $message = "You have been charged Rs 100 against Non Delivery of Order id: %s";
                        $this->sendStatusMessage($order, $message);
                    }else if($order->status=='ORDER_PARTIALLY_DELIVERED'){
                        $outletName = $order->outletDetails->outletName;
                        $message = "Customer has raised a complaint against $outletName for Order id: %s, at station \"$deliveryDetails->station\" As resolution of complaint an appropriate amount has been debited from Vendor Price";
                        $this->sendStatusMessage($order, $message);
                    }
                    return response()->json($order,200);
                }
                return response()->json(['message'=>'Updation failed'],200);
            //}catch(\Exception $e){
                //return response()->json(['message'=>'Updation failed'],501);
            //}
        }

        public function orderRejectAdmin(Request $request, $orderId)
        {
            try{
                //$this->authorize('pol_update_order', \App\Order::where('orderId',$orderId));
                $fields = array();
                $vendorOrder = $request->input('vendorOrder');
                $fields['requestor_user_id'] = Auth::user()->id;
                $fields['requestor_user_id'] = null;
                $fields['status_status'] = null;
                $fields['pending_status'] = null;
                $fields['status_reason'] = null;
                if($vendorOrder){
                    $update = \App\Models\VendorOrder::where('orderId',$orderId)->update($fields);
                    $order = \App\Models\VendorOrder::where('orderId',$orderId)->first();

                    $deliveryDetails = new \stdClass;
                    $deliveryDetails->pnr = $order->pnr;
                    $deliveryDetails->trainNo = $order->trainNo;
                    $deliveryDetails->trainName = $order->trainName;
                    $deliveryDetails->station = $order->stationName;
                    $deliveryDetails->stationCode = $order->stationCode;
                    $deliveryDetails->berth = $order->berth;
                    $deliveryDetails->coach = $order->coach;
                    $deliveryDetails->ETA = $order->deliveryDate;
                    $order->deliveryDetails = json_encode($deliveryDetails);

                    $order->customerDetails = json_decode($order->customer);
                    $order->customerDetails->customerName = $order->customerDetails->fullName;
                    $order->customerDetails = json_encode($order->customerDetails);

                    $priceDetails = new \stdClass;
                    $priceDetails->totalAmount = $order->totalAmount;
                    $order->priceDetails = json_encode($priceDetails);

                    $order->outletDetails = \App\Models\Outlet::find($order->outlet);
                    $order->vendorOrder = true;
                }else{
                    $update = \App\Models\Order::where('orderId',$orderId)->update($fields);
                    $order = \App\Models\Order::where('orderId',$orderId)->first();

                    $order->outletDetails = \App\Models\Outlet::find($order->outlet_id);
                    $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
                    if($order->deliveryBoy!=null){
                        $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                        $order->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
                    }else
                        $order->deliveryBoyName = '';
                    $order->vendorOrder = false;
                }
                $order->aggregatorDetails = json_decode($order->aggregatorDetails);
                $order->customerDetails = json_decode($order->customerDetails);
                $order->priceDetails = json_decode($order->priceDetails);
                $order->orderItems = json_decode($order->orderItems);

                if($update){
                    return response()->json($order,200);
                }
                return response()->json(['message'=>'Updation failed'],200);
            }catch(\Exception $e){
                return response()->json(['message'=>'Updation failed'],501);
            }
        }

	protected function sendOrder($url, $data){
		$data_string = json_encode($data);
		//Log::channel('dailyInfo')->info(print_r($data_string,true));
		//echo $data_string;
		$ch = curl_init($url);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
					'Content-Type: application/json',
					'Content-Length: ' . strlen($data_string),
					'Authorization:'.config('constkeys.irctc_token'))//dev-----
					//'Authorization:80d171c2-8bef-4b02-884f-fabffc769776')//prod----
			   );

		$result = curl_exec($ch);
		print_r($result);
		                //Log::channel('dailyInfo')->info(print_r($result,true));

	}

        public function processHydrogen1(Request $request){
            $fields = json_decode($request->getContent());
            $outlet = processOutletTest($fields->outlet->id);
            $fields->outlet = $outlet;
	    $fields->vendorOrderId = substr(str_shuffle(str_repeat("1234567890", 5)), 0, 10);
	    $pnr = $fields->pnr->value;
	    $fields->pnr = $pnr;
	    $fields->berth = $fields->berth->value;
	    $fields->coach = $fields->coach->value;
	    $fields->customer = (object)[
		'fullName'=>$fields->fullName->value,
		'email'=> $fields->email->value,
		'mobile'=> $fields->mobile->value
	     ];
	    $date = \Carbon\Carbon::now()->setTimezone('Asia/Kolkata')->format('Y-m-d h:i');
	    $fields->bookingDate = "$date IST";
	    $orderItems = array_map(function($item){
			return [
				"itemId"=> $item->id,
            			"itemName"=> $item->itemName,
            			"description"=> $item->description,
            			"basePrice"=>$item->price,
            			"sellingPrice"=> $item->price+$item->price*0.05,
            			"taxRate"=> $item->price*0.05,
            			"isVegetarian"=> $item->isVegeterian == 1? true:false,
            			"quantity"=> $item->quantity,
            			"option"=> null,
			];
		},$fields->orderItems);

	    unset($orderItems['st']);

	    $subtotal = collect($orderItems)->sum(function($item){
			return $item['sellingPrice'] * $item['quantity'];
		});
	    $txn = collect($orderItems)->sum(function($item){
			return ($item['taxRate']* $item['quantity']);
		});
	    $fields->orderItems = $orderItems;
	    $fields->deliveryDate = "$date IST";
	    $fields->amountPayable = $subtotal;
	    $fields->totalAmount = $subtotal;
	    $fields->vendorPaymentType = "CASH_ON_DELIVERY";
	    $fields->gst = $txn;
	    unset($fields->email);
	    unset($fields->fullName);
	    unset($fields->mobile);
	    #$this->sendOrder('https://stage-ecatering.ipsator.com/api/v1/order/vendor', $fields);
            return response()->json($fields,200);
        }

        public function getAmountAfterDiscount(Request $request){
        	try{
            $fields = json_decode($request->getContent());

			$orderItems = array_map(function($item){
                                //$basePrice = $item->price/1.05;
				return [
					//"itemId"=> $item->id,
							//"itemName"=> $item->itemName,
							//"description"=> $item->description,
                                                        //"basePrice"=>$basePrice,
                                                        "sellingPrice"=> $item->price,
                                                        //"taxRate"=> $item->price-$basePrice,
							//"isVegetarian"=> $item->isVegeterian == 1? true:false,
							"quantity"=> $item->quantity,
							//"option"=> null,
				];
			},$fields->orderItems->value);

			unset($orderItems['st']);

			$subtotal = collect($orderItems)->sum(function($item){
				return $item['sellingPrice'] * $item['quantity'];
			});
            $deliveryCost = \App\Models\Outlet::find($fields->outletId)->deliveryCost;
	    //$fields->amountPayable = round($subtotal+$subtotal*0.05);
	    $fields->amountPayable = round($subtotal);
            $o = null;
            $vo = null;
            $customerMobile = null;
            $d = 0;
            $wDiscount = 0;
            $balance = 0;
            $discountAmount = 0;
            $couponInvalid = false;
            $redeemCode = true;
            if(isset($fields->coupon->value)&&$fields->coupon->value!=null && $fields->coupon->value!=''){
                $o = \App\Models\Order::where('discount_code', $fields->coupon->value)
                        ->first();
                $vo = \App\Models\VendorOrder::where('discount_code', $fields->coupon->value)
                        ->first();
            }
            if($o!=null){
                $vo = $o;
                Config::set('vouchers.relation_table','order_voucher');
                Config::set('vouchers.user_model',\App\Models\Order::class);
            }
            if($vo!=null){
                $customerMobile = $vo->customerMobile;
            	if($vo->canRedeemCode($fields->coupon->value)){
            		try{
						$v1 = $vo->check($fields->coupon->value);
                                                $amountP = $fields->amountPayable;
                                                $discount = \App\Models\Discount::
                                                                where(function ($q) use($amountP){
                                                                    $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                                                                        ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString())
                                                                        ->where('amountFrom', '<=', $amountP)
                                                                        ->where('amountTo', '>=', $amountP);
                                                                })->first();
                                            if($v1->data->get('amountFrom')<=$fields->amountPayable && $v1->data->get('amountTo')>=$fields->amountPayable){
                                                $d = $v1->data->get('discount');
                                                $fields->couponCode = $v1->code;
                                                $fields->discountAmount = round(($fields->amountPayable)*$d/100);
                                                                            if($discount!=null){
                                                                                if($fields->discountAmount>$discount->upto)
                                                                                    $fields->discountAmount = (int)$discount->upto;
                                                                            }
                                                                            $discountAmount = $fields->discountAmount;
                                                $fields->amountPayable = $fields->amountPayable-$fields->discountAmount;
                                                }else if($discount!=null){
                                                        $d = $v1->data->get('discount');
                                                        $fields->couponCode = $v1->code;
                                                        $fields->discountAmount = (int)$discount->upto;
                                                        $discountAmount = $fields->discountAmount;
                                                        $fields->amountPayable = $fields->amountPayable-$fields->discountAmount;
                                                }
                                                }catch (\Exception $e) {
                                            return  response()
                                                        ->json(array('status'=>'failure', 'balance'=>$balance, 'wDiscount'=>$wDiscount,'discount'=>$discountAmount,'amountPayable'=>$fields->amountPayable,'message'=>$e->getMessage()),200);
					}
                    }else{
                                        $redeemCode = false;
                                        /**
                                        return  response()
                                            ->json(array('status'=>'failure', 'balance'=>$balance, 'wDiscount'=>$wDiscount,'discount'=>$discountAmount,'amountPayable'=>$fields->amountPayable,'message'=>'coupon code is invalid'),200);
                                        *
                                        */
                                    }
                }else{
                                if(!empty($fields->coupon->value))
                                $couponInvalid = true;
                            /*
                            return  response()
                                        ->json(array('status'=>'failure', 'balance'=>$balance, 'wDiscount'=>$wDiscount,'discount'=>$discountAmount,'amountPayable'=>$fields->amountPayable,'message'=>'coupon code is invalid'),200);
                             *
                             */
                        }
                    $user = Auth::guard('order')->user();
                    $user1 = Auth::guard('vendororder')->user();
                    $user2 = Auth::guard('apiorder')->user();
                    $wallet = 0;
                    if($user==null){
                        if($user1!=null){
                            $user = $user1;
                        }else{
                            if($user2!=null)
                                $user = $user2;
                            else
                                $user = Auth::guard('memoryuser')->user();
                        }
                    }
                    if($user!=null&&get_class($user->last())!='App\MemoryUser'){
                        $wDiscount = $user->last()->getAmountProduct1($fields->amountPayable);
                        $fields->amountPayable -= $user->last()->getAmountProduct1($fields->amountPayable);
                        $balance = $user->last()->balance;
                    }
                    if($couponInvalid||!$redeemCode)
                        return  response()
                                    ->json(array('status'=>'success','couponStatus'=>'failure','message'=>'coupon code is invalid',
                                                'amountPayable'=>$fields->amountPayable, 'balance'=>$balance, 'wDiscount'=>$wDiscount, 'mobile'=>$customerMobile, 'discount'=>$discountAmount),200);

                    return  response()
                                ->json(array('status'=>'success',
                                            'amountPayable'=>$fields->amountPayable, 'balance'=>$balance, 'wDiscount'=>$wDiscount, 'mobile'=>$customerMobile, 'discount'=>$discountAmount),200);
            }finally {
                    Config::set('vouchers.relation_table','vendor_order_voucher');
                    Config::set('vouchers.user_model',\App\Models\VendorOrder::class);
          }

		}
		
		
        public function processHydrogen(Request $request){
            
            $check = function($pnr){
                $data = (new \App\Domain\PnrService())
                                    ->getTrainPnr($pnr,['Authorization'=>config('constkeys.irctc_token')]);
                return $data ?? false;
            };
            
            
            $orderservice = new \App\Domain\OrderService();
            
            Log::channel('dailyInfo')->info('--------------Order Log Begin----------------------');
            $fields = [];
            $fields['discountAmount'] = 0;
            $fields['vendorOrderId'] = substr(str_shuffle(str_repeat("1234567890", 5)), 0, 10);
            foreach(json_decode($request->getContent(), true) as $key=>$value)
                $fields[$key] = $value['value'];
            
            if($PNRResponse = $check($fields['pnr'])){
                list($status,$message) = $orderservice->validateParcel($fields,$PNRResponse);
                
                if($status){
                    if($outlet = \App\Models\Outlet::find($fields['outlet']['id']))
                    {
                        list($hasError,$message) = $orderservice->validateRestraunt($message ,$outlet, false);
                        $fields['deliveryDate'] = $message['deliveryDate'];
                        $fields['bookingDate'] = $message['bookingDate'];
                        if($hasError){
                            return response()->json(['message' => $message,"status" => "failure"]);
                        }
                        
                        
                        $data = $orderservice->createParcel($fields,$PNRResponse);
                        $data['outlet'] = processOutletTest($outlet->id)['usable'];
                        $data['deliveryCharge'] = intval($outlet->deliveryCost);
                        $dataWithPayableAmount = $orderservice->calculatePayable(
                            $data,
                            $request->has('coupon'),
                            $request->has('walletApplicable')
                        );
                        
                        
                        $withDeliveryCharges = $orderservice->addDeliveryCharges($dataWithPayableAmount);
                        //$this->sendOrder('https://stage-ecatering.ipsator.com/api/v1/order/vendor', json_encode($withDeliveryCharges));
            
                        dd(json_encode($withDeliveryCharges));
                        
                    }else{
                        return response()->json(['message' => "Cannot fetch Restraunt..","status" => "failure"]);
                    }
                }else{
                    return response()->json(['message' => $message,"status" => "failure"]);
                }
            }else{
                return response()->json(['message' => "PNR Not Found","status" => "failure"]);
            }    
            //\\ Process Pnr
            // \\check station passed 
            // check restraunt closed
            // check cuttoff time 
            
            // process order
            // place order
            // notify
                
        }

	protected function persist_order($fields){
            try{
                $order['id'] = $fields->vendorOrderId; //todo
                $order['orderId'] = $fields->orderId;
                $order['customer'] = json_encode($fields->customer);
                $order['customerEmail'] = $fields->customer->email;
                $order['customerMobile'] = $fields->customer->mobile;
                $order['outlet'] = $fields->outlet->id;//get the local outlet id
                $order['bookingDate'] = $fields->bookingDate;
                $order['deliveryDate'] = $fields->deliveryDate;
                $deliveryDate1 = str_replace(' IST','',$order['deliveryDate']);
                $order['deliveryDateTime'] = \Carbon\Carbon::createFromFormat('Y-m-d H:i',$deliveryDate1);
                $order['pnr'] = $fields->pnr;
                $order['trainNo'] = $fields->trainNo;
                $order['trainName'] = $fields->trainName;
                $order['stationCode'] = $fields->stationCode;
                $order['stationName'] = $fields->stationName;
                $order['coach'] = $fields->coach;
                $order['berth'] = $fields->berth;
                $order['totalAmount'] = $fields->amountPayable; //todo
                $order['deliveryCharge'] = $fields->deliveryCharge;
                $order['gst'] = $fields->gst;
                //$order['deliveryCharge'] = $fields->amountPayable; //todo be found out
                //$order['discountAmount'] = $fields->discountAmount; //todo be found out
                $order['orderItems'] = json_encode($fields->orderItems);
                $order['discountAmount'] = $fields->discountAmount;
                $order['amountPayable'] = $fields->amountPayable; //todo
                $order['paymentType'] = $fields->paymentType;
                //$order['createdAt'] = $fields->createdAt; //todo
                //$order['updatedAt'] = $fields->updatedAt; //todo


                $outlet = \App\Models\Outlet::find($fields->outlet->id);
                $items = array_column($outlet->menu->items->toArray(), 'vendorPrice', 'id');
                $orderItems = collect($fields->orderItems)->map(function($oi) use ($items){
                    if(isset($items[$oi['itemId']]))
                        $oi['costPrice'] = $items[$oi['itemId']];
                    return $oi;
                });
                    $unavailable = false;
                    $subtotal1 = collect($orderItems)->sum(function($item){
                                if(isset($item['costPrice']) && $item['costPrice'] != 'N.A.' && $item['costPrice'] != ''){
                                    return $item['costPrice'] * $item['quantity'];
                                }
                                $unavailable = true;
                        });
                    if($unavailable)
                        $order['vendorPayment'] = 0;
                    else
                        $order['vendorPayment'] = $order['amountPayable']-$subtotal1;
            $orderCreate = $outlet->vendor_orders()->firstOrCreate($order);
            $order1 = \App\Models\VendorOrder::where('orderId', $orderCreate->orderId);
            $order2 = \App\Models\VendorOrder::where('orderId',$order['orderId'])->first();
            $wallet = $order2->wallet;
            $oWhere = \App\Models\VendorOrder::where('customerMobile', $order['customerMobile']);
            $count = $order2->count($order['customerMobile']);
            if($count==1)
                $totalAmt = ($orderCreate->amountPayable - $orderCreate->deliveryCharge) * (0.8);
            else
                $totalAmt = ($orderCreate->amountPayable - $orderCreate->deliveryCharge) * (0.5);

            Log::channel('dailyInfo')->info('--except:'.$orderCreate->amountPayable.'-'.$orderCreate->deliveryCharge.'-'.$totalAmt);
            $wallet->deposit($totalAmt);
            $walletCash = $totalAmt;
            //$order1->update(['wallet_id'=>$wallet->id]);
        } catch (\Exception $e) {
            throw new Exception("Could not create wallet.", 0, $e);
        }
        Log::channel('dailyInfo')->info('order request filled');

            try{
                $first = $order1->first();
                $amountP = $orderCreate->amountPayable;
                $discount = \App\Models\Discount::
                                where(function ($q) use($amountP){
                                    $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                                        ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString())
                                    ->where('amountFrom', '<=', $amountP)
                                    ->where('amountTo', '>=', $amountP);
                                })->first();
                if($discount!=null){
                    $voucher = $first->createVouchers(1, [
                        'discount' => $discount->discount,
                        'amountFrom' => $discount->amountFrom,
                        'amountTo' => $discount->amountTo
                    ],\Carbon\Carbon::now()->addDays($discount->expires))[0];
                    $order1->update(['discount_code' => $voucher->code]);
                }
            }catch (\Exception $e) {
                throw new Exception("Could not create voucher.", 0, $e);
            }
            try{
                if(!empty($fields->pay) && $fields->pay==true){
                    $api = new Api(config('razorpay.razor_key'), config('razorpay.razor_secret'));
                    $payment = $api->payment->fetch($fields->razorpay_payment_id);

                    try {
                        $response = $api->payment->fetch($fields->razorpay_payment_id)->capture(array('amount'=>$payment['amount']));
                        $order1->update(['razorpay_payment_id' => $fields->razorpay_payment_id]);
                        Log::channel('dailyInfo')->info('payment done');
                    } catch (\Exception $e) {
                        $order1->update(['status' => 'PAYMENT_FAILED']);
                        throw new Exception("Could not update payment.", 0, $e);
                        /*
                        $orderCreate->status = 'PAYMENT_FAILED';
                        $orderCreate->save();
                        return (object)[
                            'orderId'=>$orderCreate->orderId,
                            'status'=>1, //todo
                            'message'=>$e->getMessage()
                        ];
                         *
                         */
                    }
                }
            }catch (\Exception $e) {
                throw new Exception("Could not fetch payment.", 0, $e);
            }
            try{
                $message =  $this->formatMessage($orderCreate);
                $res1 = '';
                ob_start();
                $this->sendMessage($message['message'],$message['outlet']->mobile);
                $res1 = ob_get_clean();

                Log::channel('dailyInfo')->info('--sms sent to outlet:'.$outlet->outletName.', message response:'.$res1);
                Log::channel('dailyInfo')->info('customer mobiles:'.$fields->customer->mobile.', message response:'.$res1);
                Log::channel('dailyInfo')->info('sms response:'.$res1.'--');
                if($discount!=null){
                    $message =  $this->formatMessageCustomer($orderCreate,$discount,$voucher,$walletCash);
                            }else{
                    $message =  $this->formatMessage($orderCreate);
                }
                Log::channel('dailyInfo')->info('--sms sent to customer mobiles:'.$fields->customer->mobile);
                $res2 = '';
                ob_start();
                $this->sendMessage($message['message'],$fields->customer->mobile);
                $res2 = ob_get_clean();
                $res1 .= $res2;
                Log::channel('dailyInfo')->info('message response:'.$res2.'--');

            }catch (\Exception $e) {
                throw new Exception("Could not send voice message.".(isset($res1)?$res1:''), 0, $e);
            }

            try{

                $this->passEmail($order, $fields);
                Log::channel('dailyInfo')->info('email sent to customer::'.$order['customerEmail'].', outlet email:'.\App\User::find($outlet->user_id)->email);

            } catch (\Exception $e) {
                throw new Exception("Could not send email.", 0, $e);
            }
            try{
                $message1 = 'Hi ! You received a new order from o el f store for train no. '.implode(' ',str_split($fields->trainNo)).' customer name '.$fields->customer->fullName.' Thank You';
                $mobiles = explode(",",$outlet->user->mobile);
                foreach ($mobiles as $value) {
                    $res2 = '';
                    ob_start();
                    $this->sendVoiceMessage($message1, $value);
                    $res2 = ob_get_clean();

                    Log::channel('dailyInfo')->info('--voice message sent to outlet mobile:'.$value);
                    Log::channel('dailyInfo')->info('voice message response:'.$res2.'--');
                }
            }catch (\Exception $e) {
                throw new Exception("Could not send voice message.".(isset($res1)&&isset($res2)?$res1.'--end'.res2:''), 0, $e);
            }

            $result = (object)[
                    'orderId'=>$orderCreate->id,
                    'status'=>0, //todo
                    'message'=>$message
            ];

            return $result;
	}

    public function saveDeliveryBoy($orderId, $deliveryBoyId, $vendorOrder){
        if($vendorOrder=='true'){
            $orderWhere = \App\Models\VendorOrder::where('orderId', $orderId);
            $update = $orderWhere->update(['deliveryBoy' => $deliveryBoyId]);
            $order = \App\Models\VendorOrder::where('orderId',$orderId)->first();

            $deliveryDetails = new \stdClass;
            $deliveryDetails->pnr = $order->pnr;
            $deliveryDetails->trainNo = $order->trainNo;
            $deliveryDetails->trainName = $order->trainName;
            $deliveryDetails->station = $order->stationName;
            $deliveryDetails->stationCode = $order->stationCode;
            $deliveryDetails->berth = $order->berth;
            $deliveryDetails->coach = $order->coach;
            $deliveryDetails->ETA = $order->deliveryDate;
            $order->deliveryDetails = json_encode($deliveryDetails);

            $order->customerDetails = json_decode($order->customer);
            $order->customerDetails->customerName = $order->customerDetails->fullName;
            $order->customerDetails = json_encode($order->customerDetails);

            $priceDetails = new \stdClass;
            $priceDetails->totalAmount = $order->totalAmount;
            $order->priceDetails = json_encode($priceDetails);

            $order->outletDetails = \App\Models\Outlet::find($order->outlet);
            $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
            if($order->deliveryBoy!=null){
                $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                $order->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
            }else
                $order->deliveryBoyName = '';

            $order->vendorOrder = true;
        }else if($vendorOrder=='false'){
            $orderWhere = \App\Models\Order::where('orderId', $orderId);
            $update = $orderWhere->update(['deliveryBoy' => $deliveryBoyId]);
            $order = \App\Models\Order::where('orderId',$orderId)->first();
            $deliveryDetails = json_decode($order->deliveryDetails);
            $order->outletDetails = \App\Models\Outlet::find($order->outlet_id);
            $order->outletDetails->deliveryBoys1 = $order->outletDetails->deliveryBoys;
            if($order->deliveryBoy!=null){
                $deliveryBoy = \App\Models\DeliveryBoy::find($order->deliveryBoy);
                $order->deliveryBoyName = $deliveryBoy->first_name.' '.$deliveryBoy->last_name;
            }else
                $order->deliveryBoyName = '';
            $order->vendorOrder = false;
        }
        $order->aggregatorDetails = json_decode($order->aggregatorDetails);
        $order->customerDetails = json_decode($order->customerDetails);
        $order->priceDetails = json_decode($order->priceDetails);
        $order->orderItems = json_decode($order->orderItems);
        $this->sendMessage('Hi Delivery person '.$order->deliveryBoyName.' is on the way to deliver your order. Kindly be on your seat, in case of any query please feel free to contact him on '.$deliveryBoy->mobile.' Regards https://www.olfstores.com',$order->customerDetails->mobile);
        if($update){
            return response()->json($order,200);
        }
        return response()->json(['status'=>'failure','message'=>'Updation failed'],200);

    }
    public function getDeliveryBoys($outletId){
        $deliveryBoys = \App\Models\Outlet::find($outletId)->deliveryBoys;
        return response()->json($deliveryBoys,200);
    }
    private function formatMessageCustomer($data,$discount,$voucher,$walletCash){
		$aggrigator = 'Olf Store'; //todo
		//$delivery = $data->deliveryDate; //todo
		$p = $data->totalAmount;
		$customer = json_decode($data->customer);
		$orderId =  $data->orderId;
		$items = json_decode($data->orderItems);
		$booking = $data->bookingDate;
		$deliveryDate = substr($data->deliveryDate,0,10);
		$userId = \App\Models\Outlet::where('id',$data->outlet)->first()->user_id;
		//dd($userId);
		$outlet = \App\Models\User::find($userId);
		$message = "Order id:{$orderId} For: {$data->trainNo}/{$data->trainName} coach {$data->coach}-{$data->berth} at {$data->stationName} to {$customer->fullName} {$customer->mobile} on {$deliveryDate} \n";
		$oitems = array_map(function($item){
                                return "{$item->quantity} -{$item->itemName}";
			},$items);

		$message.= "ITEMS:".join($oitems);

		$message .= " ,Amount: {$data['amountPayable']}-{$data['paymentType']}";
		$message .= " ,Use coupon code $voucher->code on your next order and avail discount of $discount->discount% . Additionally an amount of Rs.".$walletCash." has been deposited to your wallet. Which you can use by logging in to https://www.olfstores.com. Terms and conditions apply";
		$message .= ",OLFSTORES";
		return ['message'=>$message,'outlet'=>$outlet];
	}
        //todo
    private function formatMessage($data){
		$aggrigator = 'Olf Store'; //todo
		//$delivery = $data->deliveryDate; //todo
		$p = $data->totalAmount;
		$customer = json_decode($data->customer);
		$orderId =  $data->orderId;
		$items = json_decode($data->orderItems);
		$booking = $data->bookingDate;
		$deliveryDate = substr($data->deliveryDate,0,10);
		$userId = \App\Models\Outlet::where('id',$data->outlet)->first()->user_id;
		//dd($userId);
		$outlet = \App\Models\User::find($userId);
		$message = "Order id:{$orderId} For {$data->trainNo}/{$data->trainName} coach {$data->coach}-{$data->berth} at {$data->stationName} to {$customer->fullName} {$customer->mobile} on {$deliveryDate} ";
		$oitems = array_map(function($item){
                                return "{$item->quantity} -{$item->itemName}";
			},$items);

		$message.= "ITEMS:".join($oitems);

		$message .= " ,Amount: {$data['amountPayable']}-{$data['paymentType']}";
		$message .= ",OLFSTORES";
		return ['message'=>$message,'outlet'=>$outlet];
	}

	public function testMessage(Request $request){
	    Mail::raw('Text to e-mail', function ($message) {
                 $message->from('crec_ajay@rediffmail.com', 'ajay');
                $message->to('ajay.kumar.khatri24@gmail.com');
            });
	}

	public function testMessage1(Request $request){
	    $this->sendMessage("test","9945040843");
	}

	private function sendMessage($string, $numbers){
            $key = "191164Aup6j2p05a4cb419";
            $url = sprintf('https://api.msg91.com/api/sendhttp.php?authkey=%s&mobiles=%s&route=%s&sender=%s&message=%s&country=%s',$key,$numbers,'4',urlencode('OLFSTR'),urlencode($string),'91');

            /*
	    $nums = explode(",",$numbers);
		$data = (object)[
			'sender'=>'OLFSTR',
			'route'=>4,
			'country'=>91,
			'sms'=>[
			   (object)['message'=>$string,'to'=>$nums]
			]

		];
                */
		//dd(json_encode($data));
		$curl = curl_init();

		curl_setopt_array($curl, array(
					CURLOPT_URL => "http://api.msg91.com/api/v2/sendsms",
                    			CURLOPT_URL => $url,
                    			CURLOPT_RETURNTRANSFER => true,
					CURLOPT_ENCODING => "",
					CURLOPT_MAXREDIRS => 10,
					CURLOPT_TIMEOUT => 30,
					CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
					//CURLOPT_CUSTOMREQUEST => "POST",
                                        CURLOPT_CUSTOMREQUEST => "GET",
					//CURLOPT_POSTFIELDS => json_encode($data),
					CURLOPT_SSL_VERIFYHOST => 0,
					CURLOPT_SSL_VERIFYPEER => 0,
                    /*
					CURLOPT_HTTPHEADER => array(
						"content-type: application/json"
						),*/
					));

		$response = curl_exec($curl);
		$err = curl_error($curl);

		curl_close($curl);

		/*if ($err) {
			echo "cURL Error #:" . $err;
		} else {
			echo $response;
		}*/
		print_r($response);
	}
        private function sendStatusMessage($data, $message){
		$orderId =  $data->orderId;
                if($data->vendorOrder)
                    $outletId = $data->outlet;
                else
                    $outletId = $data->outlet_id;
		$userId = \App\Models\Outlet::where('id',$outletId)->first()->user_id;
		//dd($userId);
		$user = \App\Models\User::find($userId);
                $message = sprintf($message,$orderId);
		$message .= ",OLFSTORES";
                //echo $message;
                $this->sendMessage($message,$user->mobile);

	}
	private function sendVoiceMessage($string, $number){

		$data = http_build_query(
                            array(
                                'from'=>'919522996999',
                                'message'=>$string,
                                'authkey'=>'191164Aup6j2p05a4cb419',
                                'to'=>'91'.$number,
                            ), '&');

		//dd($data);
		$curl = curl_init();

		curl_setopt_array($curl, array(
					CURLOPT_URL => "http://control.msg91.com/api/sendVoiceCall.php",
					CURLOPT_RETURNTRANSFER => true,
					CURLOPT_ENCODING => "",
					CURLOPT_MAXREDIRS => 10,
					CURLOPT_TIMEOUT => 30,
					CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
					CURLOPT_CUSTOMREQUEST => "POST",
					CURLOPT_POSTFIELDS => $data,
					CURLOPT_SSL_VERIFYHOST => 0,
					CURLOPT_SSL_VERIFYPEER => 0
					));

		$response = curl_exec($curl);
		$err = curl_error($curl);

		curl_close($curl);

		/*if ($err) {
			echo "cURL Error #:" . $err;
		} else {
			echo $response;
		}*/

		print_r($response);
	}

	private function passEmail($data, $fields){
        $delivery = (object)array(
            'pnr'=>$data['pnr'],
            'trainNo'=>$data['trainNo'],
            'trainName'=>$data['trainName'],
            'station'=>$data['stationName'],
            'stationCode'=>$data['stationCode'],
            'berth'=>$data['berth'],
            'coach'=>$data['coach'],
            'ETA'=>$data['deliveryDate'],//todo
                );
        $items = json_decode($data['orderItems']);
        $customer = json_decode($data['customer']);
        //$customerEmail = $customer['email'];
        //$customerName = $customer['fullName'];
        $discountAmount = 0;
        if(array_key_exists('discountAmount',$data))
            $discountAmount = $data['discountAmount'];
        $pricing = (object)array(
            'totalAmount'=>$data['totalAmount'],
            'gst'=>$data['gst'],
            'amountPayable'=>$data['amountPayable'],
            'discountAmount'=>$discountAmount
                ); //todo
        $customer->customerName = $customer->fullName;
        //$items = json_decode($data['orderItems']);
        $orderId = $data['orderId'];
        $bookingDate = $data['bookingDate'];

        $deliveryDate = $data['deliveryDate'];
        $paymentType = $data['paymentType'];
        $outlet= \App\Models\Outlet::find($data['outlet']);

		Mail::send('mail.emailVendor',
			  compact('delivery','items','customer','outlet','delivery','orderId','bookingDate','deliveryDate','paymentType','pricing'),
			   function ($message) use($outlet,$customer,$orderId){
				    $y = \App\Models\User::find($outlet->user_id);
					$message->from('noreply@olfstores.com', 'OLF STORE');
					$message->sender('noreply@olfstores.com','OLF STORE');
					$message->to($y->email, $outlet->outletName);
					$message->cc($customer->email, $customer->customerName);
					$message->bcc('sanjayjdn@gmail.com','Sanjay Jadon');
					$message->subject('ORDER ID:'.$orderId);
					$message->priority(3);
				});

    }
	public function test12(){
            Log::channel('dailyInfo')->info('--------------Testing----------------------');
            /*
            $order1 = \App\Order::where('orderId', 9175150);

            $o = $order1->first();

            Config::set('vouchers.relation_table','order_voucher');
            Config::set('vouchers.user_model',\App\Order::class);

            $discount = \App\Discount::
                        where(function ($q){
                                    $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                                            ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString());
                        })->first();
            if($discount!=null){
                $voucher = $o->createVouchers(1, [
                    'discount_id' => $discount->id,
                ])[0];
                echo $voucher->code;
                $o->update(['discount_code' => $voucher->code]);
            }
             *
             $code = 'EU3L-2FKV';
             $mobile = '9945040843';
             $fields = new \stdClass;
             $fields->amountPayable = 145;
             $fields->coupon = new \stdClass;
             $fields->coupon->value = $code;
             //print_r(Voucher::whereCode($code)->first());
            //$this->sendMessage("Order For: 11123/Barauni coach s4-28 at LUCKNOW to ajay 9945040843 on 2019-06-04 ITEMS:1 -Veg Mini Thali ,Amount: 145-CASH_ON_DELIVERY ,Use coupon code DNNL-T2GY on your next order and avail discount of 25% for order amount (0-1000),OLFSTORES", "9945040843");
            try{
                $o = \App\Order::where('customerMobile', $mobile)
                        ->where('discount_code', $code)
                        ->first();
                $vo = \App\VendorOrder::where('customerMobile', $mobile)
                        ->where('discount_code', $code)
                        ->first();
            if($o!=null){
                $vo = $o;
                Config::set('vouchers.relation_table','order_voucher');
                Config::set('vouchers.user_model',\App\Order::class);
            }
            	echo $vo->canRedeemCode($code);
            	if($vo->canRedeemCode($code)){
            		//try{
						$v2 = $vo->check($code);
						if($v2->data->get('amountFrom')<=$fields->amountPayable && $v2->data->get('amountTo')>=$fields->amountPayable){
							$v1 = $vo->redeemCode($fields->coupon->value);
							echo 'redeemed';
							$d = $v1->data->get('discount');
							$fields->couponCode = $v1->code;
							$fields->discountAmount = round($fields->amountPayable*$d/100);
							$fields->amountPayable = $fields->amountPayable-$fields->discountAmount;
						}
					//}catch (\Exception $e) {
						//voucher doesn't exists or couldn't be redeemed
						//echo $e->getMessage();
					//}

				}
            }finally {
                    Config::set('vouchers.relation_table','vendor_order_voucher');
                    Config::set('vouchers.user_model',\App\VendorOrder::class);
          }
*/
            //print(\App\Outlet::find(3)->deliveryBoys);

        }
   public function generateOTP(){
        $otp = mt_rand(100000,999999);
        return $otp;
    }

    public function sendOtp(Request $request) {
        if (!$request->has('mobile')) {
            return json_encode(array('status' => 'failure', 'msg' => "input parameters missing"));
        } else {
            //Auth::logout();
            //put in session
            $mobile = $request->input('mobile');
            $otp = $this->generateOTP();
            $message = 'you otp is '.$otp;
            ob_start();
            $this->sendMessage($message,$mobile);
            $res = ob_get_clean();
            session(['mobile'=> $mobile]);
            session(['otp' => $otp]);
            if($request->has('mobile')){
                $vo = \App\Models\VendorOrder::latest($mobile);
            }
            if ($vo != null) {
                $vo->password = \Illuminate\Support\Facades\Hash::make($otp);
                $vo->save();
            }else{
                $memoryUser = \App\Models\MemoryUser::where('mobile',$mobile)->first();
                if($memoryUser==null)
                    \App\Models\MemoryUser::create(['password'=>\Illuminate\Support\Facades\Hash::make($otp),'mobile'=>$mobile,'memory'=>1]);
                else{
                    $memoryUser->password = \Illuminate\Support\Facades\Hash::make($otp);
                    $memoryUser->save();
                }
            }
            return json_encode(array('status' => 'success', 'message' => 'otp sent successfully'));
        }
    }

    public function sendOtp1(Request $request) {
        if (!$request->has('mobile')) {
            return json_encode(array('status' => 'failure', 'msg' => "Mobile number not valid" . $mobile));
        } else {
            //put in session
            $mobile = $request->input('mobile');
            $otp = $this->generateOTP();
            $message = 'your otp is ' . $otp;
            ob_start();
            //$this->sendMessage($message,$mobile);
            $res = ob_get_clean();
            session(['mobile'=> $mobile]);
            session(['otp' => $otp]);
            if($request->has('mobile')){
                $controller = app()->make('\App\Http\Controllers\Auth\OrderLoginController');
                $orderUser = $controller->getUser($mobile);
                $vo = $orderUser['user'];
            }
            if($vo!=null){
                $vo->password = \Illuminate\Support\Facades\Hash::make($otp);
                $vo->save();
            }else{
                $memoryUser = \App\Models\MemoryUser::where('mobile',$mobile)->first();
                if($memoryUser==null)
                    \App\Models\MemoryUser::create(['password'=>\Illuminate\Support\Facades\Hash::make($otp),'mobile'=>$mobile,'memory'=>1]);
                else{
                    $memoryUser->password = \Illuminate\Support\Facades\Hash::make($otp);
                    $memoryUser->save();
                }
            }
            return json_encode(array('status' => 'success', 'message' => 'otp sent successfully'));
        }
    }
    public function testing(Request $request) {

        Log::channel('dailyInfo')->info('here'.print_r($request->user(),true));
    }
    public function wallet(Request $request) {
        $user = Auth::guard('vendororder')->user();
        $user1 = Auth::guard('order')->user();
        $user2 = Auth::guard('memoryuser')->user();
        $user3 = Auth::guard('apiorder')->user();

        if ($user != null) {
            if ($user1 != null && $user1->created_at->gt($user->created_at))
                $user = $user1;
        }else {
            $user = $user1;
        }

        if ($user == null){
            if ($user2 != null){
                /*
                $order = \App\Order::latest($user2->mobile);
                if($order==null)
                    $balance = 0;
                else
                    $balance = $order->balance;
                 *
                 */
                return json_encode(array('status' => 'error', 'balance' => $user2->balance));
            }else{
                if(!$user3)
                    return json_encode(array('status' => 'error', 'balance' => 'Login to proceed'));
                else
                    $user = $user3;
            }

        }
        if($request->has("payable"))
            return json_encode(array('status' => 'success', 'orderId'=>$user->orderId, 'balance' => $user->last()->balance, 'discount' => $user->last()->getAmountProduct1((int)$request->payable)));

        //Log::channel('dailyInfo')->info('--getting:previous'.print_r($user->first(),true));
        //Log::channel('dailyInfo')->info('--getting:last'.print_r($user->last(),true));
        //$user->setRelation('wallet',$user->wallet);
        return json_encode(array('status' => 'success', 'orderId'=>$user->orderId, 'balance' => $user->last()->balance));
    }
    public function mobile() {

        $user = Auth::guard('vendororder')->user();
        $user1 = Auth::guard('order')->user();
        $user2 = Auth::guard('memoryuser')->user();
        $user3 = Auth::guard('apiorder')->user();
        if ($user != null) {
            if ($user1 != null && $user1->created_at->gt($user->created_at))
                $user = $user1;
        }else {
            $user = $user1;
        }

        if ($user == null){
            if ($user2 != null){
                /*
                $order = \App\Order::latest($user2->mobile);
                if($order==null)
                    $balance = 0;
                else
                    $balance = $order->balance;
                 *
                 */
                $user = $user2;
                $mobile = $user->mobile;
            }else{
                if(!$user3)
                    return json_encode(array('status' => 'error', 'mobile' => 'Login to proceed'));
                else
                    $user = $user3;
            }

        }else
            $mobile = $user->customerMobile;


        return json_encode(array('status' => 'success', 'mobile'=>$mobile));
    }
    public function getPreviousOrders(Request $request) {
        $user = Auth::guard('vendororder')->user();
        $user1 = Auth::guard('order')->user();
        $user2 = Auth::guard('memoryuser')->user();
        $user3 = Auth::guard('apiorder')->user();

        if ($user != null) {
            if ($user1 != null && $user1->created_at->gt($user->created_at))
                $user = $user1;
        }else {
            $user = $user1;
        }

        if ($user == null){
            if ($user2 != null){
                return json_encode(array('status' => 'error', 'orders' => []));
            }else{
                if(!$user3)
                    return json_encode(array('status' => 'error', 'balance' => 'Login to proceed'));
                else
                    $user = $user3;
            }

        }
        $orders = [];
        return json_encode(array('status' => 'success', 'orders' => $user->last()->getPreviousOrders($orders)));

    }
    public function createOrderEvent(Request $request) {
        $token = \App\Models\Token::where('access_token', hash('sha256', $request->api_token))->first();
        event(new \App\Events\OrderCreatedFirebaseEvent([
            'type'=>"USER",
            'token'=>$token->firebase_token,
            'message'=>[
                'title'=>'Order Recieved',
                'content'=>"Order placed with #{123}"
            ]
        ]));

    }
}


