<?php

namespace App\Http\Controllers;

use App\Models\IRCTCDataManager;
use App\Models\Outlet;
use DB;
use File;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Http\Request;
use Mail;
use App\Models\User;
use Illuminate\Support\Facades\Log;

class OutletController extends Controller
{
    use AuthorizesRequests;

    public function __construct()
    {
        $this->service = new IRCTCDataManager(config('constkeys.irctc_token'), config('constkeys.irctc_base_url'));
    }

    public function refreshInventory(Request $request, $io_type)
    {  
        $allowed_methods = ['station', 'menu'];
        $response = null;
        if (!$request->has('id')) {
            dd("Id is Required");
        }

        if (in_array($io_type, $allowed_methods)) {

            switch ($io_type) {

                case 'station':

                    $this->service->processStation($request->query('id'));
                    break;
                case 'menu':

                    $this->service->processMenuItem1($request->query('id'));
                    break;

                default:
                    break;

            }

        }

        echo "Success";
    }

    public function GetMultipleOutet(Request $request)
    {
        $gateAllAoutlet = DB::table('outlets')->select('id')->where('status', 1)->get();
        foreach ($gateAllAoutlet as $value) {
            $url = 'https://olf.invoidea.app/jobs/menu/index?id=' . $value->id;
            $curl = curl_init($url);
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            $resp = curl_exec($curl);
            curl_close($curl);
            echo "Success";
            echo "<br>";
        }

    }

    public function files(Request $request, $name)
    {
        $path = public_path() . '/upload/' . $name;
        $image = file_get_contents($path);
        header('Content-type: image/jpeg;');
        header("Content-Length: " . strlen($image));
        echo $image;
    }

    public function getLog(Request $request)
    {
        if (!$request->has('date')) {
            $date = \Carbon\Carbon::now()->format('Y-m-d');
        } else {
            $date = $request->input('date');
        }
        $myFile = getcwd() . "/../storage/logs/orderinfo-" . $date . ".log";
        $myFileContents = array();
        //return;
        try {
            $handle = fopen($myFile, "r");
            while (($line = fgets($handle)) !== false) {
                $myFileContents[] = $line;
            }
            fclose($handle);
            /*
        $fh = fopen($myFile, 'r');
        $myFileContents = fread($fh,filesize($myFile));
        fclose($fh);*/
        } catch (\Exception $e) {

            $myFileContents[] = 'log file doesn\'t exists';
            //throw $e;
        }
        /*
        $myFileContents = '<link href="txtstyle.css" rel="stylesheet" type="text/css" />'.$myFileContents;
        $fp = fopen(getcwd().'/../storage/exports/index.html', 'w');
        fwrite($fp, $myFileContents);
        fclose($fp);*/
        return view('backend.admins.log', ['log' => $myFileContents, 'date' => $date]);
    }

    public function edit(Request $request, $outletId)
    {       
        $outlet=\App\Models\Outlet::find($outletId);
        $vendor=User::findOrFail($outlet->user_id);

        return view('backend.outlets.editoutlet',['outlet'=>$outlet,'vendor'=>$vendor]);
    }

    public function save(Request $request, $outletId)
    {   
        $post = $request->all();
        
        $outlet = \App\Models\Outlet::find($outletId);

        $fileKey = $request->file('fileUpload');
        if (isset($fileKey)) {
            $fileNameArray = explode(".", $fileKey->getClientOriginalName());
            $checkExt = end($fileNameArray);
            $RestrictExt = ['jpg', 'JPG', 'png', 'PNG', 'JPEG', 'jpeg'];
            if (in_array($checkExt, $RestrictExt)) {

                $destinationPath = public_path('static/img');
                $fileName = date("YmdHis") . '.' . $checkExt;
                $link = url('/static/img/' . $fileName);
                $fileKey->move($destinationPath, $fileName);
            } else {
                return redirect(url('manager/Outlet/edit/') . '/' . $outletId . '?img=notfound');
            }
            $post['logoImage'] = str_replace(' ', '', $link);
        }
        //Log::channel('dailyInfo')->info('input'.print_r($post,true));
        $post['openingTime'] = str_replace(' ', '', $post['openingTime']);
        $post['closingTime'] = str_replace(' ', '', $post['closingTime']);

        $outlet->update($post);
        session()->flash('success', 'outlet saved successfully.');

        $vendor=User::findOrFail($outlet->user_id);

        Log::channel('outletLog')->info(" Data of outlet ".$post['outletName']." Updated ");

        return view('backend.outlets.editoutlet', ['outlet' => $outlet, 'vendor'=>$vendor]);
    }

    public function deleteOutlet($outletId)
    {
        $outlet = Outlet::find($outletId);
        $dataarray = array(
            'out_id' => $outlet->id,
            'gstNo' => $outlet->gstNo,
            'fssaiNo' => $outlet->fssaiNo,
            'fssaiValidUpto' => $outlet->fssaiValidUpto,

        );
        $result = DB::table('OutletHestory')->insert($dataarray);
        $id = DB::getPdo()->lastInsertId();
        $updateArra = array(
            // 'outletId' => 0,
            // 'gstNo' => null,
            // 'fssaiNo' => null,
            // 'fssaiValidUpto' => null,
            'status' => 0,
            'is_delete' => 1,
        );
        DB::table('outlets')->where("id", $outlet->id)->update($updateArra);

        Log::channel('outletLog')->info(" Data of outlet ".$outlet->outletName." Deleted ");

        //$outlet->delete();
        session()->flash('success', 'outlet deleted successfully.');

        return redirect()->back();
    }

    

    public function listVendors(Request $request, $outletId)
    {
        $outlet = \App\Models\Outlet::find($outletId);
        $users = \App\Models\User::where("role", "vendor")->get();
        return view('backend.outlets.setUser', ['outletId' => $outletId, 'users' => $users, "user_id" => $outlet->user_id, "message" => ""]);
    }
    public function setuser(Request $request, $outletId)
    {
        $outlet = \App\Models\Outlet::find($outletId);
        $outlet->user_id = $request->input("user_id");
        $outlet->save();
        $users = \App\Models\User::where("role", "vendor")->get();

        Log::channel('outletLog')->info("Vendor set to outlet ".$outlet->outletName." successfully ");        

        return view('backend.outlets.setUser', ['outletId' => $outletId, "user_id" => $outlet->user_id, 'users' => $users, "message" => "user set succesfully"]);
    }
    public function setPaymentMode(Request $request, $outletId)
    {
        if (!isset($outletId) || !$request->filled('payment_mode')) {
            session()->flash('error', 'Request input parameters are missing.');
            return redirect()->back();
        }
        $outlet = \App\Models\Outlet::find($outletId);
        $outlet->payment_mode = $request->input('payment_mode');
        $outlet->save();
        session()->flash('success', 'Payment mode set successfully for ' . $outlet->outletName);
        return redirect()->back();
    }

    public function closePermanently(Request $request)
    {
        $input = $request->all();
        $outletId = $input['outletId'];
        $outlet = \App\Models\Outlet::find($outletId);
        $outlet->active = $input['active'];
        $outlet->save();

        return back();

    }

    private function calculateTax($basePrice)
    {
        $data = round(floatval($basePrice) * 0.05, 2);
        return $data;
    }
    private function calculateSellingPrice($basePrice, $tax)
    {
        $data = round(floatval($basePrice + $tax), 2);
        return $data;
    }

    public function listMenuWithItems(Request $request, $id)
    {
        //$this->authorize('pol_view_menu', Outlet::find($id));
        $out = \App\Models\Outlet::find($id);
        if (!isset($out->menu)) {
            $result = (object) [
                'menuItems' => array(),
            ];
            return response(json_encode($result), 200);
        }
        $menu = $out->menu;
        $closed = $this->isVegNonvegClosed($out);

        $items = $out->items->filter(function ($item) use ($out, $closed) {
            if ($closed) {
                return !$this->isItemClosed($out, $item);
            }

            return true;

        })->values()->map(function ($item) {

            $item->tags = explode(',', $item->tags);
            $item->basePrice = (int) $item->basePrice;
            $item->tax = $this->calculateTax(floatval($item->basePrice));
            $item->sellingPrice = $this->calculateSellingPrice(floatval($item->basePrice), $item->tax);
            $item->isVegeterian = (boolean) $item->isVegeterian == "1";
            $item->isNonVegeterian = (boolean) $item->isNonVegeterian == "1";
            $item->foodType = strtoupper($item->foodType);
            if (!isset($item->image)) {
                $item->image = 'http:\/\/olfstores.com\/trains\/public\/files\/foodIcon.png';
            }

            $item1 = (object) [
                'itemId' => (int) $item->id,
                'itemName' => $item->itemName,
                'basePrice' => $item->basePrice,
                'description' => $item->description,
                'openingTime' => $item->openingTime,
                'closingTime' => $item->closingTime,
                'tax' => $item->tax,
                'sellingPrice' => $item->sellingPrice,
                'isVegeterian' => $item->isVegeterian,
                'isNonVegeterian' => $item->isNonVegeterian,
                'tags' => $item->tags,
                'image' => $item->image,
                'cuisine' => $item->cuisine,
                'foodType' => $item->foodType,
                'customisations' => null,
            ];

            return $item1;
        });
        $data1 = [];
        foreach ($items as $item) {
            $data1[] = $item;
        }
        return response()->json(array('menuItems' => $data1), 200);
    }

    public function isItemClosed($outlet, $item)
    {
        if ($outlet->veg_non_veg != 2) {
            if ($outlet->veg_non_veg == 0) {
                if ($item->isNonVegeterian == 1) {
                    return true;
                }
// $item->visibility == 1 ? true : false;
            } else {
                if ($item->isVegeterian == 1) {
                    return true;
                }
//$item->visibility == 1 ? true : false;
            }

        }

        return false;
        // return $item->visibility == 1 ? true : false;
    }
    public function isVegNonvegClosed($outlet)
    {
        if ($outlet->closingPeriod_veg_non_veg == null) {
            $closingPeriods = json_decode('[]');
        } else {
            $closingPeriods = json_decode($outlet->closingPeriod_veg_non_veg);
        }

        $outlet->veg_non_veg_closed = false;
        $now = \Carbon\Carbon::now();
        foreach ($closingPeriods as $closingPeriod) {
            if ($now->greaterThanOrEqualTo(\Carbon\Carbon::createFromFormat('Y-m-d', $closingPeriod->closedFrom_veg_non_veg)->startOfDay())
                && $now->lessThanOrEqualTo((\Carbon\Carbon::createFromFormat('Y-m-d', $closingPeriod->closedTo_veg_non_veg)->endOfDay()))) {
                $outlet->veg_non_veg_closed = true;
                break;
            }
        }
        if (!$outlet->veg_non_veg_closed) {
            $weekMap = [
                0 => 'SUN',
                1 => 'MON',
                2 => 'TUE',
                3 => 'WED',
                4 => 'THU',
                5 => 'FRI',
                6 => 'SAT',
            ];
            $now = \Carbon\Carbon::now();
            if ($outlet->weeklyClosed_veg_non_veg == '' || $outlet->weeklyClosed_veg_non_veg == null) {
                $weekly = json_decode(json_encode([]));
            } else {
                $weekly = json_decode($outlet->weeklyClosed_veg_non_veg);
            }

            array_map(function ($w) use ($outlet, $now, $weekMap) {
                if ($weekMap[$now->dayOfWeek] == $w) {
                    $outlet->veg_non_veg_closed = true;
                }

            }, $weekly);
        }
        $closed = $outlet->veg_non_veg_closed;
        unset($outlet->veg_non_veg_closed);
        return $closed;
    }

    public function createMenu(Request $request, $outletId)
    {
        $this->authorize('pol_add_menu', Outlet::find($outletId));

        $outlet = \App\Models\Outlet::find($outletId);
        if ($outlet) {
            $outlet->menu()->create($request->all());
            $data = (object) ['message' => 'Menu Created successfully'];
            return response()->json($data, 200);

        } else {
            $data = (object) ['message' => 'No Outlets Found'];
            return response()->json($data, 200);

        }
    }

    public function updateMenu(Request $request, $outletId, $id)
    {
        $outlet = \App\Models\Outlet::find($outletId);
        $this->authorize('pol_update_menu', $outlet->menu()->find($id));

        if ($outlet) {
            $menu = $outlet->menu()->find($id);
            if ($menu) {
                $menu->update($request->all());
                $data = (object) ['message' => 'Menu updated successfully'];
                return response()->json($data, 200);
            } else {
                $data = (object) ['message' => 'No Menu Found'];
                return response()->json($data, 200);
            }

        } else {
            $data = (object) ['message' => 'No Outlets Found'];
            return response()->json($data, 200);
        }
    }

    public function deleteMenu(Request $request, $outletId, $id)
    {
        $outlet = \App\Models\Outlet::find($outletId);
        $this->authorize('pol_delete_menu', $outlet->menu()->find($id));
        if ($outlet) {
            $menu = $outlet->menu()->find($id);
            if ($menu) {
                $menu->delete();
                $data = (object) ['message' => 'Menu updated successfully'];
                return response()->json($data, 200);
            } else {
                $data = (object) ['message' => 'No Menu Found'];
                return response()->json($data, 200);
            }

        } else {
            $data = (object) ['message' => 'No Outlets Found'];
            return response()->json($data, 200);
        }
    }

    public function addItem(Request $request, $outletId, $id)
    {
        $outlet = \App\Models\Outlet::find($outletId);
        $this->authorize('pol_add_menu', $outlet);

        if ($outlet) {
            $menu = $outlet->menu;
            if ($menu) {
                $input = $request->all();
                $input['outlet_id'] = $outlet->id;
                $menu->items()->create($input);
                return response()->json(['message' => 'item created'], 200);
            }
        } else {
            return response()->json(['message' => 'Error'], 200);
        }

    }

    public function updateItem(Request $request, $outletId, $menuId, $id)
    {
        $outlet = \App\Models\Outlet::find($outletId);
        $this->authorize('pol_update_menu_item', $outlet->menu->items()->find($id));

        if ($outlet) {
            $menu = $outlet->menu;
            if ($menu) {
                $item = $menu->items()->find($id);
                if ($item) {
                    $item->update($request->all());
                    return response()->json(['message' => 'item updated'], 200);
                } else {
                    return response()->json(['message' => 'Error'], 200);
                }

            }
        } else {
            return response()->json(['message' => 'Error'], 200);
        }
    }

    public function deleteItem(Request $request, $outletId, $menuId, $id)
    {
        $outlet = \App\Models\Outlet::find($outletId);
        $this->authorize('pol_delete_menu_item', $outlet->menu->items()->find($id));
        if ($outlet) {
            $menu = $outlet->menu;
            if ($menu) {
                $item = $menu->items()->find($id);
                if ($item) {
                    $item->delete();
                    return response()->json(['message' => 'item deleted'], 200);
                } else {
                    return response()->json(['message' => 'Error'], 200);
                }

            }
        } else {
            return response()->json(['message' => 'Error'], 200);
        }
    }

    public function updateOutlet(Request $request)
    {

    }

    public function setClosingPeriod(Request $request)
    {

    }

    public function removeEmpty($array)
    {
        return array_filter($array);
    }

    public function updateClosing(Request $request, $outletId)
    {   
        try {

            $fields = $request->all();
            $out = \App\Models\Outlet::find($outletId);
            $now = \Carbon\Carbon::now();
            $out->open = true;
            $out->closingPeriod = json_encode([]);
            $out->weeklyClosed = json_encode([]);
            if (empty($fields['reason'])) {
                $out->reason = '';
            } else {
                $out->reason = trim($fields['reason']);
            }

            $i = 0;
            $fields['startDate1'] = array();
            $fields['endDate1'] = array();

            foreach ($fields['startDate'] as $start) {
                if (!empty($start) && !empty($fields['endDate'][$i])) {
                    $fields['startDate1'][] = $fields['startDate'][$i];
                    $fields['endDate1'][] = $fields['endDate'][$i];
                }
                $i++;
            }

            if (isset($fields['startDate1'][0]) && !empty($fields['startDate1'][0])) {
                $period = array_map(function ($s, $i) use ($fields, $now, $out) {
                    if (empty($s) || empty($s)) {
                        return;
                    }

                    $f = \Carbon\Carbon::createFromFormat("Y-m-d", $s)->startOfDay();
                    $t = \Carbon\Carbon::createFromFormat("Y-m-d", $fields['endDate1'][$i])->endOfDay();
                    if ($now->gte($f) && $now->lte($t)) {
                        $out->open = false;
                    }

                    $period = new \stdClass;
                    $period->closedFrom = $f->format('Y-m-d');
                    $period->closedTo = $t->format('Y-m-d');
                    return $period;
                }, $fields['startDate1'], array_keys($fields['startDate1']));
                $out->closingPeriod = json_encode($period); //print_r($period); die;
            }

            $weekMap = [
                0 => 'SUN',
                1 => 'MON',
                2 => 'TUE',
                3 => 'WED',
                4 => 'THU',
                5 => 'FRI',
                6 => 'SAT',
            ];

            $days1 = array();

            if (isset($fields['days'])) {
                foreach ($fields['days'] as $day) {
                    if (!empty($day)) {
                        $days1[] = $day;
                    }

                }
            }
            $fields['days'] = $days1;
            if (isset($fields['days'])) {
                array_map(function ($d) use ($out, $now, $weekMap) {
                    if ($weekMap[$now->dayOfWeek] == $d) {
                        $out->open = false;
                    }

                    return $d;
                }, $fields['days']);
                $out->weeklyClosed = json_encode($fields['days']);
            }
            $open = $out->open;
            unset($out->open);
           
            if($request->outlet_status==2){
                $out->active=0;
                $out->status=2;
            }
            else{
                $out->active=1;
                $out->status=$request->outlet_status;
            }
            $out->save();  
            $statusName="";

            if($request->outlet_status==0)
            $statusName="Inactive";
            elseif($request->outlet_status==1)
            $statusName="Active";
            else
            $statusName="Close Permanent";

            Log::channel('outletLog')->info("Outlet ".$out->outletName ." Status Updated To ".$statusName);
            
            return response()->json(['status' => 'success', "outletId" => $out->id, "open" => $open, 'reason' => $out->reason], 200);
        } catch (\Exception $e) {
            throw $e;
            return response(['status' => 'failure'], 200);
        }
    }

    public function updateClosingVegNonveg(Request $request, $outletId)
    {
        try {
            $fields = $request->all();
            $out = \App\Models\Outlet::find($outletId);
            $now = \Carbon\Carbon::now();
            $out->veg_non_veg = 2;
            $out->veg_non_vegValue = true;
            $out->closingPeriod_veg_non_veg = json_encode([]);
            $out->weeklyClosed_veg_non_veg = json_encode([]);
            if (empty($fields['reason'])) {
                $out->reason_veg_non_veg = '';
            } else {
                $out->reason_veg_non_veg = trim($fields['reason']);
            }

            $i = 0;
            $fields['startDate1'] = array();
            $fields['endDate1'] = array();
            foreach ($fields['startDate'] as $start) {
                if (!empty($start) && !empty($fields['endDate'][$i])) {
                    $fields['startDate1'][] = $fields['startDate'][$i];
                    $fields['endDate1'][] = $fields['endDate'][$i];
                }
                $i++;
            }
            if (isset($fields['startDate1'][0]) && !empty($fields['startDate1'][0])) {
                $period = array_map(function ($s, $i) use ($fields, $now, $out) {
                    if (empty($s) || empty($s)) {
                        return;
                    }

                    $f = \Carbon\Carbon::createFromFormat("Y-m-d", $s)->startOfDay();
                    $t = \Carbon\Carbon::createFromFormat("Y-m-d", $fields['endDate1'][$i])->endOfDay();
                    if ($now->gte($f) && $now->lte($t)) {
                        $out->veg_non_vegValue = false;
                    }

                    $period = new \stdClass;
                    $period->closedFrom_veg_non_veg = $f->format('Y-m-d');
                    $period->closedTo_veg_non_veg = $t->format('Y-m-d');
                    return $period;
                }, $fields['startDate1'], array_keys($fields['startDate1']));
                $out->closingPeriod_veg_non_veg = json_encode($period);
            }
            $weekMap = [
                0 => 'SUN',
                1 => 'MON',
                2 => 'TUE',
                3 => 'WED',
                4 => 'THU',
                5 => 'FRI',
                6 => 'SAT',
            ];
            $days1 = array();
            foreach ($fields['days'] as $day) {
                if (!empty($day)) {
                    $days1[] = $day;
                }

            }
            $fields['days'] = $days1;
            if (isset($fields['days'])) {
                array_map(function ($d) use ($out, $now, $weekMap) {
                    if ($weekMap[$now->dayOfWeek] == $d) {
                        $out->veg_non_vegValue = false;
                    }

                    return $d;
                }, $fields['days']);
                $out->weeklyClosed_veg_non_veg = json_encode($fields['days']);
            }

            if (!$out->veg_non_vegValue) {
                if ($fields['non_veg_only'] == '0') {
                    $out->veg_non_veg = 0;
                } else {
                    $out->veg_non_veg = 1;
                }

            } else {
                $out->veg_non_veg = 2;
            }
            $veg_non_vegValue = $out->veg_non_vegValue;
            unset($out->veg_non_vegValue);
            $out->save();

            return response()->json(['status' => 'success', "outletId" => $out->id, 'test' => $veg_non_vegValue, "veg_non_veg" => $out->veg_non_veg, 'reason' => $out->reason_veg_non_veg], 200);
        } catch (\Exception $e) {
            throw $e;
            return response(['status' => 'failure'], 200);
        }
    }
    public function getOrderStats(Request $request)
    {
        $orders = array();
        $outletId = null;
        $range = null;
        if ($request->has('outletId')) {
            $outletId = $request->input('outletId');
        }

        if ($request->has('range')) {
            $range = $request->input('range');
            switch ($range) {
                case 0:
                    $start = \Carbon\Carbon::now()->startOfDay();
                    $end = $start->copy()->addDay();
                    break;
                case 1:
                    $start = \Carbon\Carbon::now()->startOfDay();
                    $end = $start->copy()->addDays(7);
                    break;
                case 2:
                    $start = \Carbon\Carbon::now()->startOfDay();
                    $end = $start->copy()->endOfMonth()->startOfDay()->addDay();
                    break;
            }
        }
        if ($range == null) {
            if ($request->filled('start')) {
                $start = \Carbon\Carbon::createFromFormat('Y-m-d', $request->input('start'), 'Asia/Kolkata')->startOfDay();
            }
            if ($request->filled('end')) {
                $end = \Carbon\Carbon::createFromFormat('Y-m-d', $request->input('end'), 'Asia/Kolkata')->startOfDay()->addDay();
            }
        }
        if (!isset($start) || !isset($end) || $start == null || $end == null) {
            $start = \Carbon\Carbon::now()->startOfDay();
            $end = $start->copy()->addDay()->startOfDay();
        }
        $orders[] = \App\Models\Order::
            where(function ($q) use ($start, $end) {
            $q->where('deliveryDateTime', '>=', $start)
                ->where('deliveryDateTime', '<', $end);
        })
            ->where(function ($q) use ($outletId) {
                if ($outletId != null) {
                    $q->where('outlet_id', $outletId);
                }

            })->count();
        $orders[0] += \App\Models\VendorOrder::
            where(function ($q) use ($start, $end) {
            $q->where('deliveryDateTime', '>=', $start)
                ->where('deliveryDateTime', '<', $end);
        })
            ->where(function ($q) use ($outletId) {
                if ($outletId != null) {
                    $q->where('outlet', $outletId);
                }

            })->count();
        if ($outletId == null) {
            $orders[] = DB::table('orders')
                ->where(function ($q) use ($outletId) {
                    if ($outletId != null) {
                        $q->where('outlet_id', $outletId);
                    }

                })->where(function ($q) use ($start, $end) {
                $q->where('deliveryDateTime', '>=', $start)
                    ->where('deliveryDateTime', '<', $end);
            })->sum('orders.vendorPayment');

            $orders[1] += DB::table('vendor_orders')
                ->where(function ($q) use ($outletId) {
                    if ($outletId != null) {
                        $q->where('outlet', $outletId);
                    }

                })->where(function ($q) use ($start, $end) {
                $q->where('deliveryDateTime', '>=', $start)
                    ->where('deliveryDateTime', '<', $end);
            })->sum('vendor_orders.vendorPayment');
        } else {
            $orders[] = DB::table('orders')
                ->select(DB::raw('sum(orders.amountPayable-orders.vendorPayment) AS earnings'))
                ->where(function ($q) use ($outletId) {
                    if ($outletId != null) {
                        $q->where('outlet_id', $outletId);
                    }

                })->where(function ($q) use ($start, $end) {
                $q->where('deliveryDateTime', '>=', $start)
                    ->where('deliveryDateTime', '<', $end);

            })->first()->earnings;

            $orders[1] += DB::table('vendor_orders')
                ->select(DB::raw('sum(vendor_orders.amountPayable-vendor_orders.vendorPayment) AS earnings'))
                ->where(function ($q) use ($outletId) {
                    if ($outletId != null) {
                        $q->where('outlet', $outletId);
                    }

                })->where(function ($q) use ($start, $end) {
                $q->where('deliveryDateTime', '>=', $start)
                    ->where('deliveryDateTime', '<', $end);

            })->first()->earnings;

        }
        $orders[] = \App\Models\Order::
            where(function ($q) use ($start, $end) {
            $q->where('deliveryDateTime', '>=', $start)
                ->where('deliveryDateTime', '<', $end);
        })
            ->where(function ($q) {
                $q->where('status', 'ORDER_DELIVERED');

            })
            ->where(function ($q) use ($outletId) {
                if ($outletId != null) {
                    $q->where('outlet_id', $outletId);
                }

            })->count();
        $orders[2] += \App\Models\VendorOrder::
            where(function ($q) use ($start, $end) {
            $q->where('deliveryDateTime', '>=', $start)
                ->where('deliveryDateTime', '<', $end);
        })
            ->where(function ($q) {
                $q->where('status', 'ORDER_DELIVERED');

            })
            ->where(function ($q) use ($outletId) {
                if ($outletId != null) {
                    $q->where('outlet', $outletId);
                }

            })->count();
        $orders[] = \App\Models\Order::
            where(function ($q) use ($start, $end) {
            $q->where('deliveryDateTime', '>=', $start)
                ->where('deliveryDateTime', '<', $end);
        })
            ->where(function ($q) {
                $q->where('status', 'ORDER_UNDELIVERED');

            })
            ->where(function ($q) use ($outletId) {
                if ($outletId != null) {
                    $q->where('outlet_id', $outletId);
                }

            })->count();
        $orders[3] += \App\Models\VendorOrder::
            where(function ($q) use ($start, $end) {
            $q->where('deliveryDateTime', '>=', $start)
                ->where('deliveryDateTime', '<', $end);
        })
            ->where(function ($q) {
                $q->where('status', 'ORDER_UNDELIVERED');

            })
            ->where(function ($q) use ($outletId) {
                if ($outletId != null) {
                    $q->where('outlet', $outletId);
                }

            })->count();
        $orders[] = \App\Models\Order::
            where(function ($q) use ($start, $end) {
            $q->where('deliveryDateTime', '>=', $start)
                ->where('deliveryDateTime', '<', $end);
        })
            ->where(function ($q) {
                $q->where('status', 'ORDER_CANCELLED');

            })
            ->where(function ($q) use ($outletId) {
                if ($outletId != null) {
                    $q->where('outlet_id', $outletId);
                }

            })->count();
        $orders[4] += \App\Models\VendorOrder::
            where(function ($q) use ($start, $end) {
            $q->where('deliveryDateTime', '>=', $start)
                ->where('deliveryDateTime', '<', $end);
        })
            ->where(function ($q) {
                $q->where('status', 'ORDER_CANCELLED');

            })
            ->where(function ($q) use ($outletId) {
                if ($outletId != null) {
                    $q->where('outlet', $outletId);
                }

            })->count();
        $liveOut = 0;
        $outlets = \App\Models\Outlet::all()->map(function ($out, $i) use ($liveOut) {
            $out->open = true;
            $now = \Carbon\Carbon::now();
            $weekMap = [
                0 => 'SUN',
                1 => 'MON',
                2 => 'TUE',
                3 => 'WED',
                4 => 'THU',
                5 => 'FRI',
                6 => 'SAT',
            ];
            if (isset($out->closingPeriod) && $out->closingPeriod != null) {
                $period = json_decode($out->closingPeriod);
                array_map(function ($p) use ($out, $now) {
                    $f = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedFrom)->startOfDay();
                    $t = \Carbon\Carbon::createFromFormat("Y-m-d", $p->closedTo)->endOfDay();
                    if ($now->gte($f) && $now->lte($t)) {
                        $out->open = false;
                    }

                }, $period);
            }
            if ($out->open) {
                if (isset($out->weeklyClosed) && $out->weeklyClosed != null) {
                    $weekly = json_decode($out->weeklyClosed);
                    array_map(function ($w) use ($out, $now, $weekMap) {
                        if ($weekMap[$now->dayOfWeek] == $w) {
                            $out->open = false;
                        }

                    }, $weekly);
                }
            }

            if ($out->open) {
                $out->open = 'checked';
            } else {
                unset($out->open);
            }

            $out->weeklyClosed = json_decode($out->weeklyClosed);
            $out->closingPeriod = json_decode($out->closingPeriod);

            return $out;
        });
        foreach ($outlets as $outlet) {
            if (isset($outlet->open) && $outlet->open == 'checked') {
                $liveOut = $liveOut + 1;
            }
        }
        $orders[] = $start->format('Y-m-d');
        $orders[] = $end->addDay(-1)->format('Y-m-d');
        $orders[] = $liveOut;
        //Log::channel('dailyInfo')->info('--:'.print_r($orders,true));

        return $orders;
    }
    public function removeAttachment(Request $request, $file)
    {
        try {
            if (!isset($file)) {
                session()->flash('success', 'File name is missing in the input.');
                return redirect()->back();
            }
            if ($request->session()->has('files')) {
                $files = session('files');
                $toRem = null;
                foreach ($files as $file1 => $original) {
                    if ($original == $file) {
                        $toRem = $file1;
                    }

                }
                if ($toRem != null) {
                    unset($files[$toRem]);
                    session(['files' => $files]);
                    unlink(storage_path() . '/app/' . $toRem);
                }
            }

            session()->flash('success', 'File is removed successfully.');
        } catch (\Exception $e) {
            session()->flash('error', $e->getMessage());
        }
        return view('backend.admins.vendorEmail');

    }
    public function uploadbg(Request $request)
    {
        //Log::channel('dailyInfo')->info('--:'.print_r($request,true));
        if ($request->file('file')) {
            $path = $request->file('file')->store('mails');
            $request->file('file')->store('./', ['disk' => 'public']);

            session(['bgimage' => $path]);
            session()->flash('success', 'File attached successfully.');
            return view('backend.admins.vendorEmail');
        }
        session()->flash('failure', 'please select a file for upload.');

        return view('backend.admins.vendorEmail');
    }
    public function fontsize(Request $request)
    {
        //Log::channel('dailyInfo')->info('--:'.print_r($request,true));
        if ($request->has('fontsize')) {

            session(['fontsize' => $request->input('fontsize')]);
            session()->flash('success', 'Font size set successfully.');
            return view('backend.admins.vendorEmail');
        }
        session()->flash('failure', 'font size is missing from the input.');

        return view('backend.admins.vendorEmail');
    }
    public function fonttype(Request $request)
    {
        //Log::channel('dailyInfo')->info('--:'.print_r($request,true));
        if ($request->has('fonttype')) {

            session(['fonttype' => $request->input('fonttype')]);
            session()->flash('success', 'Font type set successfully.');
            return view('backend.admins.vendorEmail');
        }
        session()->flash('failure', 'font type is missing from the input.');

        return view('backend.admins.vendorEmail');
    }
    public function fontcolor(Request $request)
    {
        //Log::channel('dailyInfo')->info('--:'.print_r($request,true));
        if ($request->has('fontcolor')) {

            session(['fontcolor' => $request->input('fontcolor')]);
            session()->flash('success', 'Font color set successfully.');
            return view('backend.admins.vendorEmail');
        }
        session()->flash('failure', 'font color is missing from the input.');

        return view('backend.admins.vendorEmail');
    }
    public function upload(Request $request)
    {
        //Log::channel('dailyInfo')->info('--:'.print_r($request,true));
        if ($request->file('file')) {
            $path = $request->file('file')->store('mails');
            if ($request->session()->has('files')) {
                session(['files' => array_merge(session('files'), [$path => $request->file('file')->getClientOriginalName()])]);
            } else {
                session(['files' => [$path => $request->file('file')->getClientOriginalName()]]);
            }

            session()->flash('success', 'File attached successfully.');
            return view('backend.admins.vendorEmail');
        }
        session()->flash('failure', 'please select a file for upload.');

        return view('backend.admins.vendorEmail');
    }
    public function emailVendorsForm()
    {
        return view('backend.admins.vendorEmail');
    }
    public function emailVendors(Request $request)
    {
        if ($request->filled('sub') || $request->filled('content') || $request->session()->has('files')) {
            $files = null;
            $sub = null;
            $send = false;
            $content = null;
            if ($request->filled('sub')) {
                $sub = $request->input('sub');
                $send = true;
            }
            if ($request->filled('content')) {
                $content = $request->input('content');
                $send = true;
            }
            if ($request->session()->has('files')) {
                $files = session('files');
                $send = true;
            }
            $outlets = \App\Models\Outlet::all();
            $emails = $outlets->map(function ($out) {
                $y = \App\Models\User::find($out->user_id);
                return $y->email;
            });
            $emailCc = ['sanjayjdn@gmail.com', 'dharamjdn@gmail.com'];
            if ($send) {
                //Log::channel('dailyInfo')->info('--:files'.print_r($content,true));
                if ($request->filled('test')) {
                    $this->emailTest($emails->toArray(), $emailCc, $sub, $content, $files);
                } else {
                    $this->email($emails->toArray(), $emailCc, $sub, $content, $files);
                }

                $request->session()->forget('files');
                session()->flash('success', 'Email sent successfully.');
            } else {
                session()->flash('failure', 'Email coundn\'t be sent Please provide subject/content/attach files.');
            }
            return view('backend.admins.vendorEmail');
        }

        session()->flash('failure', 'Email coundn\'t be sent Please provide subject/content/attach files.');
        return view('backend.admins.vendorEmail');
    }
    private function email($emails, $emailCc, $sub, $content, $files)
    {
        //Log::channel('dailyInfo')->info('--:emails'.print_r($emails,true));
        Mail::send('mail.emailAllVendors',
            compact('content'),
            function ($message) use ($emails, $emailCc, $files, $sub) {
                $message->from('noreply@olfstores.com', 'OLF STORE');
                $message->sender('noreply@olfstores.com', 'OLF STORE');
                $message->to($emails);
                $message->cc($emailCc);
                if ($sub != null) {
                    $message->subject($sub);
                }

                $message->priority(3);
                if ($files != null) {
                    foreach ($files as $file => $original) {
                        $message->attach(storage_path() . '/app/' . $file, ['as' => $original]);
                    }
                }
            });

    }
    private function emailTest($emails, $emailCc, $sub, $content, $files)
    {
        //Log::channel('dailyInfo')->info('--:emails test'.print_r($emails,true));
        Mail::send('mail.emailAllVendors',
            compact('content'),
            function ($message) use ($emails, $emailCc, $files, $sub) {
                $message->from('noreply@olfstores.com', 'OLF STORE');
                $message->sender('noreply@olfstores.com', 'OLF STORE');
                //$message->to($emails);
                $message->to($emailCc);
                //$message->cc($emailCc);
                if ($sub != null) {
                    $message->subject($sub);
                }

                $message->priority(3);
                if ($files != null) {
                    foreach ($files as $file => $original) {
                        $message->attach(storage_path() . '/app/' . $file, ['as' => $original]);
                    }
                }
            });

    }

    public function fssaiExpire(Request $request)
    {
        $fromdate = "";
        $todate = "";
        $outletName = $request->outletName;
        $status = "";
        $crntdate = date('Y-m-d');
        $futureDate = date('Y-m-d', strtotime('+30 days'));
        if ($request->fromdate != "" && $request->todate != "") {
            $fromdate = date("Y-m-d", strtotime($request->fromdate));
            $todate = date("Y-m-d", strtotime($request->todate));
        }
        if ($request->status) {
            $status = $request->status;
        }
        
  
        $outlets = DB::table('outlets')
            ->where(function ($query) use ($fromdate, $todate, $futureDate, $crntdate) {

                if ($fromdate != "" && $todate != "") {
                    $query->whereBetween(DB::raw("DATE_FORMAT(fssaiValidUpto ,'%Y-%m-%d')"), [$fromdate, $todate]);
                } else {
                    $query->whereBetween(DB::raw("DATE_FORMAT(fssaiValidUpto, '%Y-%m-%d')"), [$crntdate, $futureDate]);
                }

            })
            ->where(function ($query) use ($outletName) {
                if ($outletName != "") {
                    $query->where("outletName", 'like', '%' . $outletName . '%');
                    $query->orWhere("fssaiNo", 'like', '%' . $outletName . '%');
                }
            })

            ->where(function ($query) use ($status, $crntdate) {

                if ($status == "1") {

                    $query->where(DB::raw("DATE_FORMAT(fssaiValidUpto ,'%Y-%m-%d')"), "<=", $crntdate);
                } elseif ($status == "2") {

                    $query->where(DB::raw("DATE_FORMAT(fssaiValidUpto ,'%Y-%m-%d')"), ">", $crntdate);
                }

            })
            ->orderBy('fssaiValidUpto', 'asc')
            ->paginate(10);
        
        return view('backend.fssaiExpire.index', ['outlets' => $outlets]);

    }

    public function dateExtande(Request $request)
    {
        $strDate = $request->faslvl . ' ' . $request->time;
        $Date = date("Y-m-d H:i", strtotime($strDate));
        DB::table('outlets')->where("id", $request->Id)->update(["fssaiValidUpto" => $Date]);

    }

    public function updateStatus(Request $request)
    {
        $Id = $request->Id;
        $Status = $request->fssaiStatus;

        DB::table('outlets')->where("id", $Id)->update(["status" => $Status]);
        $response = DB::table('outlets')->where("id", $Id)->first()->status;
        echo json_encode(["response" => $response]);

    }

    public function fssaiEdit(Request $request,$outlet_id)
    {
        $fssaiData = Outlet::select('id', 'fssaiNo', 'fssaiValidUpto', 'fssai_file','outletName')
        ->where('id', $outlet_id)
        ->first();
        
        $html = view('backend.fssaiExpire.edit', ['fssaiData' => $fssaiData])->render();

        return response()->json(['html' => $html]);
    }

    public function fssaiUpdate(Request $request) {

        $fileName=null;
        if($request->hasFile('fssai_file')){
            $extn =  $request->file('fssai_file')->getClientOriginalExtension();
            $fileName = "fssai"."_".time().".".$extn;
            $request->file('fssai_file')->move(public_path('/fssai'),$fileName);
        }

        $strDate = $request->faslvl . ' ' ."00:00";
        $Date = date("Y-m-d H:i", strtotime($strDate));

        Outlet::where('id',$request->id)->update([
            'fssaiNo'=>$request->fssaiNo,
            'fssaiValidUpto'=>$Date,
            'fssai_file'=>$fileName,
        ]);

        return redirect('/backend/fssaiExpire');
    }


    // Push Outlet On The Server

    public function pushOutletOnServer($id) {
        
        $outlet=$this->service->getOutletData($id);
        
        $outletStation=$this->service->getStation($id);
        $station_code=$outletStation->code;
    
        Outlet::where('id',$id)->update([
            'pushed_at'=>date('Y-m-d H:i:s'),
            'pushed_by'=>\Auth::id(),
        ]);
        //  dd($outlet);
        $response=$this->service->postOutlet($station_code,$outlet);
    //   dd($response);    
        session()->flash('success', 'Outlet Pushed Successfully.');
       return redirect()->back();

    }

}
