<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;

use App\Http\Requests;
use App\Models\Outlet;
use File;
use Log;
use DB;
use Mail;
use App\Models\IRCTCDataManager;

class OutletHistoryController extends Controller
{
    use AuthorizesRequests;
    public function __construct(){
        $this->service = new IRCTCDataMAnager(config('constkeys.irctc_token'), config('constkeys.irctc_base_url'));
    }

    public function index(Request $request){

        $outletName = $request->outletName;

        // Fetch the outlets with their histories grouped
        $outlets = DB::table('outlets')
            ->leftJoin('OutletHestory', 'outlets.id', '=', 'OutletHestory.out_id')
            ->select('outlets.id','outlets.is_delete','outlets.outletName', DB::raw('MAX(OutletHestory.created_at) as latest_history'))
            ->when($outletName, function ($query, $outletName) {
                return $query->where('outlets.outletName', 'like', "%{$outletName}%");
            })
            ->groupBy('outlets.id', 'outlets.outletName')
            ->orderBy('latest_history', 'DESC')
            ->paginate(10);

        $outletIds = $outlets->pluck('id');

        // Fetch histories for the paginated outlets
        $histories = DB::table('OutletHestory')
            ->whereIn('out_id', $outletIds)
            ->orderBy('created_at', 'DESC')
            ->get();

        // Group histories by outlet_id
        $groupedHistories = $histories->groupBy('out_id');

        // Attach histories to outlets
        foreach ($outlets as $outlet) {
            $outlet->histories = $groupedHistories->get($outlet->id, collect());
        }

        return  view('backend.outlets.OutletHistory',["title" =>"Outlet History","result" => $outlets]);
    }


    public function restoreOutlet($outletId){

        $outlet = Outlet::find($outletId);
        $dataarray = array(
            'out_id' => $outlet->id,
            'gstNo' => $outlet->gstNo,
            'fssaiNo' => $outlet->fssaiNo,
            'fssaiValidUpto' => $outlet->fssaiValidUpto,

        );
        $result = DB::table('OutletHestory')->insert($dataarray);
        $id = DB::getPdo()->lastInsertId();
        $updateArra = array(
            // 'outletId' => 0,
            // 'gstNo' => null,
            // 'fssaiNo' => null,
            // 'fssaiValidUpto' => null,
            'status' => 1,
            'is_delete' => 0,
        );

        DB::table('outlets')->where("id", $outlet->id)->update($updateArra);

        Log::channel('outletLog')->info(" Data of outlet ".$outlet->outletName." Deleted ");

        //$outlet->delete();
        session()->flash('success', 'Outlet Restore Successfully.');

        return redirect()->back();

    }

}