<?php

namespace App\Http\Controllers\Roles;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
// use Spatie\Permission\Models\Role;
// use Spatie\Permission\Models\Permission;
use App\Models\Role;
use App\Models\Permission;
use Illuminate\Support\Str;

class RolesController extends Controller
{
    # construct
    // public function __construct()
    // {
    //     $this->middleware(['permission:roles_and_permissions'])->only('index');
    //     $this->middleware(['permission:add_roles_and_permissions'])->only(['create', 'store']);
    //     $this->middleware(['permission:edit_roles_and_permissions'])->only(['edit', 'update']);
    //     $this->middleware(['permission:delete_roles_and_permissions'])->only(['delete']);
    // }

    # role list
    public function index(Request $request)
    {   
        $this->authorize('role-list');
        $searchKey =$roles= null;
        $roles = Role::oldest();
     
        if ($request->search != null) {
            $roles = $roles->where('name', 'like', '%' . $request->search . '%');
            $searchKey = $request->search;
        }

        $roles = $roles->paginate(50);
        
        return view('backend.roles.index', compact('roles', 'searchKey'));
    }

    # return view of create form
    public function create()
    {   
        $this->authorize('create-role');
        $permission_groups = Permission::all()->groupBy('group_name');
        return view('backend.roles.create', compact('permission_groups'));
    }

    # role store
    public function store(Request $request)
    {
        $this->authorize('create-role');
        // Create a new role with the provided name
        $role = Role::create([
            'name' => $request->name,
            'slug' => Str::slug($request->name, '-'),
        ]);
        
        // Assuming $request->permissions contains an array of permission IDs
        $role->permissions()->attach($request->permissions); // Use attach method to assign permissions
        
        // Flash success message
        session()->flash('success', 'New Role has been added successfully');
        
        // Redirect to the index page
        return redirect()->route('admin.roles.index');
    }

    # edit role
    public function edit(Request $request, $id)
    {   
        $this->authorize('edit-role');
        $role = Role::findOrFail($id);
        $permission_groups = Permission::all()->groupBy('group_name');
        return view('backend.roles.edit', compact('role', 'permission_groups'));
    }

    # update role
    public function update(Request $request)
    {   
        $this->authorize('edit-role');
        $role = Role::findOrFail($request->id);
        $role->name = $request->name;
        $role->slug =Str::slug($request->name, '-');
        
        
        // Assuming $request->permissions contains an array of permission IDs
        $role->permissions()->sync($request->permissions); // Use sync method instead of syncPermissions
        
        $role->save();
        
        session()->flash('success', 'Role has been updated successfully');
        return back();
    }

    # delete role
    public function delete($id)
    {   
        $this->authorize('delete-role');
        Role::destroy($id);
        session()->flash('success','Role has been deleted successfully');
        return redirect()->route('admin.roles.index');
    }
}
