<?php

namespace App\Http\Controllers;

use App\Models\Invoice;
use App\Models\InvoiceItem;
use Illuminate\Http\Request;
use App\Models\Order;
use App\Models\Outlet;
use Exception;
use Maatwebsite\Excel\Facades\Excel;
use Faker\Core\Number;
use Illuminate\Pagination\LengthAwarePaginator;

class SalesReportController extends Controller
{

    public function index(Request $request)
    {   
        $start = date('Y-m-d', strtotime(date('Y-m-01'). ' -3 MONTH'));
        $end = date("Y-m-d"); 
       
        if ($request->filled('fromdate'))
            $start = date("Y-m-d", strtotime($request->fromdate));

        if ($request->filled('todate')) 
            $end = date("Y-m-d", strtotime($request->todate));

        $requestoutletId = $request->filled('outletId') ? $request->outletId : null;
        
        $status = "ORDER_DELIVERED";
        
        // Query orders data
        $query = Order::query()->select('outlet_id', 'priceDetails','amountPayable','vendorPayment')
            ->when(isset($start), function ($query) use ($start) {
                return $query->where('created_at', '>=', $start);
            })
            ->when(isset($end), function ($query) use ($end) {
                return $query->where('created_at', '<=', $end);
            })
            ->when(isset($status), function ($query) use ($status) {
                return $query->where('status', $status);
            })
            ->when(isset($requestoutletId) && $requestoutletId != 0, function ($query) use ($requestoutletId) {
                return $query->where('outlet_id', $requestoutletId);
            })
            ->orderBy('created_at', 'desc');


        $orders = $query->get()->groupBy('outlet_id');
        
        if(!empty($orders)){
            // Process the orders data
            $orders = $orders->map(function ($orderItem,$outletId) use ($status, $start, $end) {
                $sale = $ordtax=0;
            
                $totalSale = $orderItem->map(function ($item) use (&$sale,&$ordtax) {
                    $priceDetails = json_decode($item->priceDetails, true);
                    $ordtax+=$priceDetails['gst'];
                    $sale += $priceDetails['amountPayable'] - $priceDetails['gst'];
                    return $sale;
                });

    
                // $invoiceIds = Invoice::where(function ($q) use ($start, $end, $status,$outletId) {
                //     if (isset($start)) {
                //         $q->where('period_start', '>=', $start);
                //     }
                //     if (isset($end)) {
                //         $q->where('period_end', '<=', $end);
                //     }
                //     if (isset($outletId)) {
                //         $q->where('outlet_id', $outletId);
                //     }

                // })->pluck('id');

                //$invoiceSum=InvoiceItem::whereIn('invoice_id',$invoiceIds)->sum('vendorPayment');

                $ordAmountPayable= $orderItem->sum('amountPayable');
                $ordVendorPayment= $orderItem->sum('vendorPayment');

                $profit = $vendorPayment =0;
                // if(isset($sale) && $sale>0){
                //     $profit =  number_format($sale*0.31, 2, '.','')??0;
                //     $vendorPayment = (float)$sale - $profit;
                // }

                $vendorPayment = $ordAmountPayable - $ordVendorPayment;
                $profit= $ordVendorPayment-$ordtax;

                $vendorData = Outlet::find($outletId);
                $ordTempArray = [
                    'outlet_name' => !empty($vendorData) ? $vendorData->outletName : 'n/a',
                    'pan_number' => !empty($vendorData) ? $vendorData->vendorPanNumber : 'n/a',
                    'gst_number' => !empty($vendorData) ? $vendorData->gstNo : 'n/a',
                    'totalSale' => $sale,
                    'vendor_total' => $vendorPayment,
                    'profit_margin' => $profit
                ];
                return $ordTempArray ;
            });
        }
        
        // Paginate the grouped results
        $perPage = 50; // Number of items per page
        $page = $request->has('page') ? $request->query('page') : 1; // Current page

        $offset = ($page - 1) * $perPage; // Calculate the offset

        // Get a slice of grouped orders for the current page
        $groupedOrdersForPage = $orders->slice($offset, $perPage);

        // Create a paginator for the sliced grouped orders
        $orderDataArray = new LengthAwarePaginator(
            $groupedOrdersForPage,
            $orders->count(), // Total number of grouped orders
            $perPage,
            $page,
            ['path' => $request->fullUrl(), 'query' => $request->query()]
        );

        $outLets = Outlet::get();

        if ($request->filled('fromdate'))
            $start = date("d-m-Y", strtotime($start));
        else
            $start=null;

        if ($request->filled('todate'))
            $end = date("d-m-Y", strtotime($end));
        else
            $end=null;

        return view('backend.reports.sales_report', compact('orderDataArray', 'outLets', 'orders','requestoutletId','start','end'));
    }

    # For Export

    public function reportExport(Request $request)
    {
        $start = date('Y-m-d', strtotime(date('Y-m-01'). ' -3 MONTH'));
        $end = date("Y-m-d"); 
    
        if ($request->filled('fromdate'))
            $start = date("Y-m-d", strtotime($request->fromdate));

        if ($request->filled('todate')) 
            $end = date("Y-m-d", strtotime($request->todate));

        $requestoutletId = $request->filled('outletId') ? $request->outletId : null;
        
        $status = "ORDER_DELIVERED";
        
        // Query orders data
        $query = Order::query()->select('outlet_id', 'priceDetails','amountPayable','vendorPayment')
            ->when(isset($start), function ($query) use ($start) {
                return $query->where('created_at', '>=', $start);
            })
            ->when(isset($end), function ($query) use ($end) {
                return $query->where('created_at', '<=', $end);
            })
            ->when(isset($status), function ($query) use ($status) {
                return $query->where('status', $status);
            })
            ->when(isset($requestoutletId) && $requestoutletId != 0, function ($query) use ($requestoutletId) {
                return $query->where('outlet_id', $requestoutletId);
            })
            ->orderBy('created_at', 'desc');


        $orders = $query->get()->groupBy('outlet_id');
        
        if(!empty($orders)){
            // Process the orders data
            $orders = $orders->map(function ($orderItem,$outletId) use ($status, $start, $end) {
                $sale = $ordtax=0;
        
                $totalSale = $orderItem->map(function ($item) use (&$sale,&$ordtax) {
                    $priceDetails = json_decode($item->priceDetails, true);
                    $ordtax+=$priceDetails['gst'];
                    $sale += $priceDetails['amountPayable'] - $priceDetails['gst'];
                    return $sale;
                });

                $ordAmountPayable= $orderItem->sum('amountPayable');
                $ordVendorPayment= $orderItem->sum('vendorPayment');

                $profit = $vendorPayment =0;
                // if(isset($sale) && $sale>0){
                //     $profit =  number_format($sale*0.31, 2, '.','')??0;
                //     $vendorPayment = (float)$sale - $profit;
                // }

                $vendorPayment = $ordAmountPayable - $ordVendorPayment;
                $profit= $ordVendorPayment-$ordtax;

                $vendorData = Outlet::find($outletId);
                $ordTempArray = [
                    'outlet_name' => !empty($vendorData) ? $vendorData->outletName : 'n/a',
                    'pan_number' => !empty($vendorData) ? $vendorData->vendorPanNumber : 'n/a',
                    'gst_number' => !empty($vendorData) ? $vendorData->gstNo : 'n/a',
                    'totalSale' => $sale,
                    'vendor_total' => $vendorPayment,
                    'profit_margin' => $profit
                ];
                return $ordTempArray ;
            });
        }
        
        $orderData=$orders->toArray();

        try{

            $excelFile = 'Sales_Report_OLF_Store' . '-' . \Carbon\Carbon::now()->format('Y-M-d') . '.xlsx';
            Excel::store(new \App\Exports\SalesReportExcel($orderData), $excelFile);

            $filePath = storage_path('app/' . $excelFile);
        }
        catch(Exception $e){
            // dd($e->getMessage());
        }
       
        // Return a response with appropriate headers to trigger download
        return response()->download($filePath, $excelFile);
        

    }



}
