<?php namespace App\Http\Controllers;

use App\Http\Requests;
use App\Http\Controllers\Controller;
use App\Models\Search;
use Illuminate\Http\Request;
use Illuminate\Pipeline\Pipeline;


function array_keys_exists(array $keys, array $arr) {
   return !array_diff_key(array_flip($keys), $arr);
}


class SearchApiController extends Controller {
	private $api = null;
	public function __construct(){
		$this->api = Search::getInstance();
	}

	public function process(Request $request)
	{
		$query = $request->query();
		if(isset($query['step'])){
				$type = $query['step'];
				switch($type){
						case '2':{
							$result = $this->api->provideStations();
							return response()->json(
											compact('result'),200
											);
					 }
					 break;

			case '3':{
						if(isset($query['station'])){
							$id = intval($query['station']);
							$result = $this->api->queryOutletsOnStation($id);
							return response()->json(compact('result'),200);
						}else{
							$result = 'Not found';
							return response()->json(compact('result'),404);

						}				
		
					 }break;



			case '4':{

						if(isset($query['outlet'])){
							$id = intval($query['outlet']);
							$result = $this->api->queryItemsForOutlets($id);
							return response()->json(compact('result'),200);
						}else{
							$result = 'Not found';
							return response()->json(compact('result'),404);

						}				

					 }break;




			default:
					 echo 'exhausted';
					 break;

				}


		}else{
			echo 'Wrong format';
		}

	}
	
	
	public function vendorOrders(Request $request, \App\Models\Outlet $vendor){
	    list( $hasLocalError,$localOrders) = $this->fetchOrders($request,$vendor,"LOCAL");
	    list( $hasRemoteError,$remoteOrders) = $this->fetchOrders($request,$vendor,"REMOTE");
	    if($hasLocalError || $hasRemoteError ){
	        return response()->json(['message' => $localOrders || $remoteOrders],200);
	    }else{
	        $orders = $remoteOrders->toArray();
	        foreach($localOrders as $item)
	            array_push($orders,$item);
	            
	        if(count($orders) < 1){
	            $message = "No Orders Here";
	            return response()->json(['orders'=> []],200);
	        }
	        
	        return response()->json(compact('orders'),200);
	    }
	}
	
	private function fetchOrders(Request $request, \App\Models\Outlet $vendor, $type){
	    $allowed = ['sort','fdate','daily','monthly','weekly'];
	    $data = $type == "REMOTE" ? $vendor->orders() : $vendor->vendor_orders();
	    try{
	        $orders = app(Pipeline::class)
	            ->send($data)
	            ->through([
	                \App\AppFilters\WeekFilter::class,
	                \App\AppFilters\DateFilter::class,
	                \App\AppFilters\MonthFilter::class,
	                \App\AppFilters\DailyFilter::class,
	                \App\AppFilters\SortFilter::class,
	                \App\AppFilters\RangeFilter::class,
	 
	                ])->then(function($data){
	                    return $data;
	                })->get()->map(function($order) use($vendor,$type){
	                    $temp = $order;
	                    $temp->status = ($order->status != null || $order->status != "")? $order->status :"ORDER_CONFIRMED";
	                    $temp->priceDetails = json_decode($order->priceDetails);
	                    
	                    $items = json_decode($order->orderItems);
	                    
	                    $temp->orderItems = array_map(function($i){
	                        if(isset($i->taxRate)){
	                            $i->taxRate = intval($i->taxRate);
	                        }
	                        if(isset($i->tax)){
	                            $i->tax = intval($i->tax);
	                        }
	                        $i->basePrice = intval($i->basePrice);
	                        $i->sellingPrice = intval($i->sellingPrice);
	                        
	                        return $i;
	                    }, $items);
	                    
	                    
	                    $temp->aggregatorDetails = json_decode($order->aggregatorDetails);
	                    $temp->customerDetails = json_decode($order->customerDetails);
	                    $temp->deliveryDetails = json_decode($order->deliveryDetails);
	                    if($type=="LOCAL"){
	                        $temp->aggregatorDetails = (object)[
	                            "customerCareNumbers"=> [
                                        "9522557555"
                                    ],
                                    "id"=> $order->orderId,
                                    "name"=> "Olf Store",
                                    "outletId"=> $vendor->id,
                                    "outletName"=> $vendor->companyName
	                            ];
	                            
	                    $temp->aggregatorOrderId = $order->id;
	                    $temp->isAggregatorOrder = 1;
	                    $c = json_decode($temp->customer); 
	                    unset($temp->customer);
	                    $temp->customerDetails = (object)[
	                        'customerName'=>$c->fullName ?? $c->customerName ?? "",
	                        'mobile' => $c->mobile,
	                        'email'=> $c->email,
	                        'alternateMobile' => $c->mobile,
	                    ];
	                    
	                    $temp->deliveryDetails = (object)[
	                          'berth' =>        $order->berth,
                              'coach' =>        $order->coach,
                              'ETA' =>          $order->deliveryDateTime,
                              'pnr' =>          $order->pnr,
                              'station' =>      $order->stationName,
                              'stationCode' =>  $order->stationCode,
                              'trainName' =>    $order->trainName,
                              'trainNo' =>      $order->trainNo,
	                      ];
	                    
	                    $temp->amountPayable = intval($temp->amountPayable);
	                    $temp->comment = "NA";
	                    $temp->remarks = "NA";
	                    $temp->outlet_id = $vendor->id;
	                    $temp->previous_id = "";
	                    
	                    $temp->priceDetails = array (
                              'amountPayable'  => intval($order->amountPayable),
                              'deliveryCharge' => intval($order->deliveryCharge),
                              'discountAmount' => intval($order->discountAmount),
                              'gst'            => floatval($order->gst),
                              'otherCharges'   => 0,
                              'totalAmount'    => intval($order->totalAmount),
                              'totalMargin'    => 0,
                            );
                            
                            
                        $unsetables = [
	                        'outlet',
	                        'id',
	                        'deliveryCharge',
	                        'discountAmount' ,
	                        'gst',
	                        'totalAmount',
	                        'berth',
	                        'coach',
	                        'pnr',
                            'station',
                            'stationCode',
                            'trainName',
                            'trainNo',
	                        ];
	                        
	                        foreach($unsetables as $key=>$value){
	                           unset($temp{$value});
	                        }
	                        
	                    }
	                    
	                    
	                    return $temp;
	                });
	                
	        return [ false ,$orders];
	    }catch(Exception $e){
	        return [true, []];
	    }
	}
	
	public function vendorOrdersTest(Request $request, \App\Models\Outlet $vendor){
	    list( $hasLocalError,$localOrders) = $this->fetchOrders($request,$vendor,"LOCAL");
	    list( $hasRemoteError,$remoteOrders) = $this->fetchOrders($request,$vendor,"REMOTE");
	    if($hasLocalError || $hasRemoteError ){
	        return response()->json(['message' => $localOrders || $remoteOrders],200);
	    }else{
	        $orders = $remoteOrders->toArray();
	        foreach($localOrders as $item)
	            array_push($orders,$item);
	            
	        if(count($orders) < 1){
	            $message = "No Orders Here";
	            return response()->json(['orders'=> []],200);
	        }
	        
	        return response()->json(['orders'=>$orders],200,[]);
	    }
	}

}
