<?php

namespace App\Http\Controllers;

use App\Repositories\StationContentRepository;
use App\Repositories\BlogRepository;
use App\Services\ScrappingService;
use App\Models\Station;
use App\Models\StationMeta;
use Illuminate\Http\Request;

class StationContentController extends Controller
{
    private $repository;
    private $blog;

    function __construct(StationContentRepository $repository, BlogRepository $blog )
    {
        $this->repository = $repository;
        $this->blog = $blog;

    }
    public function index()
    {   
        $this->authorize('page-list');
        $stations = $this->repository->getStations();
        return view('backend.station.content', compact('stations'));
    }


    public function edit(Request $request, $code)
    {   
        $this->authorize('edit-page');
        list($station, $content) = $this->repository->fetchContent($code);
        return view('backend.station.content_edit', compact('station', 'content'));
    }

    public function update(Request $request, $code)
    {   
        $this->authorize('edit-page');
        $data = $request->all();
        if ($response = $this->repository->update($code, $data)) {
            return redirect(route('stations.content'));
        } else {
            return redirect(route('editing.error'));
        }
    }

    public function delete(Request $request, $code)
    {
    }

    public function getOne($slug)
    {   
        
        #$_terms = preg_split("/\-/", $slug);
        preg_match_all("/[A-Z]{2,}/",$slug, $_terms);
        $item = end($_terms);


        $code = trim(last($item));

        $posts = $this->blog->recentPosts();
        list($station, $content) = $this->repository->fetchContent($code);
        $data = StationMeta::where('station_id', '=', $station->id)->first();
        $valid = $data == null ? false : true;

        return view('backend.station.content_view', compact('station', 'content', 'data', 'valid'));
    }

    public function scrapLiveStatus(Request $request, ScrappingService $service, $trainNumber, $dateTime)
    {
        $_parts = preg_split("/\-/", $dateTime);
        $_date = sprintf("%s%s%s", $_parts[2], $_parts[1], $_parts[0]);
        $_url = "https://runningstatus.in/status/$trainNumber-on-$_date";
        $content = $service->load($_url);

        return response()->json($content);
    }

    public function fetchStationOverview(){
        $stations = \App\Models\Station::all();
        return view('v2.pages.locations', compact('stations'));

    }
}
