<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use App\Http\Requests;
use App\Domain\StationService;
use Log;

class StationController extends Controller
{
    public function index(){
        return view('Station.index');
    }

    public function viewStationList(){
        $this->authorize('station-list');
        return view('backend.station.index');
    }

    public function all(){
        return response(\App\Models\Station::all(), 200);
    }

    public function create(Request $request){
    
        
        $this->authorize('create-station');
        if(\App\Models\Station::create($request->all())){
            $message = (object)['message'=>'created'];
            return response()->json($message);
        }else{
            $message = (object)['message'=>'Record Already Exists'];
            return response()->json($message);
        }
    }

    public function update(Request $request){
        
        $this->authorize('edit-station');
        $id = $request->id;
        if($Station = \App\Models\Station::find($id)){
            $Station->update($request->all());
        }else{
            $message = (object)['message'=>'Record Not Found'];
            return response()->json($message);
        }
    }

    public function delete($id){
         
        $this->authorize('delete-station');
        if($Station = \App\Models\Station::find($id)){
            $Station->delete();
            $message = (object)['message'=>'Record Deleted'];
            return response()->json($message);
        }else{
            $message = (object)['message'=>'No Record Found'];
            return response()->json($message);
        }
    }

    public function createOutlet(Request $request, $stationId){
        
        $this->authorize('create-outlet');

        $station = \App\Models\Station::find($stationId);
        if($station){
            $outlet = $station->outlets()->create($request->all());
            return response()->json(['outletStatus'=>$outlet], 200);

        }else{
            return response()->json(['message'=>'No Station Found'],200);
        }
    }

    public function updateOutlet(Request $request, $stationId, $outletId){

        $this->authorize('edit-outlet');

        $station = \App\Models\Station::find($stationId);
        if($station){
            $outlet = \App\Models\Outlet::find($outletId);
            if($outlet){
                $outlet->update($request->all());
                $data = (object)['message'=>'updated Successfully'];
                return response()->json($data,200);

            }else{
                $data = (object)['message'=>'No Outlets found for updation'];
                return response()->json($data,200);
            }
        }else{
            return response()->json(['message'=>'No Station Found'],200);

        }
    }

    public function listOutlet(Request $request, $stationCode, StationService $stationService){


            // $this->authorize('station-outlet-list');

            $data = $stationService->getOutletsByStation($stationCode);
            //$data = \App\Station::where('code',$stationCode)->with('outlets')->get();
            if(count($data) >0){
                $data1 = [];
                foreach($data as $outlet){
                    /*
                    $outlet->tags = explode(',',$outlet->tags);
                    $outlet->closingPeriod = json_decode($outlet->closingPeriod);
                    $outlet->weeklyClosed = json_decode($outlet->weeklyClosed);
                    if($outlet->payment_mode == 1){
                        $outlet->prepaid = true;
                    }else{
                        $outlet->prepaid = false;
                    }
                    */
                    unset($outlet->payment_mode);
                    unset($outlet->id);
                    unset($outlet->station_id);
                    unset($outlet->created_at);
                    unset($outlet->updated_at);
                    unset($outlet->user_id);

                    $data1[] = $outlet;
                }
                return response()->json(array('outlets'=>$data1),200);
            }else{

                return response()->json(array('outlets'=>[]),200);
            }
    }

    public function getStation(Request $request, $id){

        $station = \App\Models\Station::find(\App\Models\Outlet::find($id)->station_id);
        
        return response()->json(['code'=>$station->code],200);
    }
    
    public function getOutlet(Request $request, $id, StationService $stationService){

            $this->authorize('outlet-list');

            $data = $stationService->getOutlet($id);
            if(count($data) >0){
                $data1 = [];
                foreach($data as $outlet){
                    //$outlet->tags = explode(',',$outlet->tags);
                    //$outlet->closingPeriod = json_decode($outlet->closingPeriod);
                    /*
                    if(isset($outlet->weeklyClosed))
                        $outlet->weeklyClosed = json_decode($outlet->weeklyClosed);
                    else
                        $outlet->weeklyClosed = json_decode([]);

                    if($outlet->payment_mode == 1){
                        $outlet->prepaid = true;
                    }else{
                        $outlet->prepaid = false;
                    }
                    */
                    //$outlet->stnCode = \App\Station::find($outlet->station_id)->code;
                    unset($outlet->payment_mode);
                    unset($outlet->id);
                    unset($outlet->station_id);
                    unset($outlet->created_at);
                    unset($outlet->updated_at);
                    unset($outlet->user_id);

                    $data1[] = $outlet;
                }
                return response()->json(array('outlets'=>$data1),200);
            }else{

                return response()->json(array('outlets'=>[]),200);
            }
    }

    public function dropOutlet(Request $request, $stationCode, $outletId){

        $this->authorize('delete-outlet');

        $station = \App\Models\Station::where('code','=', $stationCode)->first();
        
        if($station){
            $outlet = \App\Models\Outlet::find($outletId)->delete();
            if($outlet){
                dd("deleting outlet $outletId");
            }else{
                dd("No Outlet found");
            }
        }else{
            return response()->json(['message'=>'No Station Found'],200);
        }
    }
    
    public function stationList(Request $request){
        
        $stations = \App\Models\Station::all()->filter(function($station){
            $valid = \Carbon\Carbon::createFromFormat("d/m/Y",$station->validTill);
            return $valid->gt(\Carbon\Carbon::now());
            
        })->pluck('code')->flatten()->toArray();
        return response()->json($stations);
    }

    public function GetFoodDetails(Request $request){
        $id = $request->getid;
        $station = \App\Models\Station::where('id','=', $id)->first();
        return response()->json($station,200);
    }

}
