<?php

namespace App\Http\Controllers;

use App\Models\Trains;
use App\Models\TrainsContent;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class TrainController extends Controller
{
    public function index(Request $request)
    {       
        $this->authorize('train-list');

        $trains = json_decode(file_get_contents(public_path()."/trains.json"));
        return view('backend.trains.index',compact('trains')); 
    }

    public function stepOne(Request $request){

        $this->authorize('create-train');

        $trains = json_decode(file_get_contents(public_path()."/trains.json"));
        $trains=$trains->data;
        // Find train based on trainId using array_filter()
        function findTrainById($trains, $trainId) {
            $filtered = array_filter($trains, function($train) use ($trainId) {
                return $train->trainId == $trainId;
            });

            // If found, return the first element of filtered array, otherwise null
            return count($filtered) ? reset($filtered) : null;
        }

        $trainId=$request->train_id;

        $foundTrain = findTrainById($trains, $trainId);

        $DBTrain=Trains::where('number',$foundTrain->trainNumber)->first();

        if(!empty($DBTrain)){
            return redirect()->back()->withErrors(['train_number' => 'Train already exists in the database.'])->withInput();
        }
        else{
            session()->forget('train_data');
            session(['train_data' => $foundTrain]);

            return redirect('manager/add-trains-step-two');

        }

    }


    public function stepTwo(Request $request){

        $this->authorize('create-train');

        $trainData= session('train_data');

        if(empty($trainData))
        return redirect()->back();

        return view('backend.trains.create',compact('trainData')); 
        
    }

    public function storeContent(Request $request){

        $this->authorize('create-train');

        $trainData= session('train_data');

        $DBTrain=Trains::where('number',$trainData->trainNumber)->first();

        if(!empty($DBTrain)){
            return redirect()->back()->withErrors(['train_number' => 'Train already exists in the database.'])->withInput();
        }
    
        
        $createdTrain=Trains::create([
            'name'=>$trainData->trainName,
            'number'=>$trainData->trainNumber,
        ]);

        TrainsContent::create([
            'train_id'=>$createdTrain->id,
            'content'=>$request->content,
            'description'=>$request->description,
        ]);

        return redirect('manager/trains');
    }

    public function editContent($content_id){
        
        $this->authorize('edit-train');

        $train=Trains::find($content_id);

        if(!empty($train)){
            $conetentObj=TrainsContent::where('train_id',$train->id)->first();
            return view('backend.trains.edit',compact('conetentObj','train')); 
        }
        else{
            return redirect()->back(); 
        }
    }

    public function updateContent(Request $request){

        $this->authorize('edit-train');

        TrainsContent::where('id',$request->content_id)->update([
            'content'=>$request->content,
            'description'=>$request->description,
        ]);

        return redirect('manager/trains');
    }

    public function deleteContent($train_id){

        $this->authorize('delete-train');

        $train=Trains::find($train_id);

        if(!empty($train)){
            TrainsContent::where('train_id',$train->id)->delete();
            $train->delete();
        }

        return redirect()->back(); 
    }

}
