<?php namespace App\Http\Controllers;

use App\Http\Requests;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Input;
use Illuminate\Http\Request;

class UserController extends Controller {

    public function __construct()
    {
        $this->middleware('auth', ['except' => ['index','show']]);
        $this->middleware('role:admin', ['except' => ['index','show']]);
        $this->middleware('auth', ['only' => ['index','show']]);
        $this->middleware('role:admin', ['only' => ['index','show']]);
    }



    public function activate($id) {
        $user = \App\Models\User::find($id);
        $user->active = 1;
        $user->save();

        return redirect('backend/users');
    }

    public function index() {
        
        $users = \App\Models\User::where('role','!=','vendor')->orderBy('created_at','desc')->paginate(100);
        return view('backend/users.index', compact(['users']));
    }
    
    public function show($id) {
        $user = \App\Models\User::find($id);
        return view('backend/users.show', compact('user'));
    }


    public function create() {
        $role = \App\Models\Role::get();
        return view('backend/users.create',compact('role'));
    }

    public function store(Request $request) {
       
        $this->validate($request, [
        ]);
        if (\App\Models\User::whereEmail($request->email)->get()->count()) {
            session()->flash('error', 'Duplicate Entry For email field.');
            return redirect()->back()->withInput(Input::all());
        }
        
        $data = $request->only(['first_name', 'last_name', 'email', 'passwrod', 'mobile', 'dob','address']);
        $data['password'] = \Illuminate\Support\Facades\Hash::make($data['passwrod']);
        $data['role'] = $request->role;
        $data['active']= $request->role !="vendor" ? 1 : 0;
        
        $user = \App\Models\User::create($data);
        $mobiles = explode(',', $user->mobile);
        $first = 1;
        foreach ($mobiles as $mobile) {
            if($first)
                $user->mobiles()->create(['mobile'=>$mobile, 'role'=>1]);
            else
                $user->mobiles()->create(['mobile'=>$mobile]);
            $first = 0;
        }
        $user_role = \App\Models\Role::where('slug',$data['role'])->first();
        $user->roles()->attach($user_role);
        
        session()->flash('success', 'Your User created successfully.');
        return redirect()->back();
    }

    public function edit($id) {
        $user = \App\Models\User::find($id);
        return view('backend/users.update', compact('user'));
    }

  public function update(Request $request, $id) {
        $this->validate($request, [
        ]);
        $faq = \App\Models\User::find($id);
        $faq->updateOld($request->all());
        session()->flash('success', 'Your User  Updated successfully.');
        return redirect()->back();
    }

    public function destroy($id) {
        $user = \App\Models\User::find($id)->delete();
        session()->flash('success', 'User deleted successfully.');
        return redirect()->back();
    }

}
