<?php

namespace App\Http\Controllers;

use App\Models\BufferOrder;
use App\Domain\Excel;
use App\Domain\ItemService;
use App\Domain\PersistableTrait;
use App\Domain\PnrService;
use App\Domain\StationService;
use App\Domain\WordService;
use App\Exports\ConsolidatedReport;
use App\Jobs\CustomerInvoiceJob;
use App\Jobs\DepositBalanceJob;
use App\Jobs\ScheduleOrderJob;
use App\Models\MenuItem;
use App\Models\Outlet;
use App\Services\ScrappingService;
use App\Services\SessionCart;
use App\Models\VendorOrder;
use BeyondCode\Vouchers\Exceptions\VoucherExpired;
use BeyondCode\Vouchers\Exceptions\VoucherIsInvalid;
use BeyondCode\Vouchers\Models\Voucher;
use BeyondCode\Vouchers\Vouchers;
use Carbon\Carbon as CarbonCarbon;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Razorpay\Api\Api;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Log;
use TCPDF;
use Illuminate\Support\Facades\DB;





/* else{
    try{
       Log::channel('dailyInfo')->info('coupon:'.$fields->coupon->value);
     
     if(!empty($fields->coupon->value)){
         $voucher1 = \BeyondCode\Vouchers\Models\Voucher::where('code', $fields->coupon->value)->where('type', 1)->first();
         if($voucher1!=null){
             if($this->canRedeemGeneralCode($voucher1)){
                 $fields->couponCode = $voucher1->code;
                 $discnt1 = (int)$voucher1->data[0];
                 $fields->discountAmount = (int)$discnt1*$fields->amountPayable/100;
                 $fields->amountPayable = $fields->amountPayable-$fields->discountAmount;
                 $voucher1->total_discount_amount += $fields->discountAmount;
                 $voucher1->save();
             }else
                 $couponInvalid = true;
         }
         else{
             $couponInvalid = true;
         }
     }
     
//added code for vouchers
 
 }catch (\Exception $e) {
     report($e);
       //Log::channel('dailyInfo')->info('coupon:'.print_r($e, true));
         //voucher doesn't exists or couldn't be redeemed
 }
     /*
     return  response()
                 ->json(array('status'=>'failure', 'balance'=>$balance, 'wDiscount'=>$wDiscount,'discount'=>$discountAmount,'amountPayable'=>$fields->amountPayable,'message'=>'coupon code is invalid'),200);
      *
      
 } 

 */

class WebController extends Controller
{

    use PersistableTrait;

    private $store;
    private $api;

    public function __construct(SessionCart $cart)
    {
        $this->store = $cart;
        $this->store->defaultKey = 'ORDER_STORAGE';
        $this->api = new Api(config('razorpay.razor_key'), config('razorpay.razor_secret'));
        #$this->api = new Api(env('KEY_ID'), env('KEY_SECRET'));
    }

    

    public function fetchPNR(Request $request, $pnr,$orderSource=null)
    {       
        $this->cleanUp();
        $service = new PnrService();
        $result =  $service->getStationWithArrival($pnr, ['Authorization' => config('constkeys.irctc_token')]);
      
        if (!isset($result->stations) || $result->stations->count() < 1) {
            return back()->with('error', 'No Stations found for this PNR')->with('queryPNR', $pnr);
        }
        if ($result != null || (!count($result) < 1)) {

            $stations = $result->stations->map(function ($station) {
                $station->deliveryDate  = Carbon::createFromFormat('Y-m-d H:i', sprintf('%s %s', $station->arrDate, $station->arrival ?? $station->departure), 'Asia/Kolkata');
                return $station;
            })->filter(function ($station) {
                return $station->deliveryDate->gte(now());
            });

            session()->put('stations', $stations);
            $this->setPNR($pnr);
            $this->setTrainInfo($result->trainInfo);
            $this->setSeatInfo($result->seatInfo);
            $this->setStationInfo($result->stations);
            $this->store->add('paymentType', 'CASH_ON_DELIVERY');
            $this->store->add('discountAmount', 0);

            if($orderSource != "admin")
            return view('v2.pages.stations', ['stations' => $stations]);
            else
            return view('backend.AdminOrders.create', ['stations' => $stations,'pnr'=>$pnr]);
        }
        if($orderSource != "admin")
        return view('v2.pages.stations', ['stations' => []]);
        else
        return view('backend.AdminOrders.create', ['stations' => [],'pnr'=>$pnr]);
    }

    private function isFirstTimeUser()
    {
        $user = $this->getCurrentUser();
        dd($user);
    }

    public function restraunts($code, $date,$orderSource=null)
    {
        $formatted_date = base64_decode($date);
       
       $formatted_date = Carbon::createFromFormat("Y-m-d H:i:s", $formatted_date)->format('Y-m-d H:i') . " IST";
     
        $_restraunts = (new StationService())->getOutletsByStationXX($code, $formatted_date);
        //dd($_restraunts->toArray());

        if(empty($orderSource)){
            return response()->json(['restraunts' => $_restraunts->flatten()->map(function ($restraunt) use ($code) {
                $restraunt->url = route('web.items', ['id' => $restraunt->id, 'code' => $code]);
                return $restraunt;
            })->toArray()]);
        }
        else{

            return response()->json(['restraunts' => $_restraunts->flatten()->map(function ($restraunt) use ($code,$orderSource) {
                $restraunt->url = route('web.items', ['id' => $restraunt->id, 'code' => $code,'source'=>$orderSource]);
                return $restraunt;
            })->toArray()]);

        }
    }


    private function formatDate($str)
    {
        $_lc = preg_split('/\s/', $str);
        unset($_lc[2]);
        $_dt = join(" ", $_lc);
        return \Carbon\Carbon::createFromFormat('Y-m-d H:i', $_dt, 'Asia/Kolkata');
    }


    public function fetchItems(Request $request,  $code, $id,$orderSource=null)
    {
        $i = new ItemService();
        $outlet = Outlet::find($id);
        $this->selectStation(($code));


        $_items = $i->getItemFromOutlet($id, true)->map(function ($item) {
            $tax = round(floatval($item->basePrice) * 0.05, 2);
            $item->taxRate =  $tax;
            $item->sellingPrice = floatval($item->basePrice) + $tax;
            return $item;
        });


        $outletId = 0;
        try {
            $outletId = $this->store->get('outlet')['id'];
        } catch (\Throwable $th) {
            $outletId = 0;
        }


        if ((session()->has('items')  && $this->store->get('outlet')['id'] != $id))
            session()->put('items', $_items->map(function ($item) {
                return (object)array_merge($item->only(
                    ['itemName', 'description', 'basePrice', 'sellingPrice', 'foodType', 'taxRate']
                ), [
                    'itemId' => $item->id,
                    'quantity' => 0, 'isVegetarian' => $item->isVegeterian == 1 ? true : false,
                    'option' => $item->baseOption
                ]);
            }));
        else if (!session()->has('items')) {
            session()->put('items', $_items->map(function ($item) {
                return (object)array_merge($item->only(
                    ['itemName', 'description', 'basePrice', 'sellingPrice', 'foodType', 'taxRate']
                ), [
                    'itemId' => $item->id,
                    'quantity' => 0, 'isVegetarian' => $item->isVegeterian == 1 ? true : false,
                    'option' => $item->baseOption
                ]);
            }));
        }

        $this->setOutlet($outlet);

        $items = session('items')->groupBy('foodType');

        $carty = $this->generateCartResponse();
        $expanded = $items->keys()->first();

        $overview = $this->generateCartOverview(session('items'));

        if($orderSource=="admin")
        return view('backend.AdminOrders.items', compact('items', 'outlet', 'carty', 'overview', 'expanded'));
        else
        return view('v2.pages.items', compact('items', 'outlet', 'carty', 'overview', 'expanded'));
    }


    public function generateCartOverview($items)
    {
        $_items = $items->filter(function ($item) {
            return $item->quantity > 0;
        });

        $subTotal = $_items->map(function ($i) {
            return $i->quantity * $i->basePrice;
        })->reduce(function ($carry, $item) {
            return $carry + $item;
        }, 0);

        $gst = $_items->sum(function ($item) {
            return $item->taxRate * $item->quantity;
        });

        $_outlet = \App\Models\Outlet::find($this->store->getAll()['outlet']['id']);

        $deliveryCost = $_outlet->deliveryCost;


        $discounts = 0;
        
        try{
            $discounts = $this->store->get('discountAmount');
        }catch(\Exception $e){
            $discounts = 0;
            return redirect("/");
        }

        $amountPayable = $subTotal + $deliveryCost + $gst - $discounts;

        $minimumAmount = round(floatval($_outlet->minOrderAmount), 2);
        $satisfied = $amountPayable < $minimumAmount ? false : true;
        $total = $items->pluck('quantity')->sum();
        return view('v2.partials.cart_overview', compact('amountPayable', 'total', 'minimumAmount', 'satisfied'));
    }


    public function overviewCart()
    {
        $items = session('items');
        return $this->generateCartOverview($items);
    }

    private function selectStation($code)
    {
        
        $stations = collect(session('stations'));
        $station = $stations->filter(function ($st) use ($code) {
            return $st->code == $code;
        })->first();

        if($station == null) {
            $this->cleanUp();
            return redirect()->back();
        }
        $time = ($station->schArrivalTime == "--" ? $station->departure : $station->schArrivalTime);

        $this->store->add('stationCode', $station->code);
        $this->store->add('stationName', $station->name);
        $this->store->add('deliveryDate', sprintf('%s %s IST', $station->schArrivalDate, $time));
        $this->store->add('bookingDate', sprintf('%s IST', \Carbon\Carbon::now()->setTimezone('Asia/Kolkata')->format('Y-m-d H:i')));
    }




    public function cartConfirm(Request $req, $action = 'LOAD')
    {
        if ($this->getCurrentUser() == null) {
            return back();
        }
        $_items = session('items')->filter(function ($item) {
            return $item->quantity > 0;
        });

        //dd($this->store->getAll());

        $subTotal = $_items->map(function ($i) {
            return $i->quantity * $i->basePrice;
        })->reduce(function ($carry, $item) {
            return $carry + $item;
        }, 0);

        $gst = $_items->sum(function ($item) {
            return $item->taxRate * $item->quantity;
        });

        $_outlet = \App\Models\Outlet::find($this->store->getAll()['outlet']['id']);

        $deliveryCost = $_outlet->deliveryCost;


        try{
            $discounts = $this->store->get('discountAmount');
        }catch(\Exception $e) {
            $discounts = 0;
            return redirect("/");
        }
    
        $amountPayable = $subTotal + (int)$deliveryCost + $gst - $discounts;
      

        $this->store->add('amountPayable', $amountPayable);

        if ($amountPayable < round(floatval($_outlet->minOrderAmount), 2)) {
            if ($action == "REFRESH") {
                return response(route('web.items', ['id' => $_outlet->id]), 302);
            } else {
                return redirect()->back();
            }
        }

        $frozen = false;

        if (session()->has('claim') || session()->has('couponCode'))
            $frozen = true;

        return view($action == 'REFRESH' ? 'v2.partials.cartDetail' : 'v2.pages.cart', [
            'items' => $_items,
            'subTotal' => $subTotal,
            'gst' => $gst,
            'deliveryCost' => $deliveryCost,
            'discounts' => $discounts,
            'amountPayable' => $amountPayable,
            'claim' => session('claim', 'NONE'),
            'frozen' => $frozen
        ])->render();
    }

    private function setTrainInfo($trainInfo)
    {
        $this->store->add('trainNo', $trainInfo->trainNo);
        $this->store->add('trainName', $trainInfo->name);
    }

    private function setStationInfo($payload)
    {
        session()->put('stations', $payload->toArray());
    }

    private function setSeatInfo($payload)
    {
        $this->store->add('coach',   $payload->coach);
        $this->store->add('berth', $payload->berth);
    }

    private function setOutlet($payload)
    {
        $this->store->add('outlet', array_merge($payload->only(['id', 'address', 'city', 'state',]), [
            'name' => trim(preg_replace('/\t/', '', $payload->outletName)),
            'contactNumbers' => $payload->mobile,
            'gstNumber' => trim(preg_replace('/\t/', '', $payload->gstNo)),
            'pinCode' => "474011",
            'fssaiCutOffDate' => $payload->fssaiValidUpto . " IST",
            'relationshipManagerName' => "Sanjay Jadoun",
            'relationshipManagerPhone' => $payload->mobile,
            'relationshipManagerEmail' => 'olfstores@gmail.com',
            'fssaiNumber' => trim(preg_replace('/\t/', '', $payload->fssaiNo))
        ]));
    }

    private function setPNR($pnr)
    {
        $this->store->add('pnr', $pnr);
    }


    public function confirm()
    {
        $_items = session('items')->map(function ($i) {
            return $i->except(['foodType']);
        })->filter(function ($i) {
            return $i->quantity > 0;
        });


        $this->store->add('orderItems', $_items);
    }

    public function addItemsToCart(Request $request)
    {
        $_data  = json_decode($request->getContent());

        $itemId = $_data->itemId;
        $quantity = $_data->quantity;

        session()->put('items', session('items')->map(function ($item) use ($itemId, $quantity) {
            if ($itemId == $item->itemId)
                $item->quantity = intval($quantity);
            return $item;
        }));

        return response()->json(['message' => 'item added']);
    }


    public function removeFromCart(Request  $req, $id)
    {
        session()->put('items', session('items')->filter(function ($item) use ($id) {
            return $id != $item->itemId && $item->quantity > 0;
        }));

        return response()->json(['message' => 'item removed']);
    }

    public function cart()
    {
        $_response =  $this->generateCartResponse();
        return response($_response);
    }

    private function generateCartResponse()
    {

        $_items = session('items')->filter(function ($item) {
            return $item->quantity > 0;
        });

        $subTotal = $_items->map(function ($i) {
            return $i->quantity * $i->basePrice;
        })->reduce(function ($carry, $item) {
            return $carry + $item;
        }, 0);

        $gst = $_items->sum(function ($item) {
            return $item->taxRate * $item->quantity;
        });

        $_outlet = \App\Models\Outlet::find($this->store->getAll()['outlet']['id']);

        $deliveryCost = $_outlet->deliveryCost;


        $discounts = 0;

        $amountPayable = $subTotal + $deliveryCost + $gst - $discounts;

        $minimumAmount = round(floatval($_outlet->minOrderAmount), 2);

        $_response = view('v2.partials.cartInfo', [
            'items' => $_items,
            'subTotal' => $subTotal,
            'gst' => $gst,
            'deliveryCost' => $deliveryCost,
            'discounts' => $discounts,
            'amountPayable' => $amountPayable,
            'minAmountSatisfied' => $subTotal < $minimumAmount ? false : true,
            'minAmount' => $minimumAmount
        ])->render();

        return $_response;
    }



    public function cartConfirmation()
    {
        $user = collect(['memoryuser', 'vendororder', 'order'])->map(function ($guard) {
            return [$guard, auth($guard)->check()];
        })->filter(function ($i) {
            return $i[1];
        })->first();

        $_items = session('items')->filter(function ($item) {
            return $item->quantity > 0;
        })->flatten();



        $subTotal = $_items->map(function ($i) {
            return $i->quantity * $i->basePrice;
        })->reduce(function ($carry, $item) {
            return $carry + $item;
        }, 0);

        $gst = $_items->sum(function ($item) {
            return $item->taxRate * $item->quantity;
        });

        $_outlet = \App\Models\Outlet::find($this->store->getAll()['outlet']['id']);

        $deliveryCost = $_outlet->deliveryCost;

        $minimumAmount = $_outlet->minOrderAmount;

        if ($subTotal < $minimumAmount)
            session()->flash('cart_error', "Minimum Order Amount is {$minimumAmount}");

        $discounts = 0;
        try {
            $discounts = $this->store->get('discountAmount');
        } catch (\Throwable $th) {
            dd($th);
        }

        $amountPayable = $subTotal + $deliveryCost + $gst - $discounts;

        $this->store->add('totalAmount', $subTotal);
        $this->store->add('gst', $gst);
        $this->store->add('vendorOrderId', substr(str_shuffle(str_repeat("1234567890", 5)), 0, 10));
        $this->store->add('deliveryCharge', floatval($deliveryCost));
        $this->store->add('discountAmount', $discounts);
        $this->store->add('amountPayable', $amountPayable);
        $this->store->add('orderItems', $_items);



        try {
            $data = view('v2.pages.customerDetails', ['details' => (object)$this->store->getAll(), 'userguard' => $user[0]])->render();
            return $data;
        } catch (\Throwable $th) {

            return redirect('/');
        }
    }


    public function createRequest()
    {
        $amt =  $this->store->get('amountPayable');
        $this->store->add('paymentType', 'PRE_PAID');
        $order = $this->api->order->create(['amount' => $amt * 100, 'currency' => 'INR']);
        $id = $order->id;
        return response()->json(compact('id'), 200);
    }

    public function postRazorJunk(Request $request)
    {
        $this->store->add('paymentType', 'PRE_PAID');
        $_data = $this->store->getAll();
        $posted = $request->all();

        try {
            #$response = app(OrderController::class)->sendDebugOrdertoIRCTC('https://stage-ecatering.ipsator.com/api/v1/order/vendor', $_data);
            $response = app(OrderController::class)->sendOrdertoIRCTC(config('constkeys.irctc_base_url').'/api/v1/order/vendor', $_data);

            $res = json_decode($response);
           
            if ($res != null) {
                if ($res->status == 'success') {
                    try {
                        $paymentResponse = $this->api->payment->fetch($posted['razorpay_payment_id'])->capture(array('amount' => $this->store->get('amountPayable') * 100));
                        if ($paymentResponse != null) {
                            $newPayload = $this->formatResponse($res);
                            \App\Models\BufferOrder::create([
                                'payload' => $newPayload,
                                'pay' => 1,
                                'razorpay_payment_id' => $paymentResponse['id']
                            ]);

                            $this->cleanUp();

                            return $this->generateOrderSuccess($res->result->id, $res->result->deliveryDate);
                        }
                    } catch (\Exception $th) {
                        Log::error($th);
                        return $this->generateErrorResponse();
                    }
                } else {

                    Log::info(['payload' => $res, 'status' => 'REJECTED']);
                    # carry out refund process
                    return $this->generateErrorResponse();
                }
            } else {
                Log::info('[Order] Response was NULL');
                Log::info(compact('posted', '_data'));

                return $this->generateErrorResponse();
            }
        } catch (\Exception $th) {
            Log::error($th);
            return $this->generateErrorResponse();
        }
    }

    public function postCustomerDetails(Request $request)
    {
        $this->storeDetails($request);
        return response()->json(['message' => 'saved successfully']);
    }


    public function checkoutCOD(Request $request)
    {   
                        
        Log::channel('dailyInfo')->info('
            ------------------------------------------------------
            |--------------Order Log Start----------------------|
            ------------------------------------------------------
            ');


        $this->storeDetails($request);
        $this->store->add('paymentType', $request->input('paymentType'));
        $_data = $this->store->getAll();
    
        #$response = app(OrderController::class)->sendDebugOrdertoIRCTC('https://stage-ecatering.ipsator.com/api/v1/order/vendor', $_data);
        $response = app(OrderController::class)->sendOrdertoIRCTC(config('constkeys.irctc_base_url').'/api/v1/order/vendor', $_data);
     
        
        // Log::channel('dailyInfo')->info("xxxxxxxxxxxxxxxx START REQUEST LOG xxxxxxxxxxxxxxxxx");
        // Log::channel('dailyInfo')->info($_data);
        // Log::channel('dailyInfo')->info("xxxxxxxxxxxxxxxx END REQUEST LOG xxxxxxxxxxxxxxxxx");

        // Log::channel('dailyInfo')->info("xxxxxxxxxxxxxxxx START RESPONSE LOGGING xxxxxxxxxxxxxxxxx");
        // Log::channel('dailyInfo')->info($response);
        // Log::channel('dailyInfo')->info("xxxxxxxxxxxxxxxx END RESPONSE LOGGING xxxxxxxxxxxxxxxxx");
        

        $this->cleanUp();

        //try {
        
            $res = json_decode($response);
            
            if ($res->status == 'success') {

                Log::channel('dailyInfo')->info("PNR "."                             ".$res->result->pnr);
                Log::channel('dailyInfo')->info("PNR Details Fetched "."                             "."Yes");
                Log::channel('dailyInfo')->info("Select Stattion "."                             ".$res->result->stationName);
                Log::channel('dailyInfo')->info("Select Outlet "."                             ".$res->result->outlet->vendor->name);
                Log::channel('dailyInfo')->info("Selected Items");

                foreach($res->result->orderItems as $apiordItem){
                    Log::channel('dailyInfo')->info($apiordItem->itemName."  "."Item Id ". $apiordItem->itemId); 
                }


                Log::channel('dailyInfo')->info("Login Number"."                  ".$res->result->customer->mobile);
                Log::channel('dailyInfo')->info("Coupon Used"."                  ". empty($res->result->couponCode) ? "NO" : "Yes");

                Log::channel('dailyInfo')->info("Wallet Used "."                            "."N/A");
                Log::channel('dailyInfo')->info("Order Booked "."                            ".date("Y-m-d H:i:s A"));
                Log::channel('dailyInfo')->info("Order ID Generated "."                            ".$res->result->id);


                $res->result->outlet->id=$_data['outlet']['id'];
               
                $newPayload = $this->formatResponse($res);
                
                BufferOrder::create([
                    'payload' => $newPayload,
                    'pay' => 0,
                    'razorpay_payment_id' => null
                ]);


            } else {
                # carry out refund process
                return $this->generateErrorResponse();
            }

            Log::channel('dailyInfo')->info('
                                        ------------------------------------------------------
                                        |--------------Order Log End------------------------|
                                        ------------------------------------------------------
                                        ');

            // dd($res);
            return $this->generateOrderSuccess($res->result->id, $res->result->deliveryDate);


        // } catch (\Exception $e) {
        //     // Log::error($e);
        //     dd("Error "." ".$e->getMessage());
        //     return $this->generateErrorResponse();
        // }
    }

    private function cleanUp()
    {
        $this->store->clear();
        session()->forget(['stations', 'items']);
        $this->forgetIf('couponCode');
        $this->forgetIf('claim');
        $this->forgetIf('applied');
    }

    private function forgetIf($key)
    {
        if (session()->has($key))
            session()->forget($key);
    }


    private function generateOrderSuccess($orderId, $deliveryTime)
    {
        $data = base64_encode(json_encode([$orderId, $deliveryTime]));
        return redirect("/order/success?data=$data");
    }

    private function generateErrorResponse()
    {
        return redirect(route('orderpost.error'));
    }


    public function serveErrorPage()
    {
        return view('v2.pages.order_failed');
    }

    public function serveSuccess(Request $request)
    {
        $data = $request->query('data');
        $params = json_decode(base64_decode($data), true);

        try {
            list($orderId, $deliveryTime) = $params;
            return view('v2.pages.order_success', compact('orderId', 'deliveryTime'));
        } catch (\Throwable $th) {

            die('Malformed Request');
        }
    }

    public function storeDetails(Request $request)
    {
        $data = $request->only(['fullName', 'mobile', 'email']);
        $mobile = $data['mobile'];

        $this->store->add('alternateMobileNumber', $mobile);
        $this->store->add('customer', $data);
        $this->store->add('comment', 'comment ,');
    }


    public function applyDiscounts(Request $request)
    {
        $_data = json_decode($request->getContent(), true);

        switch ($_data['claim']) {
            case 'COUPON':
                $this->processCoupon($_data['code'] ?? "");
                break;

            case 'WALLET':
                $this->processWallet();
                break;

            default:
                # code...
                break;
        }

        return  $this->cartConfirm($request, 'REFRESH');
    }

    private function reset()
    {
        Config::set('vouchers.relation_table', 'vendor_order_voucher');
        Config::set('vouchers.user_model', \App\Models\VendorOrder::class);
    }

    private function processCoupon($code)
    {

        list($orderType, $order) = $this->getOrderForDiscount($code);

        switch ($orderType) {
            case 'general':
                $this->processGeneralCoupon($order);
                break;

            case 'Order':
                $this->applyCouponsFromIRCTC();
                $this->processPromoCode($code, $order);
                break;

            case 'vendorOrder':
                $this->processPromoCode($code, $order);
                break;

            default:
                # code...
                break;
        }

        $this->reset();
    }

    private function processPromoCode($code, $order)
    {


        if ($order->canRedeemCode($code)) {
            $checkResult = $order->check($code);

            $amountPayable = $this->store->get('amountPayable');

            $_discount = \App\Models\Discount::where(function ($q) use ($amountPayable) {
                $q->where('validFrom', '<=', now()->toDateString())
                    ->where('validTo', '>=', now()->toDateString())
                    ->where('amountFrom', '<=', $amountPayable)
                    ->where('amountTo', '>=', $amountPayable);
            })->first();


            if (
                $checkResult->data->get('amountFrom') <= $amountPayable
                && $checkResult->data->get('amountTo') >= $amountPayable
            ) {
                $discount = $checkResult->data->get('discount');

                $tempDiscount = round($amountPayable * $discount / 100);

                if ($tempDiscount > $_discount->upto)
                    $this->store->add('discountAmount', intval($_discount->upto));
                else
                    $this->store->add('discountAmount', $tempDiscount);

                $discount = $checkResult->data->get('discount');
                $this->store->add('discountAmount', $discount);
                session()->put('couponCode', $code);
                session()->put('claim', 'COUPON');
                session()->put('applied', true);
                session()->flash('discount_success', 'coupon reedimed successfully');
            } else if ($_discount != null) {
                $discount = $checkResult->data->get('discount');
                $this->store->add('discountAmount', $discount);
                session()->put('couponCode', $code);
                session()->put('claim', 'COUPON');
                session()->put('applied', true);
                session()->flash('discount_success', 'coupon reedimed successfully');
            } else {
                session()->flash('discount_failed', 'coupon cannot be reedimed');
            }
        } else {
            session()->flash('discount_failed', 'coupon cannot be reedimed');
        }
    }

    private function canRedeemGeneralCode($code)
    {
        try {
            $voucher = Voucher::whereCode($code)->first();

            if ($voucher === null) {
                throw VoucherIsInvalid::withCode($code);
            }
            if ($voucher->isExpired()) {
                throw VoucherExpired::create($voucher);
            }

            return $voucher;
        } catch (\Throwable $th) {
            return false;
        }
    }

    private function processGeneralCoupon($coupon)
    {

        if ($this->canApplyFirstTimeCoupon()) {
            $voucher1 = \BeyondCode\Vouchers\Models\Voucher::where('code', $coupon)->where('type', 1)->first();

            if ($voucher1 != null) {
                if ($this->canRedeemGeneralCode($voucher1->code)) {
                    $amtPayable = $this->store->get('amountPayable');

                    /* $fields->couponCode = $voucher1->code;
                 */
                    $data = (object)json_decode($voucher1->data[0]);
                    $discnt1 = (int)$data->discount;
                    $maxAmount = (int)$data->amountTo;
                    $minimumAmount = (int)$data->amountFrom;
                    /*  $fields->discountAmount = (int)$discnt1*$fields->amountPayable/100; */

                    $discountAmount = $this->store->get('discountAmount');

                    $tempDiscount = 0;

                    if ($amtPayable < $minimumAmount) {
                        session()->flash('discount_failed', 'coupon cannot be reedimed');
                        return;
                    }

                    if ($data->strategy == 'UPTO') {
                        $tempDiscount = (int)$discnt1 * ($amtPayable / 100);
                        $tempDiscount += $discountAmount;

                        if ($tempDiscount > $maxAmount) {
                            $this->store->add('discountAmount', $maxAmount);
                        } else {
                            $this->store->add('discountAmount', $tempDiscount);
                        }
                    } else {
                        $tempDiscount = (int)$data->discount;
                        $tempDiscount += $discountAmount;
                        $this->store->add('discountAmount', $tempDiscount);
                    }



                    /*$fields->amountPayable = $fields->amountPayable-$fields->discountAmount;*/


                    session()->put('couponCode', $coupon);
                    session()->put('claim', 'COUPON');
                    session()->put('applied', true);
                    session()->flash('discount_success', 'coupon reedimed successfully');
                    $voucher1->save();
                } else
                    session()->flash('discount_failed', 'coupon cannot be reedimed');
            } else {
                session()->flash('discount_failed', 'coupon cannot be reedimed');
            }
        } else {
            session()->flash('discount_failed', 'coupon cannot be reedimed');
        }
    }

    private function getOrderForDiscount($code)
    {
        $_orders = collect([['vendorOrder', \App\Models\VendorOrder::where('discount_code', $code)->first()], ['Order', \App\Models\Order::where('discount_code', $code)->first()]]);
        try {
            $_f =  $_orders->filter(function ($order) {
                return $order[1] != null;
            });

            return  $_f->count() > 0 ? $_f->first() : ['general', $code];
        } catch (\Throwable $th) {
            return ['general', $code];
        }
    }

    private function processWallet()
    {
        list($guard, $user) = $this->getCurrentUser();
        if ($guard != 'memoryuser') {
            $wDiscount = $user->last()->getAmountProduct1($this->store->get('amountPayable'));


            $this->store->add('discountAmount', $wDiscount);
            session()->flash('discount_success', 'Wallet amount applied');
            session()->put('claim', 'WALLET');
            session()->put('applied', true);
        }
    }

    private function applyCouponsFromIRCTC()
    {
        Config::set('vouchers.relation_table', 'order_voucher');
        Config::set('vouchers.user_model', \App\Models\Order::class);
    }

    private function getCurrentUser()
    {
        $user = collect(['vendororder', 'order', 'memoryuser', 'apiorder'])->map(function ($guard) {
            return [$guard, auth($guard)->check()];
        })->filter(function ($guardWithValue) {
            return $guardWithValue[1];
        })->map(function ($guard) {
            return [$guard[0], auth($guard[0])->user()];
        })->first();

        return $user;
    }

    private function canApplyFirstTimeCoupon()
    {
        $currentUSer = $this->getCurrentUser();
        return $currentUSer[0] == 'memoryuser';
    }
    
    private function print_mem()
    {
       /* Currently used memory */
       $mem_usage = memory_get_usage();
       
       /* Peak memory usage */
       $mem_peak = memory_get_peak_usage();
    
       echo 'The script is now using: <strong>' . round($mem_usage / 1024) . 'KB</strong> of memory.<br>';
       echo 'Peak usage: <strong>' . round($mem_peak / 1024) . 'KB</strong> of memory.<br><br>';
    }

    public function pushTest(Request $request, ScrappingService $service)
    {
        
    }



    public function cancelOrder(Request $request)
    {
        $this->cleanUp();
        return redirect('/');
    }

    private function convertDate($date)
    {
        $_parts = preg_split("/\s/", $date);
        unset($_parts[2]);
        $_rt = join(" ", $_parts);
        $_date = Carbon::createFromFormat("Y-m-d H:i", $_rt, "UTC")->setTimeZone("Asia/Kolkata");
        return $_date->format('Y-m-d H:i') . " IST";
    }

    public function formatResponse($response)
    {
        $_response = $response;
        $result = $_response->result;
        $result->deliveryDate = $this->convertDate($result->deliveryDate);
        $result->bookingDate = $this->convertDate($result->bookingDate);
        $_response->result = $result;

        return json_encode($_response);
    }

    public function breakfast()
    {
        return view('v2.pages.breakfast'); 
    }

    public function dinner()
    {
        return view('v2.pages.dinner'); 
    }

    public function lunch()
    {
        return view('v2.pages.lunch'); 
    }

    public function policy()
    {
        return view('v2.pages.terms');
    }
}
