<?php

namespace App\Http\Controllers;

use App\Http\Requests;
use App\Http\Controllers\Controller;
use App\Services\WaService;
use Illuminate\Http\Request;
use Bavix\Wallet\Services\WalletService;
use BeyondCode\Vouchers\Models\Voucher;
use Event;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Log;
use Exception;
use DB;
use Illuminate\Support\Facades\Storage;


class WorkFlowController extends Controller
{

	private $wa = null;
	public function __construct(WaService $service)
	{
		$this->wa = $service;
	}

	public function step1(Request $request,$stationCode=null)
	{	
		$method = $request->method();
		
		switch ($method) {
			case 'POST':
				if ($request->has('select_vendor')) {
					$user = \App\Models\User::find($request->select_vendor);
				} else {
					if (\App\Models\User::whereEmail($request->email)->get()->count()) {
						session()->flash('error', 'Duplicate Entry For email field.');
						return redirect()->back()->withInput($request->all());
					}

					$data = $request->only([
						'first_name', 'last_name', 'email', 'password', 'mobile', 'dob',
						'address'
					]);
					$data['password'] = bcrypt($data['password']);
					$data['role'] = 'vendor';
					$data['active']= 0;
					$user = \App\Models\User::create($data);
					$mobiles = explode(',', $user->mobile);
					$first = 1;
					foreach ($mobiles as $mobile) {
						if ($first)
							$user->mobiles()->create(['mobile' => $mobile, 'role' => 1]);
						else
							$user->mobiles()->create(['mobile' => $mobile]);
						$first = 0;
					}
					$user_role = \App\Models\Role::where('slug', $data['role'])->first();
					$user->roles()->attach($user_role);
				}

				return view('backend.workflow.outlet', ['userId' => $user->id,'station'=>$request->stationCode]);

			case 'GET':
				return view('backend.workflow.user',['stationCode'=>$stationCode]);
		}
	}

	public function step1_0(Request $request)
	{
		$vendors = \App\Models\Role::where('slug', 'vendor')->first()->users;
		return view('backend.workflow.user_select', ['vendors' => $vendors]);
	}

	public function step2(Request $request, $id)
	{
		$method = $request->method();
		$user = \App\Models\User::find($id);
		$request1 = $request->all();

		$file = $request->file('logoFile');

		if (isset($file) && $file->isValid()) {
			// Generate a unique file name
			$fileName = date('YmdHis') . '_' . $file->getClientOriginalName();
			$path = $file->move('public/assets/img', $fileName);

			// Generate the link to the uploaded file
			$link = asset($path);

			// Merge the link with the request
			$request->merge(['logoImage' => $link]);
		} else {
			// Default image link if no file was uploaded or if the file is invalid
			$request->merge(['logoImage' => '/static/img/Logo.jpg']);
		}
		
		$request->merge(['status' => 0]);
		$request->merge(['breakfastStartTime' => '00:10']);
		$request->merge(['breakfastEndTime' => '11:50']);
		$request->merge(['lunchStartTime' => '11:50']);
		$request->merge(['lunchEndTime' => '18:00']);
		$request->merge(['dinnerStartTime' => '18:00']);
		$request->merge(['dinnerEndTime' => '23:58']);
		$request->merge(['closingPeriod' => '[]']);
		$request->merge(['weeklyClosed' => '[]']);
		$request->merge(['active' => 1]);
		$request->merge(['email' => 'olforder@gmail.com']);
		$request->merge(['mobile' => '07514920669']);


		switch ($method) {
			case 'POST':
				$outlet = new \App\Models\Outlet;
				$outlet->fill($request->all());
				$outlet->user_id = $user->id;
				$outlet->startDate=date('Y-m-d');
				 $outlet->save();
				 $insertedId = $outlet->id;
                 $updateArra=array(
		          'outletId'=>$insertedId,
		         
		        );
                DB::table('outlets')->where("id",$insertedId)->update($updateArra);
				session()->flash('error', 'Outlet created success Next Step Add Menus');
				return redirect('/home');

			case 'GET':
				return view('backend.workflow.outlet');
		}
	}



	public function menu(Request $request, $outletId)
	{
		$outlet = \App\Models\Outlet::find($outletId);
		switch ($request->method()) {
			case 'GET':
				return view('backend.workflow.items', ['identifier' => $outlet->id, 'outlet' => $outlet]);
			case 'POST':

				$outlet->menu()->create($request->except(['_token']));
				session()->flash('success', 'Menu has been assigned to selected vendor Now add Items to Menus');
				return redirect()->back();
		}
	}

	public function items(Request $request, $outletId)
	{
		$method = $request->method();
		switch ($method) {
			case 'POST':
				$outlet = \App\Models\Outlet::find($outletId);
				$input = $request->all();
				$input['outlet_id'] = $outlet->id;
				$outlet->menu->items()->create($input);
				session()->flash('error', 'Item Added Successfully');
				return redirect('/');

			case 'GET':
				dd("getting what expected");
				return view('backend.outlets.index');
		}
	}

	public function outlets(Request $request)
	{
		$outlets = \App\Models\Outlet::all();
		view('backend.outlets.index', ['outlets' => $outlets]);
	}


	public function orders(Request $request)
	{	
		try {
			try {
				
				Log::channel('dailyInfo')->info('--------------Order On IRCTC Server Log Begin----------------------');

				Log::channel('dailyInfo')->info('Order id:' . $request->input('orderId'));
				// Log::channel('database_log')->info('order id:' . $request->input('orderId'));
			
				// $outletId = $request->input('aggregatorDetails')['outletId'];
				$outletId = $request->input('aggregatorDetails')['aggregatorOutletId'];
			
				$aggregatorDetails = $request->input('aggregatorDetails');
				$aggregatorDetails['outletId']=$outletId;
			
				$priceDetails = $request->input('priceDetails');
				$deliveryDetails = $request->input('deliveryDetails');
				$orderItems = $request->input('orderItems');
				$customerDetails = $request->input('customerDetails');
				$order =  $request->all();
				//Log::channel('dailyInfo')->info('--request:'.$request);
				//Log::channel('dailyInfo')->info('order:'.print_r($order,true).'--');

				$order['aggregatorDetails'] = json_encode($aggregatorDetails);
				$order['priceDetails'] = json_encode($priceDetails);
				$order['amountPayable'] = $priceDetails['amountPayable'];
				$order['deliveryDetails'] = json_encode($deliveryDetails);
				$deliveryDate1 = str_replace(' IST', '', $order['deliveryDate']);
				$order['deliveryDateTime'] = \Carbon\Carbon::createFromFormat('m-d-Y H:i', $deliveryDate1);
				$order['orderItems'] = json_encode($orderItems);
				$order['customerDetails'] = json_encode($customerDetails);
				$order['customerEmail'] = array_key_exists("email",$customerDetails) ? $customerDetails['email'] : '';
				$order['customerMobile'] = $customerDetails['mobile'];
				//$order['amountPayable'] = $priceDetails->amountPayable;

				$order['status'] = "ORDER_CONFIRMED";
				$outlet = \App\Models\Outlet::find($outletId);
				$orderFind = \App\Models\Order::where('orderId', $order['orderId'])->where('outlet_id', $outletId)->first();

				if ($orderFind) {
					$result = (object)[
						'orderId' => $orderFind->orderId,
						'aggregatorOrderId' => $orderFind->orderId,
						'status' => $orderFind->status
					];

					echo json_encode($result);
					return;
				}
				$items = array_column($outlet->menu->items->toArray(), 'vendorPrice', 'id');
				$orderItems = collect(json_decode($order['orderItems'], true))->map(function ($oi) use ($items) {
					if (isset($items[$oi['itemId']]))
						$oi['costPrice'] = $items[$oi['itemId']];
					return $oi;
				});
				$unavailable = false;
				$subtotal1 = collect($orderItems)->sum(function ($item) {
					if (isset($item['costPrice']) && $item['costPrice'] != 'N.A.' && $item['costPrice'] != '') {
						return $item['costPrice'] * $item['quantity'];
					}
					$unavailable = true;
				});
				if ($unavailable)
					$order['vendorPayment'] = 0;
				else
					$order['vendorPayment'] = $priceDetails['amountPayable'] - $subtotal1;

				$order2 = \App\Models\Order::where('orderId', $order['orderId'])->first();
				if ($order2 != null) {
					$order2->paymentType = $order2->paymentType;
					$order2->save();
					$result = (object)[
						'orderId' => $order2->orderId,
						'aggregatorOrderId' => $order2->orderId,
						'status' => $order2->status
					];
					echo json_encode($result);
					return;
				}
				$orderCreate = $outlet->orders()->firstOrCreate($order);
				
				Log::channel('dailyInfo')->info('Order Detail: ' . json_encode($request->all()));

			
				$order1 = \App\Models\Order::where('orderId', $orderCreate->orderId);
				$order2 = \App\Models\Order::where('orderId', $order['orderId'])->first();

				//event(new \App\Events\OrderCreatedWallet($order2));    
				/*
                $walletDatabase = \Bavix\Wallet\Models\Wallet::where('holder_id', $orderCreate->orderId)->first();
                if($walletDatabase){
                    $walletDatabase->transactions->map(function($transaction) {
                        $transaction->delete();
                    });
                    $walletDatabase->delete();
                    $order2 = \App\Order::where('orderId',$order2->orderId)->first();
                    event(new \App\Events\OrderCreatedWallet($order2));    
                }
                */


				$wallet = $order2->wallet;
				$oWhere = \App\Models\Order::where('customerMobile', $order['customerMobile']);
		
				if ($order2 != null) 
					$count = $oWhere->count();
				else
					$count = 0;

				
				
				if ($count == 1) {
					$totalAmt = ($orderCreate->amountPayable - json_decode($order['priceDetails'])->deliveryCharge) * (0.8);
				} else {
					$totalAmt = ($orderCreate->amountPayable - json_decode($order['priceDetails'])->deliveryCharge) * (0.5);
				}

				Log::channel('dailyInfo')->info('Depositing here* '.'  '. $order['orderId'] . $totalAmt);

	
				$wallet->deposit($totalAmt);
				$walletCash = $totalAmt;
				$amountP = $orderCreate->amountPayable;
				//$order1->update(['wallet_id'=>$wallet->id]);

			} catch (\Exception $e) {

				// dd($e->getMessage()." "."Line 258");
				Log::channel('errorLogs')->info('exception:' . $e->getTraceAsString());
				throw new Exception("Could not read fields.", 0, $e);
			}

			try {

				$first = $order1->first();
				$discount = \App\Models\Discount::where(function ($q) use ($amountP) {
					$q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
						->where('validTo', '>=', \Carbon\Carbon::now()->toDateString())
						->where('amountFrom', '<=', $amountP)
						->where('amountTo', '>=', $amountP);
				})->first();
				if ($discount != null) {
					$voucher = $first->createVouchers(1, [
						'discount' => $discount->discount,
						'amountFrom' => $discount->amountFrom,
						'amountTo' => $discount->amountTo
					], \Carbon\Carbon::now()->addDays($discount->expires))[0];
					$order1->update(['discount_code' => $voucher->code]);
				}
			} catch (\Exception $e) {
				// dd($e->getMessage()." "."Line 279");
				throw new Exception("voucher creation failed.", 0, $e);
			}

			Log::channel('dailyInfo')->info('DB Order id:' . $orderCreate->orderId);

            $message =  $this->formatMessage($orderCreate);
			/*try {
				$message =  $this->formatMessage($orderCreate);
				ob_start();
				$this->sendMessage($message['message'], $message['outlet']->mobile, env('VENDOR_MSG_TEMLPLATE_ID', NULL));
				$res1 = ob_get_clean();
				Log::channel('dailyInfo')->info('--message body:' . $message['message'] . '--');
				Log::channel('dailyInfo')->info('--sms sent to outlet:' . $outlet->outletName);
				Log::channel('dailyInfo')->info('mobile:' . $message['outlet']->mobile);
				Log::channel('dailyInfo')->info('sms response:' . $res1 . '--');
			} catch (\Exception $e) {
				throw new Exception("message sending failed.", 0, $e);
			}*/

			// WA JUNK

			try {
				$this->wa->sendMessageIRCTC($message['outlet']->mobile, $orderCreate);
				$this->wa->sendMessageIRCTC($customerDetails['mobile'], $orderCreate, true);
			} catch (\Exception $th) {
				// dd($e->getMessage()." "."Line 304");
				Log::channel('errorLogs')->info(['[WA] Message Sending Failed']);
				Log::info($th);
			}


			try {

				if ($discount != null) {
					$linkMsg = ", Download the app https://play.google.com/store/apps/details?id=com.olfstores.olf_user or visit https://www.olfstores.com. Terms and conditions apply";

					$message =  $this->formatMessageCustomer($orderCreate, $discount, $voucher, $walletCash, $linkMsg);
					ob_start();
					$this->sendMessage($message['message'], $customerDetails['mobile'], env('CUSTOMER_MSG_TEMPLATE_ID', NULL));
					$res2 = ob_get_clean();
					//Log::channel('dailyInfo')->info('--message body:'.$message['message'].'--');

					Log::channel('dailyInfo')->info('--sms sent to customer:' . $customerDetails['customerName']);
					Log::channel('dailyInfo')->info('mobile:' . $customerDetails['mobile']);
					Log::channel('dailyInfo')->info('sms response:' . $res2 . '--');
				}
			} catch (\Exception $e) {
				// dd($e->getMessage()." "."Line 323");
				throw new Exception("message coupon code sending to customer mobile failed.", 0, $e);
			}

			try {

				$this->passEmail($order);
				Log::channel('dailyInfo')->info('email sent successfully to outlet:' . $outlet->outletName);
				Log::channel('dailyInfo')->info('email:' . \App\Models\User::find($outlet->user_id)->email);
			} catch (\Exception $e) {
				// dd($e->getMessage()." "."Line 331");
				throw new Exception("email sending failed.", 0, $e);
			}

			try {

				$message1 = 'Hi ! You received a new order from oelf store for train no. ' . implode(' ', str_split($deliveryDetails['trainNo'])) . ' customer name ' . $customerDetails['customerName'] . ' Thank You';
				$mobiles = explode(",", $message['outlet']->mobile);
				Log::channel('dailyInfo')->info('--sending voice message to outlet:' . $outlet->outletName);
				foreach ($mobiles as $value) {
					$res2 = '';
					ob_start();
					$this->sendVoiceMessage($message1, $value);
					$res2 = ob_get_clean();
					Log::channel('dailyInfo')->info('mobile:' . $value);
					Log::channel('dailyInfo')->info('response:' . $res2 . '--');
				}
			} catch (\Exception $e) {

				// dd($e->getMessage()." "."Line 348");
				throw new Exception("voice message sending failed.", 0, $e);
			}

			// $outlet->user->token->map(function ($token) use ($orderCreate) {
			// 	//Log::channel('dailyInfo')->info('token:'.$token->firebase_token);
			// 	if (isset($token->firebase_token))
			// 		event(new \App\Events\OrderCreatedFirebaseEvent([
			// 			'type' => "USER",
			// 			'token' => $token->firebase_token,
			// 			'message' => [
			// 				'title' => 'Order Recieved',
			// 				'content' => "Order placed with #{$orderCreate->orderId}"
			// 			]
			// 		]));
			// });

			$result = (object)[
				'orderId' => $orderCreate->orderId,
				'aggregatorOrderId' => $orderCreate->orderId,
				'status' => $orderCreate->status
			];

			Log::channel('dailyInfo')->info('--------------Order On IRCTC Server Log End----------------------');

		} catch (\Exception $e) {

			// dd($e->getMessage()." "."Line 371");
			Mail::raw($e->getMessage() . $e->getTraceAsString(), function ($message) {
				$message->from('no-reply@olfstores.com', 'olfstores.com');
				$message->to(['sanjayjdn@gmail.com', 'dharamjdn@gmail.com']);
				$message->cc('ajay.kumar.khatri24@gmail.com');
			});
			Log::channel('errorLogs')->error($e);
			throw $e;
			/*
                    return  response()->json(array('status'=>'failure',
                                                    'message'=>$e->getMessage()),200);
                     *
                     */
		}
		
		/* if (isset($orderCreate) && isset($orderCreate->orderId))
			WalletService::removeWallet($orderCreate->orderId);*/
			
		echo json_encode($result);
	}

	private function formatMessageCustomer($data, $discount, $voucher, $walletCash, $linkMsg = null)
	{
		$orderId =  $data->orderId;
		$out = \App\Models\Outlet::where('id', $data->outlet_id)->first();
		$userId = $out->user_id;
		//dd($userId);
		$customerName = json_decode($data->customerDetails)->customerName;
		$outlet = \App\Models\User::find($userId);
		$st = json_decode($data->deliveryDetails)->station;
		$message = "Hi $customerName, Thanks for order from OLFSTORE, IRCTC Order id: $orderId";
		$message .= " will be delivered at $st, by $out->outletName. Congratulations! You have received discount coupon from OLF, use coupon code $voucher->code to get $discount->discount% discount on your next order. Additionally an amount of Rs." . $walletCash . " has been deposited to your wallet. To avail (coupon wallet) discount " . $linkMsg;
		$message .= " ,OLFSTORES";
		return ['message' => $message, 'outlet' => $outlet];
	}

	private function formatMessage($data, $linkMsg = null)
	{
		$aggrigator = json_decode($data->aggregatorDetails);
		$delivery = json_decode($data->deliveryDetails);
		$p = json_decode($data->priceDetails);
		$customer = json_decode($data->customerDetails);
		$orderId =  json_decode($data->orderId);
		$items = json_decode($data->orderItems);
		$booking = $data->bookingDate;
		$deliveryDate = substr($data->deliveryDate, 0, 10);
		$userId = \App\Models\Outlet::where('id', $data->outlet_id)->first()->user_id;
		//dd($userId);
		$outlet = \App\Models\User::find($userId);
		$message = "IRCTC Order id:" . $orderId . " For: {$delivery->trainNo}/{$delivery->trainName} ETA {$delivery->ETA} coach {$delivery->coach}-{$delivery->berth} at {$delivery->station} to {$customer->customerName} {$customer->mobile} on {$deliveryDate} ";

		$oitems = array_map(function ($item) {
			return "{$item->quantity} -{$item->itemName}";
		}, $items);

		$message .= "ITEMS:" . join($oitems);

		$message .= "Amount: {$p->amountPayable}-{$data->paymentType}";

		if (isset($linkMsg))
			$message .= $linkMsg;

		$message .= ",olfstores";

		return ['message' => $message, 'outlet' => $outlet];
	}



	public function testMessage(Request $request)
	{
		Mail::raw('Text to e-mail', function ($message) {
			$message->to('ar31213@gmail.com');
		});
	}

	private function sendMessage1($string, $numbers)
	{

		$nums = explode(",", $numbers);
		$data = (object)[
			'sender' => 'OLFSTR',
			'route' => 4,
			'country' => 91,
			'sms' => [
				(object)['message' => $string, 'to' => $nums]
			]

		];

		//dd(json_encode($data));
		$curl = curl_init();

		curl_setopt_array($curl, array(
			CURLOPT_URL => "http://api.msg91.com/api/v2/sendsms",
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_ENCODING => "",
			CURLOPT_MAXREDIRS => 10,
			CURLOPT_TIMEOUT => 30,
			CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
			CURLOPT_CUSTOMREQUEST => "POST",
			CURLOPT_POSTFIELDS => json_encode($data),
			CURLOPT_SSL_VERIFYHOST => 0,
			CURLOPT_SSL_VERIFYPEER => 0,
			CURLOPT_HTTPHEADER => array(
				"authkey:191164Aup6j2p05a4cb419",
				"content-type: application/json"
			),
		));

		$response = curl_exec($curl);
		$err = curl_error($curl);

		curl_close($curl);

		/*if ($err) {
			echo "cURL Error #:" . $err;
		} else {
			echo $response;
		}*/
	}
	private function sendMessage($string, $numbers, $DLT_TE_ID = null)
	{
		$key = "191164Aup6j2p05a4cb419";
		$url = sprintf('https://api.msg91.com/api/sendhttp.php?authkey=%s&mobiles=%s&route=%s&sender=%s&message=%s&country=%s', $key, $numbers, '4', urlencode('OLFSTR'),  urlencode(urlencode($string)), '91');

		if ($DLT_TE_ID != null) {
			$url = $url . sprintf('&DLT_TE_ID=%s', $DLT_TE_ID);
		}
		/*
	    $nums = explode(",",$numbers);
		$data = (object)[
			'sender'=>'OLFSTR',
			'route'=>4,
			'country'=>91,
			'sms'=>[
			   (object)['message'=>$string,'to'=>$nums]
			]

		];
                */
		//dd(json_encode($data));
		$curl = curl_init();

		curl_setopt_array($curl, array(
			//CURLOPT_URL => "http://api.msg91.com/api/v2/sendsms",
			CURLOPT_URL => $url,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_ENCODING => "",
			CURLOPT_MAXREDIRS => 10,
			CURLOPT_TIMEOUT => 30,
			CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
			//CURLOPT_CUSTOMREQUEST => "POST",
			CURLOPT_CUSTOMREQUEST => "GET",
			//CURLOPT_POSTFIELDS => json_encode($data),
			CURLOPT_SSL_VERIFYHOST => 0,
			CURLOPT_SSL_VERIFYPEER => 0,
			/*
					CURLOPT_HTTPHEADER => array(
						"content-type: application/json"
						),*/
		));

		$response = curl_exec($curl);
		$err = curl_error($curl);

		curl_close($curl);

		/*if ($err) {
			echo "cURL Error #:" . $err;
		} else {
			echo $response;
		}*/

		print_r($response);
	}

	private function sendVoiceMessage($string, $number)
	{

		$data = http_build_query(
			array(
				'from' => '919522996999',
				'message' => $string,
				'authkey' => '191164Aup6j2p05a4cb419',
				'to' => '91' . $number,
			),
			'&'
		);

		//dd($data);
		$curl = curl_init();

		curl_setopt_array($curl, array(
			CURLOPT_URL => "http://control.msg91.com/api/sendVoiceCall.php",
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_ENCODING => "",
			CURLOPT_MAXREDIRS => 10,
			CURLOPT_TIMEOUT => 30,
			CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
			CURLOPT_CUSTOMREQUEST => "POST",
			CURLOPT_POSTFIELDS => $data,
			CURLOPT_SSL_VERIFYHOST => 0,
			CURLOPT_SSL_VERIFYPEER => 0
		));

		$response = curl_exec($curl);
		$err = curl_error($curl);

		curl_close($curl);

		/*if ($err) {
			echo "cURL Error #:" . $err;
		} else {
			echo $response;
		}*/
		print_r($response);
	}

	public function test()
	{
		$this->sendMessage('hi', '9945040843', env('VENDOR_MSG_TEMLPLATE_ID', null));
	}

	private function passEmail($data)
	{	
		$aggrigator = json_decode($data['aggregatorDetails']);

		$delivery = json_decode($data['deliveryDetails']);
		$customerIRCTC = json_decode($data['customerDetails']);
		$pricing = json_decode($data['priceDetails']);
		$items = json_decode($data['orderItems']);
		$orderId = $data['orderId'];
		$bookingDate = $data['bookingDate'];
		$deliveryDate = $data['deliveryDate'];
		$paymentType = $data['paymentType'];


		if (array_key_exists('comment', $data))
			$comment = $data['comment'];
		else
			$comment = '';


		try {
			Mail::send(
				'mail.irctc_vendor_email',
				compact('aggrigator', 'delivery', 'customerIRCTC', 'items', 'orderId', 'bookingDate', 'deliveryDate', 'paymentType', 'pricing', 'comment'),
				function ($message) use ($customerIRCTC, $aggrigator, $orderId) {
					$x = \App\Models\Outlet::find($aggrigator->outletId);
					$y = \App\Models\User::find($x->user_id);
					$message->from('no-reply@olfstores.com', 'OLF STORE');
					$message->sender('no-reply@olfstores.com', 'OLF STORE');
					$message->to($y->email, $x->outletName);
					//$message->cc($customerIRCTC->email, $customerIRCTC->customerName);
					$message->bcc('sanjayjdn@gmail.com', 'Sanjay Jadon');
					$message->subject('ORDER ID:' . $orderId);
					$message->priority(3);
				}
			);

			if(!empty($customerIRCTC->email)){
				Mail::send(
					'mail.irctc_user_email',
					compact('aggrigator', 'delivery', 'customerIRCTC', 'items', 'orderId', 'bookingDate', 'deliveryDate', 'paymentType', 'pricing', 'comment'),
					function ($message) use ($customerIRCTC, $aggrigator, $orderId) {
						$x = \App\Models\Outlet::find($aggrigator->outletId);
						$y = \App\Models\User::find($x->user_id);
						$message->from('no-reply@olfstores.com', 'OLF STORE');
						$message->sender('no-reply@olfstores.com', 'OLF STORE');
						// $message->to($y->email, $x->outletName);
						$message->to($customerIRCTC->email, $customerIRCTC->customerName);
						$message->bcc('sanjayjdn@gmail.com', 'Sanjay Jadon');
						$message->subject('ORDER ID:' . $orderId);
						$message->priority(3);
					}
				);
		    }
			
		} catch (\Throwable $th) {
			throw $th;
		}
	}


	public function checkOrder(Request $request, $order)
	{
		$_order = \App\Models\Order::where('orderId', '=', $order)->first();
		$data = $_order->toArray();

		try {
			$this->passEmail($data);
			return "ok";
		} catch (\Exception $th) {
			dd($th);
		}
		return "ok";
	}
}
