<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Requests;
use BeyondCode\Vouchers\Models\Voucher;
use BeyondCode\Vouchers\Vouchers;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Razorpay\Api\Api;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Exception;


class CouponMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        if($request['fields'] ->claimType == "CLAIM_COUPON"){
            return $this->handleRequest($request, $next);
        }else{
            return $next($request);    
        }
    }
    
    
    private function handleRequest(Request $request , Closure $next){
        try{
            Config::set('vouchers.relation_table','order_voucher');
            Config::set('vouchers.user_model',\App\Models\Order::class);
            return $this->calculateDiscount($request,$next);    
        }catch(\Exception $e){
            dd($e);
            
        }
        finally{
            Config::set('vouchers.relation_table','vendor_order_voucher');
            Config::set('vouchers.user_model',\App\Models\VendorOrder::class);
            $next($request);
        }
    }
    
    private function getCouponFromRequest($request){
        $coupon = null;
        if($orderCoupon = \App\Models\Order::where('discount_code', $request['fields']->coupon)->first())
            $coupon = $orderCoupon;
         else if($vendorCoupon = \App\Models\VendorOrder::where('discount_code', $request['fields']->coupon)->first())
            $coupon = $vendorCoupon;   
        
        return $coupon;
    }
    
    private function calculateDiscount(Request $model,Closure $next){
        if($coupon = $this->getCouponFromRequest($model))
            $this->redimCoupon($model, $coupon);
        return $next($model);
    }
    
    
    private function getDiscount($model){
        return $discount = \App\Models\Discount::where(function ($q) use($model){
                                    $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                                        ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString())
                                            ->where('amountFrom', '<=', $model['fields']->getAmtPayable())
                                                ->where('amountTo', '>=', $model['fields']->getAmtPayable());})->first();
    }
    
    
    private function redimCoupon($model, $voucher){
        try{
            if($coupon = $voucher->check($model['fields']->coupon)){

                $discount = $this->getDiscount($model);
                
                if($coupon->data->get('amountFrom')<=$model['fields']->getAmtPayable() && $coupon->data->get('amountTo')>=$model['fields']->getAmtPayable()){
                
                    $disc = $coupon->data->get('discount');
                
                    $discAmt = round(($model['fields']->getAmtPayable()) * ($disc/100));
                    
                    if($discount!=null){
                        if($discAmt>$discount->upto)
                            $discAmt = (int)$discount->upto;
                    }
                 
                     $model['fields']->setDiscount($discAmt);  
                    
                } else if($discount != null){
                
                    $dMaxAmt = (int)$discount->upto;
                
                    $model['fields']->setDiscount($dMaxAmt);
                }
                                                
                
            
        }
        
        }catch(\Exception $e){
            // LOG HERE PROBLEM
        }
    }
    
    
}
