<?php

namespace App\Jobs;

use App\Domain\InvoiceCompiler;
use App\Models\VendorOrder;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use App\Domain\WordService;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\Facades\Log;
use TCPDF;
use Illuminate\Support\Facades\Mail;
use SebastianBergmann\Environment\Console;

class CustomerInvoiceJob extends Event
{
    use SerializesModels;
    private $data;
    public $timeout = 90;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct(VendorOrder $order)
    {
        $this->data = $order;
    }

    public function getData(){
        return $this->data;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $customer = json_decode($this->data->customer);
        $d = $this->_PreparePDF($this->data->orderId)->Output('invoice.pdf', 'S');
        //file_put_contents(public_path()."/example.pdf", $d);
        $this->sendEmail($customer->email, $d);
    }

    public function getHtml(){
        $order = $this->data;
        $outlet = \App\Models\Outlet::find($order->outlet);
        $h = new InvoiceCompiler($order, $outlet, "FORTY FIVE RUPEES");
        return $h->getTable();
    }


    private function _PreparePDF($id)
    {
        $c = new WordService();
        
        $order = $this->data;
        $outlet = \App\Models\Outlet::find($order->outlet);
        $wAmount = $c->convertNumber($order->amountPayable . ".00");
        $d = new InvoiceCompiler($order, $outlet, $wAmount);
        $pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);
        $html = $d->getTable();

        // return $html;

        // set document information
        $pdf->SetCreator(PDF_CREATOR);
        $pdf->SetAuthor('OLf Stores');
        $pdf->SetTitle('Order Invoice');

        // set default header data
        $pdf->SetHeaderData('R.jpg', 15, 'OLF STORES AND TECHNOLOGIES PVT LTD', '86, Goverdhan Colony, Bhind Road Gwalior' . "\nhttps://olfstores.com", array(0, 64, 255), array(0, 64, 128));

        // set header and footer fonts
        $pdf->setHeaderFont(array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
        $pdf->setFooterFont(array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));

        // set default monospaced font
        $pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

        // set margins
        $pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
        $pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
        $pdf->SetFooterMargin(PDF_MARGIN_FOOTER);

        // set auto page breaks
        $pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);

        // set image scale factor
        $pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

        // add a page
        $pdf->AddPage();


        $pdf->writeHTML($html, true, false, false, false, '');
        // add a page

        // output the HTML content

        // reset pointer to the last page
        $pdf->lastPage();
        //Close and output PDF document
        //$pdf->Output( $order->orderId.'.pdf', 'I');
        return $pdf;
    }


    public function sendEmail($email, $attachment)
    {
        $content = ['name' => json_decode($this->data->customer)->fullName, 'id' => $this->data->orderId];

        try {
            Mail::send('mail.invoice_c', $content, function ($message) use ($email, $attachment) {
                $message->from('no-reply@olfstores.com')
                    ->to($email)
                    ->cc('sanjayjdn@gmail.com')
                    ->attachData($attachment, $this->data->orderId . ".pdf");
            });
            Log::info("Invoice for {$this->data->orderId} sent to {$email}");
        } catch (\Exception $th) {
            Log::info($th);
        }
    }
}
