<?php

namespace App\Jobs;

use App\Domain\OrderMessaging;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Support\Facades\Log;



class DepositBalanceJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels, OrderMessaging;

    public $order;
    private $totalAmt;
    private $discount;
    private $couponCode;
    private $voucher;
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($order)
    {
        $this->order = $order;
        $this->totalAmt = 0;
        $this->discount  = null;
        $this->voucher = null;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $linkMsg = ", Download the app https://play.google.com/store/apps/details?id=com.olfstores.olf_user or visit https://www.olfstores.com. Terms and conditions apply";
        $this->deposit();
        echo "Balance Deposited \n";
        $this->createCoupon();
        echo "Voucher created \n";
        
        try {
            if($this->discount != null && $this->voucher != null ){
                $message = $this->formatMessageCustomer($this->order, $this->discount, $this->voucher, $this->totalAmt, $linkMsg);
                # echo $message['message']."\n";
              $response = $this->sendMessage($message['message'], $this->order->customer->mobile, env('CUSTOMER_MSG_TEMPLATE_ID'));
              Log::info( json_encode($response));
            }
           
        } catch (\Throwable $th) {
            Log::info("Deposit Job has Some Errors");
        }
    }


    private function deposit()
    {
        $totalAmt = 0;
        $order2 = \App\Models\VendorOrder::where('orderId', $this->order->orderId)->first();
        $wallet = $order2->wallet;
        $count = $order2->count($this->order->customerMobile);
        if ($count == 1)
            $totalAmt = ($this->order->amountPayable - $this->order->deliveryCharge) * (0.8);
        else
            $totalAmt = ($this->order->amountPayable - $this->order->deliveryCharge) * (0.5);

        Log::channel('dailyInfo')->info('--except:' . $this->order->amountPayable . '-' . $this->order->deliveryCharge . '-' . $totalAmt);
        $wallet->deposit($totalAmt);
        $this->totalAmt = $totalAmt;
    }


    private function createCoupon()
    {
        $amountP = $this->order->amountPayable;
        $this->discount = $discount = \App\Models\Discount::where(function ($q) use ($amountP) {
            $q->where('validFrom', '<=', \Carbon\Carbon::now()->toDateString())
                ->where('validTo', '>=', \Carbon\Carbon::now()->toDateString())
                ->where('amountFrom', '<=', $amountP)
                ->where('amountTo', '>=', $amountP);
        })->first();
        if ($discount != null) {
            $this->voucher = $this->order->createVouchers(1, [
                'discount' => $discount->discount,
                'amountFrom' => $discount->amountFrom,
                'amountTo' => $discount->amountTo
            ], \Carbon\Carbon::now()->addDays($discount->expires))[0];
            $this->order->update(['discount_code' => $this->voucher->code]);
        }
    }


    private function formatMessageCustomer($data, $discount, $voucher, $walletCash, $linkMsg = null)
    { 
        $customer = json_decode($data->customer);
        $orderId =  $data->orderId;

        $userId = \App\Models\VendorOrder::where('customerMobile', $data->customerMobile);
        //dd($userId);
        $outlet = \App\Models\User::find($data->outlet);

        $customerName = $customer->fullName;

        $message = "Hi $customerName, Thanks for order from OLFSTORE, IRCTC Order id: $orderId";
        $message .= " will be delivered at $data->stationName, by $outlet->outletName. Congratulations! You have received discount coupon from OLF, use coupon code $voucher->code to get $discount->discount% discount on your next order. Additionally an amount of Rs." . $walletCash . " has been deposited to your wallet. To avail (coupon wallet) discount " . $linkMsg;
        $message .= " ,OLFSTORES";

        return ['message' => $message, 'outlet' => $outlet];
    }
}
