<?php

namespace App\Jobs;

use App\Domain\OrderMessaging;
use App\Services\WaService;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;

use function PHPSTORM_META\type;

class NotifyUserJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels, OrderMessaging;

    /**
     * Create a new job instance.
     *
     * @return void
     */

    public $payload;

    public function __construct($payload)
    {
        $this->payload = $payload;
    }

    private function notify()
    {
        $linkMsg = ", Download the app https://play.google.com/store/apps/details?id=com.olfstores.olf_user or visit https://www.olfstores.com. Terms and conditions apply";

        $message = $this->formatMessage($this->payload, null);
        
        $_testMessage = "IRCTC Order id:49497010 For: 03187/SDAH RPH SPL ETA 09-20-2021 09:30 IST coach D6-67 at Barddhaman Jn to RUHUL KUDDUS MOLLA 9339014518 on 09-20-2021 ITEMS:1 -Veg Paratha ThaliAmount: 160-CASH_ON_DELIVERY,olfstores";

        Log::channel('dailyInfo')->info("Sending Message to ". $this->payload->customer->mobile);

        $response = $this->sendMessage($message['message'], $this->payload->customer->mobile, "1307161519623246556");

        Log::channel('dailyInfo')->info("Message Response" . json_encode($response) ."\n");
        
        Log::channel('dailyInfo')->info("Message Sent to ". $this->payload->customer->mobile);

        
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle(WaService $service)
    {
        try {

            $this->notify(); #Send SMS
            
            $this->notifyByEmail(); #Send Email

            $this->payload->paymentType = $this->payload->vendorPaymentType;
            $service->sendMessageBuffers($this->payload->customer->mobile, $this->payload, true);
        } catch (\Exception $th) {
            Log::info("Error In Notifying User");
            Log::info($th);
        }
    }

    private function formatMessage($data, $linkMsg = null)
    {

        
        $aggrigator = 'Olf Store'; //todo
        //$delivery = $data->deliveryDate; //todo
        $p = $data->totalAmount;
        $customer = $data->customer;
        $orderId =  $data->id;
        $items = $data->orderItems;
        $booking = $data->bookingDate;
        $deliveryDate = substr($data->deliveryDate, 0, 10);
        $eta = $data->deliveryDate; 
        //dd($userId);
        $outlet = $data->outlet;
        $message = "IRCTC Order id:{$orderId} For: {$data->trainNo}/{$data->trainName} ETA {$eta} coach {$data->coach}-{$data->berth} at {$data->stationName} to {$customer->fullName} {$customer->mobile} on {$deliveryDate} ";
        $oitems = array_map(function ($item) {
            return "{$item->quantity} -{$item->itemName}";
        }, $items);

        $message .= "ITEMS:" . join($oitems);

        $message .= "Amount: {$data->amountPayable}-{$data->vendorPaymentType}";
        if (isset($linkMsg)) {
            $message .= $linkMsg;
        }
        $message .= ",olfstores";

        
        return ['message' => $message, 'outlet' => $outlet];
    }
    
    
    
    private function notifyByEmail(){
       
        try{

            $this->formatEmail($this->payload);
           
        }catch(\Exception $e){
            Log::info("Error In Sending Mail To User");
            Log::info($e);
        }
    }
    
    
    private function formatEmail($payload){

        $data = collect($payload)->toArray();

        Log::channel('dailyInfo')->info("Sending Mail To ". $data['customer']->email);

        $delivery = (object)array(
            'pnr' => $data['pnr'],
            'trainNo' => $data['trainNo'],
            'trainName' => $data['trainName'],
            'station' => $data['stationName'],
            'stationCode' => $data['stationCode'],
            'berth' => $data['berth'],
            'coach' => $data['coach'],
            'ETA' => $data['deliveryDate'], //todo
        );
        $items = $data['orderItems'];
        $customerIRCTC = $data['customer'];
        
        $discountAmount = 0;
        
        if (array_key_exists('discountAmount', $data))
            $discountAmount = $data['discountAmount'];
        
        $pricing = (object)array(
            'totalAmount' => $data['totalAmount'],
            'gst' => $data['gst'],
            'amountPayable' => $data['amountPayable'],
            'discountAmount' => $discountAmount
        ); //todo
        
        $customerIRCTC->customerName = $customerIRCTC->fullName;
        
        $orderId = $data['id'];
        $bookingDate = $data['bookingDate'];

        $deliveryDate = $data['deliveryDate'];
        $paymentType = $data['vendorPaymentType'];
        $outlet = \App\Models\Outlet::find($data['outlet']->id);
        $comment = $data['comment'] ?? "";
        
        $view = view('mail.irctc_user_email',compact('delivery', 'items', 'customerIRCTC', 'outlet', 'delivery', 'orderId', 'bookingDate', 'deliveryDate', 'paymentType', 'pricing','comment'));
        
        Mail::send(
            'mail.irctc_user_email',
            compact('delivery', 'items', 'customerIRCTC', 'outlet', 'delivery', 'orderId', 'bookingDate', 'deliveryDate', 'paymentType', 'pricing','comment'),
            function ($message) use ($outlet, $customerIRCTC, $orderId) {
                $y = \App\Models\User::find($outlet->user_id);
                $message->from('no-reply@olfstores.com', 'OLF STORE');
                $message->sender('no-reply@olfstores.com', 'OLF STORE');
                $message->to($customerIRCTC->email, $customerIRCTC->customerName ?? $customerIRCTC->fullName);
                // $message->cc('sanjayjdn@gmail.com', 'Sanjay Jadon');
                $message->subject('ORDER ID:' . $orderId);
                $message->priority(3);
            }
        );
       Log::channel('dailyInfo')->info('Mail Send To User');
    }
}
