<?php

namespace App\Jobs;

use App\Domain\OrderMessaging;
use App\Services\WaService;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Log;

class NotifyVendorJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels, OrderMessaging;

    public $order;
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($order)
    {
        $this->order = $order;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle(WaService $service)
    {
        try {
            $message = $this->formatMessage($this->order);
            
            Log::channel('dailyInfo')->info("Email  Sent to Vendor");

            // Log::info($message['message'] . "\n");

             try {
                 
               // $response = $this->sendMessage($message['message'], $message['outlet']->mobile, env('VENDOR_MSG_TEMLPLATE_ID'));
                $mobiles = preg_split('/\,/', $message['outlet']->mobile);
                Log::channel('dailyInfo')->info("Sending Whatsapp Message Start");
                foreach ($mobiles as $key => $value) {
                    $service->sendMessageVendorOrders($value,$this->order, false);
                }
                Log::channel('dailyInfo')->info("Sending Whatsapp Message End");

            } catch (\Exception $th) {
                Log::info("Message Sending ISSUES[VENDOR]");
                Log::info($th);
                            
            }
        } catch (\Exception $e) {
            Log::info("MESSAGE FORMATTING ISSUE");
            Log::info($e);
        }
        
        
       

        try {

            $this->passEmail($this->order->toArray());
        } catch (\Exception $th) {
            Log::info("Email SENDING ISSUES to VENDOR");
            Log::info($th);
        }
        
    }

    private function formatMessage($data, $linkMsg = null)
    {

        $customer = json_decode($data->customer);
        $orderId =  $data->orderId ?? $data->id;
        $items = json_decode($data->orderItems);
        $deliveryDate = substr($data->deliveryDate, 0, 10);
        $eta = substr($data->deliveryDateTime, 0, 16);
        $userId = \App\Models\Outlet::where('id', $data->outlet)->first()->user_id;
        $fullName = $customer->fullName ?? "";

        $outlet = \App\Models\User::find($userId);
        $message = "IRCTC Order id:{$orderId} For: {$data->trainNo}/{$data->trainName} ETA {$eta} coach {$data->coach}-{$data->berth} at {$data->stationName} to {$fullName} {$customer->mobile} on {$deliveryDate} ";
        $oitems = array_map(function ($item) {
            return "{$item->quantity} -{$item->itemName}";
        }, $items);

        $message .= "ITEMS:" . join($oitems);

        $message .= "Amount: {$data->amountPayable}-{$data->paymentType}";
        if (isset($linkMsg)) {
            $message .= $linkMsg;
        }
        $message .= ",olfstores";
        return ['message' => $message, 'outlet' => $outlet];
    }


    private function passEmail($data)
    {
        $delivery = (object)array(
            'pnr' => $data['pnr'],
            'trainNo' => $data['trainNo'],
            'trainName' => $data['trainName'],
            'station' => $data['stationName'],
            'stationCode' => $data['stationCode'],
            'berth' => $data['berth'],
            'coach' => $data['coach'],
            'ETA' => $data['deliveryDate'], //todo
        );
        $items = json_decode($data['orderItems']);
        $customerIRCTC = json_decode($data['customer']);
        //$customerEmail = $customer['email'];
        //$customerName = $customer['fullName'];
        $discountAmount = 0;
        if (array_key_exists('discountAmount', $data))
            $discountAmount = $data['discountAmount'];
        $pricing = (object)array(
            'totalAmount' => $data['totalAmount'],
            'gst' => $data['gst'],
            'amountPayable' => $data['amountPayable'],
            'discountAmount' => $discountAmount
        ); //todo
        $customerIRCTC->customerName = $customerIRCTC->fullName;
        //$items = json_decode($data['orderItems']);
        $orderId = $data['orderId'];
        $bookingDate = $data['bookingDate'];

        $deliveryDate = $data['deliveryDate'];
        $paymentType = $data['paymentType'];
        $outlet = \App\Models\Outlet::find($data['outlet']);

        Mail::send(
            'mail.emailVendor',
            compact('delivery', 'items', 'customerIRCTC', 'outlet', 'delivery', 'orderId', 'bookingDate', 'deliveryDate', 'paymentType', 'pricing'),
            function ($message) use ($outlet, $customerIRCTC, $orderId) {
                $y = \App\Models\User::find($outlet->user_id);
                $message->from('no-reply@olfstores.com', 'OLF STORE');
                $message->sender('no-reply@olfstores.com', 'OLF STORE');
                $message->to($y->email, $outlet->outletName);
                $message->cc("ar31213@gmail.com", $customerIRCTC->customerName ?? $customerIRCTC->fullName);
                $message->bcc('sanjayjdn@gmail.com', 'Sanjay Jadon');
                $message->subject('ORDER ID:' . $orderId);
                $message->priority(3);
            }
        );
    }
}

