<?php

namespace App\Jobs;

use App\Models\BufferOrder;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Support\Facades\Log;

class ScheduleOrderJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public $order;
    public $dispatchTime;
    public $bufferId;

    public function __construct($order, $dispatchTime, $bufferId)
    {   
        $this->order = $order;
        $this->dispatchTime = $dispatchTime;
        $this->bufferId = $bufferId;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        try {
            $order = BufferOrder::find($this->bufferId);
            $_payload = (array)$order->normalize()->result;
            $payload = (object)array_merge($_payload, $order->pay == 1 ? ['razorpay_payment_id' => $order->razorpay_payment_id, 'pay' => $order->pay] : ['pay' => $order->pay], ['orderId' => $_payload['id']]);
            #echo "Processing order {$this->bufferId} for {$this->dispatchTime}";
            SaveOrderJob::dispatch($payload);

        } catch (\Exception $th) {
            // dd($th->getMessage());
            // Log::info("Save Order broken");
        }
        

    }
}
