<?php

namespace App\Models;

use App\Jobs\NotifyUserJob;
use App\Jobs\ScheduleOrderJob;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class BufferOrder extends Model
{
    protected $fillable = ['payload', 'pay', 'razorpay_payment_id'];

    public $table = "buffers";

    public function normalize()
    {
        return json_decode($this->payload);
    }

    
    protected static function boot()
    {
        parent::boot();

        // Define the 'created' event listener
        static::created(function ($model) {
            $_payload = json_decode($model->payload)->result;

            NotifyUserJob::dispatch($_payload);
            $dispatchDate = self::calculateDispatchTime($_payload);
            Log::info('Order will be dispatched at :');
            Log::info($dispatchDate);
         
            ScheduleOrderJob::dispatch($_payload, $dispatchDate, $model->id)->delay($dispatchDate);
        });
    }
    

    public static function calculateDispatchTime($order)
    {
        
        $deliveryDate = BufferOrder::convertDate($order->deliveryDate);
        $outletId = $order->outlet->id;
        $cutOffTime = !empty(Outlet::find($outletId)) ? Outlet::find($outletId)->cancel_cut_off_time : 0;
        $dispatchDate = Carbon::createFromFormat('Y-m-d H:i', $deliveryDate)->setTimezone('Asia/Kolkata')->subMinutes(intval($cutOffTime));
        return $dispatchDate;
    }

    public static function convertDate($str)
    {
        $_str = preg_split("/\s/", $str);
        return join(" ", [$_str[0], $_str[1]]);
    }

    public function duplicate()
    {
        BufferOrder::create([
            'id' => $this->id +1,
            'payload' => $this->payload
        ]);
    }
}
