<?php

namespace App\Models;

use Exception;

interface UserCart{
     public function manageItem($item, $qty=1);
     public function getItems();
     public function getTotal();
     public function removeFromCart($id);
}

class Cart  implements UserCart
{
    private static $obj;
    private $items,$total;
    private final function __construct(){
        $this->items = [];
        $this->total = 0;
    }

    public static function getInstance(){
        if(!isset(self::$obj)){
            self::$obj = new Cart();
        }
        return self::$obj;
    }


    public function check($item)
    {
    	$found = false;
    	foreach ($this->items as $key) {
    		if ($key->id == $item->id) {
    			$found = true;
    			break;
    		}
    	}

    	return $found;
        
    }


    public function fetchNStore($item, $qty)
    {
    	$items = array_map(function($current) use($item,$qty){
    			if ($current->id == $item->id) {
    				if (($current->qty + $qty) == 0) {
                        throw new Exception("EN002", 1);
    				}else{
    					$current->qty += $qty;
    				}
    			}
    			return $current;
    	}, $this->items);

    	$this->items = $items;
    }

    public function manageItem($it, $qty=1){
         /*check if item already in cart if yes update it otherwise add it*/
         if (count($this->items) >0 ) {
         	 if ($this->check($it)) {
         	 		$this->fetchNStore($it,$qty);
         	 }else{
                if($qty== -1){
                    throw new Exception("EN001", 1);
                }
                  
         	 	$item = $it;
         	    $item->qty = $qty;
         	 	array_push($this->items, $item);
         	 	
         	 }
         	
         } else {
            if($qty== -1){
                throw new Exception("EN001", 1);
            }
         	 $item = $it;
         	 $item->qty = $qty;
         	 array_push($this->items, $item);
         }
         
    }


    public function getItem($id){
        $item = array_filter($this->items, function($k) use($id){
            return $k->id == $id;
        });

        $result = reset($item);
        return $result;
    }

    public function getItems(){
        return $this->items;
    }

    public function getTotal(){
        $total =  array_sum(array_map(function($item){
        	return ($item->sellingPrice * $item->qty);
        }, $this->items));

        return $total;
    }

    public function removeFromCart($value)
    {
    	$items = array_filter($this->items, function($item) use($value){
    			return !($item->id == $value); 
    	});
    	$this->items = $items;
    }
}
  
