<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

class Outlet extends Model
{
    protected $fillable = [
        'outletId',
        'user_id',
        'outletName',
        'order_timing',
        'minOrderAmount',
        'openingTime',
        'closingTime',
        'deliveryCost',
        'address',
        'city',
        'state',
        'breakfastStartTime',
        'breakfastEndTime',
        'lunchStartTime',
        'dinnerStartTime',
        'lunchEndTime',
        'dinnerEndTime',
        'companyName',
        'vendorPanNumber',
        'gstNo',
        'fssaiNo',
        'fssaiValidUpto',
        'tags',
        'status',
        'logoImage', 
        'email',
        'mobile', 
        'station_id',
        'cancel_cut_off_time',
        'closingPeriod',
        'weeklyClosed',
        'active',
        'invoice_frequency',
        'reason',
        'openingBalance',
        'startDate',
        'agreementStart',
        'payment_mode',
        'closingPeriod_veg_non_veg',
        'weeklyClosed_veg_non_veg',
        'veg_non_veg',
        'reason_veg_non_veg'
    ];
    protected $dates = ['startDate'];    
    public function menu(){
    	return $this->hasOne(\App\Models\Menu::class);
    }
    public function items(){
    	return $this->hasMany(\App\Models\MenuItem::class);
    }

    public function station(){
        return $this->belongsTo(\App\Models\Station::class);
    }

    public function user()
    {
        return $this->belongsTo(\App\Models\User::class);
    }

    public function orders()
    {
        return $this->hasMany(\App\Models\Order::class);
    }

    public function ordersLastMonth() {
        return $this->hasMany(\App\Models\Order::class)->whereDate('created_at', '>', \Carbon\Carbon::today()->subDays(32)->toDateString());
    }    

    public function vendor_orders()
    {
        return $this->hasMany(\App\Models\VendorOrder::class, 'outlet');
    }
    public function vendorOrdersLastMonth() {
        return $this->hasMany(\App\Models\VendorOrder::class, 'outlet')->whereDate('created_at', '>', \Carbon\Carbon::today()->subDays(32)->toDateString());
    }    
    public function deliveryBoys(){
    	return $this->hasMany(\App\Models\DeliveryBoy::class);
    }
    public function invoices(){
    	return $this->hasMany(\App\Models\Invoice::class)->orderBy('period_start', 'asc');
    }
    public function invoicesStartDate(){
    	return $this->hasMany(\App\Models\Invoice::class)->whereDate('period_start','=',$this->startDate)->orderBy('period_start', 'asc');
    }
    public function payments(){
    	return $this->hasMany(\App\Models\Payment::class);
    }
}
