<?php 

namespace App\Models;

use Illuminate\Auth\Authenticatable;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Auth\Passwords\CanResetPassword;
use Illuminate\Foundation\Auth\Access\Authorizable;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Contracts\Auth\CanResetPassword as CanResetPasswordContract;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use App\Permissions\HasPermissionsTrait;
use Illuminate\Foundation\Auth\User as Authenticatable1;
use Illuminate\Notifications\Notifiable;

class User extends Model implements AuthenticatableContract, AuthorizableContract, CanResetPasswordContract {

	use Authenticatable, Authorizable, CanResetPassword;
        use Notifiable;
        use HasPermissionsTrait;
	/**
	 * The database table used by the model.
	 *
	 * @var string
	 */
	protected $table = 'users';

	/**
	 * The attributes that are mass assignable.
	 *
	 * @var array
	 */
	protected $fillable = ['first_name','last_name','mobile','role','address', 'email', 'password', 'api_token', 'active','remember_token'];

	/**
	 * The attributes excluded from the model's JSON form.
	 *
	 * @var array
	 */
        protected $rememberTokenName = 'remember_token';
        protected $guarded = [
        'created_at', 'updated_at'
    ];
	protected $hidden = ['password', 'remember_token','api_token'];
         public function updateOld($data) {
        return $this->update($data);
    }

    public function profile(){
    	if($this->role == "vendor")
    		return $this->hasOne(\App\Models\Outlet::class);
    	else if($this->role=="user")
    		return $this->hasOne(\App\Models\UserProfile::class);
    	else if($this->role == "agent")
    		return $this->hasOne(\App\Models\AgentProfile::class);
    	else
    		return $this->hasOne(\App\Models\AdminProfile::class); 
    }

    public function isAdmin() {
        if($this->hasRole('admin'))
            return true;
                
    }
    public function mobiles()
    {
        return $this->hasMany('App\Models\UserMobile');
    }
    public function mobileAdmin()
    {
        return $this->hasOne('App\Models\UserMobile')->where('role',1);
    }
    public function outlet()
    {
        return $this->hasMany('App\Models\Outlet')->limit(1);
    }
    public function outlets()
    {
        return $this->hasMany('App\Models\Outlet');
    }
    public function documentSign(){
    	return $this->hasOne(\App\Models\DocumentSign::class)->where('type',0);
    }
    public function token(){
    	return $this->hasMany(\App\Models\Token::class);
    }

    public function userRoles() {
        return $this->belongsToMany(Role::class,'users_roles');
  
     }
     
}
