<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use App\Events\OrderCreatedEvent;
use App\Models\Outlet;
use Illuminate\Auth\Authenticatable;
use Illuminate\Auth\Passwords\CanResetPassword;
use Illuminate\Foundation\Auth\Access\Authorizable;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Contracts\Auth\CanResetPassword as CanResetPasswordContract;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use App\Permissions\HasPermissionsTrait;
use Illuminate\Foundation\Auth\User as Authenticatable1;
use Illuminate\Notifications\Notifiable;
use BeyondCode\Vouchers\Traits\HasVouchers;
use BeyondCode\Vouchers\Traits\CanRedeemVouchers;
use BeyondCode\Vouchers\Vouchers;
use BeyondCode\Vouchers\Models\Voucher;
use BeyondCode\Vouchers\Events\VoucherRedeemed;
use BeyondCode\Vouchers\Exceptions\VoucherExpired;
use BeyondCode\Vouchers\Exceptions\VoucherIsInvalid;
use BeyondCode\Vouchers\Exceptions\VoucherAlreadyRedeemed;
use Bavix\Wallet\Traits\HasWallet;
use Bavix\Wallet\Interfaces\Wallet;

use Bavix\Wallet\Interfaces\Product;
use Bavix\Wallet\Interfaces\Customer;
use App\Events\OrderCreatedWallet;
use App\Jobs\DepositBalanceJob;
use App\Jobs\NotifyVendorJob;
use Illuminate\Database\Eloquent\Relations\MorphOne;
use Auth;
use Log;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\MorphTo;

class VendorOrder extends Model implements AuthenticatableContract, AuthorizableContract, CanResetPasswordContract, Wallet
{
    use Authenticatable, Authorizable, CanResetPassword;
    use Notifiable;
    use HasPermissionsTrait;
    use HasWallet;
    use HasFactory;

    protected $dispatchesEvents = [
        'created' => \App\Events\VendorOrderCreatedWallet::class,
    ];

    protected $guard = 'vendororder';
    protected $with = ['wallet'];
    use HasVouchers, CanRedeemVouchers {
        HasVouchers::vouchers as vouchers1;
        CanRedeemVouchers::vouchers as vouchers2;
    }
    public function canBuy(Customer $customer, int $quantity = 1, bool $force = null)
    {
        /**
         * If the service can be purchased once, then
         *  return !$customer->paid($this);
         */
        return true;
    }

    public function getAmountProduct()
    {
        $count = \App\Models\Order::where('customerMobile', $this->customerMobile)
            ->count();
        if ($count == 0) {
            $count = \App\Models\VendorOrder::where('customerMobile', $this->customerMobile)
                ->count();
            if ($count == 0)
                return 0;
        }
        if ((($this->amountPayable - $this->deliveryCharge) * (0.2) * 100) > $this->balance)
            return $this->balance;
        return ($this->amountPayable - $this->deliveryCharge) * (0.2);
    }
    public function getAmountProduct1($amountPayable)
    {
        /*
            $count = \App\Order::where('customerMobile', $this->customerMobile)
                    ->count();
            if($count==0){
                $count = \App\VendorOrder::where('customerMobile', $this->customerMobile)
                        ->count();
                if($count==0)
                    return 0;
            }
             *
             */
        if (($amountPayable) * (0.2) > $this->balance)
            $wDiscount = $this->balance;
        else
            $wDiscount = round(($amountPayable) * (0.2));

        if ($wDiscount > 200)
            $wDiscount = 200;
        return $wDiscount;
    }

    public function getMetaProduct()
    {
        return [
            'title' => $this->title,
            'description' => 'Purchase of Product #' . $this->id,
            'price' => $this->getAmountProduct(),
        ];
    }

    public function getUniqueId()
    {
        return (string)$this->getKey();
    }
    /**
     * The database table used by the model.
     *
     * @var string
     */
    protected $table = 'vendor_orders';

    /**
     * The attributes excluded from the model's JSON form.
     *
     * @var array
     */
    protected $rememberTokenName = 'remember_token';
    protected $guarded = [
        'created_at', 'updated_at'
    ];
    protected $hidden = [
        'remember_token',
        'api_token'
    ];
    public function getAuthPassword()
    {
        return $this->password;
    }
    public function vouchers()
    {
        $this->vouchers2();
    }
    public function canRedeemCode(string $code)
    {
        try {
            $voucher = $this->check($code);

            if ($voucher->users()->wherePivot('vendor_order_Id', $this->id)->exists()) {
                throw VoucherAlreadyRedeemed::create($voucher);
            }
            if ($voucher->isExpired()) {
                throw VoucherExpired::create($voucher);
            }

            return true;
        } catch (\Exception $e) {
            return  false;
        }
    }
    public function check(string $code)
    {
        $voucher = Voucher::whereCode($code)->first();
        if ($voucher === null) {
            throw VoucherIsInvalid::withCode($code);
        }
        if ($voucher->isExpired()) {
            throw VoucherExpired::create($voucher);
        }

        return $voucher;
    }
    public function redeemCode(string $code)
    {
        $voucher = $this->check($code);

        if ($voucher->users()->wherePivot('vendor_order_Id', $this->id)->exists()) {
            throw VoucherAlreadyRedeemed::create($voucher);
        }
        if ($voucher->isExpired()) {
            throw VoucherExpired::create($voucher);
        }

        $this->vouchers2()->attach($voucher, [
            'redeemed_at' => now()
        ]);

        event(new VoucherRedeemed($this, $voucher));

        return $voucher;
    }

    // protected $casts = ['id' => 'bigint'];

    protected $primaryKey = 'id';
    public $incrementing = false;
    protected $keyType = 'bigint';
    protected $fillable = [
        'id',
        'customer',
        'customerEmail',
        'customerMobile',
        'outlet',
        'bookingDate',
        'deliveryDate',
        'pnr',
        'trainNo',
        'trainName',
        'stationCode',
        'stationName',
        'coach',
        'berth',
        'totalAmount',
        'gst',
        'deliveryCharge',
        'discountAmount',
        'orderItems',
        'amountPayable',
        'paymentType',
        'razorpay_payment_id',
        'createdAt',
        'updatedAt',
        'orderId',
        'deliveryDateTime',
        'deliveryBoy',
        'vendorPayment',
        'remember_token',
        'password',
        'wallet_id',
        'next_id',
        'next_type',
        'previous_id',
        'previous_type',
        'vendor',
        'created_at',
        'api_token'
    ];

    protected static function boot()
    {   
       
        parent::boot();
        static::created(function ($order) {
            $order->user_id = $order->outlet()->first()->user_id;

            event(new OrderCreatedEvent($order));
        });
        
        
        static::updated(function($order){
            if(!$order->status == 'ORDER_DELIVERED')
                event(new \App\Jobs\CustomerInvoiceJob($order));
        });
    }
    public function outlet()
    {
        return $this->belongsTo(Outlet::class, 'outlet');
    }
    public function outlet1()
    {
        return $this->belongsTo(Outlet::class, 'outlet');
    }
    public function deliveryBoy()
    {
        return $this->hasOne(\App\Models\DeliveryBoy::class);
    }
    public function scopeApprovalReq($query)
    {
        return $query->where(function ($q) {
            $q->where('pending_status', '=', 'ORDER_CANCELLED')
                ->orWhere('pending_status', '=', 'ORDER_DELIVERED')
                ->orWhere('pending_status', '=', 'ORDER_PREPARED')
                ->orWhere('pending_status', '=', 'ORDER_OUT_FOR_DELIVERY');
        });
    }
    public function previous1()
    {
        if ($this->previous_type == 'App\Models\Order') {
            return $this->morphOne(\App\Models\Order::class, 'next');
            //Log::channel('dailyInfo')->info('--getting123:');
            //return $this->hasOne(\App\Order::class,'next_id','id');

        } else
            return $this->hasOne(\App\Models\VendorOrder::class, 'next_id', 'id');
    }
    public function next1()
    {
        if ($this->next_type == 'App\Models\Order')
            return $this->morphOne(\App\Models\Order::class, 'previous');
        else
            return $this->hasOne(\App\Models\VendorOrder::class, 'previous_id', 'id');
    }
    public function next(): MorphTo
    {
        return $this->morphTo();
    }
    public function first()
    {
        if ($this->previous1 == null)
            return $this;
        return $this->previous1->first();
    }
    public function last()
    {
        if ($this->next1 == null)
            return $this;
        return $this->next1->last();
        //Log::channel('dailyInfo')->info('--getting:here'.print_r($next,true));
    }
    public function getPreviousOrders($orders)
    {
        $order = clone $this;
        unset($order->status_reason);
        unset($order->status_status);
        unset($order->pending_status);
        unset($order->requestor_user_id);
        unset($order->status_history);
        unset($order->discount_code);
        unset($order->deliveryBoy);
        unset($order->vendorPayment);
        unset($order->password);
        unset($order->remember_token);
        unset($order->previous_id);
        unset($order->previous_type);
        unset($order->next_id);
        unset($order->next_type);
        unset($order->vendor);
        unset($order->updated);
        unset($order->api_token);
        unset($order->wallet_id);
        unset($order->otp);
        unset($order->razorpay_payment_id);
        unset($order->vendorPayment);
        $order->orderItems = json_decode($order);
        $order->customer = json_decode($order->customer);

        $orders[] = $order;
        if ($this->previous1 == null)
            return $orders;
        return $this->previous1->getPreviousOrders($orders);
    }

    public static function count($customerMobile)
    {
        // $res = DB::select(DB::raw("select created_at, count(orderId) as total from ((select orders.created_at as created_at, orders.orderId as orderId from `orders` where `customerMobile` = :customerMobile) union all (select vendor_orders.created_at as created_at, vendor_orders.orderId as orderId from `vendor_orders` where `customerMobile` = :customerMobile1)) created_at order by created_at desc"), array(
        //     'customerMobile' => $customerMobile, 'customerMobile1' => $customerMobile
        // ));

        // if (isset($res) && is_array($res)) {
        //     return $res[0]->total;
        // } else
        //     return 0;

        $res = Order::where('customerMobile', $customerMobile)
            ->select('created_at', \DB::raw('count(orderId) as total'))
            ->unionAll(
                VendorOrder::where('customerMobile', $customerMobile)
                    ->select('created_at', \DB::raw('count(orderId) as total'))
            )
            ->orderBy('created_at', 'desc')
            ->get();

        if ($res->isNotEmpty()) {
            return $res->first()->total;
        } else {
            return 0;
        }

    }
    public function previous(): MorphTo
    {
        return $this->morphTo();
    }

    public static function latest($customerMobile, $except = null)
    {
        // if (!isset($except))
        //     $res = DB::select(DB::raw("select created_at, orderId, vendor from ((select orders.created_at as created_at,  orders.orderId as orderId, orders.vendor as vendor from `orders` where `customerMobile` = :customerMobile) union all (select  vendor_orders.created_at as created_at,  vendor_orders.orderId as orderId, vendor_orders.vendor as vendor from `vendor_orders` where `customerMobile` = :customerMobile1)) created_at order by created_at desc limit 1"), array(
        //         'customerMobile' => $customerMobile, 'customerMobile1' => $customerMobile
        //     ));
        // else
        //     $res = DB::select(DB::raw("select created_at, orderId, vendor from ((select orders.created_at as created_at,  orders.orderId as orderId, orders.vendor as vendor from `orders` where `customerMobile` = :customerMobile) union all (select  vendor_orders.created_at as created_at,  vendor_orders.orderId as orderId, vendor_orders.vendor as vendor from `vendor_orders` where `customerMobile` = :customerMobile1)) created_at where orderId != :except order by created_at desc limit 1"), array(
        //         'customerMobile' => $customerMobile, 'customerMobile1' => $customerMobile, 'except' => $except
        //     ));
            
        if (!isset($except)) {
            $query = DB::table('orders')
                ->selectRaw('created_at, orderId, vendor')
                ->where('customerMobile', $customerMobile);
        
            $unionQuery = DB::table('vendor_orders')
                ->selectRaw('created_at, orderId, vendor')
                ->where('customerMobile', $customerMobile)
                ->unionAll($query);
        
            $res = $unionQuery->orderBy('created_at', 'desc')->limit(1)->get();
        } else {
            $query = DB::table('orders')
                ->selectRaw('created_at, orderId, vendor')
                ->where('customerMobile', $customerMobile);
        
            $unionQuery = DB::table('vendor_orders')
                ->selectRaw('created_at, orderId, vendor')
                ->where('customerMobile', $customerMobile)
                ->unionAll($query)
                ->where('orderId', '!=', $except);
        
            $res = $unionQuery->orderBy('created_at', 'desc')->limit(1)->get();
        }
        
    
        if (isset($res) && isset($res[0])) {
            if ($res[0]->vendor) {
                return \App\Models\VendorOrder::where('orderId', $res[0]->orderId)->first();
            } else
                return \App\Models\Order::where('orderId', $res[0]->orderId)->first();
        } else
            return null;
    }
    /*
    public function wallet(): MorphOne
    {
        if($this instanceof WalletModel)
            $ref = $this->holder;
        else
            $ref = $this;
        return $ref->morphOne(config('wallet.wallet.model'), 'holder')
                        ->where('slug', config('wallet.wallet.default.slug'));
    }
     *
     */
    public function attachWallet(): MorphOne
    {
        $withDefaults = true;
        $controller = app()->make('\App\Http\Controllers\Auth\OrderLoginController');
        $user = $controller->getUser($this->customerMobile, true);

        if ($user != null) {
            $vo = $user['user'];
            if ($vo != null)
                $vendor = $user['vendor'];
            $user = $vo;
            if ($vo != null && $vo->orderId != $this->orderId) {
                $withDefaults = false;
                $classVendor = false;
                $thisVendor = get_class($this) == 'App\Models\VendorOrder' ? true : false;
                if ($vendor == $thisVendor && $vendor || $vendor != $thisVendor && !$vendor)
                    $classVendor = true;
                if ($vendor) {
                    $holderId = $vo->id;
                } else
                    $holderId = $vo->orderId;
                if ($thisVendor)
                    $thisHolderId = $this->id;
                else
                    $thisHolderId = $this->orderId;
                DB::table('wallets')
                    ->where('holder_id', $holderId)
                    ->update(['holder_id' => $thisHolderId, 'holder_type' => get_class($this)]);
                $wallet = DB::table('wallets')
                    ->where('holder_id', $thisHolderId)->first();
                DB::table('transactions')
                    ->where('wallet_id', $wallet->id)
                    ->update(['payable_type' => $wallet->holder_type, 'payable_id' => $thisHolderId]);
                //Log::channel('dailyInfo')->info('sms response:'.print_r(get_class($this).$this->orderId.'-'.$vo->orderId,true).'--');
                if ($classVendor) {
                    \App\Models\VendorOrder::where('orderId', $this->orderId)->update(['wallet_id' => $wallet->id]);
                } else
                    \App\Models\Order::where('orderId', $this->orderId)->update(['wallet_id' => $wallet->id]);
                //$this->setRelation('wallet',$wallet);
                //Log::channel('dailyInfo')->info('--gettingwallet1'.$wallet->id);
                $this->wallet_id = $wallet->id;
                $this->save();
            }
        }
        if ($this instanceof WalletModel)
            $ref = $this->holder;
        else
            $ref = $this;

        if ($withDefaults)
            return $ref->morphOne(config('wallet.wallet.model'), 'holder')
                ->where('slug', config('wallet.wallet.default.slug'))
                ->withDefault([
                    'name' => config('wallet.wallet.default.name'),
                    'slug' => config('wallet.wallet.default.slug'),
                    'balance' => 0,
                ]);
        else return $ref->morphOne(config('wallet.wallet.model'), 'holder')
            ->where('slug', config('wallet.wallet.default.slug'));
    }


    public function publish()
    {
        dispatch(new NotifyVendorJob($this));
        dispatch(new DepositBalanceJob($this));
    }
}
