<?php

namespace App\Providers;


use App\Repositories\BlogRepository;
use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Schema;
use Illuminate\Support\Facades\View;
use App\Repositories\StationContentRepository;

Route::singularResourceParameters(false);

class AppServiceProvider extends ServiceProvider
{

	/**
	 * Bootstrap any application services.
	 *
	 * @return void
	 */
	public function boot()
	{
		Schema::defaultStringLength(191);
		$filePath = base_path() . '/config.json';
		putenv("FIREBASE_CREDENTIALS=$filePath");

		View::composer('*', function ($view) {
			$guards = ['memoryuser', 'order', 'vendororder'];
			$user_ = collect($guards)->map(function ($gaurd) {
				return auth($gaurd)->user();
			})->filter(function ($x) {
				return $x != null;
			})->first();
			return $view->with('Customer', $user_);
		});
	}

	/**
	 * Register any application services.
	 *
	 * This service provider is a great spot to register your various container
	 * bindings with the application. As you can see, we are registering our
	 * "Registrar" implementation here. You can add your own bindings too!
	 *
	 * @return void
	 */
	public function register()
	{
		$this->app->bind(
			'Illuminate\Contracts\Auth\Registrar',
			'App\Services\Registrar'
		);

		$this->app->bind(BlogRepository::class, function ($container) {
			return new BlogRepository();
		});

		$this->app->bind(StationContentRepository::class, function ($container) {
			return new StationContentRepository();
		});
	}
}
