<?php

namespace App\Providers;


use App\Models\Order;
use App\Models\Menu;
use App\Models\MenuItem;
use App\Models\Outlet;
use App\Models\Station;
use App\Models\User;
use App\Models\Token;
use App\Policies\OrderPolicy;
use App\Policies\MenuPolicy;
use App\Policies\MenuItemPolicy;
use App\Policies\OutletPolicy;
use App\Policies\StationPolicy;
use App\Policies\UserPolicy;
use App\Extensions\AccessTokenGuard;
use App\Extensions\TokenToUserProvider;
use App\Extensions\TokenToUserMobileProvider;


use Illuminate\Contracts\Auth\Access\Gate as GateContract;
use Illuminate\Foundation\Support\Providers\AuthServiceProvider as ServiceProvider;

use Gate;
use Auth;
use App;
use Log;

class AuthServiceProvider extends ServiceProvider
{
    /**
     * The policy mappings for the application.
     *
     * @var array
     */
    protected $policies = [
      
          User::class =>UserPolicy::class,
          Order::class =>OrderPolicy::class,
          Menu::class =>MenuPolicy::class,
          MenuItem::class =>MenuItemPolicy::class,
          Outlet::class =>OutletPolicy::class,
          Station::class =>StationPolicy::class,
    ];

    /**
     * Register any application authentication / authorization services.
     *
     * @param  \Illuminate\Contracts\Auth\Access\Gate  $gate
     * @return void
     */
    public function boot()
    {   
        $this->registerPolicies();
        $this->register();
        $user = new User();
        $token = new Token();
        $order = new Order();

        $this->app->bind('App\Extensions\TokenToUserProvider', function ($app) use($user, $token) {
          return new TokenToUserProvider($user, $token);
        });
        $this->app->bind('App\Extensions\TokenToUserMobileProvider', function ($app) use($user, $token) {
          return new TokenToUserMobileProvider($user, $token);
        });
        $this->app->bind('App\Extensions\TokenToOrderProvider', function ($app) use($order, $token) {
          return new App\Extensions\TokenToOrderProvider($order, $token);
        });
        Auth::extend('api_token_mobile', function ($app, $name, array $config) {
                    // automatically build the DI, put it as reference
                    $userProvider = App::make('App\Extensions\TokenToUserMobileProvider');
                    $request = app('request');
                    return new App\Extensions\MobileTokenGuard($userProvider, $request, $config, true);
            });
        Auth::extend('access_token', function ($app, $name, array $config) use($user, $token){
                    // automatically build the DI, put it as reference
                    $userProvider = App::make('App\Extensions\TokenToUserProvider');
                    $request = app('request');
                    return new AccessTokenGuard($userProvider, $request, $config, true);
            });
        Auth::extend('order_token', function ($app, $name, array $config) use($user, $token){
                    // automatically build the DI, put it as reference
                    $userProvider = App::make('App\Extensions\TokenToOrderProvider');
                    $request = app('request');
                    return new App\Extensions\OrderTokenGuard($userProvider, $request, $config, true);
            });
    }
}

