<?php

namespace App\Repositories;

use App\Models\BlogCategory;
use App\Models\Post;

class BlogRepository{
    public function postByCategory($slug, $byIndex = false)
    {
        $category = $this->findCategory($slug);

        if ($category == null) {
            return [
                'category' => $this->trimPadding($slug),
                'posts' => []
            ];
        }
        $posts = $category->posts->reverse()->map(function($post){
            $_content = base64_decode($post->content);
            $post->content = $_content;
            $post->url = route('blog.postBySlug', ['slug' => $post->slug]);
            return $post;
        });

        return $byIndex ? [$category, $posts] :[
            'category' => $category->name,
            'posts' => $posts
        ];

    }


    public function categorySet()
    {
        return BlogCategory::all()->map(function($cat){
            $cat->url = route('blog.categoryPosts', ['slug' => $cat->slug]);
            return $cat;
        });
    }


    public function recentPosts()
    {
        return Post::orderBy('created_at','DESC')->limit(5)->get()->map(function($post){
            $post->url = route('blog.postBySlug', ['slug' => $post->slug]);
            return $post;
        });
    }

    public function lastPost()
    {
        return Post::orderBy('created_at', 'DESC')->limit(1)->get()->first();
    }


    public function createCategory($payload)
    {
        $category = BlogCategory::firstOrCreate($payload);
        return $category;
    }


    public function findCategory($slug)
    {
        $category = BlogCategory::where('slug','=',$slug)->first();
        return $category;
    }

    public function getPost($slug)
    {
        $post = Post::where('slug','=', $slug)->first();
        return $post;
    }

    public function createPost($categoryId, $payload)
    {
        $c = BlogCategory::find($categoryId);
        $post = $c->posts()->create($payload);
        return $post;
    }

    public  function trimPadding($value)
    {
        return preg_replace('/\-/',' ', $value);
    }

    public function editPost($postId, $payload)
    {

        $posted = Post::find($postId);
        $posted->fill($payload);
        $posted->content = base64_encode($payload['content']);
        $posted->save();

    }



    public function removePost($postId)
    {
        return Post::find($postId)->delete();
    }

    public function editCategory($categoryId, $name)
    {
        $slug = $this->generateSlug($name);
        $category = BlogCategory::find($categoryId);
        $category->fill(compact('name','slug'));
        $category->save();
    }


    public function generateSlug($value){
        return preg_replace("/\s/","-", $value);
    }

    public function encodeFile($path)
    {

        $data =  base64_encode(file_get_contents($path));
        return gzcompress($data, 9);
    }

    public function decode($data)
    {
        return gzuncompress($data);
    }



    public function encodeData($dataString)
    {
        return gzencode($dataString, 9);
    }

}
