<?php

namespace App\Repositories;

use App\Models\Station;

class StationContentRepository
{

    public function getStations()
    {
        return Station::all()->map(function($station){
            return $this->format($station);
        });
    }

    public function format(Station $station)
    {
        $meta = $this->genSlug($station);
        $code = trim($meta[1]);
        $name = strtolower($meta[0]);
        $station->title = "order-food-in-train-at-{$name}-{$code}";
        return $station;
    }

    public function genSlug(Station $station)
    {
        $stationName = preg_replace("/\s/", "-", trim($station->name));
        $code = $station->code;
        return [$stationName, $code];
    }

    public function fetchContent($code)
    {

        if ($station = $this->fetchStation($code)) {
            if ($content = $station->content) {
                $title = $this->format($station)->title;
                $station->title = $title;
                return [$station, $content];
            }
            return [$station, null];
        }
        return null;

    }

    private function fetchStation($code)
    {
        return Station::where('code', '=', $code)->first();
    }

    public function update($code, array $content)
    {
        $station =  $this->fetchStation($code);
        return $station->content()->updateOrCreate(['station_id' => $station->id], $content);
    }

    public function delete($code)
    {
        return $this->fetchStation($code)->content()->delete();
    }
}
