<?php

namespace App\Services;

use DOMDocument;
use DOMXPath;
use GuzzleHttp\Client;
use ParagonIE\Sodium\Compat;

class ScrappingService
{
    private $client;

    function __construct(Client $client)
    {
        $this->client = $client;
    }



    private function processNodeContent($content){
        list($train, $_status) = preg_split('/\-/', $content);
        $status = preg_replace("/\n/", " ", preg_split("/\|/", $_status)[0] );
        return compact('train', 'status');
    }

    private function processContent($_content){
        //dd(strpos($_content, 'Late'));
        $message = $_content;
        $parts = [];
        $status = "";
        if (str_contains($_content,"Reached its Destination")) {
            $status = 'JOURNEY_COMPLETED';
        } else if(str_contains($_content, 'Not Started')){
            $status = 'AWAITING_DEPARTURE';
        } else if(str_contains($_content,'Late')){
            $status = 'LATE';
            $parts = $this->parseLateResponse($_content);
        } else if(str_contains($_content,'On Time')){
            $status = 'ON_TIME';
            $parts = $this->parseOnTime($_content);
        }

        return compact('status', 'message', 'parts');
    }

    private function parseLateResponse($_content)
    {
        
     
       $date = "";
       $time = "";
       $station = "";

       preg_match_all("/(\d{2} \w{3}\, \d{4})/", $_content, $date);
       preg_match_all("/(\d{0,2}M Late)/", $_content, $time);
       preg_match_all("/from((\s\w*){0,3})/", $_content, $station);

       
       return ['date'=>$date[0][0], 'time'=> $time[0][0], 'lastDeparted'=> trim(substr($station[0][0], 4))];
    }

    private function parseOnTime($_content){
        $date = "";
       $station = "";

       preg_match_all("/(\d{2} \w{3}\, \d{4})/", $_content, $date);
       preg_match_all("/from((\s\w*){0,3})/", $_content, $station);

       return ['date'=>$date[0][0], 'LastDeparted'=> trim(substr($station[0][0], 4))];

    }
    
    public function load($url)
    {
        $_response = $this->client->get($url);

        $_content = $_response->getBody();


        $__content =  (string)$_content;

        $document = new \DOMDocument('1.0', 'UTF-8');

        // set error level
        $internalErrors = libxml_use_internal_errors(true);

        // load HTML
        $document->loadHTML($__content);

        // Restore error level
        libxml_use_internal_errors($internalErrors);

        $__path = new DOMXPath($document);

        $_C = $__path->query('/html/body/div[1]/div/div/div/div/div[1]/div/div[1]');

        $__nodes = [];

        foreach ($_C as $key => $value) {
            array_push($__nodes, $value);
        }

        return $this->processContent($this->processNodeContent($__nodes[0]->textContent)['status']);
    }
}
