<?php

namespace App\Services;

use App\Models\Order;
use App\Models\VendorOrder;

class WaMessageFormatterService
{

    public function getMessage($klass, $instance,$type)
    {
        if($type == "CUSTOMER"){
            return $this->formatForCustomer($klass, $instance);
        }else{
            return $this->formatForVendor($klass, $instance);
        }
    }

    public function formatForCustomer($klass, $instance)
    {
        $message = "";
        switch ($klass) {
            case Order::class:
                $message = $this->_prepareTemplate('wa.customer', $this->_getOrderArgs($instance));
                break;

            case VendorOrder::class:
                $message = $this->_prepareTemplate('wa.customer', $this->_getPayloadOVendorOrderArgs($instance, true));
                break;

            default:
                $message = $this->_prepareTemplate('wa.customer', $this->_getPayloadOVendorOrderArgs($instance, false));
                break;
        }

        return $message;
    }


    public function formatForVendor($klass, $instance)
    {
        $message = "";
        switch ($klass) {
            case Order::class:
                $message = $this->_prepareTemplate('wa.vendor', $this->_getOrderArgs($instance));
                break;

            case VendorOrder::class:
                $message = $this->_prepareTemplate('wa.vendor', $this->_getPayloadOVendorOrderArgs($instance, true));
                break;

            default:
                $message = $this->_prepareTemplate('wa.vendor', $this->_getPayloadOVendorOrderArgs($instance, false));
                break;
        }
        return $message;
    }

    private function _prepareTemplate($templateName, $templateArgs)
    {
        return view($templateName, $templateArgs)->render();
    }



    private function _getPayloadOVendorOrderArgs($data, $decode)
    {
        
        $_details = $data->customer ?? $data->customerDetails;
        $customer = $decode ? json_decode($_details) : $_details;
        $orderId =  $data->orderId ?? $data->id;
        
       
        $items = collect($decode ? json_decode($data->orderItems) : $data->orderItems)->map(function ($e) {
            return "{$e->quantity}-{$e->itemName}";
        })->implode(", ");

        
        $deliveryDate = substr($data->deliveryDate, 0, 10);
        $eta =  substr($decode ? $data->deliveryDateTime : $data->deliveryDate, 0, 16);
        
        $customerName = $customer->customerName ?? $customer->fullName;
        $mobile = $customer->mobile;
        
        $trainNo = $data->trainNo;
        $trainName = $data->trainName;
        $coach = $data->coach;
        $berth = $data->berth;
        $station = $data->stationName;
        
        $amount = $data->amountPayable;
        $paymentMode = $data->paymentType;
        
        $link = "https://bit.ly/3i6Hnt4";

        return compact('orderId', 'items', 'deliveryDate', 'eta', 'customerName', 'mobile', 'trainNo', 'trainName', 'coach', 'berth', 'station', 'amount', 'paymentMode', 'link');
    }


    /* 
    {"pnr":"8642067325","trainNo":"02025","trainName":"NGP ASR AC SPL","station":"Ludhiana Jn","stationCode":"LDH","berth":"27","coach":"A2","ETA":"09-26-2021 16:45 IST","ETD":"09-26-2021 16:55 IST"}
    */

    private function _getOrderArgs($data)
    {
        $customer = json_decode($data->customerDetails);
        $_delv = json_decode($data->deliveryDetails);

        $orderId =  $data->orderId ?? $data->id;

        $items = collect(json_decode($data->orderItems))->map(function ($e) {
            return "{$e->quantity}-{$e->itemName}";
        })->implode(", ");

        $deliveryDate = substr($data->deliveryDate, 0, 10);
        $eta =  substr($data->deliveryDateTime, 0, 16);

        $customerName = $customer->customerName;
        $mobile = $customer->mobile;

        $trainNo = $_delv->trainNo;
        $trainName = $_delv->trainName;
        $coach = $_delv->coach;
        $berth = $_delv->berth;
        $station = $_delv->station;


        $amount = $data->amountPayable;
        $paymentMode = $data->paymentType;

        $link = "https://bit.ly/3i6Hnt4";

        return compact('orderId', 'items', 'deliveryDate', 'eta', 'customerName', 'mobile', 'trainNo', 'trainName', 'coach', 'berth', 'station', 'amount', 'paymentMode', 'link');
    }
}
