<?php

namespace App\Services;

use App\Models\Order;
use App\Repositories\WARepository;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Log;
use App\Services\WaMessageFormatingService;
use App\Models\VendorOrder;

class WaService
{
    private $repository = null;
    private $client = null;
    private $userId = null;
    private $password = null;
    private $baseUrl = null;
    private $formatter = null;

    function __construct(WARepository $repository, Client $client)
    {
        $this->repository = $repository;
        $this->client = $client;
        $this->userId = config("wa.username");
        $this->password = config("wa.password");
        $this->baseUrl = config("wa.baseUrl");
        $this->formatter = new WaMessageFormatterService();
    }


    private function checkConsent($number, $customer)
    {
        $result = $this->repository->checkIfExists($number, $customer);
        return $result;
    }


    public function sendMessageIRCTC($number, $order, $c=false){

        if(!$this->checkConsent($number, 'xyz')){
            $this->_sendConsent($number, $c? 'WA_VENDOR':'WA_USER' );
        }

        try {
           $response =  $this->_sendMessageIRCTC($number, $order,$c);

           Log::channel('dailyInfo')->info(["WA/$number" => $response]);
           
           if($response->status == 'error'){
               Log::info($response);
           }
        } catch (\Throwable $th) {
            //throw $th;
        }
    }

    public function _sendMessageIRCTC($number, $order, $c=false){
        try {
            $message = $c ? $this->formatter->getMessage(Order::class, $order, "CUSTOMER") : $this->formatter->getMessage(Order::class ,$order, "VENDOR");
            $body = [
                'auth_scheme' => 'plain',
                'send_to' => '91' . $number,
                'msg_type' => 'Text',
                'userid' => $this->userId,
                'password' => $this->password,
                'method' => 'SendMessage',
                'v' => '1.1',
                'format' => 'json',
                'msg' => sprintf("%s", $message)
            ];

            // dd($message->render());

            $response = $this->client->request('POST', $this->baseUrl, [
                'form_params' => $body
            ]);

            $_data =  json_decode((string)$response->getBody());
            $_response = $_data->response;
            return $_response;

        } catch (\Exception $th) {
            Log::info($th);
        }
    }

    /* payload should be buffer order result */

    public function  sendMessageVendorOrders($number, $order, $c)
    {
        if(!$this->checkConsent($number, 'xyz')){
            $this->_sendConsent($number, $c ? 'WA_VENDOR':'WA_USER' );
        }

        try {
           $response =  $this->_sendMessageVendorOrders($number, $order,$c);
        
           Log::channel('dailyInfo')->info(["WA No. / $number" => $response->status]);
           if($response->status == 'error'){
            //    Log::info((array)$response);
           }
        } catch (\Exception $th) {
            Log::info($th);
        }
    }

    public function  _sendMessageVendorOrders($number, $order, $c)
    {
        try {
            $message = $c ? $this->formatter->getMessage(VendorOrder::class, $order, "CUSTOMER") : $this->formatter->getMessage(VendorOrder::class,$order, "VENDOR");
            $body = [
                'auth_scheme' => 'plain',
                'send_to' => '91' . $number,
                'msg_type' => 'Text',
                'userid' => $this->userId,
                'password' => $this->password,
                'method' => 'SendMessage',
                'v' => '1.1',
                'format' => 'json',
                'msg' => sprintf("%s", $message)
            ];

            // dd($message->render());

            $response = $this->client->request('POST', $this->baseUrl, [
                'form_params' => $body
            ]);

            $_data =  json_decode((string)$response->getBody());
            $_response = $_data->response;
           return $_response;
        } catch (\Exception $th) {
            Log::info($th);
        }
    }


    public function  sendMessageBuffers($number, $order, $c){
        if(!$this->checkConsent($number, 'xyz')){
            $this->_sendConsent($number, $c ? 'WA_VENDOR':'WA_USER' );
        }

        try {
    
           $response =  $this->_sendMessageBuffers($number, $order,$c);

           if($response->status == 'error'){
               Log::info($response);
           }

           Log::channel('dailyInfo')->info(["WA/$number" => $response]);
        } catch (\Throwable $th) {
            //throw $th;
        }
    }

    public function  _sendMessageBuffers($number, $order, $c)
    {
        try {
            $message = $c ? $this->formatter->getMessage(null, $order, "CUSTOMER") : $this->formatter->getMessage(null,$order, "VENDOR");
            
            $body = [
                'auth_scheme' => 'plain',
                'send_to' => '91' . $number,
                'msg_type' => 'Text',
                'userid' => $this->userId,
                'password' => $this->password,
                'method' => 'SendMessage',
                'v' => '1.1',
                'format' => 'json',
                'msg' => sprintf("%s", $message)
            ];

            // dd($message->render());

            $response = $this->client->request('POST', $this->baseUrl, [
                'form_params' => $body
            ]);

            $_data =  json_decode((string)$response->getBody());
            $_response = $_data->response;

            return $_response;

        } catch (\Exception $th) {
            Log::info($th);
        }
    }

    private function _sendConsent($number, $customer)
    {
        $query = http_build_query([
            'method' => 'OPT_IN', 'format' => 'json', 'userid' => '2000200435', 'password' => 'SydwWaLn', 'phone_number' => $number, 'v' => '1.1', 'auth_scheme' => 'plain', 'channel' => 'WHATSAPP'
        ]);
        $url = sprintf("%s?%s", $this->baseUrl, $query);
        $_response = $this->client->get($url);

        $_data =  json_decode((string)$_response->getBody());
        $response = $_data->response;
        if ($response->status == 'success') {
            Log::info('[WA] User Opted In');
            $this->repository->save($number, $customer ? 'WA_CUSTOMER' : 'WA_VENDOR');
        }
    }

    
}
