/// <reference path="vue.js" />

const fetchFeedbacks = async (url) => {
    return await fetch(url).then(_r => _r.json()).catch(err => ({ 'message': 'Cannot Fetch' }))
}


Vue.component('feedback-row', {
    props: ['feedback'],
    template:
        ` <tr>
        <td>
            <a href='#'>
                {{feedback.id}}
            </a>
        </td>

        <td>
            <a href='#'>
                {{feedback.orderId}}
            </a>
        </td>

        <td>
           Outlet: {{feedback.outletName}}
           Vendor: {{feedback.vendorName}}
        </td>

        <td>
            {{feedback.payment_type}}
        </td>

        <td>
            {{feedback.amountPayable}}
        </td>

        <td>
             {{feedback.starRating}}
        </td>

        <td>
            {{feedback.feedbackText ?? "-"}}
        </td>

        <td>
            <span v-if="feedback.complain" class="badge badge-danger" > Complain</span>
            <span v-else class="badge badge-success" > Feedback</span>
        </td>
        
        <td>
            {{feedback.createdAt}}
        </td>
    </tr>
    `
})

const app = new Vue({
    'el': '#feedbacks',

    data: {
        'headline': 'Feedback Management',
        'subtitle': 'Listing all the feedbacks Here',
        'baseUrl': 'https://www.olfstores.com/manager/feedback',
        'exportUrl':'https://www.olfstores.com/manager/feedbacks/exports',
        'feedbacks': [],
        'segment': {
            'complainFromDate': '',
            'complainToDate': '',
            'orderId': '',
            'feedbackId': '',
            'stationCode': '',
            'complainType': '',
            'paymentType': ''
        },

        'loading':false
    },

    mounted: function () {
        this.loadFeedbacks(this.baseUrl)
    },

    methods: {
        loadFeedbacks: async function (url) {
            this.loading = true
            try {
                let _data = await fetchFeedbacks(url)
                const { result: { feedbacks } } = _data
                this.feedbacks = feedbacks
                this.loading = false
            } catch (error) {
                this.loading = false
            }

        },

        reset: function () {
            this.segment = {
                'page':1,
                'complainFromDate': '',
                'complainToDate': '',
                'orderId': '',
                'feedbackId': '',
                'stationCode': '',
                'complainType': '',
                'paymentType': ''
            };
        },

        makeRequest: function () {

            const uri = this.baseUrl + "?" + Object.entries(this.segment).filter(
                ([k, v]) => v != ""
            )
                .map(([k, v]) => {
                    if (['complainFromDate', 'complainToDate'].includes(k)) {
                        const _parts = v.split('T');
                        const _value = _parts[0] + " " + _parts[1].padEnd(5, ":00") + " IST"
                        return [k, _value]
                    }
                    return [k, v]
                })
                .map(([k, v]) => `${k}=${v}`).join('&')


            this.loadFeedbacks(encodeURI(uri))


        },

        nextPage:function(){
            
            this.$set(this.segment, 'page', this.segment.page != undefined ? this.segment.page +1 : 2)
            this.makeRequest()
            
        },
        prevPage:function(){
            if(this.segment.page > 1){
                this.$set(this.segment, 'page', this.segment.page != undefined ? this.segment.page -1 : 1)
                this.makeRequest()
            }
        },


        exports:function(){
            const uri = this.exportUrl + "?" + Object.entries(this.segment).filter(
                ([k, v]) => v != ""
            )
                .map(([k, v]) => {
                    if (['complainFromDate', 'complainToDate'].includes(k)) {
                        const _parts = v.split('T');
                        const _value = _parts[0] + " " + _parts[1].padEnd(5, ":00") + " IST"
                        return [k, _value]
                    }
                    return [k, v]
                })
                .map(([k, v]) => `${k}=${v}`).join('&')


               var link = encodeURI(uri)
            
                window.location.href= link

        }

    }
})


window.app = app;
